@@ -28,35 +28,40 @@
 #include <QRect>
 #include <QWidgetAction>
 
+#include "appmodes.h"
 #include "canvas.h"
 #include "fpoint.h"
 #include "fpointarray.h"
-#include "hruler.h"
-#include "vruler.h"
 #include "hyphenator.h"
-#include "insertTable.h"
-#include "oneclick.h"
+#include "pageitem_table.h"
 #include "pageitem_textframe.h"
-#include "pageselector.h"
 #include "prefscontext.h"
 #include "prefsfile.h"
 #include "prefsmanager.h"
-#include "propertiespalette.h"
 #include "scraction.h"
-#include "scrapbookpalette.h"
 #include "scribus.h"
+#include "scribusXml.h"
 #include "scribusdoc.h"
 #include "scribusview.h"
-#include "scribusXml.h"
 #include "selection.h"
+#include "text/frect.h"
+#include "ui/basepointwidget.h"
+#include "ui/hruler.h"
+#include "ui/insertTable.h"
+#include "ui/oneclick.h"
+#include "ui/pageselector.h"
+#include "ui/propertiespalette.h"
+#include "ui/scrapbookpalette.h"
+#include "ui/vruler.h"
 #include "undomanager.h"
 #include "units.h"
 #include "util.h"
-#include "util_icon.h"
 #include "util_math.h"
 
 
 
+
+
 CreateMode::CreateMode(ScribusView* view) : CanvasMode(view) 
 {
 	canvasPressCoord.setXY(-1.0, -1.0);
@@ -64,9 +69,9 @@ CreateMode::CreateMode(ScribusView* view) : CanvasMode(view)
 	inItemCreation = false;
 	createObjectMode     = 0;
 	createObjectSubMode  = 0;
-	modifiers            = 0;
+	modifiers            = Qt::NoModifier;
 	m_MouseButtonPressed = false;
-	m_createTransaction  = NULL;
+	m_createTransaction  = nullptr;
 }
 
 
@@ -80,7 +85,7 @@ void CreateMode::drawControls(QPainter* p)
 
 	if (createObjectMode != modeDrawLine)
 	{
-		QRectF bounds=QRectF(topLeft, btRight).normalized();
+		QRectF bounds = QRectF(topLeft, btRight).normalized();
 		//Lock Height to Width for Control Modifier for region drawing
 		if (modifiers==Qt::ControlModifier)
 		{
@@ -90,7 +95,6 @@ void CreateMode::drawControls(QPainter* p)
 			if (btRight.x()<topLeft.x() && btRight.y()>topLeft.y())
 				bounds.moveTop(topLeft.y());
 		}
-
 		QRect localRect = m_canvas->canvasToLocal(bounds);
 		if (localRect.width() <= 0 || localRect.height() <= 0)
 			return;
@@ -112,9 +116,24 @@ void CreateMode::drawControls(QPainter* p)
 		{
 			p->drawEllipse(localRect);
 		}
+		else if (createObjectMode == modeDrawArc)
+		{
+			QPainterPath path;
+			path.moveTo(localRect.width() / 2.0, localRect.height() / 2.0);
+			path.arcTo(0.0, 0.0, localRect.width(), localRect.height(), m_doc->itemToolPrefs().arcStartAngle, m_doc->itemToolPrefs().arcSweepAngle);
+			path.closeSubpath();
+			p->translate(localRect.left(), localRect.top());
+			p->drawPath(path);
+		}
 		else if (createObjectMode == modeDrawRegularPolygon)
 		{
-			QPainterPath path = RegularPolygon(localRect.width(), localRect.height(), m_doc->toolSettings.polyC, m_doc->toolSettings.polyS, m_doc->toolSettings.polyF, m_doc->toolSettings.polyR, m_doc->toolSettings.polyCurvature);
+			QPainterPath path = regularPolygonPath(localRect.width(), localRect.height(), m_doc->itemToolPrefs().polyCorners, m_doc->itemToolPrefs().polyUseFactor, m_doc->itemToolPrefs().polyFactor, m_doc->itemToolPrefs().polyRotation, m_doc->itemToolPrefs().polyCurvature, m_doc->itemToolPrefs().polyInnerRot, m_doc->itemToolPrefs().polyOuterCurvature);
+			p->translate(localRect.left(), localRect.top());
+			p->drawPath(path);
+		}
+		else if (createObjectMode == modeDrawSpiral)
+		{
+			QPainterPath path = spiralPath(localRect.width(), localRect.height(), m_doc->itemToolPrefs().spiralStartAngle, m_doc->itemToolPrefs().spiralEndAngle, m_doc->itemToolPrefs().spiralFactor);
 			p->translate(localRect.left(), localRect.top());
 			p->drawPath(path);
 		}
@@ -159,7 +178,7 @@ void CreateMode::drawControls(QPainter* p)
 inline bool CreateMode::GetItem(PageItem** pi)
 { 
 	*pi = m_doc->m_Selection->itemAt(0); 
-	return (*pi) != NULL; 
+	return (*pi) != nullptr;
 }
 
 // the following code was moved from scribusview.cpp:
@@ -176,7 +195,6 @@ void CreateMode::enterEvent(QEvent *)
 
 void CreateMode::leaveEvent(QEvent *e)
 {
-
 }
 
 
@@ -189,9 +207,8 @@ void CreateMode::activate(bool fromGesture)
 		if (m_createTransaction)
 		{
 //			qDebug() << "canceling left over create Transaction";
-			m_createTransaction->cancel();
-			delete m_createTransaction;
-			m_createTransaction = NULL;
+			m_createTransaction.cancel();
+			m_createTransaction.reset();
 		}
 		canvasPressCoord.setXY(-1.0, -1.0);
 		mouseGlobalCoord.setXY(-1.0, -1.0);
@@ -208,9 +225,8 @@ void CreateMode::deactivate(bool forGesture)
 		if (m_createTransaction)
 		{
 //			qDebug() << "CreateMode::deactivate: canceling left over create Transaction";
-			m_createTransaction->cancel();
-			delete m_createTransaction;
-			m_createTransaction = NULL;
+			m_createTransaction.cancel();
+			m_createTransaction.reset();
 		}
 	}
 }
@@ -227,14 +243,15 @@ void CreateMode::mouseMoveEvent(QMouseEvent *m)
 {
 	const FPoint mousePointDoc = m_canvas->globalToCanvas(m->globalPos());
 	modifiers = m->modifiers();
-
+	
 	double newX, newY;
 	PageItem *currItem;
-	QPoint np, np2, mop;
 	QPainter p;
-	QRect tx;
+//	QRect tx;
 	m->accept();
 //	qDebug() << "legacy mode move:" << m->x() << m->y() << m_canvas->globalToCanvas(m->globalPos()).x() << m_canvas->globalToCanvas(m->globalPos()).y();
+//	emit MousePos(m->x()/m_canvas->scale(),// + m_doc->minCanvasCoordinate.x(), 
+//				  m->y()/m_canvas->scale()); // + m_doc->minCanvasCoordinate.y());
 
 	if (commonMouseMove(m))
 		return;
@@ -254,10 +271,10 @@ void CreateMode::mouseMoveEvent(QMouseEvent *m)
 			newY = mousePointDoc.y();
 			if (createObjectMode == modeDrawLine)
 			{
-				if (m_doc->useRaster)
+				if (m_doc->SnapGrid)
 				{
-					newX = qRound(newX / m_doc->guidesSettings.minorGrid) * m_doc->guidesSettings.minorGrid;
-					newY = qRound(newY / m_doc->guidesSettings.minorGrid) * m_doc->guidesSettings.minorGrid;
+					newX = qRound(newX / m_doc->guidesPrefs().minorGridSpacing) * m_doc->guidesPrefs().minorGridSpacing;
+					newY = qRound(newY / m_doc->guidesPrefs().minorGridSpacing) * m_doc->guidesPrefs().minorGridSpacing;
 				}
 				if (m->modifiers() & Qt::ControlModifier)
 				{
@@ -265,7 +282,7 @@ void CreateMode::mouseMoveEvent(QMouseEvent *m)
 					double newRot = xy2Deg(bounds.width(), bounds.height());
 					if (newRot < 0.0)
 						newRot += 360;
-					newRot = constrainAngle(newRot, m_doc->toolSettings.constrain);
+					newRot = constrainAngle(newRot, m_doc->opToolPrefs().constrain);
 					double len = qMax(0.01, distance(bounds.width(), bounds.height()));
 					bounds.setSize(len * QSizeF(cosd(newRot), sind(newRot)));
 					newX = bounds.right();
@@ -279,11 +296,17 @@ void CreateMode::mouseMoveEvent(QMouseEvent *m)
 				double nx = np2.x();
 				double ny = np2.y();
 				m_doc->ApplyGuides(&nx, &ny);
+				m_doc->ApplyGuides(&nx, &ny,true);
+				if (nx!=np2.x())
+					xSnap = nx;
+				if (ny!=np2.y())
+					ySnap = ny;
 				// #8959 : suppress qRound here as this prevent drawing line with angle constrain
 				// precisely and does not allow to stick precisely to grid or guides
 				newX = /*qRound(*/nx/*)*/;
 				newY = /*qRound(*/ny/*)*/;
 			//}
+
 			canvasCurrCoord.setXY(newX, newY);
 			m_view->HaveSelRect = true;
 
@@ -325,9 +348,11 @@ void CreateMode::mousePressEvent(QMouseEvent *m)
 	m_doc->DragP = false;
 	m_doc->leaveDrag = false;
 	inItemCreation = false;
+//	oldClip = 0;
 	m->accept();
 	m_view->registerMousePress(m->globalPos());
-	QRect mpo(m->x()-m_doc->guidesSettings.grabRad, m->y()-m_doc->guidesSettings.grabRad, m_doc->guidesSettings.grabRad*2, m_doc->guidesSettings.grabRad*2);
+//	QRect mpo(m->x()-m_doc->guidesPrefs().grabRadius, m->y()-m_doc->guidesPrefs().grabRadius, m_doc->guidesPrefs().grabRadius*2, m_doc->guidesPrefs().grabRadius*2);
+//	mpo.moveBy(qRound(m_doc->minCanvasCoordinate.x() * m_canvas->scale()), qRound(m_doc->minCanvasCoordinate.y() * m_canvas->scale()));
 	canvasPressCoord = mousePointDoc;
 	createObjectMode = m_doc->appMode;
 	createObjectSubMode = m_doc->SubMode;
@@ -337,8 +362,10 @@ void CreateMode::mousePressEvent(QMouseEvent *m)
 	canvasPressCoord.setX(qRound(Rxp));
 	Ryp  = m_doc->ApplyGridF(canvasPressCoord).y();
 	canvasPressCoord.setXY(qRound(Rxp), qRound(Ryp));
+
 	canvasCurrCoord = canvasPressCoord;
 	m_doc->ApplyGuides(&Rxp, &Ryp);
+	m_doc->ApplyGuides(&Rxp, &Ryp,true);
 	createObjectPos.setXY(Rxp, Ryp);
 
 	if (m->button() == Qt::MidButton)
@@ -360,9 +387,9 @@ void CreateMode::mousePressEvent(QMouseEvent *m)
 
 	switch (m_doc->appMode)
 	{
-	case modeDrawTable:
-		m_view->Deselect(false);
-		break;
+		case modeDrawTable2:
+			m_view->Deselect(false);
+			break;
 	}
 
 	inItemCreation = true;
@@ -380,39 +407,28 @@ void CreateMode::mouseReleaseEvent(QMouseEvent *m)
 	m->accept();
 //	m_view->stopDragTimer();
 
-	m_createTransaction = new UndoTransaction(Um::instance()->beginTransaction("creating"));
+	m_createTransaction = Um::instance()->beginTransaction("creating");
 	currItem = doCreateNewObject();
-	if (m_createTransaction && currItem && currItem->isGroupControl)
-	{
-		// We created a table, just commit transaction
-		m_view->resetMousePressed();
-		m_createTransaction->commit();
-		m_doc->changed();
-		delete m_createTransaction;
-		m_createTransaction = NULL;	
-	}
-	else if (m_createTransaction && currItem)
+	if (m_createTransaction && currItem)
 	{
 		m_view->resetMousePressed();
 		currItem->checkChanges(true);
 		QString targetName = Um::ScratchSpace;
 		if (currItem->OwnPage > -1)
 			targetName = m_doc->Pages->at(currItem->OwnPage)->getUName();
-		m_createTransaction->commit(targetName, currItem->getUPixmap(),
+		m_createTransaction.commit(targetName, currItem->getUPixmap(),
 									Um::Create + " " + currItem->getUName(),  "", Um::ICreate);
+		m_createTransaction.reset();
 		m_doc->changed();
-		delete m_createTransaction;
-		m_createTransaction = NULL;	
 		/*currItem->update();
 		currItem->emitAllToGUI();*/
 	}
 	else if (m_createTransaction)
 	{
-		m_createTransaction->cancel();
-		delete m_createTransaction;
-		m_createTransaction = NULL;
+		m_createTransaction.cancel();
+		m_createTransaction.reset();
 	}
-	if (!PrefsManager::instance()->appPrefs.stickyTools)
+	if (!PrefsManager::instance().appPrefs.uiPrefs.stickyTools)
 	{
 		m_view->requestMode(modeNormal);
 	}
@@ -432,7 +448,8 @@ void CreateMode::selectPage(QMouseEvent *m)
 	m_MouseButtonPressed = true;
 	FPoint mousePointDoc = m_canvas->globalToCanvas(m->globalPos());
 	canvasPressCoord     = mousePointDoc;
-	QRect mpo(m->x()-m_doc->guidesSettings.grabRad, m->y()-m_doc->guidesSettings.grabRad, m_doc->guidesSettings.grabRad*2, m_doc->guidesSettings.grabRad*2);
+//	QRect mpo(m->x()-m_doc->guidesPrefs().grabRadius, m->y()-m_doc->guidesPrefs().grabRadius, m_doc->guidesPrefs().grabRadius*2, m_doc->guidesPrefs().grabRadius*2);
+//	mpo.moveBy(qRound(Doc->minCanvasCoordinate.x() * m_canvas->scale()), qRound(m_doc->minCanvasCoordinate.y() * m_canvas->scale()));
 	m_doc->nodeEdit.deselect();
 	m_view->Deselect(false);
 	if (!m_doc->masterPageMode())
@@ -445,7 +462,7 @@ void CreateMode::selectPage(QMouseEvent *m)
 			if (docCurrPageNo != j)
 			{
 				m_doc->setCurrentPage(m_doc->Pages->at(j));
-				m_view->setMenTxt(j);
+				m_view->m_ScMW->slotSetCurrentPage(j);
 				m_view->DrawNew();
 			}
 		}
@@ -476,25 +493,11 @@ void CreateMode::selectPage(QMouseEvent *m)
 void CreateMode::SetupDrawNoResize(int nr)
 {
 	PageItem* currItem = m_doc->Items->at(nr);
+	//	currItem->setFont(Doc->toolSettings.textFont);
+	//	currItem->setFontSize(Doc->toolSettings.textSize);
 	m_doc->m_Selection->delaySignalsOn();
 	m_doc->m_Selection->clear();
 	m_doc->m_Selection->addItem(currItem);
-	// #10618 : Select table items if needed otherwise
-	// a crash will be triggered if user tries to ungroup
-	if (currItem->Groups.count() > 0)
-	{
-		for (int i = 0; i < m_doc->Items->count(); ++i)
-		{
-			PageItem* item = m_doc->Items->at(i);
-			if (item->Groups.count() == 0)
-				continue;
-			if (item->Groups.top() != currItem->Groups.top())
-				continue;
-			if (item->ItemNr != currItem->ItemNr)
-				m_doc->m_Selection->addItem(item, true);
-			item->isSingleSel = false;
-		}
-	}
 	m_doc->m_Selection->delaySignalsOff();
 //	emit DocChanged();
 	currItem->Sizing =  false /*currItem->asLine() ? false : true*/;
@@ -541,10 +544,19 @@ void CreateMode::getFrameItemTypes(int& itemType, int& frameType)
 		frameType = (int) PageItem::Unspecified;
 		break;
 	case modeDrawRegularPolygon:
-		itemType  = (int) PageItem::Polygon;
+		itemType  = (int) PageItem::RegularPolygon;
+		frameType = (int) PageItem::Unspecified;
+		break;
+	case modeDrawArc:
+		itemType  = (int) PageItem::Arc;
+		frameType = (int) PageItem::Unspecified;
+		break;
+	case modeDrawSpiral:
+		itemType  = (int) PageItem::Spiral;
 		frameType = (int) PageItem::Unspecified;
 		break;
 	case modeInsertPDFButton:
+	case modeInsertPDFRadioButton:
 	case modeInsertPDFTextfield:
 	case modeInsertPDFCheckbox:
 	case modeInsertPDFCombobox:
@@ -554,27 +566,31 @@ void CreateMode::getFrameItemTypes(int& itemType, int& frameType)
 		itemType  = (int) PageItem::TextFrame;
 		frameType = (int) PageItem::Unspecified;
 		break;
-	case modeDrawTable:
-		itemType  = (int) PageItem::TextFrame;
+	case modeDrawTable2:
+		itemType  = (int) PageItem::Table;
+		frameType = (int) PageItem::Unspecified;
+		break;
+	case modeInsertPDF3DAnnotation:
+		itemType  = (int) PageItem::OSGFrame;
 		frameType = (int) PageItem::Unspecified;
 		break;
 	}
 }
 
-PageItem* CreateMode::doCreateNewObject(void)
+PageItem* CreateMode::doCreateNewObject()
 {
 	int z = -1;
 	double rot, len;
 
 	double wSize = canvasCurrCoord.x() - createObjectPos.x();
 	double hSize = canvasCurrCoord.y() - createObjectPos.y();
-
 	bool   skipOneClick = (modifiers == Qt::ShiftModifier);
-	if ((createObjectMode == modeDrawLine) || (createObjectMode == modeDrawTable) ||
+	if ((createObjectMode == modeDrawLine) || (createObjectMode == modeDrawTable2) ||
 		(createObjectMode == modeInsertPDFButton) || (createObjectMode == modeInsertPDFTextfield) ||
 		(createObjectMode == modeInsertPDFTextfield) || (createObjectMode == modeInsertPDFCheckbox) ||
 		(createObjectMode == modeInsertPDFCombobox) || (createObjectMode == modeInsertPDFListbox) ||
-		(createObjectMode == modeInsertPDFTextAnnotation) || (createObjectMode == modeInsertPDFLinkAnnotation))
+		(createObjectMode == modeInsertPDFTextAnnotation) || (createObjectMode == modeInsertPDFLinkAnnotation) ||
+		(createObjectMode == modeInsertPDF3DAnnotation) || (createObjectMode == modeInsertPDFRadioButton))
 	{
 		skipOneClick = false;
 	}
@@ -584,21 +600,21 @@ PageItem* CreateMode::doCreateNewObject(void)
 		{
 			if (!doOneClick(createObjectPos, canvasCurrCoord))
 			{
-				return NULL;
+				return nullptr;
 			}
 		}
 	}
 
 	wSize = canvasCurrCoord.x() - createObjectPos.x();
 	hSize = canvasCurrCoord.y() - createObjectPos.y();
+	//Lock Height to Width for Control Modifier for final item creation
 	if (createObjectMode != modeDrawLine)
 	{
-		//Lock Height to Width for Control Modifier for final item creation
 		if (modifiers == Qt::ControlModifier)
 			hSize = wSize;
 	}
 
-	PageItem *newObject = NULL, *currItem = NULL;
+	PageItem *newObject = nullptr, *currItem = nullptr;
 	// FIXME for modeDrawLine
 	QRectF createObjectRect(createObjectPos.x(), createObjectPos.y(), wSize, hSize);
 	if (createObjectMode != modeDrawLine)
@@ -615,8 +631,6 @@ PageItem* CreateMode::doCreateNewObject(void)
 				createObjectRect.translate(0.0, createObjectRect.height());
 		}
 	}
-
-
 	double Rxp  = createObjectRect.x();
 	double Ryp  = createObjectRect.y();
 	double Rxpd = createObjectRect.width();
@@ -627,285 +641,207 @@ PageItem* CreateMode::doCreateNewObject(void)
 	case modeDrawShapes:
 		switch (createObjectSubMode)
 		{
-		case 0:
-			if (modifiers == Qt::ShiftModifier)
-			{
-				z = m_doc->itemAddArea(PageItem::Polygon, PageItem::Rectangle, Rxp, Ryp, m_doc->toolSettings.dWidth, m_doc->toolSettings.dBrush, m_doc->toolSettings.dPen, true);
+			case 0:
+				if (modifiers == Qt::ShiftModifier)
+					z = m_doc->itemAddArea(PageItem::Polygon, PageItem::Rectangle, Rxp, Ryp, m_doc->itemToolPrefs().shapeLineWidth, m_doc->itemToolPrefs().shapeFillColor, m_doc->itemToolPrefs().shapeLineColor);
+				else
+					z = m_doc->itemAdd(PageItem::Polygon, PageItem::Rectangle, Rxp, Ryp, Rxpd, Rypd, m_doc->itemToolPrefs().shapeLineWidth, m_doc->itemToolPrefs().shapeFillColor, m_doc->itemToolPrefs().shapeLineColor);
 				m_doc->Items->at(z)->FrameType = 0;
-			}
-			else
-			{
-				z = m_doc->itemAdd(PageItem::Polygon, PageItem::Rectangle, Rxp, Ryp, Rxpd, Rypd, m_doc->toolSettings.dWidth, m_doc->toolSettings.dBrush, m_doc->toolSettings.dPen, true);
-				m_doc->Items->at(z)->FrameType = 0;
-			}
-			break;
-		case 1:
-			if (modifiers == Qt::ShiftModifier)
-			{
-				z = m_doc->itemAddArea(PageItem::Polygon, PageItem::Ellipse, Rxp, Ryp, m_doc->toolSettings.dWidth, m_doc->toolSettings.dBrush, m_doc->toolSettings.dPen, true);
+				break;
+			case 1:
+				if (modifiers == Qt::ShiftModifier)
+					z = m_doc->itemAddArea(PageItem::Polygon, PageItem::Ellipse, Rxp, Ryp, m_doc->itemToolPrefs().shapeLineWidth, m_doc->itemToolPrefs().shapeFillColor, m_doc->itemToolPrefs().shapeLineColor);
+				else
+					z = m_doc->itemAdd(PageItem::Polygon, PageItem::Ellipse, Rxp, Ryp, Rxpd, Rypd, m_doc->itemToolPrefs().shapeLineWidth, m_doc->itemToolPrefs().shapeFillColor, m_doc->itemToolPrefs().shapeLineColor);
 				m_doc->Items->at(z)->FrameType = 1;
-			}
-			else
-			{
-				z = m_doc->itemAdd(PageItem::Polygon, PageItem::Ellipse, Rxp, Ryp, Rxpd, Rypd, m_doc->toolSettings.dWidth, m_doc->toolSettings.dBrush, m_doc->toolSettings.dPen, true);
-				m_doc->Items->at(z)->FrameType = 1;
-			}
-			break;
-		default:
-			if (modifiers == Qt::ShiftModifier)
-			{
-				z = m_doc->itemAddArea(PageItem::Polygon, PageItem::Unspecified, Rxp, Ryp, m_doc->toolSettings.dWidth, m_doc->toolSettings.dBrush, m_doc->toolSettings.dPen, true);
+				break;
+			default:
+				if (modifiers == Qt::ShiftModifier)
+					z = m_doc->itemAddArea(PageItem::Polygon, PageItem::Unspecified, Rxp, Ryp, m_doc->itemToolPrefs().shapeLineWidth, m_doc->itemToolPrefs().shapeFillColor, m_doc->itemToolPrefs().shapeLineColor);
+				else
+					z = m_doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->itemToolPrefs().shapeLineWidth, m_doc->itemToolPrefs().shapeFillColor, m_doc->itemToolPrefs().shapeLineColor);
 				m_doc->Items->at(z)->SetFrameShape(m_doc->ValCount, m_doc->ShapeValues);
-				m_doc->AdjustItemSize(m_doc->Items->at(z));
+				m_doc->adjustItemSize(m_doc->Items->at(z));
 				m_doc->setRedrawBounding(m_doc->Items->at(z));
 				m_doc->Items->at(z)->FrameType = createObjectSubMode + 2;
-			}
-			else
-			{
-				z = m_doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->toolSettings.dWidth, m_doc->toolSettings.dBrush, m_doc->toolSettings.dPen, true);
-				m_doc->Items->at(z)->SetFrameShape(m_doc->ValCount, m_doc->ShapeValues);
-				m_doc->AdjustItemSize(m_doc->Items->at(z));
-				m_doc->setRedrawBounding(m_doc->Items->at(z));
-				m_doc->Items->at(z)->FrameType = createObjectSubMode + 2;
-			}
-			break;
+				break;
 		}
 		break;
 	case modeDrawLine:
 		Rxp = createObjectPos.x();
 		Ryp = createObjectPos.y();
 		m_doc->ApplyGuides(&Rxp, &Ryp);
+		m_doc->ApplyGuides(&Rxp, &Ryp,true);
 		rot = xy2Deg(Rxpd, Rypd);
 		if (rot < 0.0) 
 			rot += 360;
 		len = qMax(0.01, distance(Rxpd, Rypd));
-		z = m_doc->itemAdd(PageItem::Line, PageItem::Unspecified, Rxp, Ryp, len, 1, m_doc->toolSettings.dWidthLine, CommonStrings::None, m_doc->toolSettings.dPenLine, true);
+		z = m_doc->itemAdd(PageItem::Line, PageItem::Unspecified, Rxp, Ryp, len, 1, m_doc->itemToolPrefs().lineWidth, CommonStrings::None, m_doc->itemToolPrefs().lineColor);
 		m_doc->Items->at(z)->setRotation(rot);
 		m_doc->Items->at(z)->setRedrawBounding();
 		break;
 	case modeDrawLatex:
 		if (modifiers == Qt::ShiftModifier)
-		{
-			z = m_doc->itemAddArea(PageItem::LatexFrame, PageItem::Unspecified, Rxp, Ryp, 1, m_doc->toolSettings.dBrushPict, CommonStrings::None, true);
-		}
+			z = m_doc->itemAddArea(PageItem::LatexFrame, PageItem::Unspecified, Rxp, Ryp, 1, m_doc->itemToolPrefs().imageFillColor, m_doc->itemToolPrefs().imageStrokeColor);
 		else
-		{
-			z = m_doc->itemAdd(PageItem::LatexFrame, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->toolSettings.dWidth, m_doc->toolSettings.dBrushPict, CommonStrings::None, true);
-		}
+			z = m_doc->itemAdd(PageItem::LatexFrame, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->itemToolPrefs().shapeLineWidth, m_doc->itemToolPrefs().imageFillColor, m_doc->itemToolPrefs().imageStrokeColor);
 		break;
 	case modeDrawImage:
 		if (modifiers == Qt::ShiftModifier)
-		{
-			z = m_doc->itemAddArea(PageItem::ImageFrame, PageItem::Unspecified, Rxp, Ryp, 1, m_doc->toolSettings.dBrushPict, CommonStrings::None, true);
-		}
+			z = m_doc->itemAddArea(PageItem::ImageFrame, PageItem::Unspecified, Rxp, Ryp, 1, m_doc->itemToolPrefs().imageFillColor, m_doc->itemToolPrefs().imageStrokeColor);
 		else
-		{
-			z = m_doc->itemAdd(PageItem::ImageFrame, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->toolSettings.dWidth, m_doc->toolSettings.dBrushPict, CommonStrings::None, true);
-		}
+			z = m_doc->itemAdd(PageItem::ImageFrame, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->itemToolPrefs().shapeLineWidth, m_doc->itemToolPrefs().imageFillColor, m_doc->itemToolPrefs().imageStrokeColor);
 		break;
 	case modeDrawText:
 		if (modifiers == Qt::ShiftModifier)
-		{
-			z = m_doc->itemAddArea(PageItem::TextFrame, PageItem::Unspecified, Rxp, Ryp, m_doc->toolSettings.dWidth, CommonStrings::None, m_doc->toolSettings.dPenText, true);
-		}	
+			z = m_doc->itemAddArea(PageItem::TextFrame, PageItem::Unspecified, Rxp, Ryp, m_doc->itemToolPrefs().shapeLineWidth, CommonStrings::None, m_doc->itemToolPrefs().textFont);
 		else
-		{
-			z = m_doc->itemAdd(PageItem::TextFrame, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->toolSettings.dWidth, CommonStrings::None, m_doc->toolSettings.dPenText, true);
-		}
+			z = m_doc->itemAdd(PageItem::TextFrame, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->itemToolPrefs().shapeLineWidth, CommonStrings::None, m_doc->itemToolPrefs().textFont);
 		break;
 	case modeDrawRegularPolygon:
-		{
-			if (modifiers == Qt::ShiftModifier)
-				z = m_doc->itemAddArea(PageItem::Polygon, PageItem::Unspecified, Rxp, Ryp, m_doc->toolSettings.dWidth, m_doc->toolSettings.dBrush, m_doc->toolSettings.dPen, true);
-			else
-			{
-				z = m_doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->toolSettings.dWidth, m_doc->toolSettings.dBrush, m_doc->toolSettings.dPen, true);
-			}
-			currItem = m_doc->Items->at(z);
-			QPainterPath path = RegularPolygon(currItem->width(), currItem->height(), m_doc->toolSettings.polyC, m_doc->toolSettings.polyS, m_doc->toolSettings.polyF, m_doc->toolSettings.polyR, m_doc->toolSettings.polyCurvature);
-			currItem->PoLine.fromQPainterPath(path);
-			m_doc->AdjustItemSize(currItem);
-			currItem->Clip = FlattenPath(currItem->PoLine, currItem->Segments);
-			/*
-			m_view->setCursor(QCursor(Qt::SizeFDiagCursor));
-			m_doc->m_Selection->clear();
-			m_doc->m_Selection->addItem(currItem);
-			currItem->update();
-			inItemCreation = true;
-//FIXME:				m_canvas->m_viewMode.operItemResizing = true;
-			if (m->modifiers() == Qt::ShiftModifier)
-			{
-				m_view->requestMode(modeNormal);
-// itemAdd calls PageItem::update					emit DocChanged();
-				inItemCreation = false;
-//					m_view->updateContents();
-			}
-			else
-			{
-//FIXME:					m_canvas->m_viewMode.operItemMoving = true;
-				m_canvas->setRenderModeFillBuffer();
-			}*/
-			break;
-		}
+		if (modifiers == Qt::ShiftModifier)
+			z = m_doc->itemAddArea(PageItem::RegularPolygon, PageItem::Unspecified, Rxp, Ryp, m_doc->itemToolPrefs().shapeLineWidth, m_doc->itemToolPrefs().shapeFillColor, m_doc->itemToolPrefs().lineColor);
+		else
+			z = m_doc->itemAdd(PageItem::RegularPolygon, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->itemToolPrefs().shapeLineWidth, m_doc->itemToolPrefs().shapeFillColor, m_doc->itemToolPrefs().lineColor);
+		break;
+	case modeDrawArc:
+		if (modifiers == Qt::ShiftModifier)
+			z = m_doc->itemAddArea(PageItem::Arc, PageItem::Unspecified, Rxp, Ryp, m_doc->itemToolPrefs().shapeLineWidth, m_doc->itemToolPrefs().shapeFillColor, m_doc->itemToolPrefs().lineColor);
+		else
+			z = m_doc->itemAdd(PageItem::Arc, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->itemToolPrefs().shapeLineWidth, m_doc->itemToolPrefs().shapeFillColor, m_doc->itemToolPrefs().lineColor);
+		m_doc->setRedrawBounding(m_doc->Items->at(z));
+		break;
+	case modeDrawSpiral:
+		if (modifiers == Qt::ShiftModifier)
+			z = m_doc->itemAddArea(PageItem::Spiral, PageItem::Unspecified, Rxp, Ryp, m_doc->itemToolPrefs().shapeLineWidth, CommonStrings::None, m_doc->itemToolPrefs().lineColor);
+		else
+			z = m_doc->itemAdd(PageItem::Spiral, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->itemToolPrefs().shapeLineWidth, CommonStrings::None, m_doc->itemToolPrefs().lineColor);
+		m_doc->adjustItemSize(m_doc->Items->at(z));
+		m_doc->setRedrawBounding(m_doc->Items->at(z));
+		break;
 	case modeInsertPDFButton:
+	case modeInsertPDFRadioButton:
 	case modeInsertPDFTextfield:
 	case modeInsertPDFCheckbox:
 	case modeInsertPDFCombobox:
 	case modeInsertPDFListbox:
 	case modeInsertPDFTextAnnotation:
 	case modeInsertPDFLinkAnnotation:
-		z = m_doc->itemAdd(PageItem::TextFrame, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->toolSettings.dWidth, CommonStrings::None, m_doc->toolSettings.dPenText, true);
+		z = m_doc->itemAdd(PageItem::TextFrame, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->itemToolPrefs().shapeLineWidth, CommonStrings::None, m_doc->itemToolPrefs().textColor);
 		currItem = m_doc->Items->at(z);
 		currItem->setIsAnnotation(true);
 		currItem->AutoName = false;
 		switch (m_doc->appMode)
 		{
 		case modeInsertPDFButton:
-			currItem->annotation().setType(2);
-			currItem->annotation().setFlag(65536);
+			currItem->annotation().setType(Annotation::Button);
+			currItem->annotation().setFlag(Annotation::Flag_PushButton);
+			currItem->setItemName( CommonStrings::itemName_PushButton + QString("%1").arg(m_doc->TotalItems));
+			break;
+		case modeInsertPDFRadioButton:
+			currItem->annotation().setType(Annotation::RadioButton);
+			currItem->annotation().setFlag(Annotation::Flag_Radio | Annotation::Flag_NoToggleToOff);
+			currItem->setItemName( CommonStrings::itemName_RadioButton + QString("%1").arg(m_doc->TotalItems));
 			break;
 		case modeInsertPDFTextfield:
-			currItem->annotation().setType(3);
+			currItem->annotation().setType(Annotation::Textfield);
+			currItem->setItemName( CommonStrings::itemName_TextField + QString("%1").arg(m_doc->TotalItems));
 			break;
 		case modeInsertPDFCheckbox:
-			currItem->annotation().setType(4);
+			currItem->annotation().setType(Annotation::Checkbox);
+			currItem->setItemName( CommonStrings::itemName_CheckBox + QString("%1").arg(m_doc->TotalItems));
 			break;
 		case modeInsertPDFCombobox:
-			currItem->annotation().setType(5);
-			currItem->annotation().setFlag(131072);
+			currItem->annotation().setType(Annotation::Combobox);
+			currItem->annotation().setFlag(Annotation::Flag_Combo);
+			currItem->setItemName( CommonStrings::itemName_ComboBox + QString("%1").arg(m_doc->TotalItems));
 			break;
 		case modeInsertPDFListbox:
-			currItem->annotation().setType(6);
+			currItem->annotation().setType(Annotation::Listbox);
+			currItem->setItemName( CommonStrings::itemName_ListBox + QString("%1").arg(m_doc->TotalItems));
 			break;
 		case modeInsertPDFTextAnnotation:
-			currItem->annotation().setType(10);
+			currItem->annotation().setType(Annotation::Text);
+			currItem->setItemName( CommonStrings::itemName_TextAnnotation + QString("%1").arg(m_doc->TotalItems));
 			break;
 		case modeInsertPDFLinkAnnotation:
-			currItem->annotation().setType(11);
+			currItem->annotation().setType(Annotation::Link);
 			currItem->annotation().setZiel(m_doc->currentPage()->pageNr());
 			currItem->annotation().setAction("0 0");
+			currItem->setItemName( CommonStrings::itemName_LinkAnnotation + QString("%1").arg(m_doc->TotalItems));
 			currItem->setTextFlowMode(PageItem::TextFlowDisabled);
 			break;
 		}
 		break;
-	case modeDrawTable:
-		/*m_view->Deselect(false);
-		Rxp = mousePointDoc.x();
-		Ryp = mousePointDoc.y();
-		npf = m_doc->ApplyGridF(FPoint(Rxp, Ryp));
-		Rxp = npf.x();
-		Ryp = npf.y();
-		m_doc->ApplyGuides(&Rxp, &Ryp);
-		canvasPressCoord.setXY(qRound(Rxp), qRound(Ryp));
-		canvasCurrCoord = canvasPressCoord;
-		mouseGlobalCoord.setXY(m->globalPos().x(), m->globalPos().y());
-		m_view->redrawMarker->setGeometry(m->globalPos().x(), m->globalPos().y(), 1, 1);
-		m_view->redrawMarker->show();*/
-
+	case modeDrawTable2:
+		// TODO: Figure out what these conditions actually do.
 		if ((m_doc->m_Selection->count() == 0) && (m_view->HaveSelRect) && (!m_view->MidButt))
 		{
-			int    Cols, Rows;
-			double Tx, Ty, Tw, Th;
-			double deltaX, deltaY, offX, offY;
-			UndoTransaction * activeTransaction = NULL;
 			m_view->HaveSelRect = false;
-			FPoint np2 = m_doc->ApplyGridF(canvasPressCoord);
-			double Tx1 = np2.x();
-			double Ty1 = np2.y();
-			m_doc->ApplyGuides(&Tx1, &Ty1);
-			canvasPressCoord.setXY(qRound(Tx1), qRound(Ty1));
-			np2 = m_doc->ApplyGridF(canvasCurrCoord);
-			double Tx2 = np2.x();
-			double Ty2 = np2.y();
-			m_doc->ApplyGuides(&Tx2, &Ty2);
-			canvasCurrCoord.setXY(qRound(Tx2), qRound(Ty2));
-			Tx = qMin(Tx1, Tx2);
-			Ty = qMin(Ty1, Ty2);
-			Tw = qAbs(Tx2 - Tx1);
-			Th = qAbs(Ty2 - Ty1);
-			if ((Th < 6) || (Tw < 6))
+			// Calculate table rectangle.
+			FRect tableRect = adjustedRect(canvasPressCoord, canvasCurrCoord);
+			if (tableRect.width() < 6 || tableRect.height() < 6)
 			{
+				// Ignore tiny tables.
 				m_view->requestMode(submodePaintingDone);
 				break;
 			}
-			//qApp->changeOverrideCursor(QCursor(Qt::ArrowCursor));
-			InsertTable *dia = new InsertTable(m_view, static_cast<int>(Th / 6), static_cast<int>(Tw / 6));
+			// Show table insert dialog.
+		//	qApp->changeOverrideCursor(QCursor(Qt::ArrowCursor));
+			InsertTable *dia = new InsertTable(m_view, static_cast<int>(tableRect.height()/6), static_cast<int>(tableRect.width()/6));
 			if (!dia->exec())
 			{
 				m_view->requestMode(submodePaintingDone);
 				delete dia;
-				dia = NULL;
+				dia = nullptr;
 				break;
 			}
-			Cols = dia->Cols->value();
-			Rows = dia->Rows->value();
+			int numRows = dia->Rows->value();
+			int numColumns = dia->Cols->value();
 			delete dia;
-			dia=NULL;
-			deltaX = Tw / Cols;
-			deltaY = Th / Rows;
-			offX = 0.0;
-			offY = 0.0;
-			m_doc->m_Selection->clear();
-			if (UndoManager::undoEnabled())
-				activeTransaction = new UndoTransaction(Um::instance()->beginTransaction(m_doc->currentPage()->getUName(),
-																						 Um::ITable, Um::CreateTable,
-																						 QString(Um::RowsCols).arg(Rows).arg(Cols),
-																						 Um::ICreate));
-			m_doc->m_Selection->delaySignalsOn();
-			for (int rc = 0; rc < Rows; ++rc)
-			{
-				for (int cc = 0; cc < Cols; ++cc)
-				{
-					z = m_doc->itemAdd(PageItem::TextFrame, PageItem::Unspecified, Tx + offX, Ty + offY, deltaX, deltaY, m_doc->toolSettings.dWidth, CommonStrings::None, m_doc->toolSettings.dPenText, true);
-					currItem = m_doc->Items->at(z);
-					currItem->isTableItem = true;
-					currItem->setTextFlowMode(PageItem::TextFlowUsesBoundingBox);
-					m_doc->m_Selection->addItem(currItem);
-					offX += deltaX;
-				}
-				offY += deltaY;
-				offX = 0.0;
-			}
-			for (int rc = 0; rc < Rows; ++rc)
-			{
-				for (int cc = 0; cc < Cols; ++cc)
-				{
-					currItem = m_doc->m_Selection->itemAt((rc * Cols) + cc);
-					if (rc == 0)
-						currItem->TopLink = 0;
-					else
-						currItem->TopLink = m_doc->m_Selection->itemAt(((rc-1)*Cols)+cc);
-					if (rc == Rows-1)
-						currItem->BottomLink = 0;
-					else
-						currItem->BottomLink = m_doc->m_Selection->itemAt(((rc+1)*Cols)+cc);
-					if (cc == 0)
-						currItem->LeftLink = 0;
-					else
-						currItem->LeftLink = m_doc->m_Selection->itemAt((rc*Cols)+cc-1);
-					if (cc == Cols-1)
-						currItem->RightLink = 0;
-					else
-						currItem->RightLink = m_doc->m_Selection->itemAt((rc*Cols)+cc+1);
-				}
-			}
-			m_doc->itemSelection_GroupObjects(false, false);
-			z = (m_doc->Items->count() - 1) - Rows * Cols;
-			if (activeTransaction)
-			{
-				activeTransaction->commit();
-				delete activeTransaction;
-				activeTransaction = NULL;
-			}
-			m_doc->m_Selection->delaySignalsOff();
+			dia = nullptr;
+			// Add the table item.
+			// TODO: This should be done in an undo transaction.
+			m_doc->dontResize = true;
+			z = m_doc->itemAdd(PageItem::Table, PageItem::Unspecified,
+						   tableRect.x(),
+						   tableRect.y(),
+						   tableRect.width(),
+						   tableRect.height(),
+						   0,                    // Unused.
+						   CommonStrings::None,  // Unused.
+						   CommonStrings::None); // Unused.
+			PageItem_Table *table = m_doc->Items->at(z)->asTable();
+			table->insertRows(0, numRows - 1);
+			table->insertColumns(0, numColumns - 1);
+			table->adjustTableToFrame();
+			table->adjustFrameToTable();
+			m_doc->dontResize = false;
+			m_doc->setRedrawBounding(table);
 		}
 		break;
+	case modeInsertPDF3DAnnotation:
+		if (modifiers == Qt::ShiftModifier)
+		{
+			z = m_doc->itemAddArea(PageItem::OSGFrame, PageItem::Unspecified, Rxp, Ryp, 1, m_doc->itemToolPrefs().imageFillColor, m_doc->itemToolPrefs().imageStrokeColor);
+		}
+		else
+		{
+			z = m_doc->itemAdd(PageItem::OSGFrame, PageItem::Unspecified, Rxp, Ryp, Rxpd, Rypd, m_doc->itemToolPrefs().shapeLineWidth, m_doc->itemToolPrefs().imageFillColor, m_doc->itemToolPrefs().imageStrokeColor);
+		}
+		currItem = m_doc->Items->at(z);
+		currItem->setIsAnnotation(true);
+		currItem->AutoName = false;
+		currItem->annotation().setType(Annotation::Annot3D);
+		currItem->setItemName( tr("3DAnnot") + QString("%1").arg(m_doc->TotalItems));
+		break;
 	}
 	if (z >= 0)
 	{
 		SetupDrawNoResize(z);
 		newObject = m_doc->Items->at(z);
 		newObject->ContourLine = newObject->PoLine.copy();
+		m_doc->setRedrawBounding(newObject);
 	}
 	return newObject;
 }
@@ -919,7 +855,7 @@ bool CreateMode::doOneClick(FPoint& startPoint, FPoint& endPoint)
 	if (QApplication::keyboardModifiers() & Qt::ControlModifier)
 		return true;
 
-	PrefsContext* sizes = PrefsManager::instance()->prefsFile->getContext("ObjectSize");
+	PrefsContext* sizes = PrefsManager::instance().prefsFile->getContext("ObjectSize");
 	bool doRemember     = sizes->getBool("Remember", true);
 
 	int lmode = (createObjectMode == modeDrawLine) ? 1 : 0;
@@ -935,17 +871,16 @@ bool CreateMode::doOneClick(FPoint& startPoint, FPoint& endPoint)
 		ySize = sizes->getDouble("defAngle", 0.0);
 		originPoint = sizes->getInt("OriginL", 0);
 	}
-
-	//qApp->changeOverrideCursor(QCursor(Qt::ArrowCursor));
-	OneClick *dia = new OneClick(m_view, ScribusView::tr("Enter Object Size"), m_doc->unitIndex(), xSize, ySize, doRemember, originPoint, lmode);
-	if (dia->exec())
+	//#12577 Remove one click dialog
+	//#12577 OneClick *dia = new OneClick(m_view, ScribusView::tr("Enter Object Size"), m_doc->unitIndex(), xSize, ySize, doRemember, originPoint, lmode);
+	//#12577 if (dia->exec())
 	{
-		doRemember = dia->checkRemember->isChecked();
+		//#12577 doRemember = dia->checkRemember->isChecked();
 		if (lmode == 0)
 		{
-			xSize = dia->spinWidth->value() / unitGetRatioFromIndex(m_doc->unitIndex());
-			ySize = dia->spinHeight->value() / unitGetRatioFromIndex(m_doc->unitIndex());
-			originPoint = dia->RotationGroup->checkedId();
+			//#12577 xSize = dia->spinWidth->value() / unitGetRatioFromIndex(m_doc->unitIndex());
+			//#12577 ySize = dia->spinHeight->value() / unitGetRatioFromIndex(m_doc->unitIndex());
+			//#12577 originPoint = dia->RotationGroup->checkedId();
 			if (doRemember)
 			{
 				sizes->set("defWidth", xSize);
@@ -978,9 +913,9 @@ bool CreateMode::doOneClick(FPoint& startPoint, FPoint& endPoint)
 		else
 		{
 			FPoint oldStart = startPoint;
-			xSize = dia->spinWidth->value() / unitGetRatioFromIndex(m_doc->unitIndex());
-			ySize = dia->spinHeight->value();
-			originPoint = dia->RotationGroup->checkedId();
+			//#12577 xSize = dia->spinWidth->value() / unitGetRatioFromIndex(m_doc->unitIndex());
+			//#12577 ySize = dia->spinHeight->value();
+			//#12577 originPoint = dia->RotationGroup->checkedId();
 			if (doRemember)
 			{
 				sizes->set("defLength", xSize);
@@ -1006,32 +941,55 @@ bool CreateMode::doOneClick(FPoint& startPoint, FPoint& endPoint)
 		sizes->set("Remember", doRemember);
 		doCreate = true;
 	}
-	delete dia;
+	//delete dia;
 	return doCreate;
 }
 
+FRect CreateMode::adjustedRect(FPoint &firstPoint, FPoint &secondPoint)
+{
+	// Lock to grid.
+	FPoint first = m_doc->ApplyGridF(firstPoint);
+	FPoint second = m_doc->ApplyGridF(secondPoint);
+
+	// Lock to guides and items.
+	double firstX = first.x();
+	double firstY = first.y();
+	m_doc->ApplyGuides(&firstX, &firstY);
+	m_doc->ApplyGuides(&firstX, &firstY,true);
+
+	double secondX = second.x();
+	double secondY = second.y();
+	m_doc->ApplyGuides(&secondX, &secondY);
+	m_doc->ApplyGuides(&secondX, &secondY,true);
+
+	// Return normalized rectangle.
+	FRect rect(firstX, firstY, secondX - firstX, secondY - firstY);
+
+	return rect.normalize();
+}
+
 // void CreateMode::setResizeCursor(int how)
 // {
 // 	switch (how)
 // 	{
 // 		case 1:
 // 		case 2:
-// 			m_view->setCursor(QCursor(Qt::SizeFDiagCursor));
+// 			qApp->changeOverrideCursor(QCursor(Qt::SizeFDiagCursor));
 // 			break;
 // 		case 3:
 // 		case 4:
-// 			m_view->setCursor(QCursor(Qt::SizeBDiagCursor));
+// 			qApp->changeOverrideCursor(QCursor(Qt::SizeBDiagCursor));
 // 			break;
 // 		case 5:
 // 		case 8:
-// 			m_view->setCursor(QCursor(Qt::SizeVerCursor));
+// 			qApp->changeOverrideCursor(QCursor(Qt::SizeVerCursor));
 // 			break;
 // 		case 6:
 // 		case 7:
-// 			m_view->setCursor(QCursor(Qt::SizeHorCursor));
+// 			qApp->changeOverrideCursor(QCursor(Qt::SizeHorCursor));
 // 			break;
 // 		default:
-// 			m_view->setCursor(QCursor(Qt::SizeAllCursor));
+// 			qApp->changeOverrideCursor(QCursor(Qt::SizeAllCursor));
 // 			break;
 // 	}
 // }
