@@ -19,16 +19,20 @@
 #include <QPainter>
 #include <QTimer>
 
+#include "appmodes.h"
 #include "canvas.h"
-#include "contextmenu.h"
 #include "fpoint.h"
+#include "iconmanager.h"
 #include "pageitem.h"
-#include "pageselector.h"
 #include "prefsmanager.h"
+#include "scribus.h"
+#include "scribusdoc.h"
 #include "scribusview.h"
 #include "selection.h"
+#include "ui/contextmenu.h"
+#include "ui/pageselector.h"
+#include "ui/scrspinbox.h"
 #include "undomanager.h"
-#include "util_icon.h"
 #include "util_math.h"
 
 CanvasMode_Rotate::CanvasMode_Rotate(ScribusView* view) : CanvasMode(view)
@@ -47,7 +51,7 @@ CanvasMode_Rotate::CanvasMode_Rotate(ScribusView* view) : CanvasMode(view)
 inline bool CanvasMode_Rotate::GetItem(PageItem** pi)
 { 
 	*pi = m_doc->m_Selection->itemAt(0); 
-	return (*pi) != NULL; 
+	return (*pi) != nullptr;
 }
 
 void CanvasMode_Rotate::drawControls(QPainter* p)
@@ -65,6 +69,8 @@ void CanvasMode_Rotate::drawItemOutlines(QPainter* p)
 	double itemRotation;
 
 	p->save();
+	/*p->scale(m_canvas->scale(), m_canvas->scale());
+	p->translate(-m_doc->minCanvasCoordinate.x(), -m_doc->minCanvasCoordinate.y());*/
 
 	QColor  drawColor = qApp->palette().color(QPalette::Active, QPalette::Highlight);
 	p->setRenderHint(QPainter::Antialiasing);
@@ -91,10 +97,10 @@ void CanvasMode_Rotate::getNewItemPosition(PageItem* item, FPoint& pos, double&
 	double newAngle = xy2Deg(m_canvasCurrCoord.x() - m_rotCenter.x(), m_canvasCurrCoord.y() - m_rotCenter.y());
 	if (m_angleConstrained)
 	{
-		newAngle = constrainAngle(newAngle, m_doc->toolSettings.constrain);
-		/*double oldAngle = constrainAngle(m_startAngle, m_doc->toolSettings.constrain);
+		newAngle = constrainAngle(newAngle, m_doc->opToolPrefs().constrain);
+		/*double oldAngle = constrainAngle(m_startAngle, m_doc->opToolPrefs.constrain);
 		newAngle = m_doc->m_Selection->isMultipleSelection() ? (newAngle - oldAngle) : newAngle;*/
-		m_view->oldW = constrainAngle(m_view->oldW, m_doc->toolSettings.constrain);
+		m_view->oldW = constrainAngle(m_view->oldW, m_doc->opToolPrefs().constrain);
 		newAngle = m_doc->m_Selection->isMultipleSelection() ? (newAngle - m_view->oldW) : newAngle;
 	}
 	else if (m_doc->m_Selection->isMultipleSelection())
@@ -103,7 +109,7 @@ void CanvasMode_Rotate::getNewItemPosition(PageItem* item, FPoint& pos, double&
 		newAngle = item->rotation() - (m_startAngle - newAngle);
 	if (m_doc->m_Selection->isMultipleSelection())
 	{
-		QMatrix ma;
+		QTransform ma;
 		ma.translate(m_rotCenter.x(), m_rotCenter.y());
 		ma.scale(1, 1);
 		ma.rotate(newAngle);
@@ -114,7 +120,7 @@ void CanvasMode_Rotate::getNewItemPosition(PageItem* item, FPoint& pos, double&
 	else if (m_rotMode != 0)
 	{
 		FPoint n(0,0);
-		QMatrix ma;
+		QTransform ma;
 		ma.translate(item->xPos(), item->yPos());
 		ma.scale(1, 1);
 		ma.rotate(item->rotation());
@@ -147,6 +153,11 @@ void CanvasMode_Rotate::getNewItemPosition(PageItem* item, FPoint& pos, double&
 		pos.setXY(item->xPos(), item->yPos());
 		rotation = newAngle;
 	}
+
+	while (rotation < 0)
+		rotation += 360.0;
+	while (rotation > 360)
+		rotation -= 360.0;
 }
 
 void CanvasMode_Rotate::activate(bool fromGesture)
@@ -169,7 +180,7 @@ void CanvasMode_Rotate::activate(bool fromGesture)
 
 void CanvasMode_Rotate::deactivate(bool)
 {
-	m_view->redrawMarker->hide();
+	m_view->setRedrawMarkerShown(false);
 	m_inItemRotation = false;
 }
 
@@ -183,7 +194,6 @@ void CanvasMode_Rotate::enterEvent(QEvent *)
 
 void CanvasMode_Rotate::leaveEvent(QEvent *)
 {
-
 }
 
 void CanvasMode_Rotate::mousePressEvent(QMouseEvent *m)
@@ -194,15 +204,15 @@ void CanvasMode_Rotate::mousePressEvent(QMouseEvent *m)
 	double Rxp = 0,  Ryp = 0;
 	PageItem *currItem;
 	m_canvas->PaintSizeRect(QRect());
-	QRect tx;
-	QMatrix pm;
+//	QRect tx;
+	QTransform pm;
 	m_canvas->m_viewMode.m_MouseButtonPressed = true;
 	m_canvas->m_viewMode.operItemMoving = false;
 	m_view->HaveSelRect = false;
 	m_doc->leaveDrag = false;
 	m->accept();
 	m_view->registerMousePress(m->globalPos());
-	QRect mpo(m->x()-m_doc->guidesSettings.grabRad, m->y()-m_doc->guidesSettings.grabRad, m_doc->guidesSettings.grabRad*2, m_doc->guidesSettings.grabRad*2);
+	QRect mpo(m->x()-m_doc->guidesPrefs().grabRadius, m->y()-m_doc->guidesPrefs().grabRadius, m_doc->guidesPrefs().grabRadius*2, m_doc->guidesPrefs().grabRadius*2);
 	Rxp  = m_doc->ApplyGridF(m_canvasPressCoord).x();
 	m_canvasPressCoord.setX( qRound(Rxp) );
 	Ryp  = m_doc->ApplyGridF(m_canvasPressCoord).y();
@@ -219,7 +229,7 @@ void CanvasMode_Rotate::mousePressEvent(QMouseEvent *m)
 	if (GetItem(&currItem))
 	{
 		m_inItemRotation = true;
-		m_oldRotMode   = m_rotMode   = m_doc->RotMode;
+		m_oldRotMode   = m_rotMode   = m_doc->rotationMode();
 		m_oldRotCenter = m_rotCenter = m_view->RCenter;
 		if (m_doc->m_Selection->isMultipleSelection())
 		{
@@ -236,7 +246,7 @@ void CanvasMode_Rotate::mousePressEvent(QMouseEvent *m)
 					m_rotCenter = FPoint(gxR, gyR);
 					m_rotMode   = 0;
 				}
-				m_doc->RotMode  = m_rotMode;
+				m_doc->setRotationMode  ( m_rotMode );
 				m_view->RCenter = m_rotCenter;
 			}
 			m_startAngle = xy2Deg(mousePointDoc.x() - m_view->RCenter.x(), mousePointDoc.y() - m_view->RCenter.y());
@@ -244,7 +254,7 @@ void CanvasMode_Rotate::mousePressEvent(QMouseEvent *m)
 		}
 		else
 		{
-			QMatrix mat;
+			QTransform mat;
 			m_canvas->Transform(currItem, mat);
 			m_rotMode   = 2;
 			m_rotCenter = FPoint(currItem->width()/2, currItem->height()/2, 0, 0, currItem->rotation(), 1, 1, false);
@@ -273,7 +283,7 @@ void CanvasMode_Rotate::mousePressEvent(QMouseEvent *m)
 						m_rotMode   = 1;
 					}	
 				}
-				m_doc->RotMode  = m_rotMode;
+				m_doc->setRotationMode  ( m_rotMode );
 				m_view->RCenter = m_rotCenter;
 //			}
 			m_view->RCenter = m_rotCenter = FPoint(currItem->xPos()+ m_view->RCenter.x(), currItem->yPos()+ m_view->RCenter.y()); //?????
@@ -308,24 +318,20 @@ void CanvasMode_Rotate::mouseReleaseEvent(QMouseEvent *m)
 		{
 			m_view->startGroupTransaction(Um::Rotate, "", Um::IRotate);
 		}
-		double newW = xy2Deg(mousePointDoc.x()-m_view->RCenter.x(), mousePointDoc.y()-m_view->RCenter.y()); //xy2Deg(m->x()/sc - m_view->RCenter.x(), m->y()/sc - m_view->RCenter.y());
+		double angle = 0;
+		double newW  = xy2Deg(mousePointDoc.x()-m_view->RCenter.x(), mousePointDoc.y()-m_view->RCenter.y()); //xy2Deg(m->x()/sc - m_view->RCenter.x(), m->y()/sc - m_view->RCenter.y());
 		if (m->modifiers() & Qt::ControlModifier)
 		{
-			newW=constrainAngle(newW, m_doc->toolSettings.constrain);
-			m_view->oldW=constrainAngle(m_view->oldW, m_doc->toolSettings.constrain);
+			newW=constrainAngle(newW, m_doc->opToolPrefs().constrain);
+			m_view->oldW=constrainAngle(m_view->oldW, m_doc->opToolPrefs().constrain);
 			//RotateGroup uses MoveBy so its pretty hard to constrain the result
-			if (m_doc->m_Selection->isMultipleSelection())
-				m_doc->rotateGroup(newW-m_view->oldW, m_view->RCenter);
-			else
-				m_doc->RotateItem(newW, currItem->ItemNr);
+			angle = m_doc->m_Selection->isMultipleSelection() ? (newW-m_view->oldW) : newW;
 		}
 		else
 		{
-			if (m_doc->m_Selection->isMultipleSelection())
-				m_doc->rotateGroup(newW - m_view->oldW, m_view->RCenter);
-			else
-				m_doc->RotateItem(currItem->rotation() - (m_view->oldW - newW), currItem->ItemNr);
+			angle = m_doc->m_Selection->isMultipleSelection() ? (newW - m_view->oldW) : (currItem->rotation() - (m_view->oldW - newW));
 		}
+		m_doc->itemSelection_Rotate(angle);
 		m_view->oldW = newW;
 		m_canvas->setRenderModeUseBuffer(false);
 		if (!m_doc->m_Selection->isMultipleSelection())
@@ -336,9 +342,9 @@ void CanvasMode_Rotate::mouseReleaseEvent(QMouseEvent *m)
 				m_view->updateContents();
 		}
 	}
-	m_doc->RotMode  = m_oldRotMode;
+	m_doc->setRotationMode( m_oldRotMode );
 	m_view->RCenter = m_oldRotCenter;
-	if (!PrefsManager::instance()->appPrefs.stickyTools)
+	if (!PrefsManager::instance().appPrefs.uiPrefs.stickyTools)
 		m_view->requestMode(modeNormal);
 	else
 	{
@@ -369,9 +375,9 @@ void CanvasMode_Rotate::mouseReleaseEvent(QMouseEvent *m)
 	}
 	for (int i = 0; i < m_doc->m_Selection->count(); ++i)
 		m_doc->m_Selection->itemAt(i)->checkChanges(true);
-	//Make sure the Zoom spinbox and page selector dont have focus if we click on the canvas
-	m_view->zoomSpinBox->clearFocus();
-	m_view->pageSelector->clearFocus();
+	//Make sure the Zoom spinbox and page selector don't have focus if we click on the canvas
+	m_view->m_ScMW->zoomSpinBox->clearFocus();
+	m_view->m_ScMW->pageSelector->clearFocus();
 }
 
 void CanvasMode_Rotate::mouseMoveEvent(QMouseEvent *m)
@@ -400,14 +406,14 @@ void CanvasMode_Rotate::mouseMoveEvent(QMouseEvent *m)
 			{
 				double gx, gy, gh, gw;
 				m_doc->m_Selection->getVisualGroupRect(&gx, &gy, &gw, &gh);
-				int how = m_canvas->frameHitTest(QPointF(mousePointDoc.x(),mousePointDoc.y()), QRectF(gx, gy, gw, gh));
+				int how = m_canvas->frameHitTest(QPointF(mousePointDoc.x(), mousePointDoc.y()), QRectF(gx, gy, gw, gh));
 				if (how >= 0)
 				{
 					if (how > 0)
 					{
 						setResizeCursor(how);
 					}
-					m_view->setCursor(QCursor(loadIcon("Rotieren2.png")));
+					m_view->setCursor(IconManager::instance().loadCursor("Rotieren2.png"));
 				}
 				else
 				{
@@ -420,14 +426,14 @@ void CanvasMode_Rotate::mouseMoveEvent(QMouseEvent *m)
 				currItem = m_doc->m_Selection->itemAt(a);
 				if (currItem->locked())
 					break;
-				QMatrix p;
+				QTransform p;
 				m_canvas->Transform(currItem, p);
-				QRect mpo = QRect(m->x()-m_doc->guidesSettings.grabRad, m->y()-m_doc->guidesSettings.grabRad, m_doc->guidesSettings.grabRad*2, m_doc->guidesSettings.grabRad*2);
-				if ((QRegion(p.map(QPolygon(QRect(-3, -3, static_cast<int>(currItem->width()+6), static_cast<int>(currItem->height()+6))))).contains(mpo)))
+				QRect mpo = QRect(m->x() - m_doc->guidesPrefs().grabRadius, m->y() - m_doc->guidesPrefs().grabRadius, m_doc->guidesPrefs().grabRadius * 2, m_doc->guidesPrefs().grabRadius * 2);
+				if ((QRegion(p.map(QPolygon(QRect(-3, -3, static_cast<int>(currItem->width() + 6), static_cast<int>(currItem->height() + 6))))).contains(mpo)))
 				{
 					QRect tx = p.mapRect(QRect(0, 0, static_cast<int>(currItem->width()), static_cast<int>(currItem->height())));
 					if ((tx.intersects(mpo)) && (!currItem->locked()))
-						m_view->setCursor(QCursor(loadIcon("Rotieren2.png")));
+						m_view->setCursor(IconManager::instance().loadCursor("Rotieren2.png"));
 				}
 			}
 		}
@@ -437,21 +443,19 @@ void CanvasMode_Rotate::mouseMoveEvent(QMouseEvent *m)
 		if ((m_canvas->m_viewMode.m_MouseButtonPressed) && (m->buttons() & Qt::LeftButton))
 		{
 			QPoint startP = m_canvas->canvasToGlobal(m_canvasPressCoord);
-			m_view->redrawMarker->setGeometry(QRect(startP, m->globalPos()).normalized());
-			if (!m_view->redrawMarker->isVisible())
-				m_view->redrawMarker->show();
+			m_view->redrawMarker->setGeometry(QRect(m_view->mapFromGlobal(startP), m_view->mapFromGlobal(m->globalPos())).normalized());
+			m_view->setRedrawMarkerShown(true);
 			m_view->HaveSelRect = true;
-			return;
 		}
 	}
 }
 
 void CanvasMode_Rotate::createContextMenu(PageItem* currItem, double mx, double my)
 {
-	ContextMenu* cmen=NULL;
+	ContextMenu* cmen=nullptr;
 	m_view->setObjectUndoMode();
 	m_canvasPressCoord.setXY(mx, my);
-	if(currItem!=NULL)
+	if (currItem!=nullptr)
 		cmen = new ContextMenu(*(m_doc->m_Selection), m_view->m_ScMW, m_doc);
 	else
 		cmen = new ContextMenu(m_view->m_ScMW, m_doc, mx, my);
