@@ -11,14 +11,15 @@ for which a new license (GPL+exception) is in place.
 #include <QString>
 
 #include "lcms2.h"
+#include "sccolormgmtimplelem.h"
 #include "sccolorprofiledata.h"
 
-class ScLcms2ColorProfileImpl : public ScColorProfileData
+class ScLcms2ColorProfileImpl : public ScColorProfileImplBase
 {
 	friend class ScLcms2ColorMgmtEngineImpl;
 
 public:
-	ScLcms2ColorProfileImpl(cmsHPROFILE lcmsProfile);
+	ScLcms2ColorProfileImpl(ScColorMgmtEngine& engine, cmsHPROFILE lcmsProfile);
 	virtual ~ScLcms2ColorProfileImpl();
 
 	virtual bool isNull() const;
