@@ -18,30 +18,34 @@ class ScDLThread : public QThread
 		~ScDLThread();
 		void run();
 
-		void addURL(const QUrl &url, bool overwrite, const QString& location="");
-		void addURLs(const QStringList &urlList, bool overwrite, const QString& location="");
+		void addURL(const QUrl &url, bool overwrite, const QString& location, const QString& destinationLocation);
+		void addURLs(const QStringList &urlList, bool overwrite, const QString& location, const QString& destinationLocation);
 		void startDownloads();
 		QString saveFileName(const QUrl &url, const QString &location, bool overwrite);
 
 	signals:
 		void finished();
 		void runSignal();
-		void received(const QString &);
+		void fileStarted(const QString &);
+		void fileReceived(const QString &);
+		void fileFailed(const QString &);
+		void fileDownloadProgress(qint64 bytesReceived, qint64 bytesTotal);
 
 	private slots:
 		void startNextDownload();
 		void downloadFinished();
+		void replyFinished(QNetworkReply* reply);
 		void downloadReadyRead();
 		void runSlot();
 
 	private:
-		bool urlOK(QUrl url);
+		bool urlOK(const QUrl& url);
 		QStringList m_urlList;
-		QFile output;
-		QQueue<QPair<QUrl, QString> > downloadQueue;
-		int downloadedCount;
-		int totalCount;
-		QNetworkReply *currentDownload;
-		QNetworkAccessManager manager;
+		QFile m_output;
+		QQueue<QPair<QUrl, QString> > m_downloadQueue;
+		int m_downloadedCount;
+		int m_totalCount;
+		QNetworkReply *m_currentDownload;
+		QNetworkAccessManager m_manager;
 };
 #endif
