@@ -33,6 +33,8 @@ for which a new license (GPL+exception) is in place.
 #include <QVector>
 
 #include "util.h"
+#include "util_math.h"
+#include "sclimits.h"
 
 using namespace std;
 
@@ -41,7 +43,7 @@ FPointArray FPointArray::copy() const
 { 
 	FPointArray tmp;
 	tmp << *this;
-	tmp.count = count;
+	tmp.QVector<FPoint>::squeeze();
 	return tmp; 
 }
 
@@ -49,24 +51,44 @@ FPointArray FPointArray::copy() const
 FPointArray & FPointArray::operator=( const FPointArray &a )
 { 
 	QVector<FPoint>::operator=(a);
-	count = a.count; 
-	svgState = NULL;
+	m_svgState = nullptr;
+	QVector<FPoint>::squeeze();
 	return *this; 
 }
 
-bool FPointArray::resize(uint newCount)
+
+/* optimized for speed:
+ *   never shrink
+ *   when growing, try to double size
+ *   if capacity permits, just increase count
+ */
+bool FPointArray::resize(int newCount)
 {
-	if (newCount == 0)
+	if (newCount <= 0)
 	{
 		QVector<FPoint>::resize(0);
 		QVector<FPoint>::squeeze();
 	}
 	else
+	{
 		QVector<FPoint>::resize(newCount);
-	count = newCount;
+	}
 	return true;
 }
 
+void FPointArray::reverse()
+{
+	FPointArray tmp;
+	tmp << *this;
+	tmp.QVector<FPoint>::squeeze();
+	QVector<FPoint>::resize(0);
+	QVector<FPoint>::squeeze();
+	for (int a = 0; a < tmp.count()-1; a += 2)
+	{
+		QVector<FPoint>::prepend(tmp.point(a+1));
+		QVector<FPoint>::prepend(tmp.point(a));
+	}
+}
 
 bool FPointArray::setPoints( int nPoints, double firstx, double firsty, ... )
 {
@@ -91,7 +113,7 @@ bool FPointArray::setPoints( int nPoints, double firstx, double firsty, ... )
 bool FPointArray::putPoints( int index, int nPoints, double firstx, double firsty,  ... )
 {
 	va_list ap;
-	if ( index + nPoints > static_cast<int>(count) )
+	if ( index + nPoints > QVector<FPoint>::count())
 	{
 		if ( !FPointArray::resize(index + nPoints) )
 			return false;
@@ -115,7 +137,7 @@ bool FPointArray::putPoints( int index, int nPoints, double firstx, double first
 
 bool FPointArray::putPoints( int index, int nPoints, const FPointArray & from, int fromIndex )
 {
-	if ( index + nPoints > static_cast<int>(count) )
+	if ( index + nPoints > QVector<FPoint>::count() )
 	{	// extend array
 		if ( !FPointArray::resize(index + nPoints) )
 			return false;
@@ -126,40 +148,33 @@ bool FPointArray::putPoints( int index, int nPoints, const FPointArray & from, i
 	p += index;
 	ConstIterator q = from.begin();
 	q += fromIndex;
-	while( --nPoints >= 0 )
+	while (--nPoints >= 0)
 	{
 		*p++ = *q++;
     }
 	return true;
 }
 
-void FPointArray::point(uint i, double *x, double *y) const
+void FPointArray::point(int i, double *x, double *y) const
 {
-//	FPoint p = QMemArray<FPoint>::at(i);
-	ConstIterator p = begin();
-	p += i;
+	const FPoint& p = QVector<FPoint>::at(i);
 	if (x)
-		*x = p->xp;
+		*x = p.xp;
 	if (y)
-		*y = p->yp;
+		*y = p.yp;
 }
 
 
-QPoint FPointArray::pointQ(uint i) const
+QPoint FPointArray::pointQ(int i) const
 {
-//	FPoint p = QMemArray<FPoint>::at(i);
-	ConstIterator p = begin();
-	p += i;
-	QPoint r(qRound(p->xp),qRound(p->yp));
-	return r;
+	const FPoint& p = QVector<FPoint>::at(i);
+	return QPoint(qRound(p.xp), qRound(p.yp));
 }
 
-QPointF FPointArray::pointQF(uint i) const
+QPointF FPointArray::pointQF(int i) const
 {
-//	FPoint p = QMemArray<FPoint>::at(i);
-	ConstIterator p = begin();
-	p += i;
-	QPointF r(p->xp,p->yp);
+	const FPoint& p = QVector<FPoint>::at(i);
+	QPointF r(p.xp, p.yp);
 	return r;
 }
 
@@ -167,10 +182,10 @@ void FPointArray::translate( double dx, double dy )
 {
 	FPoint pt( dx, dy );
 	Iterator pend = begin();
-	pend += count;
+	pend += QVector<FPoint>::count();
 	for (Iterator p = begin(); p != pend; p++)
 	{
-		if (p->x() < 900000)
+		if (!isMarkerI(p))
 			*p += pt;
 	}
 }
@@ -178,29 +193,34 @@ void FPointArray::translate( double dx, double dy )
 void FPointArray::scale( double sx, double sy )
 {
 	Iterator pend = begin();
-	pend += count;
+	pend += QVector<FPoint>::count();
 	for (Iterator p = begin(); p != pend; p++)
 	{
-		if (p->x() < 900000) {
+		if (!isMarkerI(p))
 			p->setXY(p->x() * sx, p->y() * sy);
-		}
 	}
 }
 
-
-FPoint FPointArray::WidthHeight() const
+QRectF FPointArray::boundingRect() const
 {
-	if ( count == 0 )
+	FPoint min = getMinClipF(this);
+	FPoint max = getMaxClipF(this);
+	return QRectF(QPointF(min.x(), min.y()), QPointF(max.x(), max.y()));
+}
+
+FPoint FPointArray::widthHeight() const
+{
+	if ( QVector<FPoint>::count() == 0 )
 		return FPoint( 0.0, 0.0 );		// null rectangle
 	ConstIterator pd = begin();
 	ConstIterator pend = begin();
-	pend += count;
+	pend += QVector<FPoint>::count();
 	double minx, maxx, miny, maxy;
 	minx = maxx = pd->xp;
 	miny = maxy = pd->yp;
 	for ( ++pd; pd != pend; ++pd )
 	{	// find min+max x and y
-		if (pd->xp > 900000)
+		if (isMarkerI(pd))
 		{
 			continue;
 		}
@@ -218,7 +238,7 @@ FPoint FPointArray::WidthHeight() const
 	return FPoint(maxx - minx,maxy - miny);
 }
 
-void FPointArray::map( QMatrix m )
+void FPointArray::map( const QTransform& m )
 {
 	const double m11 = m.m11();
 	const double m12 = m.m12();
@@ -228,10 +248,10 @@ void FPointArray::map( QMatrix m )
 	const double dy  = m.dy();
 	double mx, my;
 	Iterator pend = begin();
-	pend += count;
+	pend += QVector<FPoint>::count();
 	for (Iterator p = begin(); p != pend; p++)
 	{
-		if (p->xp > 900000)
+		if (isMarkerD(p->xp, p->yp))
 		{
 			mx = p->xp;
 			my = p->yp;
@@ -248,28 +268,43 @@ void FPointArray::map( QMatrix m )
 
 void FPointArray::setMarker()
 {
-	addQuadPoint(999999.0, 999999.0,
-				999999.0, 999999.0,
-				999999.0, 999999.0,
-				999999.0, 999999.0);
+	double maxVal = std::numeric_limits<double>::max() / 2.0;
+	addQuadPoint(maxVal, maxVal, maxVal, maxVal, maxVal, maxVal, maxVal, maxVal);
+}
+
+bool FPointArray::isMarker(int pos) const
+{
+	double maxVal = std::numeric_limits<double>::max() / 3.0;
+	const FPoint& p = QVector<FPoint>::at(pos);
+	return ((p.x() >= maxVal) && (p.y() >= maxVal));
+}
+
+bool FPointArray::isMarkerI(ConstIterator p) const
+{
+	double maxVal = std::numeric_limits<double>::max() / 3.0;
+	return ((p->xp >= maxVal) && (p->yp >= maxVal));
+}
+
+bool FPointArray::isMarkerD(double x, double y) const
+{
+	double maxVal = std::numeric_limits<double>::max() / 3.0;
+	return ((x >= maxVal) && (y >= maxVal));
 }
 
 void FPointArray::addPoint(double x, double y)
 {
-	FPointArray::resize(count+1);
-	setPoint(count-1, x, y);
+	QVector<FPoint>::append(FPoint(x, y));
 }
 
-void FPointArray::addPoint(FPoint p)
+void FPointArray::addPoint(const FPoint& p)
 {
-	FPointArray::resize(count+1);
-	setPoint(count-1, p);
+	QVector<FPoint>::append(p);
 }
 
 
 bool FPointArray::hasLastQuadPoint(double x1, double y1, double x2, double y2, double x3, double y3, double x4, double y4) const
 {
-	int i = count-4;
+	int i = QVector<FPoint>::count()-4;
 	if (i < 0)
 		return false;
 	ConstIterator p = begin();
@@ -283,37 +318,23 @@ bool FPointArray::hasLastQuadPoint(double x1, double y1, double x2, double y2, d
 	if (p->xp != x3 || p->yp != y3)
 		return false;
 	++p; 
-	if (p->xp != x4 || p->yp != y4)
-		return false;
-	
-	return true;
+	return !(p->xp != x4 || p->yp != y4);
 }
 
 void FPointArray::addQuadPoint(double x1, double y1, double x2, double y2, double x3, double y3, double x4, double y4)
 {
-	const int i = count;
-	FPointArray::resize(count+4);
-	Iterator p = begin();
-	p += i;
-	p->setXY(x1, y1);
-	++p;
-	p->setXY(x2, y2);
-	++p;
-	p->setXY(x3, y3);
-	++p;
-	p->setXY(x4, y4);
+	QVector<FPoint>::append(FPoint(x1, y1));
+	QVector<FPoint>::append(FPoint(x2, y2));
+	QVector<FPoint>::append(FPoint(x3, y3));
+	QVector<FPoint>::append(FPoint(x4, y4));
 }
 
-void FPointArray::addQuadPoint(FPoint p1, FPoint p2, FPoint p3, FPoint p4)
+void FPointArray::addQuadPoint(const FPoint& p1, const FPoint& p2, const FPoint& p3, const FPoint& p4)
 {
-	const int i = count;
-	FPointArray::resize(count+4);
-	Iterator p = begin();
-	p += i;
-	*p++ = p1;
-	*p++ = p2;
-	*p++ = p3;
-	*p = p4;
+	QVector<FPoint>::append(p1);
+	QVector<FPoint>::append(p2);
+	QVector<FPoint>::append(p3);
+	QVector<FPoint>::append(p4);
 }
 
 double FPointArray::lenPathSeg(int seg) const
@@ -379,9 +400,9 @@ void FPointArray::pointTangentNormalAt( int seg, double t, FPoint* p, FPoint* tn
 	// Calculate derivative if necessary.
 	FPoint d;
 	if( tn || n )
-		pointDerivativesAt( seg, t, p, &d, 0L );
+		pointDerivativesAt( seg, t, p, &d, nullptr );
 	else
-		pointDerivativesAt( seg, t, p, 0L, 0L );
+		pointDerivativesAt( seg, t, p, nullptr, nullptr );
 	// Normalize derivative.
 	if( tn || n )
 	{
@@ -416,9 +437,9 @@ void FPointArray::pointDerivativesAt( int seg, double t, FPoint* p, FPoint* d1,
 	q[ 3 ] = point(seg+2);
 	q[ 2 ] = point(seg+3);
 	// The De Casteljau algorithm.
-	for( unsigned short j = 1; j <= 3; j++ )
+	for (unsigned short j = 1; j <= 3; j++)
 	{
-		for( unsigned short i = 0; i <= 3 - j; i++ )
+		for (unsigned short i = 0; i <= 3 - j; i++)
 		{
 			q[ i ] = ( 1.0 - t ) * q[ i ] + t * q[ i + 1 ];
 		}
@@ -439,20 +460,6 @@ void FPointArray::pointDerivativesAt( int seg, double t, FPoint* p, FPoint* d1,
 	if( p )
 		*p = q[ 0 ];
 	delete[]( q );
-	return;
-}
-
-bool FPointArray::operator==(const FPointArray &rhs) const
-{
-	return count == rhs.count && 
-	       QVector<FPoint>::operator==(rhs);
-}
-
-bool FPointArray::operator!=(const FPointArray &rhs) const
-{
-	if (count != rhs.count)
-		return true;
-	return QVector<FPoint>::operator!=(rhs);
 }
 
 
@@ -488,89 +495,59 @@ struct SVGState
 };
 
 
-QString FPointArray::svgPath() const
+QString FPointArray::svgPath(bool closed) const
 {
 	QString tmp = "";
-	FPoint np, np1, np2;
+	FPoint np, np1, np2, np3, np4, firstP;
 	bool nPath = true;
+	bool first = true;
 	if (size() > 3)
 	{
-		for (uint poi=0; poi < size()-3; poi += 4)
+		for (int poi=0; poi < size()-3; poi += 4)
 		{
-			if (point(poi).x() > 900000)
+			if (isMarker(poi))
 			{
-//				tmp += "Z ";
 				nPath = true;
 				continue;
 			}
 			if (nPath)
 			{
 				np = point(poi);
+				if ((!first) && (closed) && (np4 == firstP))
+					tmp += "Z ";
 				tmp += "M"+QString::number(np.x())+" "+QString::number(np.y())+" ";
 				nPath = false;
+				first = false;
+				firstP = np;
+				np4 = np;
 			}
-			np = point(poi+1);
-			tmp += "C"+QString::number(np.x())+" "+QString::number(np.y())+" ";
-			np1 = point(poi+3);
-			tmp += QString::number(np1.x())+" "+QString::number(np1.y())+" ";
-			np2 = point(poi+2);
-			tmp += QString::number(np2.x())+" "+QString::number(np2.y())+" ";
+			np = point(poi);
+			np1 = point(poi+1);
+			np2 = point(poi+3);
+			np3 = point(poi+2);
+			if ((np == np1) && (np2 == np3))
+				tmp += QString("L%1 %2 ").arg(np3.x()).arg(np3.y());
+			else
+				tmp += QString("C%1 %2 %3 %4 %5 %6 ").arg(np1.x()).arg(np1.y()).arg(np2.x()).arg(np2.y()).arg(np3.x()).arg(np3.y());
+			np4 = np3;
 		}
+		if (closed)
+			tmp += "Z";
 	}
 	return tmp;
-/*	Xml_string result;
-	bool hasPoint = false;
-	double x, y, x1, y1, x2, y2, xe=0, ye=0; 
-	uint i=0;
-	while (i < size())
-	{
-		point(i++, &x, &y);
-		if (x > 900000 && y > 900000)  // marker for closepath
-		{
-			hasPoint = false;
-			continue;
-		}
-		if (!hasPoint || x != xe || y != ye) // start a subpath
-		{
-			result += "M";
-			result += QString::number(x);
-			result += " ";
-			result += QString::number(y);
-			result += "C";
-			hasPoint = true;
-		}
-		else 
-			result += " ";
-		
-		point(i++, &x1, &y1);
-		point(i++, &x2, &y2);
-		point(i++, &xe, &ye);
-		result += QString::number(x1);
-		result += " ";
-		result += QString::number(y1);
-		result += " ";
-		result += QString::number(x2);
-		result += " ";
-		result += QString::number(y2);
-		result += " ";
-		result += QString::number(xe);
-		result += " ";
-		result += QString::number(ye);
-	}
-	return result;	*/
 }
 
-QPainterPath FPointArray::toQPainterPath(bool closed)
+QPainterPath FPointArray::toQPainterPath(bool closed) const
 {
 	QPainterPath m_path = QPainterPath();
 	bool nPath = true;
 	bool first = true;
-	FPoint np, np1, np2, np3;
+	FPoint np, np1, np2, np3, np4, firstP;
 	if (size() > 3)
 	{
-		for (uint poi = 0; poi < size()-3; poi += 4)
+		for (int poi = 0; poi < size()-3; poi += 4)
 		{
-			if (point(poi).x() > 900000)
+			if (isMarker(poi))
 			{
 				nPath = true;
 				continue;
@@ -578,11 +555,13 @@ QPainterPath FPointArray::toQPainterPath(bool closed)
 			if (nPath)
 			{
 				np = point(poi);
-				if ((!first) && (closed))
+				if ((!first) && (closed) && (np4 == firstP))
 					m_path.closeSubpath();
 				m_path.moveTo(np.x(), np.y());
 				nPath = false;
 				first = false;
+				firstP = np;
+				np4 = np;
 			}
 			np = point(poi);
 			np1 = point(poi+1);
@@ -592,6 +571,7 @@ QPainterPath FPointArray::toQPainterPath(bool closed)
 				m_path.lineTo(np3.x(), np3.y());
 			else
 				m_path.cubicTo(np1.x(), np1.y(), np2.x(), np2.y(), np3.x(), np3.y());
+			np4 = np3;
 		}
 		if (closed)
 			m_path.closeSubpath();
@@ -599,7 +579,7 @@ QPainterPath FPointArray::toQPainterPath(bool closed)
 	return m_path;
 }
 
-void FPointArray::fromQPainterPath(QPainterPath &path)
+void FPointArray::fromQPainterPath(QPainterPath &path, bool close)
 {
 	resize(0);
 	svgInit();
@@ -609,9 +589,9 @@ void FPointArray::fromQPainterPath(QPainterPath &path)
 		switch (elm.type)
 		{
 			case QPainterPath::MoveToElement:
-				if (svgState->WasM)
+				if (m_svgState->WasM)
 					svgClosePath();
-				svgState->WasM = true;
+				m_svgState->WasM = true;
 				svgMoveTo(elm.x, elm.y);
 				break;
 			case QPainterPath::LineToElement:
@@ -624,105 +604,112 @@ void FPointArray::fromQPainterPath(QPainterPath &path)
 				break;
 		}
 	}
+	if (close)
+		svgClosePath();
 }
 
 FPointArray::~FPointArray()
 {
-	if (svgState)
-		delete svgState;
+	delete m_svgState;
 }
 
 
 void FPointArray::svgInit()
 {
-	if (!svgState)
-		svgState = new SVGState;
-	svgState->reset(0,0);
-	svgState->FirstM = true;
-	svgState->WasM = false;
+	if (!m_svgState)
+		m_svgState = new SVGState;
+	m_svgState->reset(0,0);
+	m_svgState->FirstM = true;
+	m_svgState->WasM = false;
 }
 
 
 void FPointArray::svgMoveTo(double x, double y)
 {
-	svgState->reset(x, y);
-	svgState->WasM = true;
+	if (!m_svgState)
+		return;
+	m_svgState->reset(x, y);
+	m_svgState->WasM = true;
 }
 
 
 void FPointArray::svgLineTo(double x1, double y1)
 {
-	if (svgState->needsMarker())
-	{
+	if (!m_svgState)
+		return;
+	if (m_svgState->needsMarker())
 		setMarker();
-	}
-	svgState->FirstM = false;
-	svgState->WasM = false;
+	m_svgState->FirstM = false;
+	m_svgState->WasM = false;
 	if (size() > 3)
 	{
 		FPoint b1 = point(size()-4);
 		FPoint b2 = point(size()-3);
 		FPoint b3 = point(size()-2);
 		FPoint b4 = point(size()-1);
-		FPoint n1 = FPoint(svgState->CurrX, svgState->CurrY);
+		FPoint n1 = FPoint(m_svgState->CurrX, m_svgState->CurrY);
 		FPoint n2 = FPoint(x1, y1);
 		if ((b1 == n1) && (b2 == n1) && (b3 == n2) && (b4 == n2))
 			return;
 	}
-	addPoint(FPoint(svgState->CurrX, svgState->CurrY));
-	addPoint(FPoint(svgState->CurrX, svgState->CurrY));
+	addPoint(FPoint(m_svgState->CurrX, m_svgState->CurrY));
+	addPoint(FPoint(m_svgState->CurrX, m_svgState->CurrY));
 	addPoint(FPoint(x1, y1));
 	addPoint(FPoint(x1, y1));
-	svgState->move(x1, y1, 4);
+	m_svgState->move(x1, y1, 4);
 }
 
 
 void FPointArray::svgCurveToCubic(double x1, double y1, double x2, double y2, double x3, double y3)
 {
-	if (svgState->needsMarker())
-	{
+	if (!m_svgState)
+		return;
+	if (m_svgState->needsMarker())
 		setMarker();
-	}
-	svgState->FirstM = false;
-	svgState->WasM = false;
-	if (svgState->PathLen > 3)
+	m_svgState->FirstM = false;
+	m_svgState->WasM = false;
+	if (m_svgState->PathLen > 3)
 	{
 		FPoint b1 = point(size()-4);
 		FPoint b2 = point(size()-3);
 		FPoint b3 = point(size()-2);
 		FPoint b4 = point(size()-1);
-		FPoint n1 = FPoint(svgState->CurrX, svgState->CurrY);
+		FPoint n1 = FPoint(m_svgState->CurrX, m_svgState->CurrY);
 		FPoint n2 = FPoint(x1, y1);
 		FPoint n3 = FPoint(x3, y3);
 		FPoint n4 = FPoint(x2, y2);
 		if ((b1 == n1) && (b2 == n2) && (b3 == n3) && (b4 == n4))
 			return;
 	}
-	addPoint(FPoint(svgState->CurrX, svgState->CurrY));
+	addPoint(FPoint(m_svgState->CurrX, m_svgState->CurrY));
 	addPoint(FPoint(x1, y1));
 	addPoint(FPoint(x3, y3));
 	addPoint(FPoint(x2, y2));
-	svgState->move(x3, y3, 4);
+	m_svgState->move(x3, y3, 4);
 }
 
 
 void FPointArray::svgClosePath()
 {
-	if (svgState->PathLen > 2)
+	if (!m_svgState)
+		return;
+	if (m_svgState->PathLen > 2)
 	{
-		if ((svgState->PathLen == 4) || (point(size()-2).x() != svgState->StartX) || (point(size()-2).y() != svgState->StartY))
+		if ((m_svgState->PathLen == 4) || (point(size()-2).x() != m_svgState->StartX) || (point(size()-2).y() != m_svgState->StartY))
 		{
 			addPoint(point(size()-2));
 			addPoint(point(size()-3));
-			addPoint(FPoint(svgState->StartX, svgState->StartY));
-			addPoint(FPoint(svgState->StartX, svgState->StartY));
+			addPoint(FPoint(m_svgState->StartX, m_svgState->StartY));
+			addPoint(FPoint(m_svgState->StartX, m_svgState->StartY));
 		}
 	}
 }
 
 void FPointArray::svgArcTo(double r1, double r2, double angle, bool largeArcFlag, bool sweepFlag, double x1, double y1)
 {
-	calculateArc(false, svgState->CurrX, svgState->CurrY, angle, x1, y1, r1, r2, largeArcFlag, sweepFlag);
+	if (!m_svgState)
+		return;
+	calculateArc(false, m_svgState->CurrX, m_svgState->CurrY, angle, x1, y1, r1, r2, largeArcFlag, sweepFlag);
 }
 
 void FPointArray::calculateArc(bool relative, double &curx, double &cury, double angle, 
@@ -797,7 +784,7 @@ void FPointArray::calculateArc(bool relative, double &curx, double &cury, double
 	else if(th_arc > 0 && !sweepFlag)
 		th_arc -= 2 * M_PI;
 	n_segs = static_cast<int>(ceil(fabs(th_arc / (M_PI * 0.5 + 0.001))));
-	for(i = 0; i < n_segs; i++)
+	for (i = 0; i < n_segs; i++)
 	{
 	{
 		double sin_th, cos_th;
@@ -859,12 +846,12 @@ static const char * getCoord( const char *ptr, double &number )
 	}
 	
 	// read the integer part
-	while(*ptr != '\0' && *ptr >= '0' && *ptr <= '9')
+	while (*ptr != '\0' && *ptr >= '0' && *ptr <= '9')
 		integer = (integer * 10) + *(ptr++) - '0';
 	if(*ptr == '.') // read the decimals
 	{
 		ptr++;
-		while(*ptr != '\0' && *ptr >= '0' && *ptr <= '9')
+		while (*ptr != '\0' && *ptr >= '0' && *ptr <= '9')
 			decimal += (*(ptr++) - '0') * (frac *= 0.1);
 	}
 	
@@ -882,7 +869,7 @@ static const char * getCoord( const char *ptr, double &number )
 		}
 		
 		exponent = 0;
-		while(*ptr != '\0' && *ptr >= '0' && *ptr <= '9')
+		while (*ptr != '\0' && *ptr >= '0' && *ptr <= '9')
 		{
 			exponent *= 10;
 			exponent += *ptr - '0';
@@ -904,7 +891,7 @@ bool FPointArray::parseSVG(const QString& svgPath)
 	QString d = svgPath;
 	d = d.replace( QRegExp( "," ), " ");
 	bool ret = false;
-	if (!d.isEmpty() )
+	if (!d.isEmpty())
 	{
 		d = d.simplified();
 		QByteArray pathData = d.toLatin1();
@@ -913,6 +900,7 @@ bool FPointArray::parseSVG(const QString& svgPath)
 		double contrlx, contrly, curx, cury, subpathx, subpathy, tox, toy, x1, y1, x2, y2, xc, yc;
 		double px1, py1, px2, py2, px3, py3;
 		bool relative;
+		int moveCount = 0;
 		svgInit();
 		char command = *(ptr++), lastCommand = ' ';
 		subpathx = subpathy = curx = cury = contrlx = contrly = 0.0;
@@ -923,16 +911,23 @@ bool FPointArray::parseSVG(const QString& svgPath)
 			relative = false;
 			switch (command)
 			{
+			case 'f':
+			case 'F':
+				{
+					ptr = getCoord( ptr, tox );
+					break;
+				}
 			case 'm':
 				relative = true;
 			case 'M':
 				{
 					ptr = getCoord( ptr, tox );
 					ptr = getCoord( ptr, toy );
-					svgState->WasM = true;
+					m_svgState->WasM = true;
 					subpathx = curx = relative ? curx + tox : tox;
 					subpathy = cury = relative ? cury + toy : toy;
 					svgMoveTo(curx, cury );
+					moveCount++;
 					break;
 				}
 			case 'l':
@@ -1107,15 +1102,14 @@ bool FPointArray::parseSVG(const QString& svgPath)
 				contrly = cury;
 			}
 		}
-		if ((lastCommand != 'z') && (lastCommand != 'Z'))
+		if (((lastCommand != 'z') && (lastCommand != 'Z')) || (moveCount > 1))
 			ret = true;
 		if (size() > 2)
 		{
-			if ((point(0).x() == point(size()-2).x()) && (point(0).y() == point(size()-2).y()))
+			if ((point(0).x() == point(size()-2).x()) && (point(0).y() == point(size()-2).y()) && (moveCount == 1))
 				ret = false;
 		}
 	}
 	return ret;
-
 }
 
