@@ -37,6 +37,7 @@ for which a new license (GPL+exception) is in place.
 #include <io.h>
 
 #include <QApplication>
+#include <QMessageBox>
 #include <exception>
 using namespace std;
 
@@ -46,6 +47,7 @@ using namespace std;
 #include "scribusapp.h"
 #include "scribuscore.h"
 #include "scribus.h"
+#include "scimagecachemanager.h"
 
 #include "scconfig.h"
 
@@ -60,7 +62,7 @@ static QString exceptionDescription(DWORD exceptionCode);
 static void defaultCrashHandler(DWORD exceptionCode);
 
 // Console IO redirection handling
-void messageHandler( QtMsgType type, const char *msg );
+void messageHandler(QtMsgType type, const QMessageLogContext &context, const QString &msg);
 bool consoleOptionEnabled(int argc, char* argv[]);
 void redirectIOToConsole(void);
 
@@ -72,21 +74,31 @@ extern const char ARG_CONSOLE[];
 extern const char ARG_CONSOLE_SHORT[];
 
 ScribusCore SCRIBUS_API *ScCore;
-ScribusMainWindow SCRIBUS_API *ScMW;
 ScribusQApp SCRIBUS_API *ScQApp;
 bool emergencyActivated;
 
+#if !defined(_CONSOLE)
+#if defined(_DEBUG)
+#pragma comment(lib, "qtmaind.lib")
+#else
+#pragma comment(lib, "qtmain.lib")
+#endif
+#endif
+
 int main(int argc, char *argv[])
 {
 	int result;
 	emergencyActivated = false;
+
 #if !defined(_CONSOLE)
 	if (consoleOptionEnabled(argc, argv))
 	{
 		redirectIOToConsole();
-		qInstallMsgHandler( messageHandler );
+		qInstallMessageHandler(messageHandler);
 	}
 #endif
+
+	ScribusQApp::setAttribute(Qt::AA_EnableHighDpiScaling);
 	ScribusQApp app(argc, argv);
 	setPythonEnvironment(app.applicationDirPath());
 	result =  mainApp(app);
@@ -113,10 +125,10 @@ int mainApp(ScribusQApp& app)
 	{
 #endif
 		app.parseCommandLine();
-		if (app.useGUI)
+		appRetVal = app.init();
+		if (appRetVal != EXIT_FAILURE)
 		{
-			appRetVal = app.init();
-			if (appRetVal != EXIT_FAILURE)
+			if (app.useGUI)
 				appRetVal = app.exec();
 		}
 #ifndef _DEBUG
@@ -164,7 +176,7 @@ void setPythonEnvironment(const QString& appPath)
 	tmp += ";";
 	tmp += nativePath;
 	tmp += "\\python";
-	if (oldenv != NULL) {
+	if (oldenv != nullptr) {
 		tmp += ";";
 		tmp +=  QString::fromUtf16((const ushort*) oldenv);
 	}
@@ -281,33 +293,68 @@ void defaultCrashHandler(DWORD exceptionCode)
 		QString expHdr  = QObject::tr("Scribus Crash");
 		QString expLine = "-------------";
 		QString expMsg  = QObject::tr("Scribus crashes due to the following exception : %1").arg(expDesc);
-		std::cout << (const char*) expHdr.toAscii().data() << std::endl;
-		std::cout << (const char*) expLine.toAscii().data() << std::endl;
-		std::cout << (const char*) expMsg.toAscii().data() << std::endl;
+		std::cout << expHdr.toStdString() << std::endl;
+		std::cout << expLine.toStdString() << std::endl;
+		std::cout << expMsg.toStdString() << std::endl;
 		if (ScribusQApp::useGUI)
 		{
 			ScCore->closeSplash();
-			QMessageBox::critical(ScMW, expHdr, expMsg, QObject::tr("&OK"));
-			ScMW->emergencySave();
-			ScMW->close();
+			ScribusMainWindow* mainWin = ScCore->primaryMainWindow();
+			if (mainWin)
+			{
+				ScMessageBox::critical(mainWin, expHdr, expMsg);
+				mainWin->emergencySave();
+				mainWin->close();
+			}
 		}
+		ScImageCacheManager::instance().removeMasterLock();
 	}
 	ExitProcess(255);
 }
 
-void messageHandler( QtMsgType type, const char *msg )
+void messageHandler(QtMsgType type, const QMessageLogContext &context, const QString &msg)
 {
-	cerr << msg << endl;
-	if (type == QtFatalMsg)
+	QByteArray localMsg = msg.toLocal8Bit();
+	switch (type)
 	{
+	case QtDebugMsg:
+		cerr << "Debug: " << localMsg.constData();
+		if (context.file && context.function)
+			cerr << "(" << context.file << ":" << context.line << ", " << context.function << ")";
+		cerr << endl;
+		break;
+	case QtWarningMsg:
+		cerr << "Warning: " << localMsg.constData();
+		if (context.file && context.function)
+			cerr << "(" << context.file << ":" << context.line << ", " << context.function << ")";
+		cerr << endl;
+		break;
+	case QtCriticalMsg:
+		cerr << "Critical: " << localMsg.constData();
+		if (context.file && context.function)
+			cerr << "(" << context.file << ":" << context.line << ", " << context.function << ")";
+		cerr << endl;
+		break;
+	case QtFatalMsg:
 		if (ScribusQApp::useGUI)
 		{
 			ScCore->closeSplash();
-			QString expHdr = QObject::tr("Scribus Crash");
-			QString expMsg = msg;
-			QMessageBox::critical(ScMW, expHdr, expMsg, QObject::tr("&OK"));
-			ScMW->emergencySave();
-			ScMW->close();
+			ScribusMainWindow* mainWin = ScCore->primaryMainWindow();
+			if (mainWin)
+			{
+				QString expHdr = QObject::tr("Scribus Crash");
+				QString expMsg = msg;
+				ScMessageBox::critical(mainWin, expHdr, expMsg);
+				mainWin->emergencySave();
+				mainWin->close();
+			}
+		}
+		else
+		{
+			cerr << "Fatal: " << localMsg.constData();
+			if (context.file && context.function)
+				cerr << "(" << context.file << ":" << context.line << ", " << context.function << ")";
+			cerr << endl;
 		}
 		ExitProcess(255);
 	}
@@ -355,7 +402,7 @@ void redirectIOToConsole(void)
 		if (hConHandle != -1)
 		{
 			_dup2(hConHandle, fileno(stdin));
-			setvbuf(stdin, NULL, _IONBF, 0);
+			setvbuf(stdin, nullptr, _IONBF, 0);
 			SetStdHandle(STD_INPUT_HANDLE, (HANDLE) _get_osfhandle(fileno(stdin)));
 			_close(hConHandle);
 		}
@@ -369,7 +416,7 @@ void redirectIOToConsole(void)
 		if (hConHandle != -1)
 		{
 			_dup2(hConHandle, fileno(stdout));
-			setvbuf(stdout, NULL, _IONBF, 0);
+			setvbuf(stdout, nullptr, _IONBF, 0);
 			SetStdHandle(STD_OUTPUT_HANDLE, (HANDLE) _get_osfhandle(fileno(stdout)));
 			_close(hConHandle);
 		}
@@ -383,7 +430,7 @@ void redirectIOToConsole(void)
 		if (hConHandle != -1)
 		{
 			_dup2(hConHandle, fileno(stderr));
-			setvbuf(stderr, NULL, _IONBF, 0);
+			setvbuf(stderr, nullptr, _IONBF, 0);
 			SetStdHandle(STD_ERROR_HANDLE, (HANDLE) _get_osfhandle(fileno(stderr)));
 			_close(hConHandle);
 		}
