@@ -22,143 +22,201 @@ for which a new license (GPL+exception) is in place.
  ***************************************************************************/
 
 // #include <QDebug>
+#include <QApplication>
+#include <QFontInfo>
 #include <QGridLayout>
 #include <QKeyEvent>
 
 #include <cmath>
 #include <cassert>
 
-#include "page.h"
+#include "scconfig.h"
+
+#include "appmodes.h"
+#include "commonstrings.h"
+#include "filewatcher.h"
 #include "pageitem.h"
 #include "pageitem_imageframe.h"
 #include "prefsmanager.h"
 #include "scraction.h"
+#include "scpage.h"
 #include "scpaths.h"
 #include "scpainter.h"
-#include "scribus.h"
 #include "scribusstructs.h"
+#include "scribuscore.h"
 #include "scribusdoc.h"
-#include "commonstrings.h"
+#include "scribusview.h"
+
 #include "undomanager.h"
 #include "undostate.h"
-#include "scconfig.h"
+#include "undotransaction.h"
 #include "util_formats.h"
 #include "util_color.h"
-
 #include "util.h"
 
 
-PageItem_ImageFrame::PageItem_ImageFrame(ScribusDoc *pa, double x, double y, double w, double h, double w2, QString fill, QString outline)
+PageItem_ImageFrame::PageItem_ImageFrame(ScribusDoc *pa, double x, double y, double w, double h, double w2, const QString& fill, const QString& outline)
 	: PageItem(pa, PageItem::ImageFrame, x, y, w, h, w2, fill, outline)
 {
 }
 
-void PageItem_ImageFrame::DrawObj_Item(ScPainter *p, QRectF /*e*/, double sc)
+PageItem_ImageFrame::~PageItem_ImageFrame()
 {
-	if(!m_Doc->RePos)
+	if ((imageIsAvailable) && (!Pfile.isEmpty()))
 	{
-		if (!m_Doc->layerOutline(LayerNr))
+		ScCore->fileWatcher->removeFile(Pfile);
+		QFileInfo fi(Pfile);
+		ScCore->fileWatcher->removeDir(fi.absolutePath());
+	}
+}
+
+void PageItem_ImageFrame::DrawObj_Item(ScPainter *p, QRectF /*e*/)
+{
+	if (m_Doc->RePos)
+		return;
+	if (m_Doc->layerOutline(m_layerID))
+		return;
+
+	p->setFillRule(true);
+	if ((fillColor() != CommonStrings::None) || (GrType != 0))
+	{
+		p->setupPolygon(&PoLine);
+		p->fillPath();
+	}
+	p->save();
+	if (Pfile.isEmpty())
+	{
+		if ((drawFrame()) && (m_Doc->guidesPrefs().framesShown))
 		{
-			p->setFillRule(true);
-			if ((fillColor() != CommonStrings::None) || (GrType != 0))
+			p->setPen(Qt::black, 1, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin);
+			p->drawLine(FPoint(0, 0), FPoint(m_width, m_height));
+			p->drawLine(FPoint(0, m_height), FPoint(m_width, 0));
+		}
+	}
+	else
+	{
+		//If we are missing our image, draw a red cross in the frame
+		if ((!m_imageVisible) || (!imageIsAvailable))
+		{
+			if ((drawFrame()) && (m_Doc->guidesPrefs().framesShown))
 			{
-				p->setupPolygon(&PoLine);
-				p->fillPath();
-			}
-			p->save();
-			if (Pfile.isEmpty())
-			{
-				if ((Frame) && (m_Doc->guidesSettings.framesShown))
+				p->setBrush(Qt::white);
+				QString htmlText = "";
+				QFileInfo fi = QFileInfo(Pfile);
+				if (imageIsAvailable)
 				{
 					p->setPen(Qt::black, 1, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin);
-					p->drawLine(FPoint(0, 0), FPoint(Width, Height));
-					p->drawLine(FPoint(0, Height), FPoint(Width, 0));
-				}
-			}
-			else
-			{
-				//If we are missing our image, draw a red cross in the frame
-				if ((!PicArt) || (!PictureIsAvailable))
-				{
-					if ((Frame) && (m_Doc->guidesSettings.framesShown))
+					if (isInlineImage)
+						htmlText.append( tr("Embedded Image") + "\n");
+					else
+						htmlText.append( tr("File:") + " " + fi.fileName() + "\n");
+					htmlText.append( tr("Original PPI:") + " " + QString::number(pixm.imgInfo.xres)+" x "+QString::number(pixm.imgInfo.yres) + "\n");
+					htmlText.append( tr("Actual PPI:") + " " + QString::number(qRound(72.0 / imageXScale()))+" x "+ QString::number(qRound(72.0 / imageYScale())) + "\n");
+					htmlText.append( tr("Size:") + " " + QString::number(OrigW) + " x " + QString::number(OrigH) + "\n");
+					htmlText.append( tr("Colorspace:") + " ");
+					QString ext = fi.suffix().toLower();
+					if ((extensionIndicatesPDF(ext) || extensionIndicatesEPSorPS(ext)) && (pixm.imgInfo.type != ImageType7))
+						htmlText.append( tr("Unknown"));
+					else
+						htmlText.append(colorSpaceText(pixm.imgInfo.colorspace));
+					if (pixm.imgInfo.numberOfPages > 1)
 					{
-						p->setBrush(Qt::white);
-						QString htmlText = "";
-						QFileInfo fi = QFileInfo(Pfile);
-						if (PictureIsAvailable)
-						{
-							p->setPen(Qt::black, 1, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin);
-							if (isInlineImage)
-								htmlText.append( tr("Embedded Image") + "\n");
-							else
-								htmlText.append( tr("File:") + " " + fi.fileName() + "\n");
-							htmlText.append( tr("Original PPI:") + " " + QString::number(qRound(pixm.imgInfo.xres))+" x "+QString::number(qRound(pixm.imgInfo.yres)) + "\n");
-							htmlText.append( tr("Actual PPI:") + " " + QString::number(qRound(72.0 / imageXScale()))+" x "+ QString::number(qRound(72.0 / imageYScale())) + "\n");
-							htmlText.append( tr("Size:") + " " + QString::number(OrigW) + " x " + QString::number(OrigH) + "\n");
-							htmlText.append( tr("Colorspace:") + " ");
-							QString cSpace;
-							QString ext = fi.suffix().toLower();
-							if ((extensionIndicatesPDF(ext) || extensionIndicatesEPSorPS(ext)) && (pixm.imgInfo.type != ImageType7))
-								htmlText.append( tr("Unknown"));
-							else
-								htmlText.append(colorSpaceText(pixm.imgInfo.colorspace));
-							if (pixm.imgInfo.numberOfPages > 1)
-							{
-								htmlText.append("\n");
-								if (pixm.imgInfo.actualPageNumber > 0)
-									htmlText.append( tr("Page:") + " " + QString::number(pixm.imgInfo.actualPageNumber) + "/" + QString::number(pixm.imgInfo.numberOfPages));
-								else
-									htmlText.append( tr("Pages:") + " " + QString::number(pixm.imgInfo.numberOfPages));
-							}
-						}
+						htmlText.append("\n");
+						if (pixm.imgInfo.actualPageNumber > 0)
+							htmlText.append( tr("Page:") + " " + QString::number(pixm.imgInfo.actualPageNumber) + "/" + QString::number(pixm.imgInfo.numberOfPages));
 						else
-						{
-							p->setPen(Qt::red, 1, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin);
-							htmlText = fi.fileName();
-						}
-						p->drawLine(FPoint(0, 0), FPoint(Width, Height));
-						p->drawLine(FPoint(0, Height), FPoint(Width, 0));
-						p->setFont(QApplication::font());
-						p->drawText(QRectF(0.0, 0.0, Width, Height), htmlText);
+							htmlText.append( tr("Pages:") + " " + QString::number(pixm.imgInfo.numberOfPages));
 					}
 				}
 				else
 				{
-					p->setupPolygon(&PoLine);
-					p->setClipPath();
-					if (imageFlippedH())
-					{
-						p->translate(Width, 0);
-						p->scale(-1, 1);
-					}
-					if (imageFlippedV())
-					{
-						p->translate(0, Height);
-						p->scale(1, -1);
-					}
-					if (imageClip.size() != 0)
-					{
-						p->setupPolygon(&imageClip);
-						p->setClipPath();
-					}
-					p->translate(LocalX*LocalScX, LocalY*LocalScY);
-					p->scale(LocalScX, LocalScY);
-					if (pixm.imgInfo.lowResType != 0)
-						p->scale(pixm.imgInfo.lowResScale, pixm.imgInfo.lowResScale);
-					p->drawImage(pixm.qImagePtr());
+					p->setPen(Qt::red, 1, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin);
+					htmlText = fi.fileName();
+				}
+				p->drawLine(FPoint(0, 0), FPoint(m_width, m_height));
+				p->drawLine(FPoint(0, m_height), FPoint(m_width, 0));
+				const QFont &font = QApplication::font();
+				p->setFont(PrefsManager::instance().appPrefs.fontPrefs.AvailFonts.findFont(font.family(), QFontInfo(font).styleName()), font.pointSizeF());
+				p->drawText(QRectF(0.0, 0.0, m_width, m_height), htmlText);
+			}
+		}
+		else
+		{
+			p->setupPolygon(&PoLine);
+			p->setClipPath();
+			if (imageFlippedH())
+			{
+				p->translate(m_width, 0);
+				p->scale(-1, 1);
+			}
+			if (imageFlippedV())
+			{
+				p->translate(0, m_height);
+				p->scale(1, -1);
+			}
+			if (!imageClip.empty())
+			{
+				p->setupPolygon(&imageClip);
+				p->setClipPath();
+			}
+			p->translate(m_imageXOffset*m_imageXScale, m_imageYOffset*m_imageYScale);
+			p->rotate(m_imageRotation);
+			double mscalex = 1.0 / m_imageXScale;
+			double mscaley = 1.0 / m_imageYScale;
+			p->scale(m_imageXScale, m_imageYScale);
+			if (pixm.imgInfo.lowResType != 0)
+			{
+				p->scale(pixm.imgInfo.lowResScale, pixm.imgInfo.lowResScale);
+				mscalex *= 1.0 / pixm.imgInfo.lowResScale;
+				mscaley *= 1.0 / pixm.imgInfo.lowResScale;
+			}
+			if ((GrMask == 1) || (GrMask == 2) || (GrMask == 4) || (GrMask == 5))
+			{
+				if ((GrMask == 1) || (GrMask == 2))
+					p->setMaskMode(1);
+				else
+					p->setMaskMode(3);
+				if ((!gradientMaskVal.isEmpty()) && (!m_Doc->docGradients.contains(gradientMaskVal)))
+					gradientMaskVal = "";
+				if (!(gradientMaskVal.isEmpty()) && (m_Doc->docGradients.contains(gradientMaskVal)))
+					mask_gradient = m_Doc->docGradients[gradientMaskVal];
+				p->mask_gradient = mask_gradient;
+				if ((GrMask == 1) || (GrMask == 4))
+					p->setGradientMask(VGradient::linear, FPoint(GrMaskStartX * mscalex, GrMaskStartY * mscaley), FPoint(GrMaskEndX * mscalex, GrMaskEndY * mscaley), FPoint(GrMaskStartX * mscalex, GrMaskStartY * mscaley), GrMaskScale, GrMaskSkew);
+				else
+					p->setGradientMask(VGradient::radial, FPoint(GrMaskStartX * mscalex, GrMaskStartY * mscaley), FPoint(GrMaskEndX * mscalex, GrMaskEndY * mscaley), FPoint(GrMaskFocalX * mscalex, GrMaskFocalY * mscaley), GrMaskScale, GrMaskSkew);
+			}
+			else if ((GrMask == 3) || (GrMask == 6) || (GrMask == 7) || (GrMask == 8))
+			{
+				if ((patternMaskVal.isEmpty()) || (!m_Doc->docPatterns.contains(patternMaskVal)))
+					p->setMaskMode(0);
+				else
+				{
+					p->setPatternMask(&m_Doc->docPatterns[patternMaskVal], patternMaskScaleX * mscalex, patternMaskScaleY * mscaley, patternMaskOffsetX, patternMaskOffsetY, patternMaskRotation, patternMaskSkewX, patternMaskSkewY, patternMaskMirrorX, patternMaskMirrorY);
+					if (GrMask == 3)
+						p->setMaskMode(2);
+					else if (GrMask == 6)
+						p->setMaskMode(4);
+					else if (GrMask == 7)
+						p->setMaskMode(5);
+					else
+						p->setMaskMode(6);
 				}
 			}
-			p->restore();
+			else
+				p->setMaskMode(0);
+			p->drawImage(pixm.qImagePtr());
 		}
 	}
+	p->restore();
 }
 
 void PageItem_ImageFrame::clearContents()
 {
 	if (UndoManager::undoEnabled())
 	{
-		ItemState<ScImageEffectList> *is = new ItemState<ScImageEffectList>(Um::ClearImage + "\n" + Pfile, "");
-		is->set("CLEAR_IMAGE", "clear_image");
+		ScItemState<ScImageEffectList> *is = new ScItemState<ScImageEffectList>(Um::ClearImage + "\n" + Pfile, "");
+		is->set("CLEAR_IMAGE");
 		is->set("CI_PFILE", Pfile);
 		is->set("CI_FLIPPH",imageFlippedH());
 		is->set("CI_FLIPPV",imageFlippedV());
@@ -174,27 +232,27 @@ void PageItem_ImageFrame::clearContents()
 		undoManager->action(this, is);
 	}
 	effectsInUse.clear();
-	PictureIsAvailable = false;
+	imageIsAvailable = false;
 	Pfile = "";
 	pixm = ScImage();
 
-	LocalScX = 1;
-	LocalScY = 1;
+	m_imageXScale = 1;
+	m_imageYScale = 1;
 	OrigW = 0;
 	OrigH = 0;
-	LocalX = 0;
-	LocalY = 0;
+	m_imageXOffset = 0;
+	m_imageYOffset = 0;
 	setImageFlippedH(false);
 	setImageFlippedV(false);
 	EmProfile = "";
-	ScaleType = true;
-	AspectRatio = true;
+	ScaleType = m_Doc->prefsData().itemToolPrefs.imageScaleType;;
+	AspectRatio = m_Doc->prefsData().itemToolPrefs.imageAspectRatio;
 	setFillTransparency(0.0);
 	setLineTransparency(0.0);
 	imageClip.resize(0);
-	if (tempImageFile != NULL)
-		delete tempImageFile;
-	tempImageFile = NULL;
+	if ((isTempFile) && (!Pfile.isEmpty()))
+		QFile::remove(Pfile);
+	isTempFile = false;
 	isInlineImage = false;
 	//				emit UpdtObj(Doc->currentPage->pageNr(), ItemNr);
 }
@@ -215,9 +273,18 @@ void PageItem_ImageFrame::handleModeEditKey(QKeyEvent *k, bool& keyRepeat)
 		moveBy=0.01;
 	else if (!shiftDown && altDown)
 		resizingImage=true;
+
 	double dX=0.0,dY=0.0;
 	int kk = k->key();
-	if (!resizingImage)
+	ScribusView* view = m_Doc->view();
+
+	if (kk == Qt::Key_Delete || kk == Qt::Key_Backspace || (shiftDown && controlDown && kk == Qt::Key_Delete) )	// Delete in edit mode, to empty the frame
+	{
+		clearContents();
+		view->requestMode(modeNormal);
+		update();
+	}
+	else if (!resizingImage)
 	{
 		moveBy/=m_Doc->unitRatio();//Lets allow movement by the current doc ratio, not only points
 		switch (kk)
@@ -259,37 +326,47 @@ void PageItem_ImageFrame::handleModeEditKey(QKeyEvent *k, bool& keyRepeat)
 				break;
 			default:
 				return;
-		}		
-		if (dX!=0.0)
+		}
+		UndoTransaction transaction;
+		if (UndoManager::undoEnabled())
 		{
-			double newXScale=dX / 100.0 * LocalScX;
+			if ((fitImageToFrame() || !controlDown) && (dX != 0.0 || dY != 0.0))
+				transaction = undoManager->beginTransaction(getUName(), getUPixmap(), Um::ImageScale, "", Um::IMove);
+		}
+		if (dX != 0.0)
+		{
+			double newXScale = dX / 100.0 * m_imageXScale;
+			setImageScalingMode(true, AspectRatio);
 			setImageXScale(newXScale);
 			if (!controlDown)
 			{
-				double newYScale=dX / 100.0 * LocalScY;
+				double newYScale = dX / 100.0 * m_imageYScale;
 				setImageYScale(newYScale);
 			}
 		}
-		else
-		if (dY!=0.0)
+		else if (dY != 0.0)
 		{
-			double newYScale=dY / 100.0 * LocalScY;
+			double newYScale = dY / 100.0 * m_imageYScale;
+			setImageScalingMode(true, AspectRatio);
 			setImageYScale(newYScale);
 			if (!controlDown)
 			{
-				double newXScale=dY / 100.0 * LocalScY;
+				double newXScale = dY / 100.0 * m_imageYScale;
 				setImageXScale(newXScale);
 			}
 		}
-		if (dX!=0.0 || dY!=0.0)
-			if (imageClip.size() != 0)
+		if (dX != 0.0 || dY != 0.0)
+			if (!imageClip.empty())
 			{
 				imageClip = pixm.imgInfo.PDSpathData[pixm.imgInfo.usedPath].copy();
-				QMatrix cl;
+				QTransform cl;
 				cl.translate(imageXOffset()*imageXScale(), imageYOffset()*imageYScale());
+				cl.rotate(imageRotation());
 				cl.scale(imageXScale(), imageYScale());
 				imageClip.map(cl);
 			}
+		if (transaction)
+			transaction.commit();
 		update();	
 	}
 }
@@ -303,7 +380,7 @@ bool PageItem_ImageFrame::createInfoGroup(QFrame *infoGroup, QGridLayout *infoGr
 	infoCT->setText( tr("Image"));
 	infoGroupLayout->addWidget( infoCT, 0, 0, 1, 2, Qt::AlignHCenter );
 	
-	if (PictureIsAvailable)
+	if (imageIsAvailable)
 	{
 		fileT = new QLabel(infoGroup);
 		oPpiT = new QLabel(infoGroup);
@@ -323,7 +400,7 @@ bool PageItem_ImageFrame::createInfoGroup(QFrame *infoGroup, QGridLayout *infoGr
 		
 		oPpiCT->setText( tr("Original PPI:"));
 		infoGroupLayout->addWidget( oPpiCT, 2, 0, Qt::AlignRight );
-		oPpiT->setText(QString::number(qRound(pixm.imgInfo.xres))+" x "+QString::number(qRound(pixm.imgInfo.yres)));
+		oPpiT->setText(QString::number(pixm.imgInfo.xres)+" x "+QString::number(pixm.imgInfo.yres));
 		infoGroupLayout->addWidget( oPpiT, 2, 1 );
 		
 		aPpiCT->setText( tr("Actual PPI:"));
@@ -367,81 +444,6 @@ bool PageItem_ImageFrame::createInfoGroup(QFrame *infoGroup, QGridLayout *infoGr
 	return true;
 }
 
-/*
-bool PageItem_ImageFrame::createContextMenu(QMenu *menu, int step)
-{
-	QMap<QString, QPointer<ScrAction> > actions = doc()->scMW()->scrActions;
-	static QMenu* menuResolution = 0;
-	QAction *act;
-	
-	if (menu == 0) {
-		if (menuResolution) delete menuResolution;
-		menuResolution = 0;
-		return true;
-	}
-	
-	switch (step) {
-		case 5:
-			if (pixm.imgInfo.exifDataValid)
-				menu->addAction(actions["itemImageInfo"]);
-		break;
-		case 10:
-			menu->addSeparator();
-			menu->addAction(actions["fileImportImage"]);
-			if (PictureIsAvailable)
-			{
-				if (!isTableItem)
-					menu->addAction(actions["itemAdjustFrameToImage"]);
-				menu->addAction(actions["itemAdjustImageToFrame"]);
-				if (pixm.imgInfo.valid)
-					menu->addAction(actions["itemExtendedImageProperties"]);
-				menu->addAction(actions["itemUpdateImage"]);
-			}
-			createContextMenu(menu, 11);
-			if (PictureIsAvailable && isRaster)
-			{
-				menu->addAction(actions["styleImageEffects"]);
-				menu->addAction(actions["editEditWithImageEditor"]);
-			}
-		break;
-		case 11:
-//			if (menuResolution != 0) {
-//				qDebug() << "New context menu created before old was destroyed."
-//						"Loosing some bytes of memory!";
-//			}
-			menuResolution = new QMenu();
-			act = menu->addMenu(menuResolution);
-			act->setText( tr("Preview Settings"));
-			menuResolution->addAction(actions["itemImageIsVisible"]);
-			menuResolution->addSeparator();
-			menuResolution->addAction(actions["itemPreviewLow"]);
-			menuResolution->addAction(actions["itemPreviewNormal"]);
-			menuResolution->addAction(actions["itemPreviewFull"]);
-		break;
-		case 30:
-			actions["itemConvertToTextFrame"]->setEnabled(true);
-			menu->addAction(actions["itemConvertToTextFrame"]);
-			if (!isTableItem)
-				menu->addAction(actions["itemConvertToPolygon"]);
-		break;
-		case 40:
-			if (PictureIsAvailable)
-				menu->addAction(actions["editCopyContents"]);
-			if (doc()->scMW()->contentsBuffer.sourceType==PageItem::ImageFrame)
-			{
-				menu->addAction(actions["editPasteContents"]);
-				menu->addAction(actions["editPasteContentsAbs"]);
-			}
-			if (PictureIsAvailable)
-				menu->addAction(actions["editClearContents"]);
-			return (PictureIsAvailable) || (doc()->scMW()->contentsBuffer.sourceType==PageItem::ImageFrame);
-		default:
-			return false;
-	}
-	return true;
-}
-*/
-
 void PageItem_ImageFrame::applicableActions(QStringList & actionList)
 {
 	actionList << "fileImportImage";
@@ -451,26 +453,26 @@ void PageItem_ImageFrame::applicableActions(QStringList & actionList)
 	actionList << "itemPreviewLow";
 	actionList << "itemPreviewNormal";
 
-	if (PictureIsAvailable)
+	if (imageIsAvailable)
 	{
-		if (!isTableItem)
-			actionList << "itemAdjustFrameToImage";
+		actionList << "itemAdjustFrameToImage";
 		actionList << "itemAdjustImageToFrame";
 		if (pixm.imgInfo.valid)
 			actionList << "itemExtendedImageProperties";
 		if (pixm.imgInfo.exifDataValid)
 			actionList << "itemImageInfo";
 		actionList << "itemUpdateImage";
-		actionList << "editClearContents";
 		actionList << "editCopyContents";
+		actionList << "itemToggleInlineImage";
 		if (isRaster)
 		{
 			actionList << "styleImageEffects";
 			actionList << "editEditWithImageEditor";
 		}
 	}
-	if (!isTableItem)
-		actionList << "itemConvertToPolygon";
+	if(!Pfile.isEmpty())
+		actionList << "editClearContents";
+	actionList << "itemConvertToPolygon";
 	if (doc()->scMW()->contentsBuffer.sourceType==PageItem::ImageFrame)
 	{
 		actionList << "editPasteContents";
@@ -478,22 +480,21 @@ void PageItem_ImageFrame::applicableActions(QStringList & actionList)
 	}
 }
 
-QString PageItem_ImageFrame::infoDescription()
+QString PageItem_ImageFrame::infoDescription() const
 {
 	QString htmlText;
 	htmlText.append( tr("Image") + "<br/>");
 	
-	if (PictureIsAvailable)
+	if (imageIsAvailable)
 	{
 		QFileInfo fi = QFileInfo(Pfile);
 		if (isInlineImage)
 			htmlText.append( tr("Embedded Image") + "<br/>");
 		else
 			htmlText.append( tr("File:") + " " + fi.fileName() + "<br/>");
-		htmlText.append( tr("Original PPI:") + " " + QString::number(qRound(pixm.imgInfo.xres))+" x "+QString::number(qRound(pixm.imgInfo.yres)) + "<br/>");
+		htmlText.append( tr("Original PPI:") + " " + QString::number(pixm.imgInfo.xres)+" x "+QString::number(pixm.imgInfo.yres) + "<br/>");
 		htmlText.append( tr("Actual PPI:") + " " + QString::number(qRound(72.0 / imageXScale()))+" x "+ QString::number(qRound(72.0 / imageYScale())) + "<br/>");
 		htmlText.append( tr("Colorspace:") + " ");
-		QString cSpace;
 		QString ext = fi.suffix().toLower();
 		if ((extensionIndicatesPDF(ext) || extensionIndicatesEPSorPS(ext)) && (pixm.imgInfo.type != ImageType7))
 			htmlText.append( tr("Unknown"));
