@@ -20,8 +20,8 @@ class PLUGIN_API FontPreviewPlugin : public ScActionPlugin
 		// Standard plugin implementation
 		FontPreviewPlugin();
 		virtual ~FontPreviewPlugin();
-		virtual bool run(ScribusDoc* doc, QString target = QString::null);
-		virtual bool run(QWidget *parent, ScribusDoc* doc, QString target = QString::null);
+		virtual bool run(ScribusDoc* doc, const QString& target = QString());
+		virtual bool run(QWidget *parent, ScribusDoc* doc, const QString& target = QString());
 		virtual const QString fullTrName() const;
 		virtual const AboutData* getAboutData() const;
 		virtual void deleteAboutData(const AboutData* about) const;
