@@ -5,6 +5,9 @@ a copyright and/or license notice that predates the release of Scribus 1.3.2
 for which a new license (GPL+exception) is in place.
 */
 #include "csvim.h"
+#include "gtwriter.h"
+#include "gtparagraphstyle.h"
+#include "gtframestyle.h"
 #include "scribusstructs.h"
 #include "util.h"
 
@@ -18,13 +21,12 @@ QStringList FileExtensions()
     return QStringList("csv");
 }
 
-void GetText(QString filename, QString encoding, bool /* textOnly */, gtWriter *writer)
+void GetText(const QString& filename, const QString& encoding, bool /* textOnly */, gtWriter *writer)
 {
 	CsvDialog* csvdia = new CsvDialog();
 	if (csvdia->exec())
 	{
-		CsvIm *cim = new CsvIm(filename, encoding, writer, csvdia->getFDelim(), csvdia->getVDelim(),
-                               csvdia->hasHeader(), csvdia->useVDelim());
+		CsvIm *cim = new CsvIm(filename, encoding, writer, csvdia->getFDelim(), csvdia->getVDelim(), csvdia->hasHeader(), csvdia->useVDelim());
 		cim->write();
 		delete cim;
 	}
@@ -68,7 +70,7 @@ void CsvIm::setupPStyles()
 		pstyleHeader->getFont()->setWeight(BOLD);
 	}
 	else
-		pstyleHeader = NULL;
+		pstyleHeader = nullptr;
 }
 
 void CsvIm::setFieldDelimiter(const QString& fdelim)
@@ -90,11 +92,27 @@ void CsvIm::write()
 void CsvIm::loadFile()
 {
 	QString text = "";
+	/*
+	QFile f(filename);
+	QFileInfo fi(f);
+	if (!fi.exists())
+		return;
+	QByteArray bb(f.size(), ' ');
+	if (f.open(QIODevice::ReadOnly))
+	{
+		f.read(bb.data(), f.size());
+		f.close();
+		for (int posi = 0; posi < bb.size(); ++posi)
+			text += QChar(bb[posi]);
+	}
+	text = toUnicode(text);
+	*/
 	QByteArray rawText;
 	if (loadRawText(filename, rawText))
 		text = toUnicode(rawText);
+
 	QStringList lines = text.split("\n", QString::SkipEmptyParts);
-	uint i;
+	uint i=0;
 	if (hasHeader)
 	{
 		colIndex = 0;
@@ -104,8 +122,6 @@ void CsvIm::loadFile()
 		i = 1;
 		++rowNumber;
 	}
-	else
-		i = 0;
 	for (int i2 = i; i2 < lines.size(); ++i2)
 	{
 		colIndex = 0;
@@ -115,6 +131,9 @@ void CsvIm::loadFile()
 		if (colCount < colIndex)
 			colCount = colIndex;
 	}
+	if (data.startsWith("\t"))
+		data.remove(0,1);
+	data.replace("\n\t","\n");
 }
 
 void CsvIm::parseLine(const QString& line, bool isHeader)
@@ -133,7 +152,7 @@ void CsvIm::parseLine(const QString& line, bool isHeader)
 		}
 		return; // line done
 	}
-	
+
 	int vdIndexStart = line.indexOf(valueDelimiter);
 	int vdIndexEnd   = line.indexOf(valueDelimiter, vdIndexStart + 1);
 	if (vdIndexEnd < 0)
@@ -142,12 +161,12 @@ void CsvIm::parseLine(const QString& line, bool isHeader)
 			header += "\t" + line;
 		else
 			data += "\t" + line;
-		return; // error in line, no closing valuedelimiter could be found	
+		return; // error in line, no closing valuedelimiter could be found
 	}
-	
+
 	int fdIndex = line.indexOf(fieldDelimiter, vdIndexEnd + 1);
 	QString tmpCol = "";
-	
+
 	if (fdIndex < 0)
 	{
 		if (vdIndexEnd < 0)
@@ -168,7 +187,7 @@ void CsvIm::parseLine(const QString& line, bool isHeader)
 		++colIndex;
 		return; // no more field delimiters left
 	}
-	
+
 	if (fdIndex < vdIndexStart)
 	{
 		tmpCol = line.mid(0, fdIndex);
@@ -205,7 +224,18 @@ void CsvIm::setupTabulators()
 		curValue += addition;
 	}
 }
-
+/*
+QString CsvIm::toUnicode(const QString& text)
+{
+	QTextCodec *codec;
+	if (encoding.isEmpty())
+		codec = QTextCodec::codecForLocale();
+	else
+		codec = QTextCodec::codecForName(encoding.toLocal8Bit());
+	QString dec = codec->toUnicode(text.toLocal8Bit());
+	return dec;
+}
+*/
 QString CsvIm::toUnicode(const QByteArray& rawText)
 {
 	QTextCodec *codec;
