@@ -11,40 +11,42 @@ for which a new license (GPL+exception) is in place.
 #include <QDrag>
 #include <QFile>
 #include <QList>
+#include <QMessageBox>
 #include <QMimeData>
 #include <QRegExp>
 #include <QStack>
-#include <QStack>
 
 #include <cstdlib>
 
-#include "commonstrings.h"
-#include "customfdialog.h"
 #include "importps.h"
+
+
+#include "commonstrings.h"
 #include "loadsaveplugin.h"
-#include "multiprogressdialog.h"
 #include "prefscontext.h"
 #include "prefsfile.h"
 #include "prefsmanager.h"
-#include "prefsmanager.h"
 #include "prefstable.h"
-#include "propertiespalette.h"
-#include "scconfig.h"
 #include "scclocale.h"
+#include "scconfig.h"
 #include "scmimedata.h"
 #include "scpaths.h"
-#include "scribus.h"
 #include "scribusXml.h"
 #include "scribuscore.h"
+#include "scribusdoc.h"
+#include "scribusview.h"
 #include "sctextstream.h"
 #include "selection.h"
+#include "ui/customfdialog.h"
+#include "ui/multiprogressdialog.h"
+#include "ui/propertiespalette.h"
+#include "ui/scmessagebox.h"
 #include "undomanager.h"
 #include "util.h"
 #include "util_color.h"
 #include "util_formats.h"
-#include "util_ghostscript.h"
-#include "util_icon.h"
 #include "util_math.h"
+
 #ifdef HAVE_PODOFO
 	#include <podofo/podofo.h>
 #endif
@@ -56,12 +58,18 @@ EPSPlug::EPSPlug(ScribusDoc* doc, int flags)
 {
 	tmpSel = new Selection(this, false);
 	m_Doc  = doc;
-	progressDialog = NULL;
+	progressDialog = nullptr;
 	interactive = (flags & LoadSavePlugin::lfInteractive);
 }
 
 bool EPSPlug::import(QString fName, const TransactionSettings &trSettings, int flags, bool showProgress)
 {
+#ifdef Q_OS_OSX
+	#if QT_VERSION >= 0x050300
+		showProgress = false;
+	#endif
+#endif
+
 	bool success = false;
 	interactive = (flags & LoadSavePlugin::lfInteractive);
 	cancel = false;
@@ -71,11 +79,11 @@ bool EPSPlug::import(QString fName, const TransactionSettings &trSettings, int f
 	CustColors.clear();
 	QFileInfo fi = QFileInfo(fName);
 	QString ext = fi.suffix().toLower();
-	if (!ScCore->usingGUI()) {
+	if ( !ScCore->usingGUI() ) {
 		interactive = false;
 		showProgress = false;
 	}
-	if (showProgress) 
+	if ( showProgress ) 
 	{
 		ScribusMainWindow* mw=(m_Doc==0) ? ScCore->primaryMainWindow() : m_Doc->scMW();
 		progressDialog = new MultiProgressDialog( tr("Importing: %1").arg(fi.fileName()), CommonStrings::tr_Cancel, mw);
@@ -93,14 +101,14 @@ bool EPSPlug::import(QString fName, const TransactionSettings &trSettings, int f
 		qApp->processEvents();
 	}
 	else {
-		progressDialog = NULL;
+		progressDialog = nullptr;
 	}
 	
 /* Set default Page to size defined in Preferences */
 	x = 0.0;
 	y = 0.0;
-	b = PrefsManager::instance()->appPrefs.PageWidth;
-	h = PrefsManager::instance()->appPrefs.PageHeight;
+	b = PrefsManager::instance()->appPrefs.docSetupPrefs.pageWidth;
+	h = PrefsManager::instance()->appPrefs.docSetupPrefs.pageHeight;
 	if (extensionIndicatesEPSorPS(ext))
 	{
 		QString tmp, BBox, tmp2, FarNam;
@@ -150,16 +158,16 @@ bool EPSPlug::import(QString fName, const TransactionSettings &trSettings, int f
 		try
 		{
 			PoDoFo::PdfError::EnableDebug( false );
-#if (PODOFO_VERSION >= PODOFO_MAKE_VERSION(0, 7, 0))
-			PoDoFo::PdfError::EnableLogging( false );
+#if (PODOFO_VERSION == 0 && PODOFO_MINOR > 6)
+		PoDoFo::PdfError::EnableLogging( false );
 #endif
-#if (PODOFO_VERSION >= PODOFO_MAKE_VERSION(0, 5, 99))
+#if (PODOFO_VERSION == 0 && PODOFO_MINOR == 5 && PODOFO_REVISION == 99) || PODOFO_MINOR > 5
 			PoDoFo::PdfMemDocument doc( fName.toLocal8Bit().data() );
 #else
 			PoDoFo::PdfDocument doc( fName.toLocal8Bit().data() );
 #endif
 			PoDoFo::PdfPage *curPage = doc.GetPage(0);
-			if (curPage != NULL)
+			if (curPage != nullptr)
 			{
 				PoDoFo::PdfRect rect = curPage->GetMediaBox();
 				b = rect.GetWidth() - rect.GetLeft();
@@ -202,10 +210,10 @@ bool EPSPlug::import(QString fName, const TransactionSettings &trSettings, int f
 	if ((ret) || (!interactive))
 	{
 		if (b-x > h-y)
-			m_Doc->PageOri = 1;
+			m_Doc->setPageOrientation(1);
 		else
-			m_Doc->PageOri = 0;
-		m_Doc->m_pageSize = "Custom";
+			m_Doc->setPageOrientation(0);
+		m_Doc->setPageSize("Custom");
 	}
 	ColorList::Iterator it;
 	for (it = CustColors.begin(); it != CustColors.end(); ++it)
@@ -215,12 +223,12 @@ bool EPSPlug::import(QString fName, const TransactionSettings &trSettings, int f
 	}
 	boundingBoxRect.addRect(0, 0, b-x, h-y);
 	Elements.clear();
-	m_Doc->view()->Deselect();
 	m_Doc->setLoading(true);
 	m_Doc->DoDrawing = false;
-	m_Doc->view()->updatesOn(false);
+	if (!(flags & LoadSavePlugin::lfLoadAsPattern))
+		m_Doc->view()->updatesOn(false);
 	m_Doc->scMW()->setScriptRunning(true);
-	qApp->setOverrideCursor(QCursor(Qt::WaitCursor));
+	qApp->changeOverrideCursor(QCursor(Qt::WaitCursor));
 	QString CurDirP = QDir::currentPath();
 	QDir::setCurrent(fi.path());
 	if (convert(fName, x, y, b, h))
@@ -230,64 +238,7 @@ bool EPSPlug::import(QString fName, const TransactionSettings &trSettings, int f
 		QDir::setCurrent(CurDirP);
 //		if ((Elements.count() > 1) && (interactive))
 		if (Elements.count() > 1)
-		{
-			bool isGroup = true;
-			int firstElem = -1;
-			if (Elements.at(0)->Groups.count() != 0)
-				firstElem = Elements.at(0)->Groups.top();
-			for (int bx = 0; bx < Elements.count(); ++bx)
-			{
-				PageItem* bxi = Elements.at(bx);
-				if (bxi->Groups.count() != 0)
-				{
-					if (bxi->Groups.top() != firstElem)
-						isGroup = false;
-				}
-				else
-					isGroup = false;
-			}
-			if (!isGroup)
-			{
-				double minx = 99999.9;
-				double miny = 99999.9;
-				double maxx = -99999.9;
-				double maxy = -99999.9;
-				uint lowestItem = 999999;
-				uint highestItem = 0;
-				for (int a = 0; a < Elements.count(); ++a)
-				{
-					Elements.at(a)->Groups.push(m_Doc->GroupCounter);
-					PageItem* currItem = Elements.at(a);
-					lowestItem = qMin(lowestItem, currItem->ItemNr);
-					highestItem = qMax(highestItem, currItem->ItemNr);
-					double x1, x2, y1, y2;
-					currItem->getVisualBoundingRect(&x1, &y1, &x2, &y2);
-					minx = qMin(minx, x1);
-					miny = qMin(miny, y1);
-					maxx = qMax(maxx, x2);
-					maxy = qMax(maxy, y2);
-				}
-				double gx = minx;
-				double gy = miny;
-				double gw = maxx - minx;
-				double gh = maxy - miny;
-				PageItem *high = m_Doc->Items->at(highestItem);
-				int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Rectangle, gx, gy, gw, gh, 0, m_Doc->toolSettings.dBrush, m_Doc->toolSettings.dPen, true);
-				PageItem *neu = m_Doc->Items->takeAt(z);
-				m_Doc->Items->insert(lowestItem, neu);
-				neu->Groups.push(m_Doc->GroupCounter);
-				neu->setItemName( tr("Group%1").arg(neu->Groups.top()));
-				neu->isGroupControl = true;
-				neu->groupsLastItem = high;
-				neu->setTextFlowMode(PageItem::TextFlowDisabled);
-				for (int a = 0; a < m_Doc->Items->count(); ++a)
-				{
-					m_Doc->Items->at(a)->ItemNr = a;
-				}
-				Elements.prepend(neu);
-				m_Doc->GroupCounter++;
-			}
-		}
+			m_Doc->groupObjectsList(Elements);
 		m_Doc->DoDrawing = true;
 		m_Doc->scMW()->setScriptRunning(false);
 		m_Doc->setLoading(false);
@@ -300,14 +251,17 @@ bool EPSPlug::import(QString fName, const TransactionSettings &trSettings, int f
 				m_Doc->setLoading(false);
 				m_Doc->changed();
 				m_Doc->setLoading(loadF);
-				m_Doc->m_Selection->delaySignalsOn();
-				for (int dre=0; dre<Elements.count(); ++dre)
+				if (!(flags & LoadSavePlugin::lfLoadAsPattern))
 				{
-					m_Doc->m_Selection->addItem(Elements.at(dre), true);
+					m_Doc->m_Selection->delaySignalsOn();
+					for (int dre=0; dre<Elements.count(); ++dre)
+					{
+						m_Doc->m_Selection->addItem(Elements.at(dre), true);
+					}
+					m_Doc->m_Selection->delaySignalsOff();
+					m_Doc->m_Selection->setGroupRect();
+					m_Doc->view()->updatesOn(true);
 				}
-				m_Doc->m_Selection->delaySignalsOff();
-				m_Doc->m_Selection->setGroupRect();
-				m_Doc->view()->updatesOn(true);
 			}
 			else
 			{
@@ -317,13 +271,12 @@ bool EPSPlug::import(QString fName, const TransactionSettings &trSettings, int f
 				m_Doc->m_Selection->delaySignalsOn();
 				for (int dre=0; dre<Elements.count(); ++dre)
 				{
-					m_Doc->DragElements.append(Elements.at(dre)->ItemNr);
 					tmpSel->addItem(Elements.at(dre), true);
 				}
 				tmpSel->setGroupRect();
 				ScriXmlDoc *ss = new ScriXmlDoc();
 				ScElemMimeData* md = new ScElemMimeData();
-				md->setScribusElem(ss->WriteElem(m_Doc, m_Doc->view(), tmpSel));
+				md->setScribusElem(ss->WriteElem(m_Doc, tmpSel));
 				delete ss;
 /*#ifndef Q_WS_MAC*/
 // see #2196
@@ -346,7 +299,8 @@ bool EPSPlug::import(QString fName, const TransactionSettings &trSettings, int f
 		{
 			m_Doc->changed();
 			m_Doc->reformPages();
-			m_Doc->view()->updatesOn(true);
+			if (!(flags & LoadSavePlugin::lfLoadAsPattern))
+				m_Doc->view()->updatesOn(true);
 		}
 		success = true;
 	}
@@ -361,9 +315,11 @@ bool EPSPlug::import(QString fName, const TransactionSettings &trSettings, int f
 	if (interactive)
 		m_Doc->setLoading(false);
 	//CB If we have a gui we must refresh it if we have used the progressbar
-	if ((showProgress) && (!interactive))
-		m_Doc->view()->DrawNew();
-	qApp->restoreOverrideCursor();
+	if (!(flags & LoadSavePlugin::lfLoadAsPattern))
+	{
+		if ((showProgress) && (!interactive))
+			m_Doc->view()->DrawNew();
+	}
 	return success;
 }
 
@@ -381,8 +337,8 @@ bool EPSPlug::convert(QString fn, double x, double y, double b, double h)
 	QString cmd, cmd1, cmd2, cmd3, tmp, tmp2, tmp3, tmp4;
 	// import.prolog do not cope with filenames containing blank spaces
 	// so take care that output filename does not (win32 compatibility)
-	QString tmpFile = getShortPathName(ScPaths::getTempFileDir())+ "/ps.out";
-	QString errFile = getShortPathName(ScPaths::getTempFileDir())+ "/ps.err";
+	QString tmpFile = getShortPathName(ScPaths::tempFileDir())+ "/ps.out";
+	QString errFile = getShortPathName(ScPaths::tempFileDir())+ "/ps.err";
 	QString pfad = ScPaths::instance().libDir();
 	QString pfad2 = QDir::toNativeSeparators(pfad + "import.prolog");
 	QFileInfo fi = QFileInfo(fn);
@@ -394,7 +350,7 @@ bool EPSPlug::convert(QString fn, double x, double y, double b, double h)
 	}
 /*
 // Destill the eps with ghostscript to get a clean eps file
-	QString cleanFile = getShortPathName(ScPaths::getTempFileDir())+ "/clean.eps";
+	QString cleanFile = getShortPathName(ScPaths::tempFileDir())+ "/clean.eps";
 	args.append( "-q" );
 	args.append( "-dNOPAUSE" );
 	args.append( "-sDEVICE=epswrite" );
@@ -407,20 +363,11 @@ bool EPSPlug::convert(QString fn, double x, double y, double b, double h)
 	System(getShortPathName(PrefsManager::instance()->ghostscriptExecutable()), args, errFile, errFile, &cancel);
 	args.clear();
 */
-	int major = 0, minor = 0;
-	getNumericGSVersion(major, minor);
-	int gsVersion = major * 1000 + minor;
-
 	args.append( "-q" );
 	args.append( "-dNOPAUSE" );
 	args.append( "-dNODISPLAY" );
 	args.append( "-dBATCH" );
-	// #15077: -dDELAYBIND was deprecated in 9.22 but will be restored in 9.23
-	// See http://git.ghostscript.com/?p=ghostpdl.git;a=commit;h=fa499a5809aab45b2891b5c8b2363d1bca890757
-	if (gsVersion == 9022)
-		args.append( "-dREALLYDELAYBIND" );
-	else
-		args.append( "-dDELAYBIND" );
+	args.append( "-dDELAYBIND" );
 	// Add any extra font paths being used by Scribus to gs's font search
 	// path We have to use Scribus's prefs context, not a plugin context, to
 	// get to the required information.
@@ -445,7 +392,7 @@ bool EPSPlug::convert(QString fn, double x, double y, double b, double h)
 	args.append( QString("-sTraceFile=%1").arg(QDir::toNativeSeparators(tmpFile)) );
 	QString exportPath = m_Doc->DocName + "-" + fi.baseName();
 	QFileInfo exportFi(exportPath);
-	if (!exportFi.isWritable()) {
+	if ( !exportFi.isWritable() ) {
 		PrefsContext* docContext = PrefsManager::instance()->prefsFile->getContext("docdirs", false);
 		QString docDir = ".";
 		QString prefsDocDir=PrefsManager::instance()->documentDir();
@@ -484,7 +431,7 @@ bool EPSPlug::convert(QString fn, double x, double y, double b, double h)
 		if (progressDialog)
 			progressDialog->close();
 		QString mess = tr("Importing File:\n%1\nfailed!").arg(fn);
-		QMessageBox::critical(0, tr("Fatal Error"), mess, 1, 0, 0);
+		ScMessageBox::critical(0, tr("Fatal Error"), mess);
 		return false;
 	}
 	if(progressDialog && !cancel) {
@@ -511,8 +458,8 @@ void EPSPlug::parseOutput(QString fn, bool eps)
 	bool fillRuleEvenOdd = true;
 	PageItem* ite;
 	QStack<PageItem*> groupStack;
+	QStack< QList<PageItem*> > groupStackP;
 	QStack<int>  gsStack;
-	QStack<uint> elemCount;
 	QStack<uint> gsStackMarks;
 	QFile f(fn);
 	lasttoken = "";
@@ -599,9 +546,9 @@ void EPSPlug::parseOutput(QString fn, bool eps)
 					else
 					{
 						if (ClosedPath)
-							z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColor, CommonStrings::None, true);
+							z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColor, CommonStrings::None);
 						else
-							z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColor, CommonStrings::None, true);
+							z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColor, CommonStrings::None);
 						ite = m_Doc->Items->at(z);
 						ite->PoLine = Coords.copy();  //FIXME: try to avoid copy if FPointArray when properly shared
 						ite->PoLine.translate(m_Doc->currentPage()->xOffset(), m_Doc->currentPage()->yOffset());
@@ -613,15 +560,11 @@ void EPSPlug::parseOutput(QString fn, bool eps)
 						ite->Clip = FlattenPath(ite->PoLine, ite->Segments);
 						ite->setFillTransparency(1.0 - Opacity);
 						ite->setTextFlowMode(PageItem::TextFlowDisabled);
-						m_Doc->AdjustItemSize(ite);
-						if (groupStack.count() != 0)
-						{
-							QStack<int> groupOld = groupStack.top()->Groups;
-							for (int gg = 0; gg < groupOld.count(); gg++)
-							{
-								ite->Groups.push(groupOld[gg]);
-							}
-						}
+						m_Doc->adjustItemSize(ite);
+						if (ite->itemType() == PageItem::Polygon)
+							ite->ContourLine = ite->PoLine.copy();
+						if ((groupStack.count() != 0) && (groupStackP.count() != 0))
+							groupStackP.top().append(ite);
 						Elements.append(ite);
 						lastPath = currPath;
 					}
@@ -647,9 +590,9 @@ void EPSPlug::parseOutput(QString fn, bool eps)
 					else
 					{
 						if (ClosedPath)
-							z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CommonStrings::None, CurrColor, true);
+							z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CommonStrings::None, CurrColor);
 						else
-							z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CommonStrings::None, CurrColor, true);
+							z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CommonStrings::None, CurrColor);
 						ite = m_Doc->Items->at(z);
 						ite->PoLine = Coords.copy(); //FIXME: try to avoid copy when FPointArray is properly shared
 						ite->PoLine.translate(m_Doc->currentPage()->xOffset(), m_Doc->currentPage()->yOffset());
@@ -663,17 +606,13 @@ void EPSPlug::parseOutput(QString fn, bool eps)
 						ite->setWidthHeight(wh.x(), wh.y());
 						ite->Clip = FlattenPath(ite->PoLine, ite->Segments);
 						ite->setLineTransparency(1.0 - Opacity);
-						m_Doc->AdjustItemSize(ite);
+						m_Doc->adjustItemSize(ite);
+						if (ite->itemType() == PageItem::Polygon)
+							ite->ContourLine = ite->PoLine.copy();
 						ite->setLineWidth(LineW);
 						ite->setTextFlowMode(PageItem::TextFlowDisabled);
-						if (groupStack.count() != 0)
-						{
-							QStack<int> groupOld = groupStack.top()->Groups;
-							for (int gg = 0; gg < groupOld.count(); gg++)
-							{
-								ite->Groups.push(groupOld[gg]);
-							}
-						}
+						if ((groupStack.count() != 0) && (groupStackP.count() != 0))
+							groupStackP.top().append(ite);
 						Elements.append(ite);
 					}
 					lastPath = "";
@@ -693,7 +632,7 @@ void EPSPlug::parseOutput(QString fn, bool eps)
 					if ((tmpPath.boundingRect().width() != 0) && (tmpPath.boundingRect().height() != 0))
 					{
 						clipCoords.fromQPainterPath(tmpPath);
-						z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Rectangle, baseX, baseY, 10, 10, 0, CommonStrings::None, CommonStrings::None, true);
+						z = m_Doc->itemAdd(PageItem::Group, PageItem::Rectangle, baseX, baseY, 10, 10, 0, CommonStrings::None, CommonStrings::None);
 						ite = m_Doc->Items->at(z);
 						ite->PoLine = clipCoords.copy();  //FIXME: try to avoid copy if FPointArray when properly shared
 						ite->PoLine.translate(m_Doc->currentPage()->xOffset(), m_Doc->currentPage()->yOffset());
@@ -702,22 +641,16 @@ void EPSPlug::parseOutput(QString fn, bool eps)
 						FPoint wh = getMaxClipF(&ite->PoLine);
 						ite->setWidthHeight(wh.x(),wh.y());
 						ite->Clip = FlattenPath(ite->PoLine, ite->Segments);
-						m_Doc->AdjustItemSize(ite);
-						ite->Groups.push(m_Doc->GroupCounter);
-						if (groupStack.count() != 0)
-						{
-							QStack<int> groupOld = groupStack.top()->Groups;
-							for (int gg = 0; gg < groupOld.count(); gg++)
-							{
-								ite->Groups.push(groupOld[gg]);
-							}
-						}
-						ite->isGroupControl = true;
+						m_Doc->adjustItemSize(ite, true);
+						ite->ContourLine = ite->PoLine.copy();
 						ite->setItemName( tr("Group%1").arg(m_Doc->GroupCounter));
 						ite->setTextFlowMode(PageItem::TextFlowDisabled);
 						Elements.append(ite);
+						if ((groupStack.count() != 0) && (groupStackP.count() != 0))
+							groupStackP.top().append(ite);
 						groupStack.push(ite);
-						elemCount.push(Elements.count());
+						QList<PageItem*> gElements;
+						groupStackP.push(gElements);
 						gsStackMarks.push(gsStack.count());
 						m_Doc->GroupCounter++;
 					}
@@ -735,19 +668,17 @@ void EPSPlug::parseOutput(QString fn, bool eps)
 				// #6834 : self defense against incorrectly balanced save/restore
 				if (gsStack.count() > 0)
 					gsStack.pop();
-				if (groupStack.count() != 0)
+				if ((groupStack.count() != 0) && (groupStackP.count() != 0))
 				{
 					if (gsStack.count() < static_cast<int>(gsStackMarks.top()))
 					{
 						PageItem *ite = groupStack.pop();
-						int count = elemCount.pop();
-						if (count == Elements.count())
+						QList<PageItem*> gList = groupStackP.pop();
+						for (int d = 0; d < gList.count(); d++)
 						{
-							m_Doc->Items->removeLast();
-							Elements.removeLast();
+							Elements.removeAll(gList.at(d));
 						}
-						else
-							ite->groupsLastItem = Elements.last();
+						m_Doc->groupObjectsToItem(ite, gList);
 						gsStackMarks.pop();
 					}
 				}
@@ -756,7 +687,6 @@ void EPSPlug::parseOutput(QString fn, bool eps)
 			{
 				ScTextStream Lw(&params, QIODevice::ReadOnly);
 				Lw >> LineW;
-//				currPath += params;
 			}
 			else if (token == "ld")
 			{
@@ -772,7 +702,6 @@ void EPSPlug::parseOutput(QString fn, bool eps)
 						DashPattern.append(dcp);
 					}
 				}
-//				currPath += params;
 			}
 			else if (token == "lc")
 			{
@@ -793,7 +722,6 @@ void EPSPlug::parseOutput(QString fn, bool eps)
 						CapStyle = Qt::FlatCap;
 						break;
 				}
-//				currPath += params;
 			}
 			else if (token == "lj")
 			{
@@ -814,13 +742,12 @@ void EPSPlug::parseOutput(QString fn, bool eps)
 						JoinStyle = Qt::MiterJoin;
 						break;
 				}
-//				currPath += params;
 			}
 			else if (token == "cp") {
 				ClosedPath = true;
 			}
 			else if (token == "im") {
-				if (!Image(params))
+				if ( !Image(params) )
 					++failedImages;
 			}
 			lasttoken = token;
@@ -831,21 +758,19 @@ void EPSPlug::parseOutput(QString fn, bool eps)
 			while (!groupStack.isEmpty())
 			{
 				PageItem *ite = groupStack.pop();
-				int count = elemCount.pop();
-				if (count == Elements.count())
+				QList<PageItem*> gList = groupStackP.pop();
+				for (int d = 0; d < gList.count(); d++)
 				{
-					m_Doc->Items->removeLast();
-					Elements.removeLast();
+					Elements.removeAll(gList.at(d));
 				}
-				else
-					ite->groupsLastItem = Elements.last();
+				m_Doc->groupObjectsToItem(ite, gList);
 			}
 		}
 	}
 	if (failedImages > 0)
 	{
 		QString mess = tr("Converting of %1 images failed!").arg(failedImages);
-		QMessageBox::critical(0, tr("Error"), mess, 1, 0, 0);
+		ScMessageBox::critical(0, tr("Error"), mess);
 	}
 }
 
@@ -867,13 +792,13 @@ bool EPSPlug::Image(QString vals)
 	if (device.startsWith("psd")) {
 		filename = filename.mid(0, filename.length()-3) + "psd";
 	}
-		
+
 	qDebug("%s", QString("import %7 image %1: %2x%3 @ (%4,%5) °%6").arg(filename).arg(w).arg(h).arg(x).arg(y).arg(angle).arg(device).toLocal8Bit().data());
 	QString rawfile = filename.mid(0, filename.length()-3) + "dat";
 	QStringList args;
 	args.append( "-q" );
 	args.append( "-dNOPAUSE" );
-	args.append( QString("-sDEVICE=%1").arg(device) );    
+	args.append( QString("-sDEVICE=%1").arg(device) );
 	args.append( "-dBATCH" );
 	args.append( QString("-g%1x%2").arg(horpix).arg(verpix) );
 	args.append( QString("-sOutputFile=%1").arg(QDir::toNativeSeparators(filename)) );
@@ -894,7 +819,7 @@ bool EPSPlug::Image(QString vals)
 			bool gs_error = false;
 			do {
 				len = diag.readLine(buf, 120);
-				gs_error |= (strstr(buf,"Error")==NULL);
+				gs_error |= (strstr(buf,"Error")==nullptr);
 				if (gs_error)
 					qDebug("\t%s", buf);
 			}
@@ -913,7 +838,7 @@ bool EPSPlug::Image(QString vals)
 				len = dat.readLine(buf, 120);
 				qDebug("\t%s", buf);
 			}
-			while ( len > 0 && !(strstr(buf, "image")==NULL) );
+			while ( len > 0 && !(strstr(buf, "image")==nullptr) );
 			dat.close();
 		}
 		else {
@@ -921,7 +846,7 @@ bool EPSPlug::Image(QString vals)
 		}
 	}
 	QFile::remove(rawfile);
-	int z = m_Doc->itemAdd(PageItem::ImageFrame, PageItem::Unspecified, m_Doc->currentPage()->xOffset(), m_Doc->currentPage()->yOffset(), w, h, LineW, CommonStrings::None, CurrColor, true);
+	int z = m_Doc->itemAdd(PageItem::ImageFrame, PageItem::Unspecified, m_Doc->currentPage()->xOffset(), m_Doc->currentPage()->yOffset(), w, h, LineW, CommonStrings::None, CurrColor);
 	PageItem * ite = m_Doc->Items->at(z);
 	ite->setXYPos(m_Doc->currentPage()->xOffset() + x, m_Doc->currentPage()->yOffset() + y);
 	ite->setWidthHeight(w, h);
@@ -942,7 +867,7 @@ bool EPSPlug::Image(QString vals)
 	m_Doc->loadPict(filename, ite, -1);
 	ite->setRotation(angle);
 	ite->setImageScalingMode(false, true); // fit to frame, keep ratio
-//	m_Doc->view()->AdjustItemSize(ite);
+//	m_Doc->view()->adjustItemSize(ite);
 	Elements.append(ite);
 	return ret == 0;
 }
@@ -997,9 +922,7 @@ QString EPSPlug::parseColor(QString vals, bool eps, colorModel model)
 		return ret;
 	double c, m, y, k, r, g, b;
 	ScColor tmp;
-	ColorList::Iterator it;
 	ScTextStream Code(&vals, QIODevice::ReadOnly);
-	bool found = false;
 	if (model == colorModelRGB)
 	{
 		Code >> r;
@@ -1013,21 +936,7 @@ QString EPSPlug::parseColor(QString vals, bool eps, colorModel model)
 		int Rc = qRound(r * 255);
 		int Gc = qRound(g * 255);
 		int Bc = qRound(b * 255);
-		int hR, hG, hB;
 		tmp.setColorRGB(Rc, Gc, Bc);
-		for (it = m_Doc->PageColors.begin(); it != m_Doc->PageColors.end(); ++it)
-		{
-			if (it.value().getColorModel() == colorModelRGB)
-			{
-				it.value().getRGB(&hR, &hG, &hB);
-				if ((Rc == hR) && (Gc == hG) && (Bc == hB))
-				{
-					ret = it.key();
-					found = true;
-					break;
-				}
-			}
-		}
 	}
 	else
 	{
@@ -1036,40 +945,18 @@ QString EPSPlug::parseColor(QString vals, bool eps, colorModel model)
 		Code >> y;
 		Code >> k;
 		Code >> Opacity;
-/*		int Cc = static_cast<int>(c * 255 + 0.5);
-		int Mc = static_cast<int>(m * 255 + 0.5);
-		int Yc = static_cast<int>(y * 255 + 0.5);
-		int Kc = static_cast<int>(k * 255 + 0.5); */
 		int Cc = qRound(c * 255);
 		int Mc = qRound(m * 255);
 		int Yc = qRound(y * 255);
 		int Kc = qRound(k * 255);
-		int hC, hM, hY, hK;
 		tmp.setColor(Cc, Mc, Yc, Kc);
-		for (it = m_Doc->PageColors.begin(); it != m_Doc->PageColors.end(); ++it)
-		{
-			if (it.value().getColorModel() == colorModelCMYK)
-			{
-				it.value().getCMYK(&hC, &hM, &hY, &hK);
-				if ((Cc == hC) && (Mc == hM) && (Yc == hY) && (Kc == hK))
-				{
-					ret = it.key();
-					found = true;
-					break;
-				}
-			}
-		}
-	}
-	if (!found)
-	{
-		tmp.setSpotColor(false);
-		tmp.setRegistrationColor(false);
-		QString namPrefix = "FromEPS";
-		if (!eps)
-			namPrefix = "FromPS";
-		m_Doc->PageColors.insert(namPrefix+tmp.name(), tmp);
-//		importedColors.append(namPrefix+tmp.name());
-		ret = namPrefix+tmp.name();
 	}
+	tmp.setSpotColor(false);
+	tmp.setRegistrationColor(false);
+	QString namPrefix = "FromEPS";
+	if (!eps)
+		namPrefix = "FromPS";
+	QString fNam = m_Doc->PageColors.tryAddColor(namPrefix+tmp.name(), tmp);
+	ret = fNam;
 	return ret;
 }
