@@ -37,8 +37,9 @@ class PLUGIN_API SVGImportPlugin : public LoadSavePlugin
 		virtual const AboutData* getAboutData() const;
 		virtual void deleteAboutData(const AboutData* about) const;
 		virtual void languageChange();
-		virtual bool fileSupported(QIODevice* file, const QString & fileName=QString::null) const;
+		virtual bool fileSupported(QIODevice* file, const QString & fileName=QString()) const;
 		virtual bool loadFile(const QString & fileName, const FileFormat & fmt, int flags, int index = 0);
+		virtual QImage readThumbnail(const QString& fileName);
 		virtual void addToMainWindowMenu(ScribusMainWindow *) {};
 
 	public slots:
@@ -48,7 +49,7 @@ class PLUGIN_API SVGImportPlugin : public LoadSavePlugin
 		\param filename a file name to import
 		\retval true for success
 		 */
-		virtual bool import(QString filename = QString::null, int flags = lfUseCurrentPage|lfInteractive);
+		virtual bool import(QString filename = QString(), int flags = lfUseCurrentPage|lfInteractive);
 
 	private:
 		void registerFormats();
@@ -69,42 +70,50 @@ class GradientHelper
 {
 public:
 	GradientHelper() :
-		CSpace(false),
+		cspace(false),
 		cspaceValid(true),
 		gradient(VGradient::linear),
 		gradientValid(false),
 		matrix(),
 		matrixValid(false),
 		reference(""),
-		Type(1),
+		type(1),
 		typeValid(false),
-		X1(0),
+		x1(0),
 		x1Valid(true),
-		X2(1),
+		x2(1),
 		x2Valid(true),
-		Y1(0),
+		y1(0),
 		y1Valid(true),
-		Y2(0),
-		y2Valid(true)
+		y2(0),
+		y2Valid(true),
+		fx(0),
+		fxValid(true),
+		fy(0),
+		fyValid(true)
 		{
 		}
-	bool CSpace;
+	bool cspace;
 	bool cspaceValid;
 	VGradient gradient;
 	bool gradientValid;
-	QMatrix matrix;
+	QTransform matrix;
 	bool matrixValid;
 	QString reference;
-	int Type;
+	int type;
 	bool typeValid;
-	double X1;
+	double x1;
 	bool x1Valid;
-	double X2;
+	double x2;
 	bool x2Valid;
-	double Y1;
+	double y1;
 	bool y1Valid;
-	double Y2;
+	double y2;
 	bool y2Valid;
+	double fx;
+	bool fxValid;
+	double fy;
+	bool fyValid;
 	};
 
 class SvgStyle
@@ -112,7 +121,8 @@ class SvgStyle
 public:
 	SvgStyle() :
 		Display(true),
-		CSpace(false),
+		FillCSpace(false),
+		StrokeCSpace(false),
 		CurCol("Black"),
 		dashOffset(0),
 		FontFamily(""),
@@ -122,18 +132,29 @@ public:
 		FontSize(12),
 		FillCol("Black"),
 		fillRule("nonzero"),
-		GCol1("Black"),
-		GCol2("Black"),
-		GradCo(VGradient::linear),
-		Gradient(0),
-		GX1(0),
-		GX2(0),
-		GY1(0),
-		GY2(0),
+		GFillCol1("Black"),
+		GStrokeCol1("Black"),
+		FillGradient(VGradient::linear),
+		StrokeGradient(VGradient::linear),
+		FillGradientType(0),
+		StrokeGradientType(0),
+		GradFillX1(0),
+		GradFillX2(0),
+		GradFillY1(0),
+		GradFillY2(0),
+		GradFillFX(0),
+		GradFillFY(0),
+		GradStrokeX1(0),
+		GradStrokeX2(0),
+		GradStrokeY1(0),
+		GradStrokeY2(0),
+		GradStrokeFX(0),
+		GradStrokeFY(0),
 		InherCol(false),
 		LWidth(1.0),
 		matrix(),
-		matrixg(),
+		matrixgf(),
+		matrixgs(),
 		PLineArt(Qt::SolidLine),
 		PLineEnd(Qt::FlatCap),
 		PLineJoin(Qt::MiterJoin),
@@ -142,11 +163,15 @@ public:
 		FillOpacity(1.0),
 		StrokeOpacity(1.0),
 		textAnchor("start"),
-		clipPath()
+		clipPath(),
+		forGroup(false),
+		endMarker(""),
+		startMarker("")
 		{
 		}
 	bool Display;
-	bool CSpace;
+	bool FillCSpace;
+	bool StrokeCSpace;
 	QString CurCol;
 	QVector<double> dashArray;
 	double dashOffset;
@@ -157,18 +182,29 @@ public:
 	double  FontSize;
 	QString FillCol;
 	QString fillRule;
-	QString GCol1;
-	QString GCol2;
-	VGradient	GradCo;
-	int Gradient;
-	double GX1;
-	double GX2;
-	double GY1;
-	double GY2;
+	QString GFillCol1;
+	QString GStrokeCol1;
+	VGradient FillGradient;
+	VGradient StrokeGradient;
+	int    FillGradientType;
+	int    StrokeGradientType;
+	double GradFillX1;
+	double GradFillX2;
+	double GradFillY1;
+	double GradFillY2;
+	double GradFillFX;
+	double GradFillFY;
+	double GradStrokeX1;
+	double GradStrokeX2;
+	double GradStrokeY1;
+	double GradStrokeY2;
+	double GradStrokeFX;
+	double GradStrokeFY;
 	bool InherCol;
 	double LWidth;
-	QMatrix matrix;
-	QMatrix matrixg;
+	QTransform matrix;
+	QTransform matrixgf;
+	QTransform matrixgs;
 	Qt::PenStyle PLineArt;
 	Qt::PenCapStyle PLineEnd;
 	Qt::PenJoinStyle PLineJoin;
@@ -179,7 +215,10 @@ public:
 	QString textAnchor;
 	QString textDecoration;
 	FPointArray clipPath;
+	bool forGroup;
 	QString filter;
+	QString endMarker;
+	QString startMarker;
 };
 
 class SVGPlug : public QObject
@@ -193,18 +232,19 @@ public:
 	\param fName QString
 	\param isInteractive flag to use GUI
 	 */
-	SVGPlug(ScribusMainWindow *mw, int flags);
+	SVGPlug(ScribusDoc* doc, int flags);
 	~SVGPlug();
-	bool import(QString fname, const TransactionSettings& trSettings, int flags);
-	bool loadData(QString fname);
+	bool import(const QString& fname, const TransactionSettings& trSettings, int flags);
+	QImage readThumbnail(const QString& fn);
+	bool loadData(const QString& fname);
 	void convert(const TransactionSettings& trSettings, int flags);
 	void addGraphicContext();
 	void setupNode( const QDomElement &e );
 	void setupTransform( const QDomElement &e );
-	void finishNode( const QDomNode &e, PageItem* item);
+	PageItem* finishNode( const QDomNode &e, PageItem* item);
 	bool isIgnorableNode( const QDomElement &e );
 	bool isIgnorableNodeName( const QString &n );
-	FPoint parseTextPosition(const QDomElement &e, const FPoint* pos = NULL);
+	FPoint parseTextPosition(const QDomElement &e, const FPoint* pos = nullptr);
 	QSizeF  parseWidthHeight(const QDomElement &e);
 	QRectF  parseViewBox(const QDomElement &e);
 	void parseDefs(const QDomElement &e);
@@ -213,6 +253,7 @@ public:
 	void parseFilterAttr(const QDomElement &e, PageItem* item);
 	QList<PageItem*> parseA(const QDomElement &e);
 	QList<PageItem*> parseGroup(const QDomElement &e);
+	QList<PageItem*> parseDoc(const QDomElement &e);
 	QList<PageItem*> parseElement(const QDomElement &e);
 	QList<PageItem*> parseCircle(const QDomElement &e);
 	QList<PageItem*> parseEllipse(const QDomElement &e);
@@ -234,7 +275,7 @@ public:
 	double  fromPercentage(const QString &s );
 	double  parseFontSize(const QString& fsize);
 	double  parseUnit(const QString &unit);
-	QMatrix parseTransform(const QString &transform);
+	QTransform parseTransform(const QString &transform);
 	bool    parseSVG( const QString &s, FPointArray *ite );
 	QColor  parseColorN( const QString &rgbColor );
 	QString parseColor( const QString &s );
@@ -244,6 +285,7 @@ public:
 	void parseStyle( SvgStyle *obj, const QDomElement &e );
 	void parseColorStops(GradientHelper *gradient, const QDomElement &e);
 	void parseFilter(const QDomElement &b);
+	void parseMarker(const QDomElement &b);
 	void parsePattern(const QDomElement &b);
 	void parseGradient( const QDomElement &e );
 	FPoint GetMaxClipO(FPointArray Clip);
@@ -270,15 +312,28 @@ public:
 	ScribusDoc* m_Doc;
 	Selection* tmpSel;
 	QStringList importedColors;
+	QStringList importedGradients;
+	QMap<QString, QString> importedGradTrans;
 	QStringList importedPatterns;
 	QMap<QString, QString> importedPattTrans;
 	double inGroupXOrigin;
 	double inGroupYOrigin;
+	int importerFlags;
+	bool firstLayer;
 	struct filterSpec
 	{
 		int blendMode;
 	};
+	struct markerDesc
+	{
+		double xref;
+		double yref;
+		double wpat;
+		double hpat;
+	};
 	QMap<QString, filterSpec> filters;
+	QMap<QString, markerDesc> markers;
+	QList<PageItem*> Elements;
 
 protected:
 	QVector<double> parseNumbersList(const QString& numbersStr);
