@@ -19,29 +19,27 @@ for which a new license (GPL+exception) is in place.
 #include <QPainterPath>
 #include <QRegExp>
 
-#include "customfdialog.h"
-#include "scribus.h"
-#include "scribusXml.h"
-#include "propertiespalette.h"
-#include "prefsfile.h"
-
-#include <cmath>
 #include "commonstrings.h"
 #include "fpointarray.h"
-#include "menumanager.h"
-#include "prefsmanager.h"
+#include "loadsaveplugin.h"
 #include "pageitem.h"
+#include "prefsfile.h"
+#include "prefsmanager.h"
 #include "scmimedata.h"
 #include "scraction.h"
+#include "scribusXml.h"
 #include "scribuscore.h"
 #include "scribusdoc.h"
+#include "scribusview.h"
 #include "selection.h"
+#include "ui/customfdialog.h"
+#include "ui/propertiespalette.h"
+#include "ui/scmwmenumanager.h"
 #include "undomanager.h"
-#include "loadsaveplugin.h"
 #include "util.h"
-#include "util_icon.h"
 #include "util_math.h"
-#include "fonts/scfontmetrics.h"
+
+
 
 using namespace std;
 
@@ -138,11 +136,10 @@ static const ushort symbol_to_unicode[96] = {
 	0x03C8, 0x03B6, 0x03EA, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD, 0xFFFD
 };
 
-WMFImport::WMFImport( ScribusMainWindow* mw, int flags ) :
-	QObject(mw)
+WMFImport::WMFImport( ScribusDoc* doc, int flags )
 {	
 	m_tmpSel = new Selection(this, false);
-	m_Doc    = mw->doc;
+	m_Doc    = doc;
 	unsupported = false;
 	importFailed = false;
 	importCanceled = true;
@@ -151,8 +148,10 @@ WMFImport::WMFImport( ScribusMainWindow* mw, int flags ) :
 	m_docTitle = "";
 	interactive = (flags & LoadSavePlugin::lfInteractive);
 
+	m_IsPlaceable  = false;
+	m_IsEnhanced   = false;
 	m_Valid        = false;
-	m_ObjHandleTab = NULL;
+	m_ObjHandleTab = nullptr;
 	m_Dpi          = 1440;
 }
 
@@ -160,7 +159,7 @@ WMFImport::~WMFImport()
 {
 	qDeleteAll(m_commands);
 	m_commands.clear();
-	if ( m_tmpSel) delete m_tmpSel;
+	delete m_tmpSel;
 	if ( m_ObjHandleTab ) 
 	{
 		for (int i = 0; i < MAX_OBJHANDLE; ++i)
@@ -174,36 +173,15 @@ WMFImport::~WMFImport()
 
 QString WMFImport::importColor(const QColor& color)
 {
-	bool found = false;
-	int r, g, b;
-	QColor  tmpColor;
-	QString retColorName;
-	ColorList::Iterator it;
-	for (it = m_Doc->PageColors.begin(); it != m_Doc->PageColors.end(); ++it)
-	{
-		if (it.value().getColorModel() == colorModelRGB)
-		{
-			it.value().getRGB(&r, &g, &b);
-			tmpColor.setRgb(r, g, b);
-			if (color == tmpColor)
-			{
-				retColorName = it.key();
-				found = true;
-				break;
-			}
-		}
-	}
-	if (!found)
-	{
-		ScColor tmp;
-		tmp.fromQColor(color);
-		tmp.setSpotColor(false);
-		tmp.setRegistrationColor(false);
-		m_Doc->PageColors.insert("FromWMF"+color.name(), tmp);
-		importedColors.append("FromWMF"+color.name());
-		retColorName = "FromWMF"+color.name();
-	}
-	return retColorName;
+	ScColor tmp;
+	tmp.fromQColor(color);
+	tmp.setSpotColor(false);
+	tmp.setRegistrationColor(false);
+	QString pNam = "FromWMF"+color.name();
+	QString fNam = m_Doc->PageColors.tryAddColor(pNam, tmp);
+	if (fNam == pNam)
+		importedColors.append(pNam);
+	return fNam;
 }
 
 QColor WMFImport::colorFromParam( short* params )
@@ -221,7 +199,7 @@ QColor WMFImport::colorFromParam( short* params )
 
 QTextCodec* WMFImport::codecFromCharset( int charset )
 {
-	QTextCodec* codec = NULL;
+	QTextCodec* codec = nullptr;
 	if (charset == DEFAULT_CHARSET || charset == OEM_CHARSET)
 		codec = QTextCodec::codecForLocale();
 	else if (charset == ANSI_CHARSET)
@@ -266,7 +244,8 @@ QString WMFImport::symbolToUnicode ( const QByteArray& chars )
 	QString r;
 	const unsigned char * c = (const unsigned char *) chars.data();
 	
-	if( chars.size() == 0 ) return r;
+	if (chars.size() == 0)
+		return r;
 	
 	for (int i = 0; i < chars.size(); ++i) 
 	{
@@ -299,20 +278,18 @@ FPointArray WMFImport::pointsToPolyline( const FPointArray& points, bool closePa
 	double x = 0.0, y = 0.0;
 	FPointArray polyline;
 	polyline.svgInit();
-	for( uint i = 0; i < points.size(); ++i )
+	for (int i = 0; i < points.size(); ++i )
 	{
-		FPoint point = points.point(i);
-		if( bFirst )
+		const FPoint& point = points.point(i);
+		x = point.x();
+		y = point.y();
+		if (bFirst)
 		{
-			x = point.x();
-			y = point.y();
 			polyline.svgMoveTo(x, y);
 			bFirst = false;
 		}
 		else
 		{
-			x = point.x();
-			y = point.y();
 			polyline.svgLineTo(x, y);
 		}
 	}
@@ -331,7 +308,57 @@ void WMFImport::pointsToAngle( double xStart, double yStart, double xEnd, double
 	if ( angleLength < 0 ) angleLength = 360.0 + angleLength;
 }
 
-bool WMFImport::import(QString fname, const TransactionSettings& trSettings, int flags)
+QImage WMFImport::readThumbnail(const QString& fname)
+{
+	if (!loadWMF(fname))
+	{
+		importFailed = true;
+		return QImage();
+	}
+	QString CurDirP = QDir::currentPath();
+	QFileInfo efp(fname);
+	QDir::setCurrent(efp.path());
+	double scale  = (m_Dpi > 0) ? 72.0 / m_Dpi : 0.05;
+	double width  = m_BBox.width() * scale;
+	double height = m_BBox.height() * scale;
+	m_Doc = new ScribusDoc();
+	m_Doc->setup(0, 1, 1, 1, 1, "Custom", "Custom");
+	m_Doc->setPage(width, height, 0, 0, 0, 0, 0, 0, false, false);
+	m_Doc->addPage(0);
+	m_Doc->setGUI(false, ScCore->primaryMainWindow(), nullptr);
+	m_Doc->setLoading(true);
+	m_Doc->DoDrawing = false;
+	m_Doc->scMW()->setScriptRunning(true);
+	m_Doc->PageColors.ensureDefaultColors();
+	QList<PageItem*> Elements = parseWmfCommands();
+	m_tmpSel->clear();
+	QImage tmpImage = QImage();
+	if (Elements.count() > 0)
+	{
+		if (Elements.count() > 1)
+			m_Doc->groupObjectsList(Elements);
+		m_Doc->DoDrawing = true;
+		m_Doc->m_Selection->delaySignalsOn();
+		for (int dre=0; dre<Elements.count(); ++dre)
+		{
+			m_tmpSel->addItem(Elements.at(dre), true);
+		}
+		m_tmpSel->setGroupRect();
+		double xs = m_tmpSel->width();
+		double ys = m_tmpSel->height();
+		tmpImage = Elements.at(0)->DrawObj_toImage(500);
+		tmpImage.setText("XSize", QString("%1").arg(xs));
+		tmpImage.setText("YSize", QString("%1").arg(ys));
+		m_Doc->m_Selection->delaySignalsOff();
+		m_Doc->setLoading(false);
+	}
+	m_Doc->scMW()->setScriptRunning(false);
+	delete m_Doc;
+	QDir::setCurrent(CurDirP);
+	return tmpImage;
+}
+
+bool WMFImport::import(const QString& fname, const TransactionSettings& trSettings, int flags)
 {
 	if (!loadWMF(fname))
 	{
@@ -486,7 +513,7 @@ bool WMFImport::loadWMF( QBuffer &buffer )
 
 	//----- Test header validity
 	m_Valid = ((header.mtHeaderSize == 9) && (header.mtNoParameters == 0)) || m_IsEnhanced || m_IsPlaceable;
-	if ( m_Valid )
+	if ( m_Valid && !m_IsEnhanced )
 	{
 		//----- Read Metafile Records
 		rdFunc = -1;
@@ -494,6 +521,7 @@ bool WMFImport::loadWMF( QBuffer &buffer )
 		{
 			st >> rdSize;
 			st >> rdFunc;
+
 			idx = findFunc( rdFunc );
 			rdSize -= 3;
 
@@ -513,8 +541,10 @@ bool WMFImport::loadWMF( QBuffer &buffer )
 				m_BBox.setTop ( qMin((int) cmd->params[ 0 ], m_BBox.top()) );
 			}
 			if ( rdFunc == 0x020C && !m_IsPlaceable ) {         // SETWINDOWEXT: dimensions
-				m_BBox.setWidth ( qMax((int) cmd->params[ 1 ], m_BBox.width()) );
-				m_BBox.setHeight( qMax((int) cmd->params[ 0 ], m_BBox.height()));
+				m_BBox.setWidth((int) cmd->params[ 1 ]);
+				m_BBox.setHeight((int) cmd->params[ 0 ]);
+//				m_BBox.setWidth ( qMax((int) cmd->params[ 1 ], m_BBox.width()) );
+//				m_BBox.setHeight( qMax((int) cmd->params[ 0 ], m_BBox.height()));
 			}
 
 			if ( i<rdSize )
@@ -523,18 +553,22 @@ bool WMFImport::loadWMF( QBuffer &buffer )
 				return false;
 			}
 		}
+		m_BBox = m_BBox.normalized();
 		//----- Test records validities
 		m_Valid = (rdFunc == 0) && (m_BBox.width() != 0) && (m_BBox.height() != 0);
 		if ( !m_Valid ) {
 			cerr << "WMF : incorrect file format !" << endl;
 		}
 	}
+	else if ( m_IsEnhanced ) {
+		cerr << "WMF : unsupported Enhanced Metafile !" << endl;
+	}
 	else {
 		cerr << "WMF Header : incorrect header !" << endl;
 	}
 
 	buffer.close();
-	return m_Valid;
+	return ( m_Valid && !m_IsEnhanced );
 }
 
 bool WMFImport::importWMF(const TransactionSettings& trSettings, int flags)
@@ -562,25 +596,28 @@ bool WMFImport::importWMF(const TransactionSettings& trSettings, int flags)
 	if ((ret) || (!interactive))
 	{
 		if (width > height)
-			m_Doc->PageOri = 1;
+			m_Doc->setPageOrientation(1);
 		else
-			m_Doc->PageOri = 0;
-		m_Doc->m_pageSize = "Custom";
+			m_Doc->setPageOrientation(0);
+		m_Doc->setPageSize("Custom");
 	}
-	m_Doc->view()->Deselect();
+	if ((!(flags & LoadSavePlugin::lfLoadAsPattern)) && (m_Doc->view() != nullptr))
+		m_Doc->view()->Deselect();
 	m_Doc->setLoading(true);
 	m_Doc->DoDrawing = false;
-	m_Doc->view()->updatesOn(false);
+	if ((!(flags & LoadSavePlugin::lfLoadAsPattern)) && (m_Doc->view() != nullptr))
+		m_Doc->view()->updatesOn(false);
 	m_Doc->scMW()->setScriptRunning(true);
 	qApp->setOverrideCursor(QCursor(Qt::WaitCursor));
-	//gc->Family = m_Doc->toolSettings.defFont;
+	//gc->Family = m_Doc->toolSettings.textFont;
 	m_Doc->PageColors.ensureDefaultColors();
 	//m_gc.push( gc );
 	QList<PageItem*> Elements = parseWmfCommands();
+	qApp->restoreOverrideCursor();
 	if (flags & LoadSavePlugin::lfCreateDoc)
 	{
-		m_Doc->documentInfo.setTitle(m_docTitle);
-		m_Doc->documentInfo.setComments(m_docDesc);
+		m_Doc->documentInfo().setTitle(m_docTitle);
+		m_Doc->documentInfo().setComments(m_docDesc);
 	}
 	m_tmpSel->clear();
 	if (Elements.count() == 0)
@@ -595,70 +632,11 @@ bool WMFImport::importWMF(const TransactionSettings& trSettings, int flags)
 		}
 	}
 	if (Elements.count() > 1)
-	{
-		bool isGroup = true;
-		int firstElem = -1;
-		if (Elements.at(0)->Groups.count() != 0)
-			firstElem = Elements.at(0)->Groups.top();
-		for (int bx = 0; bx < Elements.count(); ++bx)
-		{
-			PageItem* bxi = Elements.at(bx);
-			if (bxi->Groups.count() != 0)
-			{
-				if (bxi->Groups.top() != firstElem)
-					isGroup = false;
-			}
-			else
-				isGroup = false;
-		}
-		if (!isGroup)
-		{
-			double minx = 99999.9;
-			double miny = 99999.9;
-			double maxx = -99999.9;
-			double maxy = -99999.9;
-			uint lowestItem = 999999;
-			uint highestItem = 0;
-			for (int a = 0; a < Elements.count(); ++a)
-			{
-				Elements.at(a)->Groups.push(m_Doc->GroupCounter);
-				PageItem* currItem = Elements.at(a);
-				lowestItem = qMin(lowestItem, currItem->ItemNr);
-				highestItem = qMax(highestItem, currItem->ItemNr);
-				double x1, x2, y1, y2;
-				currItem->getVisualBoundingRect(&x1, &y1, &x2, &y2);
-				minx = qMin(minx, x1);
-				miny = qMin(miny, y1);
-				maxx = qMax(maxx, x2);
-				maxy = qMax(maxy, y2);
-			}
-			double gx = minx;
-			double gy = miny;
-			double gw = maxx - minx;
-			double gh = maxy - miny;
-			PageItem *high = m_Doc->Items->at(highestItem);
-			int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Rectangle, gx, gy, gw, gh, 0, m_Doc->toolSettings.dBrush, m_Doc->toolSettings.dPen, true);
-			PageItem *neu = m_Doc->Items->takeAt(z);
-			m_Doc->Items->insert(lowestItem, neu);
-			neu->Groups.push(m_Doc->GroupCounter);
-			neu->setItemName( tr("Group%1").arg(neu->Groups.top()));
-			neu->isGroupControl = true;
-			neu->groupsLastItem = high;
-			for (int a = 0; a < m_Doc->Items->count(); ++a)
-			{
-				m_Doc->Items->at(a)->ItemNr = a;
-			}
-			neu->setRedrawBounding();
-			neu->setTextFlowMode(PageItem::TextFlowDisabled);
-			Elements.prepend(neu);
-			m_Doc->GroupCounter++;
-		}
-	}
+		m_Doc->groupObjectsList(Elements);
 	m_Doc->DoDrawing = true;
 	m_Doc->scMW()->setScriptRunning(false);
 	if (interactive)
 		m_Doc->setLoading(false);
-	qApp->setOverrideCursor(QCursor(Qt::ArrowCursor));
 	if ((Elements.count() > 0) && (!ret) && (interactive))
 	{
 		if (flags & LoadSavePlugin::lfScripted)
@@ -667,32 +645,32 @@ bool WMFImport::importWMF(const TransactionSettings& trSettings, int flags)
 			m_Doc->setLoading(false);
 			m_Doc->changed();
 			m_Doc->setLoading(loadF);
-			m_Doc->m_Selection->delaySignalsOn();
-			for (int dre=0; dre<Elements.count(); ++dre)
+			if (!(flags & LoadSavePlugin::lfLoadAsPattern))
 			{
-				m_Doc->m_Selection->addItem(Elements.at(dre), true);
+				m_Doc->m_Selection->delaySignalsOn();
+				for (int dre=0; dre<Elements.count(); ++dre)
+				{
+					m_Doc->m_Selection->addItem(Elements.at(dre), true);
+				}
+				m_Doc->m_Selection->delaySignalsOff();
+				m_Doc->m_Selection->setGroupRect();
+				if (m_Doc->view() != nullptr)
+					m_Doc->view()->updatesOn(true);
 			}
-			m_Doc->m_Selection->delaySignalsOff();
-			m_Doc->m_Selection->setGroupRect();
-			m_Doc->view()->updatesOn(true);
 			importCanceled = false;
 		}
 		else
 		{
 			m_Doc->DragP = true;
-			m_Doc->DraggedElem = 0;
+			m_Doc->DraggedElem = nullptr;
 			m_Doc->DragElements.clear();
 			m_Doc->m_Selection->delaySignalsOn();
 			for (int dre=0; dre<Elements.count(); ++dre)
 			{
-				m_Doc->DragElements.append(Elements.at(dre)->ItemNr);
 				m_tmpSel->addItem(Elements.at(dre), true);
 			}
-			ScriXmlDoc *ss = new ScriXmlDoc();
 			m_tmpSel->setGroupRect();
-			ScElemMimeData* md = new ScElemMimeData();
-			md->setScribusElem(ss->WriteElem(m_Doc, m_Doc->view(), m_tmpSel));
-			delete ss;
+			ScElemMimeData* md = ScriXmlDoc::writeToMimeData(m_Doc, m_tmpSel);
 /*#ifndef QT_MAC*/
 // see #2526
 			m_Doc->itemSelection_DeleteItem(m_tmpSel);
@@ -704,7 +682,7 @@ bool WMFImport::importWMF(const TransactionSettings& trSettings, int flags)
 			TransactionSettings* transacSettings = new TransactionSettings(trSettings);
 			m_Doc->view()->handleObjectImport(md, transacSettings);
 			m_Doc->DragP = false;
-			m_Doc->DraggedElem = 0;
+			m_Doc->DraggedElem = nullptr;
 			m_Doc->DragElements.clear();
 		}
 	}
@@ -714,14 +692,14 @@ bool WMFImport::importWMF(const TransactionSettings& trSettings, int flags)
 		m_Doc->setLoading(false);
 		m_Doc->changed();
 		m_Doc->reformPages();
-		m_Doc->view()->updatesOn(true);
+		if ((!(flags & LoadSavePlugin::lfLoadAsPattern)) && (m_Doc->view() != nullptr))
+			m_Doc->view()->updatesOn(true);
 		m_Doc->setLoading(loadF);
 	}
-	qApp->restoreOverrideCursor();
 	return true;
 }
 
-QList<PageItem*> WMFImport::parseWmfCommands(void)
+QList<PageItem*> WMFImport::parseWmfCommands()
 {
 	int idx, i;
 	const WmfCmd* cmd;
@@ -731,10 +709,10 @@ QList<PageItem*> WMFImport::parseWmfCommands(void)
 
 	if ( !m_Valid )  return elements;
 
-	if ( m_ObjHandleTab ) delete[] m_ObjHandleTab;
+	delete[] m_ObjHandleTab;
 	m_ObjHandleTab = new WmfObjHandle* [ MAX_OBJHANDLE ];
 	for ( i = MAX_OBJHANDLE-1; i >= 0; i-- )
-		m_ObjHandleTab[ i ] = NULL;
+		m_ObjHandleTab[ i ] = nullptr;
 
 	if ( WMFIMPORT_DEBUG )  {
 		cerr << "Bounding box : " << m_BBox.left() << " " << m_BBox.top() << " " << m_BBox.right() << " " << m_BBox.bottom() << endl;
@@ -754,7 +732,7 @@ QList<PageItem*> WMFImport::parseWmfCommands(void)
 
 		if ( WMFIMPORT_DEBUG )  {
 			QString str = "", param;
-			if ( metaFuncTab[ idx ].name == NULL ) {
+			if ( metaFuncTab[ idx ].name == nullptr ) {
 				str += "UNKNOWN ";
 			}
 			if ( metaFuncTab[ idx ].method == &WMFImport::noop ) {
@@ -768,7 +746,7 @@ QList<PageItem*> WMFImport::parseWmfCommands(void)
 				str += param;
 				str += " ";
 			}
-			cerr << str.toAscii().data() << endl;
+			cerr << str.toLatin1().data() << endl;
 		}
 	}
 	return elements;
@@ -776,18 +754,18 @@ QList<PageItem*> WMFImport::parseWmfCommands(void)
 
 void WMFImport::finishCmdParsing( PageItem* item )
 {
-	QMatrix gcm  = m_context.worldMatrix();
+	QTransform gcm  = m_context.worldMatrix();
 	double coeff1 = sqrt(gcm.m11() * gcm.m11() + gcm.m12() * gcm.m12());
 	double coeff2 = sqrt(gcm.m21() * gcm.m21() + gcm.m22() * gcm.m22());
-	if( item->asImageFrame() )
+	if (item->asImageFrame())
 	{
 		item->moveBy(gcm.dx(), gcm.dy());
 		item->setWidthHeight(item->width() * gcm.m11(), item->height() * gcm.m22());
 		item->setLineWidth(qMax(item->lineWidth() * (coeff1 + coeff2) / 2.0, 0.25));
-		if (item->PictureIsAvailable)
+		if (item->imageIsAvailable)
 			item->setImageXYScale(item->width() / item->pixm.width(), item->height() / item->pixm.height());
 	}
-	else if( item->asTextFrame() )
+	else if (item->asTextFrame())
 	{
 		item->setLineWidth(qMax(item->lineWidth() * (coeff1 + coeff2) / 2.0, 0.25));
 	}
@@ -799,7 +777,7 @@ void WMFImport::finishCmdParsing( PageItem* item )
 		item->setLineWidth(qMax(item->lineWidth() * (coeff1 + coeff2) / 2.0, 0.25));
 		FPoint wh = getMaxClipF(&item->PoLine);
 		item->setWidthHeight(wh.x(), wh.y());
-		m_Doc->AdjustItemSize(item);
+		m_Doc->adjustItemSize(item);
 	}
 	item->setRedrawBounding();
 	item->OwnPage   = m_Doc->OnPage(item);
@@ -834,7 +812,7 @@ void WMFImport::lineTo( QList<PageItem*>& items, long, short* params )
 		double y2 = params[0];
 		double  lineWidth = m_context.pen().width();
 		QString lineColor = importColor(m_context.pen().color());
-		int z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, CommonStrings::None, lineColor, true);
+		int z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, CommonStrings::None, lineColor);
 		PageItem* ite = m_Doc->Items->at(z);
 		ite->PoLine.resize(4);
 		ite->PoLine.setPoint(0, FPoint(x1, y1));
@@ -865,9 +843,9 @@ void WMFImport::ellipse( QList<PageItem*>& items, long, short* params )
 	QString fillColor   = doFill ? importColor( m_context.brush().color() ) : CommonStrings::None;
 	QString strokeColor = doStroke ? importColor( m_context.pen().color() ) : CommonStrings::None;
 	double  lineWidth   = m_context.pen().width();
-	int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Ellipse, BaseX, BaseY, rx * 2.0, ry * 2.0, lineWidth, fillColor, strokeColor, true);
+	int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Ellipse, BaseX, BaseY, rx * 2.0, ry * 2.0, lineWidth, fillColor, strokeColor);
 	PageItem* ite = m_Doc->Items->at(z);
-	QMatrix mm(1.0, 0.0, 0.0, 1.0, px, py);
+	QTransform mm(1.0, 0.0, 0.0, 1.0, px, py);
 	ite->PoLine.map(mm);
 	finishCmdParsing(ite);
 	items.append(ite);
@@ -887,9 +865,9 @@ void WMFImport::polygon( QList<PageItem*>& items, long, short* params )
 		lineWidth = 1.0;
 	FPointArray paramPoints = pointsFromParam( params[0], &params[1] );
 	FPointArray points      = pointsToPolyline( paramPoints, true );
-	if( paramPoints.size() > 0 )
+	if (!paramPoints.empty())
 	{
-		int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, fillColor, strokeColor, true);
+		int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, fillColor, strokeColor);
 		PageItem* ite = m_Doc->Items->at(z);
 		ite->PoLine = points;
 		ite->fillRule = !m_context.windingFill();
@@ -902,16 +880,40 @@ void WMFImport::polyPolygon( QList<PageItem*>& items, long num, short* params )
 {
 	int numPolys   = params[0];
 	int pointIndex = params[0] + 1;
+	FPointArray pointsPoly;
 	for (int i = 0; i < numPolys; ++i)
 	{
 		short  numPoints  = params[i + 1];
 		short* paramArray = new short[1 + 2 * numPoints];
 		paramArray[0] = numPoints;
 		memcpy(&paramArray[1], &params[pointIndex], 2 * numPoints * sizeof(short));
-		polygon(items, num, paramArray);
+		FPointArray paramPoints = pointsFromParam( numPoints, &paramArray[1] );
+		FPointArray points      = pointsToPolyline( paramPoints, true );
+		pointsPoly += points;
+		if (numPolys > 1)
+			pointsPoly.setMarker();
+//		polygon(items, num, paramArray);
 		delete[] paramArray;
 		pointIndex += (2 * numPoints);
 	}
+	double  BaseX = m_Doc->currentPage()->xOffset();
+	double  BaseY = m_Doc->currentPage()->yOffset();
+	bool    doFill = m_context.brush().style() != Qt::NoBrush;
+	bool    doStroke = m_context.pen().style() != Qt::NoPen;
+	QString fillColor   = doFill ? importColor( m_context.brush().color() ) : CommonStrings::None;
+	QString strokeColor = doStroke ? importColor( m_context.pen().color() ) : CommonStrings::None;
+	double  lineWidth   = m_context.pen().width();
+	if (doStroke && lineWidth <= 0.0 )
+		lineWidth = 1.0;
+	if (!pointsPoly.empty())
+	{
+		int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, fillColor, strokeColor);
+		PageItem* ite = m_Doc->Items->at(z);
+		ite->PoLine = pointsPoly;
+		ite->fillRule = !m_context.windingFill();
+		finishCmdParsing(ite);
+		items.append(ite);
+	}
 }
 
 void WMFImport::polyline( QList<PageItem*>& items, long, short* params )
@@ -927,9 +929,9 @@ void WMFImport::polyline( QList<PageItem*>& items, long, short* params )
 		lineWidth = 1.0;
 	FPointArray paramPoints = pointsFromParam( params[0], &params[1] );
 	FPointArray points      = pointsToPolyline( paramPoints, false );
-	if( paramPoints.size() > 0 )
+	if (!paramPoints.empty())
 	{
-		int z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, fillColor, strokeColor, true);
+		int z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, fillColor, strokeColor);
 		PageItem* ite = m_Doc->Items->at(z);
 		ite->PoLine = points;
 		finishCmdParsing(ite);
@@ -953,9 +955,9 @@ void WMFImport::rectangle( QList<PageItem*>& items, long, short* params )
 	double y = ((params[2] - params[0]) > 0) ? params[0] : params[2];
 	double width  = fabs((double) params[3] - params[1]);
 	double height = fabs((double) params[2] - params[0]);
-	int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Rectangle, BaseX, BaseY, width, height, lineWidth, fillColor, strokeColor, true);
+	int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Rectangle, BaseX, BaseY, width, height, lineWidth, fillColor, strokeColor);
 	PageItem* ite = m_Doc->Items->at(z);
-	QMatrix mm(1.0, 0.0, 0.0, 1.0, x, y);
+	QTransform mm(1.0, 0.0, 0.0, 1.0, x, y);
 	ite->PoLine.map(mm);
 	finishCmdParsing(ite);
 	items.append(ite);
@@ -979,7 +981,7 @@ void WMFImport::roundRect( QList<PageItem*>& items, long, short* params )
 	double height = fabs((double) params[4] - params[2]);
 	double rx     = params[1] / 2.0;
 	double ry     = params[0] / 2.0;
-	int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Rectangle, BaseX, BaseY, width, height, lineWidth, fillColor, strokeColor, true);
+	int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Rectangle, BaseX, BaseY, width, height, lineWidth, fillColor, strokeColor);
 	PageItem* ite = m_Doc->Items->at(z);
 	if ((rx != 0.0) || (ry != 0.0))
 	{
@@ -987,7 +989,7 @@ void WMFImport::roundRect( QList<PageItem*>& items, long, short* params )
 		ite->SetFrameRound();
 		m_Doc->setRedrawBounding(ite);
 	}
-	QMatrix mm(1.0, 0.0, 0.0, 1.0, x, y);
+	QTransform mm(1.0, 0.0, 0.0, 1.0, x, y);
 	ite->PoLine.map(mm);
 	finishCmdParsing(ite);
 	items.append(ite);
@@ -1016,9 +1018,9 @@ void WMFImport::arc( QList<PageItem*>& items, long, short* params )
 	painterPath.arcMoveTo(x, y, xWidth, yHeight, angleStart);
 	painterPath.arcTo(x, y, xWidth, yHeight, angleStart, angleLength);
 	pointArray.fromQPainterPath(painterPath);
-	if( pointArray.size() > 0 )
+	if (!pointArray.empty())
 	{
-		int z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, fillColor, strokeColor, true);
+		int z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, fillColor, strokeColor);
 		PageItem* ite = m_Doc->Items->at(z);
 		ite->PoLine = pointArray;
 		finishCmdParsing(ite);
@@ -1053,9 +1055,9 @@ void WMFImport::chord( QList<PageItem*>& items, long, short* params )
 	painterPath.arcTo (x, y, xWidth, yHeight, angleStart, angleLength);
 	painterPath.lineTo(firstPoint);
 	pointArray.fromQPainterPath(painterPath);
-	if( pointArray.size() > 0 )
+	if (!pointArray.empty())
 	{
-		int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, fillColor, strokeColor, true);
+		int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, fillColor, strokeColor);
 		PageItem* ite = m_Doc->Items->at(z);
 		ite->PoLine = pointArray;
 		finishCmdParsing(ite);
@@ -1091,9 +1093,9 @@ void WMFImport::pie( QList<PageItem*>& items, long, short* params )
 	painterPath.lineTo(xCenter, yCenter);
 	painterPath.lineTo(firstPoint);
 	pointArray.fromQPainterPath(painterPath);
-	if( pointArray.size() > 0 )
+	if (!pointArray.empty())
 	{
-		int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, fillColor, strokeColor, true);
+		int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, fillColor, strokeColor);
 		PageItem* ite = m_Doc->Items->at(z);
 		ite->PoLine = pointArray;
 		finishCmdParsing(ite);
@@ -1111,7 +1113,7 @@ void WMFImport::setBkColor( QList<PageItem*>& /*items*/, long, short* params )
 	m_context.setBackgroundColor( colorFromParam( params ) );
 }
 
-void WMFImport::setBkMode( QList<PageItem*>& /*items*/, long, short* params )
+void WMFImport::setBkMode(QList<PageItem*>& /*items*/, long, short* params )
 {
 	if ( params[ 0 ]==1 ) 
 		m_context.setBackgroundMode( Qt::TransparentMode );
@@ -1253,9 +1255,9 @@ void WMFImport::extTextOut( QList<PageItem*>& items, long num, short* params )
 				xpos += params[idxOffset + index - 1];
 			painterPath.addText(xpos, ypos, m_context.font(), textString.at(index));
 			textPath.fromQPainterPath(painterPath);
-			if (textPath.size() > 0)
+			if (!textPath.empty())
 			{
-				int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, textColor, CommonStrings::None, true);
+				int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, textColor, CommonStrings::None);
 				PageItem* ite = m_Doc->Items->at(z);
 				ite->PoLine = textPath;
 				finishCmdParsing(ite);
@@ -1270,10 +1272,10 @@ void WMFImport::extTextOut( QList<PageItem*>& items, long num, short* params )
 		QPainterPath painterPath;
 		painterPath.addText( startX, startY, m_context.font(), textString );
 		textPath.fromQPainterPath(painterPath);
-		if (textPath.size() > 0)
+		if (!textPath.empty())
 		{
 			double  lineWidth = 0.0;
-			int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, textColor, CommonStrings::None, true);
+			int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, BaseX, BaseY, 10, 10, lineWidth, textColor, CommonStrings::None);
 			PageItem* ite = m_Doc->Items->at(z);
 			ite->PoLine = textPath;
 			finishCmdParsing(ite);
@@ -1283,7 +1285,7 @@ void WMFImport::extTextOut( QList<PageItem*>& items, long num, short* params )
 	m_context.restore();
 }
 
-void WMFImport::selectObject( QList<PageItem*>& /*items*/, long, short* params )
+void WMFImport::selectObject(QList<PageItem*>& items, long num, short* params )
 {
 	int idx = params[ 0 ];
 	if ( idx >= 0 && idx < MAX_OBJHANDLE && m_ObjHandleTab[ idx ] )
@@ -1409,7 +1411,7 @@ void WMFImport::addHandle( WmfObjHandle* handle )
 	int idx;
 
 	for ( idx =0; idx < MAX_OBJHANDLE ; idx++ )
-		if ( m_ObjHandleTab[ idx ] == NULL )  break;
+		if ( m_ObjHandleTab[ idx ] == nullptr )  break;
 
 	if ( idx < MAX_OBJHANDLE )
 		m_ObjHandleTab[ idx ] = handle;
@@ -1422,7 +1424,7 @@ void WMFImport::deleteHandle( int idx )
 	if ( idx >= 0 && idx < MAX_OBJHANDLE && m_ObjHandleTab[ idx ] )
 	{
 		delete m_ObjHandleTab[ idx ];
-		m_ObjHandleTab[ idx ] = NULL;
+		m_ObjHandleTab[ idx ] = nullptr;
 	}
 }
 
@@ -1434,7 +1436,7 @@ unsigned short WMFImport::calcCheckSum( WmfPlaceableHeader* apmfh )
 	// Start with the first word
 	wResult = *( lpWord = ( WORD16* )( apmfh ) );
 	// XOR in each of the other 9 words
-	for( i=1; i<=9; i++ )
+	for (i = 1; i <= 9; i++)
 	{
 		wResult ^= lpWord[ i ];
 	}
@@ -1452,7 +1454,7 @@ int WMFImport::findFunc( unsigned short aFunc ) const
 	return i;
 }
 
-unsigned int WMFImport::toDWord( short* params )
+unsigned int WMFImport::toDWord( const short* params )
 {
 	unsigned int l;
 #if !defined( WORDS_BIGENDIAN )
