@@ -15,25 +15,18 @@ for which a new license (GPL+exception) is in place.
 #include <QStack>
 #include <QDebug>
 
-#if defined(_MSC_VER) && !defined(_USE_MATH_DEFINES)
-#define _USE_MATH_DEFINES
-#endif
-#include <cmath>
 #include <cstdlib>
 
-#include "commonstrings.h"
-#include "customfdialog.h"
 #include "importxfig.h"
+
+#include "commonstrings.h"
 #include "loadsaveplugin.h"
-#include "missing.h"
-#include "multiprogressdialog.h"
+#include "pageitem.h"
 #include "pagesize.h"
 #include "prefscontext.h"
 #include "prefsfile.h"
 #include "prefsmanager.h"
-#include "prefsmanager.h"
 #include "prefstable.h"
-#include "propertiespalette.h"
 #include "rawimage.h"
 #include "scclocale.h"
 #include "sccolorengine.h"
@@ -41,30 +34,158 @@ for which a new license (GPL+exception) is in place.
 #include "scmimedata.h"
 #include "scpaths.h"
 #include "scpattern.h"
-#include "scpattern.h"
-#include "scribus.h"
 #include "scribusXml.h"
 #include "scribuscore.h"
+#include "scribusdoc.h"
+#include "scribusview.h"
 #include "sctextstream.h"
 #include "selection.h"
+#include "ui/customfdialog.h"
+#include "ui/missing.h"
+#include "ui/multiprogressdialog.h"
+#include "ui/propertiespalette.h"
 #include "undomanager.h"
 #include "util.h"
 #include "util_formats.h"
-#include "util_icon.h"
 #include "util_math.h"
 
-extern SCRIBUS_API ScribusQApp * ScQApp;
-
 XfigPlug::XfigPlug(ScribusDoc* doc, int flags)
 {
 	tmpSel=new Selection(this, false);
 	m_Doc=doc;
+	importerFlags = flags;
 	interactive = (flags & LoadSavePlugin::lfInteractive);
+	progressDialog = nullptr;
 }
 
-bool XfigPlug::import(QString fNameIn, const TransactionSettings& trSettings, int flags, bool showProgress)
+QImage XfigPlug::readThumbnail(const QString& fName)
+{
+	QFileInfo fi = QFileInfo(fName);
+	baseFile = QDir::cleanPath(QDir::toNativeSeparators(fi.absolutePath()+"/"));
+	double w=0.0, h=0.0, x=0.0, y=0.0;
+	parseHeader(fName, x, y, w, h);
+	docX = x;
+	docY = y;
+	if (w == 0.0)
+		w = PrefsManager::instance().appPrefs.docSetupPrefs.pageWidth;
+	if (h == 0.0)
+		h = PrefsManager::instance().appPrefs.docSetupPrefs.pageHeight;
+	docWidth = w - x;
+	docHeight = h - y;
+	progressDialog = nullptr;
+	m_Doc = new ScribusDoc();
+	m_Doc->setup(0, 1, 1, 1, 1, "Custom", "Custom");
+	m_Doc->setPage(docWidth, docHeight, 0, 0, 0, 0, 0, 0, false, false);
+	m_Doc->addPage(0);
+	m_Doc->setGUI(false, ScCore->primaryMainWindow(), nullptr);
+	baseX = m_Doc->currentPage()->xOffset();
+	baseY = m_Doc->currentPage()->yOffset();
+	Elements.clear();
+	CustColors.clear();
+	importedColors.insert(0, "Black");
+	importedColors.insert(1, "Blue");
+	importedColors.insert(2, "Green");
+	importedColors.insert(3, "Cyan");
+	importedColors.insert(4, "Red");
+	importedColors.insert(5, "Magenta");
+	importedColors.insert(6, "Yellow");
+	importedColors.insert(7, "White");
+	importedColors.insert(8, "Blue4");
+	importedColors.insert(9, "Blue3");
+	importedColors.insert(10, "Blue2");
+	importedColors.insert(11, "LtBlue");
+	importedColors.insert(12, "Green4");
+	importedColors.insert(13, "Green3");
+	importedColors.insert(14, "Green2");
+	importedColors.insert(15, "Cyan4");
+	importedColors.insert(16, "Cyan3");
+	importedColors.insert(17, "Cyan2");
+	importedColors.insert(18, "Red4");
+	importedColors.insert(19, "Red3");
+	importedColors.insert(20, "Red2");
+	importedColors.insert(21, "Magenta4");
+	importedColors.insert(22, "Magenta3");
+	importedColors.insert(23, "Magenta2");
+	importedColors.insert(24, "Brown4");
+	importedColors.insert(25, "Brown3");
+	importedColors.insert(26, "Brown2");
+	importedColors.insert(27, "Pink4");
+	importedColors.insert(28, "Pink3");
+	importedColors.insert(29, "Pink2");
+	importedColors.insert(30, "Pink");
+	importedColors.insert(31, "Gold");
+	CustColors.insert("Blue", ScColor(0, 0, 255));
+	CustColors.insert("Blue4", ScColor(0, 0, 144));
+	CustColors.insert("Blue3", ScColor(0, 0, 176));
+	CustColors.insert("Blue2", ScColor(0, 0, 208));
+	CustColors.insert("LtBlue", ScColor(135, 206, 255));
+	CustColors.insert("Cyan", ScColor(255, 0, 0, 0));
+	CustColors.insert("Cyan4", ScColor(0, 144, 144));
+	CustColors.insert("Cyan3", ScColor(0, 176, 176));
+	CustColors.insert("Cyan2", ScColor(0, 208, 208));
+	CustColors.insert("Green", ScColor(255, 0, 255, 0));
+	CustColors.insert("Green4", ScColor(0, 144, 0));
+	CustColors.insert("Green3", ScColor(0, 176, 0));
+	CustColors.insert("Green2", ScColor(0, 208, 0));
+	CustColors.insert("Red", ScColor(0, 255, 255, 0));
+	CustColors.insert("Red4", ScColor(144, 0, 0));
+	CustColors.insert("Red3", ScColor(176, 0, 0));
+	CustColors.insert("Red2", ScColor(208, 0, 0));
+	CustColors.insert("Yellow", ScColor(0, 0, 255, 0));
+	CustColors.insert("Magenta", ScColor(0, 255, 0, 0));
+	CustColors.insert("Magenta4", ScColor(144, 0, 144));
+	CustColors.insert("Magenta3", ScColor(176, 0, 176));
+	CustColors.insert("Magenta2", ScColor(208, 0, 208));
+	CustColors.insert("Brown4", ScColor(128, 48, 0));
+	CustColors.insert("Brown3", ScColor(160, 64, 0));
+	CustColors.insert("Brown2", ScColor(192, 96, 0));
+	CustColors.insert("Pink4", ScColor(255, 128, 128));
+	CustColors.insert("Pink3", ScColor(255, 160, 160));
+	CustColors.insert("Pink2", ScColor(255, 192, 192));
+	CustColors.insert("Pink", ScColor(255, 224, 224));
+	CustColors.insert("Gold", ScColor(255, 215, 0));
+	m_Doc->setLoading(true);
+	m_Doc->DoDrawing = false;
+	m_Doc->scMW()->setScriptRunning(true);
+	QString CurDirP = QDir::currentPath();
+	QDir::setCurrent(fi.path());
+	if (convert(fName))
+	{
+		tmpSel->clear();
+		QDir::setCurrent(CurDirP);
+		if (Elements.count() > 1)
+			m_Doc->groupObjectsList(Elements);
+		m_Doc->DoDrawing = true;
+		m_Doc->m_Selection->delaySignalsOn();
+		QImage tmpImage;
+		if (Elements.count() > 0)
+		{
+			for (int dre=0; dre<Elements.count(); ++dre)
+			{
+				tmpSel->addItem(Elements.at(dre), true);
+			}
+			tmpSel->setGroupRect();
+			double xs = tmpSel->width();
+			double ys = tmpSel->height();
+			tmpImage = Elements.at(0)->DrawObj_toImage(500);
+			tmpImage.setText("XSize", QString("%1").arg(xs));
+			tmpImage.setText("YSize", QString("%1").arg(ys));
+		}
+		m_Doc->scMW()->setScriptRunning(false);
+		m_Doc->setLoading(false);
+		m_Doc->m_Selection->delaySignalsOff();
+		delete m_Doc;
+		return tmpImage;
+	}
+	QDir::setCurrent(CurDirP);
+	m_Doc->DoDrawing = true;
+	m_Doc->scMW()->setScriptRunning(false);
+	delete m_Doc;
+	return QImage();
+}
+
+bool XfigPlug::import(const QString& fNameIn, const TransactionSettings& trSettings, int flags, bool showProgress)
 {
-	QString fName = fNameIn;
 	bool success = false;
 	interactive = (flags & LoadSavePlugin::lfInteractive);
 	importerFlags = flags;
@@ -104,7 +225,7 @@ bool XfigPlug::import(QString fNameIn, const TransactionSettings& trSettings, in
 	importedColors.insert(29, "Pink2");
 	importedColors.insert(30, "Pink");
 	importedColors.insert(31, "Gold");
-	CustColors.insert("Blue", ScColor(255, 0, 0, 0));
+	CustColors.insert("Blue", ScColor(0, 0, 255));
 	CustColors.insert("Blue4", ScColor(0, 0, 144));
 	CustColors.insert("Blue3", ScColor(0, 0, 176));
 	CustColors.insert("Blue2", ScColor(0, 0, 208));
@@ -134,7 +255,7 @@ bool XfigPlug::import(QString fNameIn, const TransactionSettings& trSettings, in
 	CustColors.insert("Pink2", ScColor(255, 192, 192));
 	CustColors.insert("Pink", ScColor(255, 224, 224));
 	CustColors.insert("Gold", ScColor(255, 215, 0));
-	QFileInfo fi = QFileInfo(fName);
+	QFileInfo fi = QFileInfo(fNameIn);
 	if ( !ScCore->usingGUI() )
 	{
 		interactive = false;
@@ -143,7 +264,7 @@ bool XfigPlug::import(QString fNameIn, const TransactionSettings& trSettings, in
 	baseFile = QDir::cleanPath(QDir::toNativeSeparators(fi.absolutePath()+"/"));
 	if ( showProgress )
 	{
-		ScribusMainWindow* mw=(m_Doc==0) ? ScCore->primaryMainWindow() : m_Doc->scMW();
+		ScribusMainWindow* mw=(m_Doc==nullptr) ? ScCore->primaryMainWindow() : m_Doc->scMW();
 		progressDialog = new MultiProgressDialog( tr("Importing: %1").arg(fi.fileName()), CommonStrings::tr_Cancel, mw );
 		QStringList barNames, barTexts;
 		barNames << "GI";
@@ -159,7 +280,7 @@ bool XfigPlug::import(QString fNameIn, const TransactionSettings& trSettings, in
 		qApp->processEvents();
 	}
 	else
-		progressDialog = NULL;
+		progressDialog = nullptr;
 /* Set default Page to size defined in Preferences */
 	x = 0.0;
 	y = 0.0;
@@ -170,13 +291,13 @@ bool XfigPlug::import(QString fNameIn, const TransactionSettings& trSettings, in
 		progressDialog->setOverallProgress(1);
 		qApp->processEvents();
 	}
-	parseHeader(fName, x, y, b, h);
+	parseHeader(fNameIn, x, y, b, h);
 	docX = x;
 	docY = y;
 	if (b == 0.0)
-		b = PrefsManager::instance()->appPrefs.PageWidth;
+		b = PrefsManager::instance().appPrefs.docSetupPrefs.pageWidth;
 	if (h == 0.0)
-		h = PrefsManager::instance()->appPrefs.PageHeight;
+		h = PrefsManager::instance().appPrefs.docSetupPrefs.pageHeight;
 	docWidth = b - x;
 	docHeight = h - y;
 	baseX = 0;
@@ -202,10 +323,10 @@ bool XfigPlug::import(QString fNameIn, const TransactionSettings& trSettings, in
 	}
 	if (flags & LoadSavePlugin::lfCreateDoc)
 	{
-		m_Doc->documentInfo.setAuthor(docCreator);
-		m_Doc->documentInfo.setPublisher(docOrganisation);
-		m_Doc->documentInfo.setTitle(docTitle);
-		m_Doc->documentInfo.setDate(docDate+" "+docTime);
+		m_Doc->documentInfo().setAuthor(docCreator);
+		m_Doc->documentInfo().setPublisher(docOrganisation);
+		m_Doc->documentInfo().setTitle(docTitle);
+		m_Doc->documentInfo().setDate(docDate+" "+docTime);
 	}
 	if ((!ret) && (interactive))
 	{
@@ -215,83 +336,28 @@ bool XfigPlug::import(QString fNameIn, const TransactionSettings& trSettings, in
 	if ((ret) || (!interactive))
 	{
 		if (b-x > h-y)
-			m_Doc->PageOri = 1;
+			m_Doc->setPageOrientation(1);
 		else
-			m_Doc->PageOri = 0;
-		m_Doc->m_pageSize = "Custom";
+			m_Doc->setPageOrientation(0);
+		m_Doc->setPageSize("Custom");
 	}
+	if ((!(flags & LoadSavePlugin::lfLoadAsPattern)) && (m_Doc->view() != nullptr))
+		m_Doc->view()->Deselect();
 	Elements.clear();
-	m_Doc->view()->Deselect();
 	m_Doc->setLoading(true);
 	m_Doc->DoDrawing = false;
-	m_Doc->view()->updatesOn(false);
+	if ((!(flags & LoadSavePlugin::lfLoadAsPattern)) && (m_Doc->view() != nullptr))
+		m_Doc->view()->updatesOn(false);
 	m_Doc->scMW()->setScriptRunning(true);
 	qApp->setOverrideCursor(QCursor(Qt::WaitCursor));
 	QString CurDirP = QDir::currentPath();
 	QDir::setCurrent(fi.path());
-	if (convert(fName))
+	if (convert(fNameIn))
 	{
 		tmpSel->clear();
 		QDir::setCurrent(CurDirP);
 		if ((Elements.count() > 1) && (!(importerFlags & LoadSavePlugin::lfCreateDoc)))
-		{
-			bool isGroup = true;
-			int firstElem = -1;
-			if (Elements.at(0)->Groups.count() != 0)
-				firstElem = Elements.at(0)->Groups.top();
-			for (int bx = 0; bx < Elements.count(); ++bx)
-			{
-				PageItem* bxi = Elements.at(bx);
-				if (bxi->Groups.count() != 0)
-				{
-					if (bxi->Groups.top() != firstElem)
-						isGroup = false;
-				}
-				else
-					isGroup = false;
-			}
-			if (!isGroup)
-			{
-				double minx = 99999.9;
-				double miny = 99999.9;
-				double maxx = -99999.9;
-				double maxy = -99999.9;
-				uint lowestItem = 999999;
-				uint highestItem = 0;
-				for (int a = 0; a < Elements.count(); ++a)
-				{
-					Elements.at(a)->Groups.push(m_Doc->GroupCounter);
-					PageItem* currItem = Elements.at(a);
-					lowestItem = qMin(lowestItem, currItem->ItemNr);
-					highestItem = qMax(highestItem, currItem->ItemNr);
-					double x1, x2, y1, y2;
-					currItem->getVisualBoundingRect(&x1, &y1, &x2, &y2);
-					minx = qMin(minx, x1);
-					miny = qMin(miny, y1);
-					maxx = qMax(maxx, x2);
-					maxy = qMax(maxy, y2);
-				}
-				double gx = minx;
-				double gy = miny;
-				double gw = maxx - minx;
-				double gh = maxy - miny;
-				PageItem *high = m_Doc->Items->at(highestItem);
-				int z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Rectangle, gx, gy, gw, gh, 0, m_Doc->toolSettings.dBrush, m_Doc->toolSettings.dPen, true);
-				PageItem *neu = m_Doc->Items->takeAt(z);
-				m_Doc->Items->insert(lowestItem, neu);
-				neu->Groups.push(m_Doc->GroupCounter);
-				neu->setItemName( tr("Group%1").arg(neu->Groups.top()));
-				neu->isGroupControl = true;
-				neu->groupsLastItem = high;
-				neu->setTextFlowMode(PageItem::TextFlowDisabled);
-				for (int a = 0; a < m_Doc->Items->count(); ++a)
-				{
-					m_Doc->Items->at(a)->ItemNr = a;
-				}
-				Elements.prepend(neu);
-				m_Doc->GroupCounter++;
-			}
-		}
+			m_Doc->groupObjectsList(Elements);
 		m_Doc->DoDrawing = true;
 		m_Doc->scMW()->setScriptRunning(false);
 		m_Doc->setLoading(false);
@@ -304,37 +370,32 @@ bool XfigPlug::import(QString fNameIn, const TransactionSettings& trSettings, in
 				m_Doc->setLoading(false);
 				m_Doc->changed();
 				m_Doc->setLoading(loadF);
-				m_Doc->m_Selection->delaySignalsOn();
-				for (int dre=0; dre<Elements.count(); ++dre)
+				if (!(flags & LoadSavePlugin::lfLoadAsPattern))
 				{
-					m_Doc->m_Selection->addItem(Elements.at(dre), true);
+					m_Doc->m_Selection->delaySignalsOn();
+					for (int dre=0; dre<Elements.count(); ++dre)
+					{
+						m_Doc->m_Selection->addItem(Elements.at(dre), true);
+					}
+					m_Doc->m_Selection->delaySignalsOff();
+					m_Doc->m_Selection->setGroupRect();
+					if (m_Doc->view() != nullptr)
+						m_Doc->view()->updatesOn(true);
 				}
-				m_Doc->m_Selection->delaySignalsOff();
-				m_Doc->m_Selection->setGroupRect();
-				m_Doc->view()->updatesOn(true);
 			}
 			else
 			{
 				m_Doc->DragP = true;
-				m_Doc->DraggedElem = 0;
+				m_Doc->DraggedElem = nullptr;
 				m_Doc->DragElements.clear();
 				m_Doc->m_Selection->delaySignalsOn();
 				for (int dre=0; dre<Elements.count(); ++dre)
 				{
-					m_Doc->DragElements.append(Elements.at(dre)->ItemNr);
 					tmpSel->addItem(Elements.at(dre), true);
 				}
 				tmpSel->setGroupRect();
-				ScriXmlDoc *ss = new ScriXmlDoc();
-				ScElemMimeData* md = new ScElemMimeData();
-				md->setScribusElem(ss->WriteElem(m_Doc, m_Doc->view(), tmpSel));
-				delete ss;
-/*#ifndef Q_WS_MAC*/
-// see #2196
+				ScElemMimeData* md = ScriXmlDoc::writeToMimeData(m_Doc, tmpSel);
 				m_Doc->itemSelection_DeleteItem(tmpSel);
-/*#else
-				qDebug("xfigimport: leaving items on page");
-#endif*/
 				m_Doc->view()->updatesOn(true);
 				m_Doc->m_Selection->delaySignalsOff();
 				// We must copy the TransationSettings object as it is owned
@@ -342,7 +403,7 @@ bool XfigPlug::import(QString fNameIn, const TransactionSettings& trSettings, in
 				TransactionSettings* transacSettings = new TransactionSettings(trSettings);
 				m_Doc->view()->handleObjectImport(md, transacSettings);
 				m_Doc->DragP = false;
-				m_Doc->DraggedElem = 0;
+				m_Doc->DraggedElem = nullptr;
 				m_Doc->DragElements.clear();
 			}
 		}
@@ -350,7 +411,8 @@ bool XfigPlug::import(QString fNameIn, const TransactionSettings& trSettings, in
 		{
 			m_Doc->changed();
 			m_Doc->reformPages();
-			m_Doc->view()->updatesOn(true);
+			if (!(flags & LoadSavePlugin::lfLoadAsPattern))
+				m_Doc->view()->updatesOn(true);
 		}
 		success = true;
 	}
@@ -365,38 +427,38 @@ bool XfigPlug::import(QString fNameIn, const TransactionSettings& trSettings, in
 	if (interactive)
 		m_Doc->setLoading(false);
 	//CB If we have a gui we must refresh it if we have used the progressbar
-	if ((showProgress) && (!interactive))
-		m_Doc->view()->DrawNew();
+	if (!(flags & LoadSavePlugin::lfLoadAsPattern))
+	{
+		if ((showProgress) && (!interactive))
+			m_Doc->view()->DrawNew();
+	}
 	qApp->restoreOverrideCursor();
 	return success;
 }
 
 XfigPlug::~XfigPlug()
 {
-	if (progressDialog)
-		delete progressDialog;
+	delete progressDialog;
 	delete tmpSel;
 }
 
-bool XfigPlug::parseHeader(QString fName, double &x, double &y, double &b, double &h)
+bool XfigPlug::parseHeader(const QString& fName, double &x, double &y, double &b, double &h)
 {
-	QString tmp, BBox, tmp2, FarNam;
-	ScColor cc;
 	bool found = false;
 	QFile f(fName);
 	if (f.open(QIODevice::ReadOnly))
 	{
 		QDataStream ts(&f);
-		QString version = readLinefromDataStream(ts);
+		QString version = readLineFromDataStream(ts);
 		if (!version.startsWith("#FIG 3"))
 		{
 			f.close();
 			return false;
 		}
-		QString orientation = readLinefromDataStream(ts);
-		QString justification = readLinefromDataStream(ts);
-		QString units = readLinefromDataStream(ts);
-		QString papersize = readLinefromDataStream(ts);
+		QString orientation = readLineFromDataStream(ts);
+		QString justification = readLineFromDataStream(ts);
+		QString units = readLineFromDataStream(ts);
+		QString papersize = readLineFromDataStream(ts);
 		PageSize ps(papersize);
 		if (orientation.startsWith("Landscape"))
 		{
@@ -591,7 +653,7 @@ void XfigPlug::processArrows(int forward_arrow, QString fArrowData, int backward
 			if ((End.x() != Vector.x()) || (End.y() != Vector.y()))
 			{
 				double r = atan2(End.y()-Vector.y(),End.x()-Vector.x())*(180.0/M_PI);
-				QMatrix arrowTrans;
+				QTransform arrowTrans;
 				if (arrow_typeAF == 0)
 					arrow.parseSVG("M -1, -0.5 L 0, 0 L -1, 0.5");
 				else if (arrow_typeAF == 1)
@@ -617,7 +679,7 @@ void XfigPlug::processArrows(int forward_arrow, QString fArrowData, int backward
 		}
 		else
 			iteType = PageItem::Polygon;
-		z = m_Doc->itemAdd(iteType, PageItem::Unspecified, ite->xPos(), ite->yPos(), 10, 10, arrow_thicknessAF, fillC, CurrColorStroke, true);
+		z = m_Doc->itemAdd(iteType, PageItem::Unspecified, ite->xPos(), ite->yPos(), 10, 10, arrow_thicknessAF, fillC, CurrColorStroke);
 		if (z >= 0)
 		{
 			PageItem *item = m_Doc->Items->at(z);
@@ -629,7 +691,7 @@ void XfigPlug::processArrows(int forward_arrow, QString fArrowData, int backward
 			FPoint wh = getMaxClipF(&item->PoLine);
 			item->setWidthHeight(wh.x(),wh.y());
 			item->setTextFlowMode(PageItem::TextFlowDisabled);
-			m_Doc->AdjustItemSize(item);
+			m_Doc->adjustItemSize(item);
 			item->setWidthHeight(qMax(item->width(), 1.0), qMax(item->height(), 1.0));
 			depthMap.insert(999 - depth, currentItemNr);
 			currentItemNr++;
@@ -645,13 +707,13 @@ void XfigPlug::processArrows(int forward_arrow, QString fArrowData, int backward
 		arrow_thicknessAB = arrow_thicknessAB / 80.0 * 72.0;
 		FPointArray arrow;
 		FPoint Start = ite->PoLine.point(0);
-		for (uint xx = 1; xx < ite->PoLine.size(); xx += 2)
+		for (int xx = 1; xx < ite->PoLine.size(); xx += 2)
 		{
 			FPoint Vector = ite->PoLine.point(xx);
 			if ((Start.x() != Vector.x()) || (Start.y() != Vector.y()))
 			{
 				double r = atan2(Start.y()-Vector.y(),Start.x()-Vector.x())*(180.0/M_PI);
-				QMatrix arrowTrans;
+				QTransform arrowTrans;
 				if (arrow_typeAB == 0)
 					arrow.parseSVG("M -1, -0.5 L 0, 0 L -1, 0.5");
 				else if (arrow_typeAB == 1)
@@ -677,7 +739,7 @@ void XfigPlug::processArrows(int forward_arrow, QString fArrowData, int backward
 		}
 		else
 			iteType = PageItem::Polygon;
-		z = m_Doc->itemAdd(iteType, PageItem::Unspecified, ite->xPos(), ite->yPos(), 10, 10, arrow_thicknessAB, fillC, CurrColorStroke, true);
+		z = m_Doc->itemAdd(iteType, PageItem::Unspecified, ite->xPos(), ite->yPos(), 10, 10, arrow_thicknessAB, fillC, CurrColorStroke);
 		if (z >= 0)
 		{
 			PageItem *item = m_Doc->Items->at(z);
@@ -689,7 +751,7 @@ void XfigPlug::processArrows(int forward_arrow, QString fArrowData, int backward
 			FPoint wh = getMaxClipF(&item->PoLine);
 			item->setWidthHeight(wh.x(),wh.y());
 			item->setTextFlowMode(PageItem::TextFlowDisabled);
-			m_Doc->AdjustItemSize(item);
+			m_Doc->adjustItemSize(item);
 			item->setWidthHeight(qMax(item->width(), 1.0), qMax(item->height(), 1.0));
 			depthMap.insert(999 - depth, currentItemNr);
 			currentItemNr++;
@@ -697,11 +759,11 @@ void XfigPlug::processArrows(int forward_arrow, QString fArrowData, int backward
 	}
 }
 
-void XfigPlug::processPolyline(QDataStream &ts, QString data)
+void XfigPlug::processPolyline(QDataStream &ts, const QString& data)
 {
 	QString tmp = data;
-	QString fArrowData = "";
-	QString bArrowData = "";
+	QString fArrowData;
+	QString bArrowData;
 	int		command;
 	int		subtype;				// (1: polyline, 2: box, 3: polygon, 4: arc-box, 5: imported-picture bounding-box)
 	int		line_style;				// (enumeration type)
@@ -729,18 +791,18 @@ void XfigPlug::processPolyline(QDataStream &ts, QString data)
 	Code >> command >> subtype >> line_style >> thickness >> pen_color >> fill_color >> depth >> pen_style;
 	Code >> area_fill >> style_val >> join_style >> cap_style >> radius >> forward_arrow >> backward_arrow >> npoints;
 	if (forward_arrow == 1)
-		fArrowData = readLinefromDataStream(ts);
+		fArrowData = readLineFromDataStream(ts);
 	if (backward_arrow == 1)
-		bArrowData = readLinefromDataStream(ts);
+		bArrowData = readLineFromDataStream(ts);
 	if (subtype == 5)
 	{
-		tmp = readLinefromDataStream(ts);
+		tmp = readLineFromDataStream(ts);
 		ScTextStream imf(&tmp, QIODevice::ReadOnly);
 		imf >> imgFlipped >> imgFile;
 	}
 	while (!ts.atEnd())
 	{
-		tmp = readLinefromDataStream(ts);
+		tmp = readLineFromDataStream(ts);
 		ScTextStream pts(&tmp, QIODevice::ReadOnly);
 		while (!pts.atEnd())
 		{
@@ -769,15 +831,15 @@ void XfigPlug::processPolyline(QDataStream &ts, QString data)
 	int z = -1;
 	PageItem *ite;
 	if (subtype == 1)
-		z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColorFill, CurrColorStroke, true);
+		z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColorFill, CurrColorStroke);
 	else if ((subtype == 2) || (subtype == 3) || (subtype == 4))
 	{
-		z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColorFill, CurrColorStroke, true);
+		z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColorFill, CurrColorStroke);
 		Coords.svgClosePath();
 	}
 	else if (subtype == 5)
 	{
-		z = m_Doc->itemAdd(PageItem::ImageFrame, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColorFill, CurrColorStroke, true);
+		z = m_Doc->itemAdd(PageItem::ImageFrame, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColorFill, CurrColorStroke);
 		Coords.svgClosePath();
 	}
 	else
@@ -811,7 +873,7 @@ void XfigPlug::processPolyline(QDataStream &ts, QString data)
 		FPoint wh = getMaxClipF(&ite->PoLine);
 		ite->setWidthHeight(wh.x(),wh.y());
 		ite->setTextFlowMode(PageItem::TextFlowDisabled);
-		m_Doc->AdjustItemSize(ite);
+		m_Doc->adjustItemSize(ite);
 		ite->setWidthHeight(qMax(ite->width(), 1.0), qMax(ite->height(), 1.0));
 		if (subtype == 4)
 		{
@@ -834,7 +896,7 @@ void XfigPlug::processPolyline(QDataStream &ts, QString data)
 	}
 }
 
-void XfigPlug::processSpline(QDataStream &ts, QString data)
+void XfigPlug::processSpline(QDataStream &ts, const QString& data)
 {
 	QString tmp = data;
 	QString fArrowData = "";
@@ -867,12 +929,12 @@ void XfigPlug::processSpline(QDataStream &ts, QString data)
 	Code >> command >> subtype >> line_style >> thickness >> pen_color >> fill_color >> depth >> pen_style;
 	Code >> area_fill >> style_val >> cap_style >> forward_arrow >> backward_arrow >> npoints;
 	if (forward_arrow == 1)
-		fArrowData = readLinefromDataStream(ts);
+		fArrowData = readLineFromDataStream(ts);
 	if (backward_arrow == 1)
-		bArrowData = readLinefromDataStream(ts);
+		bArrowData = readLineFromDataStream(ts);
 	while (!ts.atEnd())
 	{
-		tmp = readLinefromDataStream(ts);
+		tmp = readLineFromDataStream(ts);
 		ScTextStream pts(&tmp, QIODevice::ReadOnly);
 		while (!pts.atEnd())
 		{
@@ -898,7 +960,7 @@ void XfigPlug::processSpline(QDataStream &ts, QString data)
 	pointsRead = 0;
 	while (!ts.atEnd())
 	{
-		tmp = readLinefromDataStream(ts);
+		tmp = readLineFromDataStream(ts);
 		ScTextStream pts(&tmp, QIODevice::ReadOnly);
 		while (!pts.atEnd())
 		{
@@ -914,10 +976,10 @@ void XfigPlug::processSpline(QDataStream &ts, QString data)
 	int z = -1;
 	PageItem *ite;
 	if ((subtype == 0) || (subtype == 2) || (subtype == 4))
-		z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColorFill, CurrColorStroke, true);
+		z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColorFill, CurrColorStroke);
 	else if ((subtype == 1) || (subtype == 3) || (subtype == 5))
 	{
-		z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColorFill, CurrColorStroke, true);
+		z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColorFill, CurrColorStroke);
 		Coords.svgClosePath();
 	}
 	if (z >= 0)
@@ -943,7 +1005,7 @@ void XfigPlug::processSpline(QDataStream &ts, QString data)
 		FPoint wh = getMaxClipF(&ite->PoLine);
 		ite->setWidthHeight(wh.x(),wh.y());
 		ite->setTextFlowMode(PageItem::TextFlowDisabled);
-		m_Doc->AdjustItemSize(ite);
+		m_Doc->adjustItemSize(ite);
 		ite->setWidthHeight(qMax(ite->width(), 1.0), qMax(ite->height(), 1.0));
 		depthMap.insert(999 - depth, currentItemNr);
 		currentItemNr++;
@@ -952,7 +1014,7 @@ void XfigPlug::processSpline(QDataStream &ts, QString data)
 	}
 }
 
-void XfigPlug::processArc(QDataStream &ts, QString data)
+void XfigPlug::processArc(QDataStream &ts, const QString& data)
 {
 	QString tmp = data;
 	QString fArrowData = "";
@@ -980,18 +1042,18 @@ void XfigPlug::processArc(QDataStream &ts, QString data)
 	Code >> area_fill >> style_val >> cap_style >> direction >> forward_arrow >> backward_arrow;
 	Code >> center_x >> center_y >> x1 >> y1 >> x2 >> y2 >> x3 >> y3;
 	if (forward_arrow == 1)
-		fArrowData = readLinefromDataStream(ts);
+		fArrowData = readLineFromDataStream(ts);
 	if (backward_arrow == 1)
-		bArrowData = readLinefromDataStream(ts);
+		bArrowData = readLineFromDataStream(ts);
 	useColor(pen_color, 0, false);
 	useColor(fill_color, area_fill, true);
 	LineW = thickness / 80.0 * 72.0;
 	int z = -1;
 	PageItem *ite;
 	if (subtype == 1)
-		z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColorFill, CurrColorStroke, true);
+		z = m_Doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColorFill, CurrColorStroke);
 	else if (subtype == 2)
-		z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColorFill, CurrColorStroke, true);
+		z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, baseX, baseY, 10, 10, LineW, CurrColorFill, CurrColorStroke);
 	else
 		z = -1;
 	center_x = fig2Pts(center_x) - docX;
@@ -1060,7 +1122,7 @@ void XfigPlug::processArc(QDataStream &ts, QString data)
 		FPoint wh = getMaxClipF(&ite->PoLine);
 		ite->setWidthHeight(wh.x(),wh.y());
 		ite->setTextFlowMode(PageItem::TextFlowDisabled);
-		m_Doc->AdjustItemSize(ite);
+		m_Doc->adjustItemSize(ite);
 		ite->setWidthHeight(qMax(ite->width(), 1.0), qMax(ite->height(), 1.0));
 		depthMap.insert(999 - depth, currentItemNr);
 		currentItemNr++;
@@ -1074,7 +1136,7 @@ void XfigPlug::processArc(QDataStream &ts, QString data)
 	}
 }
 
-void XfigPlug::processEllipse(QString data)
+void XfigPlug::processEllipse(const QString& data)
 {
 	QString tmp = data;
 	int		command;			// (always 1)
@@ -1116,7 +1178,7 @@ void XfigPlug::processEllipse(QString data)
 	y += m_Doc->currentPage()->yOffset();
 	int z = -1;
 	PageItem *ite;
-	z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Ellipse, baseX + x, baseY + y, w, h, LineW, CurrColorFill, CurrColorStroke, true);
+	z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Ellipse, baseX + x, baseY + y, w, h, LineW, CurrColorFill, CurrColorStroke);
 	if (z >= 0)
 	{
 		ite = m_Doc->Items->at(z);
@@ -1125,19 +1187,19 @@ void XfigPlug::processEllipse(QString data)
 		if (line_style > 0)
 			ite->setDashes(getDashValues(LineW, line_style));
 		ite->setTextFlowMode(PageItem::TextFlowDisabled);
-		int rot = m_Doc->RotMode;
-		m_Doc->RotMode = 2;
-		m_Doc->RotateItem(-angle * 180.0 / M_PI, ite);
-		m_Doc->RotMode = rot;
+		int rot = m_Doc->rotationMode();
+		m_Doc->setRotationMode ( 2);
+		m_Doc->rotateItem(-angle * 180.0 / M_PI, ite);
+		m_Doc->setRotationMode( rot);
 		depthMap.insert(999 - depth, currentItemNr);
 		currentItemNr++;
 	}
 }
 
-QString XfigPlug::cleanText(QString text)
+QString XfigPlug::cleanText(const QString& text)
 {
-	QString ret = "";
-	QString tmp = "";
+	QString ret;
+	QString tmp;
 	bool sep = false;
 	int sepcount = 0;
 	for (int a = 1; a < text.count(); ++a)
@@ -1182,7 +1244,7 @@ QString XfigPlug::cleanText(QString text)
 	return ret;
 }
 
-void XfigPlug::processText(QString data)
+void XfigPlug::processText(const QString& data)
 {
 	QString tmp = data;
 	int		command;			// (always 4)
@@ -1213,7 +1275,7 @@ void XfigPlug::processText(QString data)
 	text = cleanText(text);
 	FPointArray textPath;
 	QPainterPath painterPath;
-	QString TFont = m_Doc->toolSettings.defFont;
+	QString TFont(m_Doc->itemToolPrefs().textFont);
 	QFont::Weight weight = QFont::Normal;
 	bool isItalic = false;
 	if (font_flags & 4)
@@ -1370,7 +1432,7 @@ void XfigPlug::processText(QString data)
 				weight = QFont::Normal;
 				break;
 			default:
-				TFont = m_Doc->toolSettings.defFont;
+				TFont = m_Doc->itemToolPrefs().textFont;
 				weight = QFont::Normal;
 				break;
 		}
@@ -1379,7 +1441,7 @@ void XfigPlug::processText(QString data)
 	{
 		switch (font)
 		{
-    		case 1: // Roman
+			case 1: // Roman
 				TFont = "Times";
 				weight = QFont::Normal;
 				break;
@@ -1401,7 +1463,7 @@ void XfigPlug::processText(QString data)
 				weight = QFont::Normal;
 				break;
 			default:
-				TFont = m_Doc->toolSettings.defFont;
+				TFont = m_Doc->itemToolPrefs().textFont;
 				weight = QFont::Normal;
 				break;
 		}
@@ -1420,7 +1482,7 @@ void XfigPlug::processText(QString data)
 	QRectF br = painterPath.boundingRect();
 	if ((angle == 0) && (br.width() > w))
 	{
-		QMatrix m;
+		QTransform m;
 		m.scale(w / br.width(), w / br.width());
 		painterPath = m.map(painterPath);
 	}
@@ -1428,7 +1490,7 @@ void XfigPlug::processText(QString data)
 	useColor(color, 0, false);
 	int z = -1;
 	PageItem *ite;
-	z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, baseX + x, baseY + y, w, h, 0, CurrColorStroke, CommonStrings::None, true);
+	z = m_Doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, baseX + x, baseY + y, w, h, 0, CurrColorStroke, CommonStrings::None);
 	if (z >= 0)
 	{
 		ite = m_Doc->Items->at(z);
@@ -1439,18 +1501,18 @@ void XfigPlug::processText(QString data)
 		FPoint wh = getMaxClipF(&ite->PoLine);
 		ite->setWidthHeight(wh.x(),wh.y());
 		ite->setTextFlowMode(PageItem::TextFlowDisabled);
-		m_Doc->AdjustItemSize(ite);
+		m_Doc->adjustItemSize(ite);
 		ite->setWidthHeight(qMax(ite->width(), 1.0), qMax(ite->height(), 1.0));
 		if (subtype == 1)
-			m_Doc->MoveRotated(ite, FPoint(-br.width() / 2.0, 0.0));
+			m_Doc->moveRotated(ite, FPoint(-br.width() / 2.0, 0.0));
 		else if (subtype == 2)
-			m_Doc->MoveRotated(ite, FPoint(-br.width(), 0.0));
+			m_Doc->moveRotated(ite, FPoint(-br.width(), 0.0));
 		depthMap.insert(999 - depth, currentItemNr);
 		currentItemNr++;
 	}
 }
 
-void XfigPlug::processData(QDataStream &ts, QString data)
+void XfigPlug::processData(QDataStream &ts, const QString& data)
 {
 	QString tmp = data;
 	int command, subtype;
@@ -1508,9 +1570,8 @@ void XfigPlug::resortItems()
 			PageItem* ite = itemList.at(elems.at(i));
 			Elements.append(ite);
 			m_Doc->Items->append(ite);
-			ite->ItemNr = m_Doc->Items->count()-1;
 			if ((importerFlags & LoadSavePlugin::lfCreateDoc) && (it > 0))
-				ite->LayerNr = currentLayer;
+				ite->m_layerID = currentLayer;
 		}
 	}
 }
@@ -1521,7 +1582,7 @@ double XfigPlug::fig2Pts(double in)
 	return in / 1200.0 * 72.0;
 }
 
-bool XfigPlug::convert(QString fn)
+bool XfigPlug::convert(const QString& fn)
 {
 	QString tmp;
 	CurrColorFill = "White";
@@ -1546,7 +1607,7 @@ bool XfigPlug::convert(QString fn)
 	QList<PageItem*> gElements;
 	groupStack.push(gElements);
 	currentItemNr = 0;
-	if(progressDialog)
+	if (progressDialog)
 	{
 		progressDialog->setOverallProgress(2);
 		progressDialog->setLabel("GI", tr("Generating Items"));
@@ -1563,26 +1624,29 @@ bool XfigPlug::convert(QString fn)
 			qApp->processEvents();
 		}
 		QDataStream ts(&f);
-		QString version = readLinefromDataStream(ts);
-		QString orientation = readLinefromDataStream(ts);
-		QString justification = readLinefromDataStream(ts);
-		QString units = readLinefromDataStream(ts);
-		QString papersize = readLinefromDataStream(ts);
-		QString magnification = readLinefromDataStream(ts);
-		QString multiple = readLinefromDataStream(ts);
-		QString transparent = readLinefromDataStream(ts);
-		tmp = readLinefromDataStream(ts);
+		QString version = readLineFromDataStream(ts);
+		QString orientation = readLineFromDataStream(ts);
+		QString justification = readLineFromDataStream(ts);
+		QString units = readLineFromDataStream(ts);
+		QString papersize = readLineFromDataStream(ts);
+		QString magnification = readLineFromDataStream(ts);
+		QString multiple = readLineFromDataStream(ts);
+		QString transparent = readLineFromDataStream(ts);
+		tmp = readLineFromDataStream(ts);
 		if (tmp.startsWith("#"))
-			tmp = readLinefromDataStream(ts);
+			tmp = readLineFromDataStream(ts);
 		QString resolution = tmp;
 		while (!ts.atEnd())
 		{
-			tmp = readLinefromDataStream(ts);
+			tmp = readLineFromDataStream(ts);
 			if (tmp.startsWith("#"))
 				continue;
 			processData(ts, tmp);
-			progressDialog->setProgress("GI", ts.device()->pos());
-			qApp->processEvents();
+			if (progressDialog)
+			{
+				progressDialog->setProgress("GI", ts.device()->pos());
+				qApp->processEvents();
+			}
 		}
 		resortItems();
 	}
