@@ -6,30 +6,33 @@ for which a new license (GPL+exception) is in place.
 */
 #include "cmdmani.h"
 #include "cmdutil.h"
-#include "propertiespalette.h" //CB argh.. noooooooooooooooooooooooooooooooooooo FIXME see other FIXME
-#include "selection.h"
 #include "scribuscore.h"
+#include "scribusdoc.h"
+#include "scribusview.h"
+#include "sctextstream.h"
+#include "selection.h"
+#include "ui/propertiespalette.h" //CB argh.. noooooooooooooooooooooooooooooooooooo FIXME see other FIXME
+#include "ui/propertiespalette_image.h"
 #include "undomanager.h"
 
+
 PyObject *scribus_loadimage(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	char *Image;
 	if (!PyArg_ParseTuple(args, "es|es", "utf-8", &Image, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *item = GetUniqueItem(QString::fromUtf8(Name));
-	if (item == NULL)
-		return NULL;
+	if (item == nullptr)
+		return nullptr;
 	if (!item->asImageFrame())
 	{
 		PyErr_SetString(WrongFrameTypeError, QObject::tr("Target is not an image frame.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
-	ScCore->primaryMainWindow()->doc->LoadPict(QString::fromUtf8(Image), item->ItemNr);
-//	Py_INCREF(Py_None);
-//	return Py_None;
+	ScCore->primaryMainWindow()->doc->loadPict(QString::fromUtf8(Image), item);
 	Py_RETURN_NONE;
 }
 
@@ -38,21 +41,40 @@ PyObject *scribus_scaleimage(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	double x, y;
 	if (!PyArg_ParseTuple(args, "dd|es", &x, &y, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *item = GetUniqueItem(QString::fromUtf8(Name));
-	if (item == NULL)
-		return NULL;
+	if (item == nullptr)
+		return nullptr;
 	if (! item->asImageFrame())
 	{
 		PyErr_SetString(ScribusException, QObject::tr("Specified item not an image frame.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
-	ScCore->primaryMainWindow()->doc->itemSelection_SetImageScale(x, y); //CB why when this is done above?
-	ScCore->primaryMainWindow()->doc->updatePic();
-//	Py_INCREF(Py_None);
-//	return Py_None;
+
+	// Grab the old selection - but use it only where is there any
+	ScribusDoc* currentDoc = ScCore->primaryMainWindow()->doc;
+	ScribusView* currentView = ScCore->primaryMainWindow()->view;
+	Selection tempSelection(*currentDoc->m_Selection);
+	bool hadOrigSelection = (tempSelection.count() != 0);
+
+	currentDoc->m_Selection->clear();
+	// Clear the selection
+	currentView->Deselect();
+	// Select the item, which will also select its group if
+	// there is one.
+	currentView->SelectItem(item);
+
+	// scale
+	currentDoc->itemSelection_SetImageScale(x, y); //CB why when this is done above?
+	currentDoc->updatePic();
+
+	// Now restore the selection.
+	currentView->Deselect();
+	if (hadOrigSelection)
+		*currentDoc->m_Selection=tempSelection;
+
 	Py_RETURN_NONE;
 }
 
@@ -61,106 +83,181 @@ PyObject *scribus_setimagescale(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	double x, y;
 	if (!PyArg_ParseTuple(args, "dd|es", &x, &y, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *item = GetUniqueItem(QString::fromUtf8(Name));
-	if (item == NULL)
-		return NULL;
-	if (! item->asImageFrame())
+	if (item == nullptr)
+		return nullptr;
+	if (!item->asImageFrame())
 	{
 		PyErr_SetString(ScribusException, QObject::tr("Specified item not an image frame.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
+
+	// Grab the old selection - but use it only where is there any
+	ScribusDoc* currentDoc = ScCore->primaryMainWindow()->doc;
+	ScribusView* currentView = ScCore->primaryMainWindow()->view;
+	Selection tempSelection(*currentDoc->m_Selection);
+	bool hadOrigSelection = (tempSelection.count() != 0);
+
+	currentDoc->m_Selection->clear();
+	// Clear the selection
+	currentView->Deselect();
+	// Select the item, which will also select its group if
+	// there is one.
+	currentView->SelectItem(item);
+
+	// scale
 	double newScaleX = x / item->pixm.imgInfo.xres * 72.0;
 	double newScaleY = y / item->pixm.imgInfo.yres * 72.0;
-	ScCore->primaryMainWindow()->doc->itemSelection_SetImageScale(newScaleX, newScaleY); //CB why when this is done above?
-	ScCore->primaryMainWindow()->doc->updatePic();
-//	Py_INCREF(Py_None);
-//	return Py_None;
+	currentDoc->itemSelection_SetImageScale(newScaleX, newScaleY); //CB why when this is done above?
+	currentDoc->updatePic();
+
+	// Now restore the selection.
+	currentView->Deselect();
+	if (hadOrigSelection)
+		*currentDoc->m_Selection=tempSelection;
+
 	Py_RETURN_NONE;
 }
-
 PyObject *scribus_setimageoffset(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	double x, y;
 	if (!PyArg_ParseTuple(args, "dd|es", &x, &y, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *item = GetUniqueItem(QString::fromUtf8(Name));
-	if (item == NULL)
-		return NULL;
-	if (! item->asImageFrame())
+	if (item == nullptr)
+		return nullptr;
+	if (!item->asImageFrame())
 	{
 		PyErr_SetString(ScribusException, QObject::tr("Specified item not an image frame.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 
 	// Grab the old selection - but use it only where is there any
+	ScribusDoc* currentDoc = ScCore->primaryMainWindow()->doc;
+	ScribusView* currentView = ScCore->primaryMainWindow()->view;
 	Selection tempSelection(*ScCore->primaryMainWindow()->doc->m_Selection);
 	bool hadOrigSelection = (tempSelection.count() != 0);
 
-	ScCore->primaryMainWindow()->doc->m_Selection->clear();
+	currentDoc->m_Selection->clear();
 	// Clear the selection
-	ScCore->primaryMainWindow()->view->Deselect();
+	currentView->Deselect();
 	// Select the item, which will also select its group if
 	// there is one.
-	ScCore->primaryMainWindow()->view->SelectItemNr(item->ItemNr);
+	currentView->SelectItem(item);
 
 	// offset
 	double newOffsetX = x / ((item->imageXScale() != 0.0) ? item->imageXScale() : 1);
 	double newOffsetY = y / ((item->imageYScale() != 0.0) ? item->imageYScale() : 1);
-	ScCore->primaryMainWindow()->doc->itemSelection_SetImageOffset(newOffsetX, newOffsetY); //CB why when this is done above?
-	ScCore->primaryMainWindow()->doc->updatePic();
+	currentDoc->itemSelection_SetImageOffset(newOffsetX, newOffsetY); //CB why when this is done above?
+	currentDoc->updatePic();
 
 	// Now restore the selection.
-	ScCore->primaryMainWindow()->view->Deselect();
+	currentView->Deselect();
 	if (hadOrigSelection)
-		*ScCore->primaryMainWindow()->doc->m_Selection=tempSelection;
+		*currentDoc->m_Selection=tempSelection;
 
-	//      Py_INCREF(Py_None);
-	//      return Py_None;
 	Py_RETURN_NONE;
 }
 
+PyObject *scribus_setimagebrightness(PyObject* /* self */, PyObject* args)
+{
+	char *Name = const_cast<char*>("");
+	double n;
+	if (!PyArg_ParseTuple(args, "d|es", &n, "utf-8", &Name))
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	PageItem *item = GetUniqueItem(QString::fromUtf8(Name));
+	if (item == nullptr)
+		return nullptr;
+	if (! item->asImageFrame())
+	{
+		PyErr_SetString(ScribusException, QObject::tr("Specified item not an image frame.","python error").toLocal8Bit().constData());
+		return nullptr;
+	}
+
+	ImageEffect ef;
+	ef.effectCode = ScImage::EF_BRIGHTNESS;
+	ScTextStream fp(&ef.effectParameters, QIODevice::WriteOnly);
+	fp << n;
+
+	item->effectsInUse.append(ef);
+	item->pixm.applyEffect(item->effectsInUse, ScCore->primaryMainWindow()->doc->PageColors, false);
+	
+	ScCore->primaryMainWindow()->doc->updatePic();
+	Py_RETURN_NONE;
+}
+
+PyObject *scribus_setimagegrayscale(PyObject* /* self */, PyObject* args)
+{
+	char *Name = const_cast<char*>("");
+	if (!PyArg_ParseTuple(args, "|es", "utf-8", &Name))
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	PageItem *item = GetUniqueItem(QString::fromUtf8(Name));
+	if (item == nullptr)
+		return nullptr;
+	if (! item->asImageFrame())
+	{
+		PyErr_SetString(ScribusException, QObject::tr("Specified item not an image frame.","python error").toLocal8Bit().constData());
+		return nullptr;
+	}
+
+	ImageEffect ef;
+	ef.effectCode = ScImage::EF_GRAYSCALE;
+
+	item->effectsInUse.append(ef);
+	item->pixm.applyEffect(item->effectsInUse, ScCore->primaryMainWindow()->doc->PageColors, false);
+	
+	ScCore->primaryMainWindow()->doc->updatePic();
+	Py_RETURN_NONE;
+}
 
 PyObject *scribus_moveobjrel(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	double x, y;
 	if (!PyArg_ParseTuple(args, "dd|es", &x, &y, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *item = GetUniqueItem(QString::fromUtf8(Name));
-	if (item==NULL)
-		return NULL;
+	if (item==nullptr)
+		return nullptr;
+
 	// Grab the old selection - but use it only where is there any
-	Selection tempSelection(*ScCore->primaryMainWindow()->doc->m_Selection);
+	ScribusDoc* currentDoc = ScCore->primaryMainWindow()->doc;
+	ScribusView* currentView = ScCore->primaryMainWindow()->view;
+	Selection tempSelection(*currentDoc->m_Selection);
 	bool hadOrigSelection = (tempSelection.count() != 0);
 
-	ScCore->primaryMainWindow()->doc->m_Selection->clear();
+	currentDoc->m_Selection->clear();
 	// Clear the selection
-	ScCore->primaryMainWindow()->view->Deselect();
+	currentView->Deselect();
 	// Select the item, which will also select its group if
 	// there is one.
-	ScCore->primaryMainWindow()->view->SelectItemNr(item->ItemNr);
+	currentView->SelectItem(item);
 	// Move the item, or items
-	if (ScCore->primaryMainWindow()->doc->m_Selection->count() > 1)
+	if (currentDoc->m_Selection->count() > 1)
 	{
-		ScCore->primaryMainWindow()->view->startGroupTransaction(Um::Move, "", Um::IMove);
-		ScCore->primaryMainWindow()->doc->moveGroup(ValueToPoint(x), ValueToPoint(y));
-		ScCore->primaryMainWindow()->view->endGroupTransaction();
+		currentView->startGroupTransaction(Um::Move, "", Um::IMove);
+		currentDoc->moveGroup(ValueToPoint(x), ValueToPoint(y));
+		currentView->endGroupTransaction();
 	}
-	else
-		ScCore->primaryMainWindow()->doc->MoveItem(ValueToPoint(x), ValueToPoint(y), item);
+	else {
+		currentDoc->moveItem(ValueToPoint(x), ValueToPoint(y), item);
+		}
 	// Now restore the selection.
-	ScCore->primaryMainWindow()->view->Deselect();
+	currentView->Deselect();
 	if (hadOrigSelection)
-		*ScCore->primaryMainWindow()->doc->m_Selection=tempSelection;
-
+		*currentDoc->m_Selection=tempSelection;
 	Py_RETURN_NONE;
 }
 
@@ -169,36 +266,39 @@ PyObject *scribus_moveobjabs(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	double x, y;
 	if (!PyArg_ParseTuple(args, "dd|es", &x, &y, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *item = GetUniqueItem(QString::fromUtf8(Name));
-	if (item == NULL)
-		return NULL;
+	if (item == nullptr)
+		return nullptr;
+
 	// Grab the old selection - but use it only where is there any
-	Selection tempSelection(*ScCore->primaryMainWindow()->doc->m_Selection);
+	ScribusDoc* currentDoc = ScCore->primaryMainWindow()->doc;
+	ScribusView* currentView = ScCore->primaryMainWindow()->view;
+	Selection tempSelection(*currentDoc->m_Selection);
 	bool hadOrigSelection = (tempSelection.count() != 0);
 
 	// Clear the selection
-	ScCore->primaryMainWindow()->view->Deselect();
+	currentView->Deselect();
 	// Select the item, which will also select its group if
 	// there is one.
-	ScCore->primaryMainWindow()->view->SelectItemNr(item->ItemNr);
+	currentView->SelectItem(item);
 	// Move the item, or items
-	if (ScCore->primaryMainWindow()->doc->m_Selection->count() > 1)
+	if (currentDoc->m_Selection->count() > 1)
 	{
-		ScCore->primaryMainWindow()->view->startGroupTransaction(Um::Move, "", Um::IMove);
+		currentView->startGroupTransaction(Um::Move, "", Um::IMove);
 		double x2, y2, w, h;
-		ScCore->primaryMainWindow()->doc->m_Selection->getGroupRect(&x2, &y2, &w, &h);
-		ScCore->primaryMainWindow()->doc->moveGroup(pageUnitXToDocX(x) - x2, pageUnitYToDocY(y) - y2);
-		ScCore->primaryMainWindow()->view->endGroupTransaction();
+		currentDoc->m_Selection->getGroupRect(&x2, &y2, &w, &h);
+		currentDoc->moveGroup(pageUnitXToDocX(x) - x2, pageUnitYToDocY(y) - y2);
+		currentView->endGroupTransaction();
 	}
 	else
-		ScCore->primaryMainWindow()->doc->MoveItem(pageUnitXToDocX(x) - item->xPos(), pageUnitYToDocY(y) - item->yPos(), item);
+		currentDoc->moveItem(pageUnitXToDocX(x) - item->xPos(), pageUnitYToDocY(y) - item->yPos(), item);
 	// Now restore the selection.
-	ScCore->primaryMainWindow()->view->Deselect();
+	currentView->Deselect();
 	if (hadOrigSelection)
-		*ScCore->primaryMainWindow()->doc->m_Selection=tempSelection;
+		*currentDoc->m_Selection=tempSelection;
 
 	Py_RETURN_NONE;
 }
@@ -208,15 +308,13 @@ PyObject *scribus_rotobjrel(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	double x;
 	if (!PyArg_ParseTuple(args, "d|es", &x, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *item = GetUniqueItem(QString::fromUtf8(Name));
-	if (item == NULL)
-		return NULL;
-	ScCore->primaryMainWindow()->doc->RotateItem(item->rotation() - x, item->ItemNr);
-//	Py_INCREF(Py_None);
-//	return Py_None;
+	if (item == nullptr)
+		return nullptr;
+	ScCore->primaryMainWindow()->doc->rotateItem(item->rotation() - x, item);
 	Py_RETURN_NONE;
 }
 
@@ -225,15 +323,13 @@ PyObject *scribus_rotobjabs(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	double x;
 	if (!PyArg_ParseTuple(args, "d|es", &x, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *item = GetUniqueItem(QString::fromUtf8(Name));
-	if (item == NULL)
-		return NULL;
-	ScCore->primaryMainWindow()->doc->RotateItem(x * -1.0, item->ItemNr);
-//	Py_INCREF(Py_None);
-//	return Py_None;
+	if (item == nullptr)
+		return nullptr;
+	ScCore->primaryMainWindow()->doc->rotateItem(x * -1.0, item);
 	Py_RETURN_NONE;
 }
 
@@ -242,36 +338,34 @@ PyObject *scribus_sizeobjabs(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	double x, y;
 	if (!PyArg_ParseTuple(args, "dd|es", &x, &y, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *item = GetUniqueItem(QString::fromUtf8(Name));
-	if (item == NULL)
-		return NULL;
-	ScCore->primaryMainWindow()->doc->SizeItem(ValueToPoint(x), ValueToPoint(y), item->ItemNr);
-//	Py_INCREF(Py_None);
-//	return Py_None;
+	if (item == nullptr)
+		return nullptr;
+	ScCore->primaryMainWindow()->doc->sizeItem(ValueToPoint(x), ValueToPoint(y), item);
 	Py_RETURN_NONE;
 }
 
 PyObject *scribus_groupobj(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
-	PyObject *il = 0;
+	PyObject *il = nullptr;
 	if (!PyArg_ParseTuple(args, "|O", &il))
-		return NULL;
+		return nullptr;
 	if (!checkHaveDocument())
-		return NULL;
-	if (il == 0 && ScCore->primaryMainWindow()->doc->m_Selection->count() < 2)
+		return nullptr;
+	if (il == nullptr && ScCore->primaryMainWindow()->doc->m_Selection->count() < 2)
 	{
 		PyErr_SetString(PyExc_TypeError, QObject::tr("Need selection or argument list of items to group", "python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
-	Selection *tempSelection=0;
-	Selection *finalSelection=0;
+	Selection *tempSelection=nullptr;
+	Selection *finalSelection=nullptr;
 	//uint ap = ScCore->primaryMainWindow()->doc->currentPage()->pageNr();
 	// If we were passed a list of items to group...
-	if (il != 0)
+	if (il != nullptr)
 	{
 		int len = PyList_Size(il);
 		tempSelection = new Selection(ScCore->primaryMainWindow(), false);
@@ -282,10 +376,10 @@ PyObject *scribus_groupobj(PyObject* /* self */, PyObject* args)
 			// so anyway.
 			Name = PyString_AsString(PyList_GetItem(il, i));
 			PageItem *ic = GetUniqueItem(QString::fromUtf8(Name));
-			if (ic == NULL)
+			if (ic == nullptr)
 			{
 				delete tempSelection;
-				return NULL;
+				return nullptr;
 			}
 			tempSelection->addItem (ic, true);
 		}
@@ -297,31 +391,34 @@ PyObject *scribus_groupobj(PyObject* /* self */, PyObject* args)
 	{
 		// We can't very well group only one item
 		PyErr_SetString(NoValidObjectError, QObject::tr("Cannot group less than two items", "python error").toLocal8Bit().constData());
-		finalSelection=0;
+		finalSelection=nullptr;
 		delete tempSelection;
-		return NULL;
+		return nullptr;
 	}
-	ScCore->primaryMainWindow()->doc->itemSelection_GroupObjects(false, false, finalSelection);
-	finalSelection=0;
+
+	const PageItem* group = ScCore->primaryMainWindow()->doc->itemSelection_GroupObjects(false, false, finalSelection);
+	finalSelection=nullptr;
 	delete tempSelection;
-	Py_RETURN_NONE;
+	
+	return (group ? PyString_FromString(group->itemName().toUtf8()) : nullptr);
 }
 
 PyObject *scribus_ungroupobj(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "|es", "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *i = GetUniqueItem(QString::fromUtf8(Name));
-	if (i == NULL)
-		return NULL;
-	ScCore->primaryMainWindow()->view->Deselect();
-	ScCore->primaryMainWindow()->view->SelectItemNr(i->ItemNr);
-	ScCore->primaryMainWindow()->UnGroupObj();
-//	Py_INCREF(Py_None);
-//	return Py_None;
+	if (i == nullptr)
+		return nullptr;
+
+	ScribusMainWindow* currentWin = ScCore->primaryMainWindow();
+	currentWin->view->Deselect();
+	currentWin->view->SelectItem(i);
+	currentWin->UnGroupObj();
+
 	Py_RETURN_NONE;
 }
 
@@ -330,27 +427,29 @@ PyObject *scribus_scalegroup(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	double sc;
 	if (!PyArg_ParseTuple(args, "d|es", &sc, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	if (sc == 0.0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot scale by 0%.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	PageItem *i = GetUniqueItem(QString::fromUtf8(Name));
-	if (i == NULL)
-		return NULL;
-	ScCore->primaryMainWindow()->view->Deselect();
-	ScCore->primaryMainWindow()->view->SelectItemNr(i->ItemNr);
-//	int h = ScCore->primaryMainWindow()->view->frameResizeHandle;
-//	ScCore->primaryMainWindow()->view->frameResizeHandle = 1;
-	ScCore->primaryMainWindow()->view->startGroupTransaction(Um::Resize, "", Um::IResize);
-	ScCore->primaryMainWindow()->doc->scaleGroup(sc, sc);
-	ScCore->primaryMainWindow()->view->endGroupTransaction();
-//	ScCore->primaryMainWindow()->view->frameResizeHandle = h;
-//	Py_INCREF(Py_None);
-//	return Py_None;
+	if (i == nullptr)
+		return nullptr;
+
+	ScribusDoc* currentDoc = ScCore->primaryMainWindow()->doc;
+	ScribusView* currentView = ScCore->primaryMainWindow()->view;
+
+	currentView->Deselect();
+	currentView->SelectItem(i);
+//	int h = currentView->frameResizeHandle;
+//	currentView->frameResizeHandle = 1;
+	currentView->startGroupTransaction(Um::Resize, "", Um::IResize);
+	currentDoc->scaleGroup(sc, sc);
+	currentView->endGroupTransaction();
+//	currentView->frameResizeHandle = h;
 	Py_RETURN_NONE;
 }
 
@@ -358,20 +457,19 @@ PyObject *scribus_getselobjnam(PyObject* /* self */, PyObject* args)
 {
 	int i = 0;
 	if (!PyArg_ParseTuple(args, "|i", &i))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	if ((i < static_cast<int>(ScCore->primaryMainWindow()->doc->m_Selection->count())) && (i > -1))
 		return PyString_FromString(ScCore->primaryMainWindow()->doc->m_Selection->itemAt(i)->itemName().toUtf8());
-	else
-		// FIXME: Should probably return None if no selection?
-		return PyString_FromString("");
+	// FIXME: Should probably return None if no selection?
+	return PyString_FromString("");
 }
 
 PyObject *scribus_selcount(PyObject* /* self */)
 {
-	if(!checkHaveDocument())
-		return NULL;
+	if (!checkHaveDocument())
+		return nullptr;
 	return PyInt_FromLong(static_cast<long>(ScCore->primaryMainWindow()->doc->m_Selection->count()));
 }
 
@@ -379,25 +477,21 @@ PyObject *scribus_selectobj(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "es", "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *i = GetUniqueItem(QString::fromUtf8(Name));
-	if (i == NULL)
-		return NULL;
-	ScCore->primaryMainWindow()->view->SelectItemNr(i->ItemNr);
-//	Py_INCREF(Py_None);
-//	return Py_None;
+	if (i == nullptr)
+		return nullptr;
+	ScCore->primaryMainWindow()->view->SelectItem(i);
 	Py_RETURN_NONE;
 }
 
 PyObject *scribus_deselect(PyObject* /* self */)
 {
-	if(!checkHaveDocument())
-		return NULL;
+	if (!checkHaveDocument())
+		return nullptr;
 	ScCore->primaryMainWindow()->view->Deselect();
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
@@ -405,12 +499,12 @@ PyObject *scribus_lockobject(PyObject* /* self */, PyObject* args)
 {
 	char *name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "|es", "utf-8", &name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *item = GetUniqueItem(QString::fromUtf8(name));
-	if (item == NULL)
-		return NULL;
+	if (item == nullptr)
+		return nullptr;
 	item->toggleLock();
 	if (item->locked())
 		return PyInt_FromLong(1);
@@ -421,37 +515,60 @@ PyObject *scribus_islocked(PyObject* /* self */, PyObject* args)
 {
 	char *name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "|es", "utf-8", &name))
-		return NULL;
+		return nullptr;
 	// FIXME: Rather than toggling the lock, we should probably let the user set the lock state
 	// and instead provide a different function like toggleLock()
-	if(!checkHaveDocument())
-		return NULL;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *item = GetUniqueItem(QString::fromUtf8(name));
-	if (item == NULL)
-		return NULL;
+	if (item == nullptr)
+		return nullptr;
 	if (item->locked())
 		return PyBool_FromLong(1);
 	return PyBool_FromLong(0);
 }
 
+PyObject *scribus_setscaleframetoimage(PyObject* /* self */, PyObject* args)
+{
+	if (!checkHaveDocument())
+		return nullptr;
+	char *Name = const_cast<char*>("");
+	if (!PyArg_ParseTuple(args, "|es", "utf-8", &Name))
+		return nullptr;
+	PageItem *item = GetUniqueItem(QString::fromUtf8(Name));
+	if (item == nullptr)
+		return nullptr;
+	if (!item->asImageFrame())
+	{
+		PyErr_SetString(WrongFrameTypeError, QObject::tr("Specified item not an image frame.","python error").toLocal8Bit().constData());
+		return nullptr;
+	}
+	Selection *sel = new Selection(ScCore->primaryMainWindow());
+	sel->addItem(item);
+	ScCore->primaryMainWindow()->doc->itemSelection_AdjustFrametoImageSize(sel);
+	delete sel;
+
+	Py_RETURN_NONE;
+}
+
 PyObject *scribus_setscaleimagetoframe(PyObject* /* self */, PyObject* args, PyObject* kw)
 {
 	char *name = const_cast<char*>("");
 	long int scaleToFrame = 0;
 	long int proportional = 1;
 	char* kwargs[] = {const_cast<char*>("scaletoframe"),
-		const_cast<char*>("proportional"), const_cast<char*>("name"), NULL};
+		const_cast<char*>("proportional"), const_cast<char*>("name"), nullptr};
 	if (!PyArg_ParseTupleAndKeywords(args, kw, "i|ies", kwargs, &scaleToFrame, &proportional, "utf-8", &name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *item = GetUniqueItem(QString::fromUtf8(name));
-	if (item == NULL)
-		return NULL;
+	if (item == nullptr)
+		return nullptr;
 	if (! item->asImageFrame())
 	{
 		PyErr_SetString(ScribusException, QObject::tr("Specified item not an image frame.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	// Set the item to scale if appropriate. ScaleType 1 is free
 	// scale, 0 is scale to frame.
@@ -460,16 +577,75 @@ PyObject *scribus_setscaleimagetoframe(PyObject* /* self */, PyObject* args, PyO
 	// set it. 1 is proportional, 0 is free aspect.
 	if (proportional != -1)
 		item->AspectRatio = proportional > 0;
-	// Force the braindead app to notice the changes
-
-	//FIXME emit or something so we dont need this
-	ScCore->primaryMainWindow()->propertiesPalette->setScaleAndOffset(item->imageXScale(), item->imageYScale(), item->imageXOffset(), item->imageYOffset());
-	item->AdjustPictScale();
-	//ScCore->primaryMainWindow()->view->AdjustPictScale(item);
 
+	item->adjustPictScale();
 	item->update();
-//	Py_INCREF(Py_None);
-//	return Py_None;
+
+	Py_RETURN_NONE;
+}
+PyObject *scribus_flipobject(PyObject* /* self */, PyObject* args)
+{
+	char *Name = const_cast<char*>("");
+	double h, v;
+	if (!PyArg_ParseTuple(args, "dd|es", &h, &v, "utf-8", &Name))
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	PageItem *item = GetUniqueItem(QString::fromUtf8(Name));
+	if (item == nullptr)
+		return nullptr;
+	
+	// Grab the old selection - but use it only where is there any
+	ScribusDoc* currentDoc = ScCore->primaryMainWindow()->doc;
+	ScribusView* currentView = ScCore->primaryMainWindow()->view;
+	Selection tempSelection(*currentDoc->m_Selection);
+	bool hadOrigSelection = (tempSelection.count() != 0);
+
+	currentDoc->m_Selection->clear();
+	// Clear the selection
+	currentView->Deselect();
+	// Select the item, which will also select its group if
+	// there is one.
+	currentView->SelectItem(item);
+
+	// flip
+	if (h == 1)
+		currentDoc->itemSelection_FlipH();
+	if (v == 1)
+		currentDoc->itemSelection_FlipV();
+	// Now restore the selection.
+	currentView->Deselect();
+	if (hadOrigSelection)
+		*currentDoc->m_Selection = tempSelection;
+
+	Py_RETURN_NONE;
+}
+
+PyObject *scribus_combinepolygons(PyObject * /* self */)
+{
+	if (!checkHaveDocument())
+		return nullptr;
+
+	ScribusDoc* currentDoc = ScCore->primaryMainWindow()->doc;
+	Selection* curSelection = currentDoc->m_Selection;
+	if (curSelection->count() <= 1)
+		Py_RETURN_NONE;
+
+	bool canUniteItems = true;
+	for (int i = 0; i < curSelection->count(); ++i)
+	{
+		PageItem* it = currentDoc->m_Selection->itemAt(i);
+		if ((!it->asPolygon()) || (!it->asPolyLine()))
+			canUniteItems = false;
+	}
+
+	if (!canUniteItems)
+	{
+		PyErr_SetString(WrongFrameTypeError, QObject::tr("Selection must contain only shapes or bezier curves.", "python error").toLocal8Bit().constData());
+		return nullptr;
+	}
+	currentDoc->itemSelection_UniteItems(nullptr);
+
 	Py_RETURN_NONE;
 }
 
@@ -478,7 +654,7 @@ with header files structure untouched (docstrings are kept near declarations)
 PV */
 void cmdmanidocwarnings()
 {
-    QStringList s;
+	QStringList s;
 	s << scribus_moveobjrel__doc__ << scribus_moveobjabs__doc__
 	  << scribus_rotobjrel__doc__ << scribus_rotobjabs__doc__
 	  << scribus_sizeobjabs__doc__ << scribus_getselobjnam__doc__
@@ -487,6 +663,7 @@ void cmdmanidocwarnings()
 	  << scribus_ungroupobj__doc__ << scribus_scalegroup__doc__
 	  << scribus_loadimage__doc__ << scribus_scaleimage__doc__
 	  << scribus_setimagescale__doc__ << scribus_lockobject__doc__
-	  << scribus_islocked__doc__ << scribus_setscaleimagetoframe__doc__
-	  << scribus_setimageoffset__doc__ ;
+	  << scribus_islocked__doc__ << scribus_setscaleimagetoframe__doc__ << scribus_setimagebrightness__doc__ << scribus_setimagegrayscale__doc__ << scribus_setimageoffset__doc__
+	  << scribus_setscaleframetoimage__doc__
+	  << scribus_flipobject__doc__;
 }
