@@ -15,33 +15,33 @@ for which a new license (GPL+exception) is in place.
 #include "prefsmanager.h"
 #include "scribuscore.h"
 #include "scribusdoc.h"
+#include "scribusview.h"
 #include "selection.h"
 #include "fonts/scfontmetrics.h"
+#include "pdfoptionsio.h"
 
 PyObject *scribus_setredraw(PyObject* /* self */, PyObject* args)
 {
 	int e;
 	if (!PyArg_ParseTuple(args, "i", &e))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	ScCore->primaryMainWindow()->doc->DoDrawing = static_cast<bool>(e);
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
 PyObject *scribus_fontnames(PyObject* /* self */)
 {
 	int cc2 = 0;
-	SCFontsIterator it2(PrefsManager::instance()->appPrefs.AvailFonts);
+	SCFontsIterator it2(PrefsManager::instance().appPrefs.fontPrefs.AvailFonts);
 	for ( ; it2.hasNext() ; it2.next())
 	{
 		if (it2.current().usable())
 			cc2++;
 	}
 	PyObject *l = PyList_New(cc2);
-	SCFontsIterator it(PrefsManager::instance()->appPrefs.AvailFonts);
+	SCFontsIterator it(PrefsManager::instance().appPrefs.fontPrefs.AvailFonts);
 	int cc = 0;
 	for ( ; it.hasNext() ; it.next())
 	{
@@ -56,8 +56,8 @@ PyObject *scribus_fontnames(PyObject* /* self */)
 
 PyObject *scribus_xfontnames(PyObject* /* self */)
 {
-	PyObject *l = PyList_New(PrefsManager::instance()->appPrefs.AvailFonts.count());
-	SCFontsIterator it(PrefsManager::instance()->appPrefs.AvailFonts);
+	PyObject *l = PyList_New(PrefsManager::instance().appPrefs.fontPrefs.AvailFonts.count());
+	SCFontsIterator it(PrefsManager::instance().appPrefs.fontPrefs.AvailFonts);
 	int cc = 0;
 	PyObject *row;
 	for ( ; it.hasNext() ; it.next())
@@ -83,7 +83,7 @@ PyObject *scribus_renderfont(PyObject* /*self*/, PyObject* args, PyObject* kw)
 	char *Name = const_cast<char*>("");
 	char *FileName = const_cast<char*>("");
 	char *Sample = const_cast<char*>("");
-	char *format = NULL;
+	char *format = nullptr;
 	int Size;
 	bool ret = false;
 	char *kwargs[] = {const_cast<char*>("fontname"),
@@ -91,25 +91,25 @@ PyObject *scribus_renderfont(PyObject* /*self*/, PyObject* args, PyObject* kw)
 					  const_cast<char*>("sample"),
 					  const_cast<char*>("size"),
 					  const_cast<char*>("format"),
-					  NULL};
+					  nullptr};
 	if (!PyArg_ParseTupleAndKeywords(args, kw, "esesesi|es", kwargs,
 				"utf-8", &Name, "utf-8", &FileName, "utf-8", &Sample, &Size, "ascii", &format))
-		return NULL;
-	if (!PrefsManager::instance()->appPrefs.AvailFonts.contains(QString::fromUtf8(Name)))
+		return nullptr;
+	if (!PrefsManager::instance().appPrefs.fontPrefs.AvailFonts.contains(QString::fromUtf8(Name)))
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Font not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
-	QString ts = QString::fromUtf8(Sample);
+	QVector<uint> ts = QString::fromUtf8(Sample).toUcs4();
 	if (ts.isEmpty())
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot render an empty sample.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	if (!format)
 		// User specified no format, so use the historical default of PPM format.
 		format =  const_cast<char*>("PPM");
-	QPixmap pm = FontSample(PrefsManager::instance()->appPrefs.AvailFonts[QString::fromUtf8(Name)], Size, ts, Qt::white);
+	QPixmap pm = FontSample(PrefsManager::instance().appPrefs.fontPrefs.AvailFonts[QString::fromUtf8(Name)], Size, ts, Qt::white);
 	// If the user specified an empty filename, return the image data as
 	// a string. Otherwise, save it to disk.
 	if (QString::fromUtf8(FileName).isEmpty())
@@ -121,37 +121,35 @@ PyObject *scribus_renderfont(PyObject* /*self*/, PyObject* args, PyObject* kw)
 		if (!ret)
 		{
 			PyErr_SetString(ScribusException, QObject::tr("Unable to save pixmap","scripter error").toLocal8Bit().constData());
-			return NULL;
+			return nullptr;
 		}
 		int bufferSize = buffer.size();
 		buffer.close();
 		// Now make a Python string from the data we generated
 		PyObject* stringPython = PyString_FromStringAndSize(buffer_string,bufferSize);
-		// Return even if the result is NULL (error) since an exception will have been
+		// Return even if the result is nullptr (error) since an exception will have been
 		// set in that case.
 		return stringPython;
 	}
-	else
+
 	// Save the pixmap to a file, since the filename is non-empty
+	ret = pm.save(QString::fromUtf8(FileName), format);
+	if (!ret)
 	{
-		ret = pm.save(QString::fromUtf8(FileName), format);
-		if (!ret)
-		{
-			PyErr_SetString(PyExc_Exception, QObject::tr("Unable to save pixmap","scripter error").toLocal8Bit().constData());
-			return NULL;
-		}
-		// For historical reasons, we need to return true on success.
-//		Py_INCREF(Py_True);
-//		return Py_True;
-//		Py_RETURN_TRUE;
-		return PyBool_FromLong(static_cast<long>(true));
+		PyErr_SetString(PyExc_Exception, QObject::tr("Unable to save pixmap","scripter error").toLocal8Bit().constData());
+		return nullptr;
 	}
+	// For historical reasons, we need to return true on success.
+	//		Py_INCREF(Py_True);
+	//		return Py_True;
+	//		Py_RETURN_TRUE;
+	return PyBool_FromLong(static_cast<long>(true));
 }
 
 PyObject *scribus_getlayers(PyObject* /* self */)
 {
-	if(!checkHaveDocument())
-		return NULL;
+	if (!checkHaveDocument())
+		return nullptr;
 	PyObject *l;
 	l = PyList_New(ScCore->primaryMainWindow()->doc->Layers.count());
 	for (int lam=0; lam < ScCore->primaryMainWindow()->doc->Layers.count(); lam++)
@@ -163,13 +161,13 @@ PyObject *scribus_setactlayer(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "es", "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == 0)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (Name == nullptr)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot have an empty layer name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	bool found = ScCore->primaryMainWindow()->doc->setActiveLayer(QString::fromUtf8(Name));
 	if (found)
@@ -177,17 +175,15 @@ PyObject *scribus_setactlayer(PyObject* /* self */, PyObject* args)
 	else
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
 PyObject *scribus_getactlayer(PyObject* /* self */)
 {
-	if(!checkHaveDocument())
-		return NULL;
+	if (!checkHaveDocument())
+		return nullptr;
 	return PyString_FromString(ScCore->primaryMainWindow()->doc->activeLayerName().toUtf8());
 }
 
@@ -196,42 +192,40 @@ PyObject *scribus_senttolayer(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	char *Layer = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "es|es", "utf-8", &Layer, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Layer == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Layer) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot have an empty layer name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	PageItem *item = GetUniqueItem(QString::fromUtf8(Name));
-	if (item == NULL)
-		return NULL;
+	if (item == nullptr)
+		return nullptr;
 	ScribusDoc* currentDoc   = ScCore->primaryMainWindow()->doc;
 	ScribusView* currentView = ScCore->primaryMainWindow()->view;
 	const ScLayer *scLayer = currentDoc->Layers.layerByName( QString::fromUtf8(Layer) );
 	if (!scLayer)
 	{
 		PyErr_SetString(ScribusException, QString("Layer not found").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	// If no name have been specified in args, process whole selection
-	currentView->SelectItemNr(item->ItemNr);
-	if ((Name == EMPTY_STRING) || (item->isGroupControl) || (item->Groups.count() > 0))
+	currentView->SelectItem(item);
+	if (strlen(Name) == 0)
 	{
 		for (int i = 0; i < currentDoc->m_Selection->count(); ++i)
 		{
 			item = currentDoc->m_Selection->itemAt(i);
-			item->LayerNr = scLayer->LNr;
+			item->m_layerID = scLayer->ID;
 		}
 	}
 	else
 	{
-		item->LayerNr = scLayer->LNr;
+		item->m_layerID = scLayer->ID;
 	}
 
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
@@ -240,13 +234,13 @@ PyObject *scribus_layervisible(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	int vis = 1;
 	if (!PyArg_ParseTuple(args, "esi", "utf-8", &Name, &vis))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QString("Cannot have an empty layer name").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	bool found = false;
 	for (int lam=0; lam < ScCore->primaryMainWindow()->doc->Layers.count(); ++lam)
@@ -261,10 +255,8 @@ PyObject *scribus_layervisible(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
@@ -273,13 +265,13 @@ PyObject *scribus_layerprint(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	int vis = 1;
 	if (!PyArg_ParseTuple(args, "esi", "utf-8", &Name, &vis))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot have an empty layer name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	bool found = false;
 	for (int lam=0; lam < ScCore->primaryMainWindow()->doc->Layers.count(); ++lam)
@@ -294,10 +286,8 @@ PyObject *scribus_layerprint(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
@@ -306,13 +296,13 @@ PyObject *scribus_layerlock(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	int vis = 1;
 	if (!PyArg_ParseTuple(args, "esi", "utf-8", &Name, &vis))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot have an empty layer name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	bool found = false;
 	for (int lam=0; lam < ScCore->primaryMainWindow()->doc->Layers.count(); ++lam)
@@ -327,10 +317,8 @@ PyObject *scribus_layerlock(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
@@ -339,13 +327,13 @@ PyObject *scribus_layeroutline(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	int vis = 1;
 	if (!PyArg_ParseTuple(args, "esi", "utf-8", &Name, &vis))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot have an empty layer name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	bool found = false;
 	for (int lam=0; lam < ScCore->primaryMainWindow()->doc->Layers.count(); ++lam)
@@ -360,10 +348,8 @@ PyObject *scribus_layeroutline(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
@@ -372,13 +358,13 @@ PyObject *scribus_layerflow(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	int vis = 1;
 	if (!PyArg_ParseTuple(args, "esi", "utf-8", &Name, &vis))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot have an empty layer name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	bool found = false;
 	for (int lam=0; lam < ScCore->primaryMainWindow()->doc->Layers.count(); ++lam)
@@ -393,10 +379,8 @@ PyObject *scribus_layerflow(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
@@ -405,13 +389,13 @@ PyObject *scribus_layerblend(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	int vis = 0;
 	if (!PyArg_ParseTuple(args, "esi", "utf-8", &Name, &vis))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QString("Cannot have an empty layer name").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	bool found = false;
 	for (int lam=0; lam < ScCore->primaryMainWindow()->doc->Layers.count(); ++lam)
@@ -426,10 +410,8 @@ PyObject *scribus_layerblend(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
@@ -438,13 +420,13 @@ PyObject *scribus_layertrans(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 	double vis = 1.0;
 	if (!PyArg_ParseTuple(args, "esd", "utf-8", &Name, &vis))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QString("Cannot have an empty layer name").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	bool found = false;
 	for (int lam=0; lam < ScCore->primaryMainWindow()->doc->Layers.count(); ++lam)
@@ -459,10 +441,8 @@ PyObject *scribus_layertrans(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
@@ -470,13 +450,13 @@ PyObject *scribus_glayervisib(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "es", "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot have an empty layer name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	int i = 0;
 	bool found = false;
@@ -492,7 +472,7 @@ PyObject *scribus_glayervisib(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	return PyInt_FromLong(static_cast<long>(i));
 }
@@ -501,13 +481,13 @@ PyObject *scribus_glayerprint(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "es", "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot have an empty layer name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	int i = 0;
 	bool found = false;
@@ -523,7 +503,7 @@ PyObject *scribus_glayerprint(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	return PyInt_FromLong(static_cast<long>(i));
 }
@@ -532,13 +512,13 @@ PyObject *scribus_glayerlock(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "es", "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot have an empty layer name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	int i = 0;
 	bool found = false;
@@ -554,7 +534,7 @@ PyObject *scribus_glayerlock(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	return PyInt_FromLong(static_cast<long>(i));
 }
@@ -563,13 +543,13 @@ PyObject *scribus_glayeroutline(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "es", "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot have an empty layer name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	int i = 0;
 	bool found = false;
@@ -585,7 +565,7 @@ PyObject *scribus_glayeroutline(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	return PyInt_FromLong(static_cast<long>(i));
 }
@@ -594,13 +574,13 @@ PyObject *scribus_glayerflow(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "es", "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot have an empty layer name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	int i = 0;
 	bool found = false;
@@ -616,7 +596,7 @@ PyObject *scribus_glayerflow(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	return PyInt_FromLong(static_cast<long>(i));
 }
@@ -625,13 +605,13 @@ PyObject *scribus_glayerblend(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "es", "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot have an empty layer name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	int i = 0;
 	bool found = false;
@@ -647,7 +627,7 @@ PyObject *scribus_glayerblend(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	return PyInt_FromLong(static_cast<long>(i));
 }
@@ -656,13 +636,13 @@ PyObject *scribus_glayertrans(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "es", "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot have an empty layer name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	double i = 1.0;
 	bool found = false;
@@ -678,7 +658,7 @@ PyObject *scribus_glayertrans(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	return PyFloat_FromDouble(i);
 }
@@ -688,18 +668,18 @@ PyObject *scribus_removelayer(PyObject* /* self */, PyObject* args)
 //FIXME: Use the docs remove layer code
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "es", "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot have an empty layer name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	if (ScCore->primaryMainWindow()->doc->Layers.count() == 1)
 	{
 		PyErr_SetString(ScribusException, QObject::tr("Cannot remove the last layer.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	bool found = false;
 	for (int lam=0; lam < ScCore->primaryMainWindow()->doc->Layers.count(); ++lam)
@@ -707,7 +687,7 @@ PyObject *scribus_removelayer(PyObject* /* self */, PyObject* args)
 		if (ScCore->primaryMainWindow()->doc->Layers[lam].Name == QString::fromUtf8(Name))
 		{
 			ScLayer it2 = ScCore->primaryMainWindow()->doc->Layers.at(lam);
-			int num2 = it2.LNr;
+			int num2 = it2.ID;
 			if (!num2)
 			{
 				// FIXME: WTF DOES THIS DO?
@@ -715,7 +695,7 @@ PyObject *scribus_removelayer(PyObject* /* self */, PyObject* args)
 				return Py_None;
 			}
 			ScCore->primaryMainWindow()->doc->removeLayer(num2);
-			ScCore->primaryMainWindow()->doc->Layers.removeLayerByNumber(num2);
+			ScCore->primaryMainWindow()->doc->Layers.removeLayerByID(num2);
 			ScCore->primaryMainWindow()->doc->setActiveLayer(0);
 			ScCore->primaryMainWindow()->changeLayer(0);
 			found = true;
@@ -725,10 +705,8 @@ PyObject *scribus_removelayer(PyObject* /* self */, PyObject* args)
 	if (!found)
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Layer not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
@@ -736,18 +714,16 @@ PyObject *scribus_createlayer(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "es", "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
-	if (Name == EMPTY_STRING)
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (strlen(Name) == 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Cannot create layer without a name.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	ScCore->primaryMainWindow()->doc->addLayer(QString::fromUtf8(Name), true);
 	ScCore->primaryMainWindow()->changeLayer(ScCore->primaryMainWindow()->doc->activeLayer());
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
@@ -776,14 +752,48 @@ PyObject *scribus_moveselectiontoback(PyObject*)
 	Py_RETURN_NONE;
 }
 
+PyObject *scribus_savepdfoptions(PyObject* /* self */, PyObject* args)
+{
+	char* file;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (!PyArg_ParseTuple(args, const_cast<char*>("es"), "utf-8", &file))
+		return nullptr;
+
+	PDFOptionsIO io(ScCore->primaryMainWindow()->doc->pdfOptions());
+	if (!io.writeTo(file))
+	{
+		PyErr_SetString(ScribusException, io.lastError().toUtf8());
+		return nullptr;
+	}
+	Py_RETURN_NONE;
+}
+
+PyObject *scribus_readpdfoptions(PyObject* /* self */, PyObject* args)
+{
+	char* file;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (!PyArg_ParseTuple(args, const_cast<char*>("es"), "utf-8", &file))
+		return nullptr;
+
+	PDFOptionsIO io(ScCore->primaryMainWindow()->doc->pdfOptions());
+	if (!io.readFrom(file))
+	{
+		PyErr_SetString(ScribusException, io.lastError().toUtf8());
+		return nullptr;
+	}
+	Py_RETURN_NONE;
+}
+
 /*! HACK: this removes "warning: 'blah' defined but not used" compiler warnings
 with header files structure untouched (docstrings are kept near declarations)
 PV */
 void cmdmiscdocwarnings()
 {
-    QStringList s;
-	s << scribus_setredraw__doc__ <<scribus_fontnames__doc__ 
-	  << scribus_xfontnames__doc__ <<scribus_renderfont__doc__ 
+	QStringList s;
+	s << scribus_setredraw__doc__ << scribus_fontnames__doc__ 
+	  << scribus_xfontnames__doc__ << scribus_renderfont__doc__ 
 	  << scribus_getlayers__doc__ << scribus_setactlayer__doc__ 
 	  << scribus_getactlayer__doc__ << scribus_senttolayer__doc__ 
 	  << scribus_layervisible__doc__ << scribus_layerprint__doc__ 
@@ -795,5 +805,6 @@ void cmdmiscdocwarnings()
 	  << scribus_glayerblend__doc__ << scribus_glayertrans__doc__ 
 	  << scribus_removelayer__doc__ << scribus_createlayer__doc__ 
 	  << scribus_getlanguage__doc__ << scribus_moveselectiontofront__doc__
-	  << scribus_moveselectiontoback__doc__<< scribus_filequit__doc__;
+	  << scribus_moveselectiontoback__doc__ << scribus_filequit__doc__
+	  << scribus_savepdfoptions__doc__ << scribus_readpdfoptions__doc__;
 }
