@@ -5,11 +5,16 @@ a copyright and/or license notice that predates the release of Scribus 1.3.2
 for which a new license (GPL+exception) is in place.
 */
 #include "cmdobj.h"
+
+#include "appmodes.h"
 #include "cmdutil.h"
+#include "commonstrings.h"
+#include "pageitem_table.h"
+#include "scribuscore.h"
+#include "scribusdoc.h"
+#include "scribusview.h"
 #include "selection.h"
 #include "util_math.h"
-#include "commonstrings.h"
-#include "scribuscore.h"
 
 
 PyObject *scribus_newrect(PyObject* /* self */, PyObject* args)
@@ -18,28 +23,26 @@ PyObject *scribus_newrect(PyObject* /* self */, PyObject* args)
 	char *Name = const_cast<char*>("");
 
 	if (!PyArg_ParseTuple(args, "dddd|es", &x, &y, &w, &h, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 //	if (ItemExists(QString::fromUtf8(Name)))
 //	{
 //		PyErr_SetString(NameExistsError, QObject::tr("An object with the requested name already exists.","python error"));
-//		return NULL;
+//		return nullptr;
 //	}
 	int i = ScCore->primaryMainWindow()->doc->itemAdd(PageItem::Polygon, PageItem::Rectangle,
 								pageUnitXToDocX(x), pageUnitYToDocY(y),
 								ValueToPoint(w), ValueToPoint(h),
-								ScCore->primaryMainWindow()->doc->toolSettings.dWidth,
-								ScCore->primaryMainWindow()->doc->toolSettings.dBrush, ScCore->primaryMainWindow()->doc->toolSettings.dPen, true);
+								ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeLineWidth,
+								ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeFillColor, ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeLineColor);
 //	ScCore->primaryMainWindow()->doc->setRedrawBounding(ScCore->primaryMainWindow()->doc->Items->at(i));
-	if (Name != EMPTY_STRING)
+	if (strlen(Name) > 0)
 	{
 		QString objName = QString::fromUtf8(Name);
 		if (!ItemExists(objName))
 			ScCore->primaryMainWindow()->doc->Items->at(i)->setItemName(objName);
 	}
-//	if (Name != EMPTY_STRING)
-//		ScCore->primaryMainWindow()->doc->Items->at(i)->setItemName(QString::fromUtf8(Name));
 	return PyString_FromString(ScCore->primaryMainWindow()->doc->Items->at(i)->itemName().toUtf8());
 }
 
@@ -49,27 +52,18 @@ PyObject *scribus_newellipse(PyObject* /* self */, PyObject* args)
 	double x, y, w, h;
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "dddd|es", &x, &y, &w, &h, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	int i = ScCore->primaryMainWindow()->doc->itemAdd(PageItem::Polygon, PageItem::Ellipse,
 										pageUnitXToDocX(x),
 										pageUnitYToDocY(y),
 										ValueToPoint(w),
 										ValueToPoint(h),
-										ScCore->primaryMainWindow()->doc->toolSettings.dWidth,
-										ScCore->primaryMainWindow()->doc->toolSettings.dBrush,
-										ScCore->primaryMainWindow()->doc->toolSettings.dPen,
-										true);
-//	if (ItemExists(QString::fromUtf8(Name)))
-//	{
-//		PyErr_SetString(NameExistsError, QObject::tr("An object with the requested name already exists.","python error"));
-//		return NULL;
-//	}
-//	ScCore->primaryMainWindow()->doc->setRedrawBounding(ScCore->primaryMainWindow()->doc->Items->at(i));
-//	if (Name != EMPTY_STRING)
-//		ScCore->primaryMainWindow()->doc->Items->at(i)->setItemName(QString::fromUtf8(Name));
-	if (Name != EMPTY_STRING)
+										ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeLineWidth,
+										ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeFillColor,
+										ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeLineColor);
+	if (strlen(Name) > 0)
 	{
 		QString objName = QString::fromUtf8(Name);
 		if (!ItemExists(objName))
@@ -84,25 +78,17 @@ PyObject *scribus_newimage(PyObject* /* self */, PyObject* args)
 	double x, y, w, h;
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "dddd|es", &x, &y, &w, &h, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	int i = ScCore->primaryMainWindow()->doc->itemAdd(PageItem::ImageFrame, PageItem::Unspecified,
 									pageUnitXToDocX(x),
 									pageUnitYToDocY(y),
 									ValueToPoint(w),
 									ValueToPoint(h),
-									1, ScCore->primaryMainWindow()->doc->toolSettings.dBrushPict,
-									CommonStrings::None, true);
-//	if (ItemExists(QString::fromUtf8(Name)))
-//	{
-//		PyErr_SetString(NameExistsError, QObject::tr("An object with the requested name already exists.","python error"));
-//		return NULL;
-//	}
-//	ScCore->primaryMainWindow()->doc->setRedrawBounding(ScCore->primaryMainWindow()->doc->Items->at(i));
-//	if (Name != EMPTY_STRING)
-//		ScCore->primaryMainWindow()->doc->Items->at(i)->setItemName(QString::fromUtf8(Name));
-	if (Name != EMPTY_STRING)
+									1, ScCore->primaryMainWindow()->doc->itemToolPrefs().imageFillColor,
+									ScCore->primaryMainWindow()->doc->itemToolPrefs().imageStrokeColor);
+	if (strlen(Name) > 0)
 	{
 		QString objName = QString::fromUtf8(Name);
 		if (!ItemExists(objName))
@@ -117,23 +103,17 @@ PyObject *scribus_newtext(PyObject* /* self */, PyObject* args)
 	double x, y, w, h;
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "dddd|es", &x, &y, &w, &h, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	int i = ScCore->primaryMainWindow()->doc->itemAdd(PageItem::TextFrame, PageItem::Unspecified,
 								pageUnitXToDocX(x),
 								pageUnitYToDocY(y),
 								ValueToPoint(w),
 								ValueToPoint(h),
-								ScCore->primaryMainWindow()->doc->toolSettings.dWidth, CommonStrings::None,
-								ScCore->primaryMainWindow()->doc->toolSettings.dPenText, true);
-//	if (ItemExists(QString::fromUtf8(Name)))
-//	{
-//		PyErr_SetString(NameExistsError, QObject::tr("An object with the requested name already exists.","python error"));
-//		return NULL;
-//	}
-//	ScCore->primaryMainWindow()->doc->setRedrawBounding(ScCore->primaryMainWindow()->doc->Items->at(i));
-	if (Name != EMPTY_STRING)
+								ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeLineWidth, CommonStrings::None,
+								ScCore->primaryMainWindow()->doc->itemToolPrefs().textColor);
+	if (strlen(Name) > 0)
 	{
 		QString objName = QString::fromUtf8(Name);
 		if (!ItemExists(objName))
@@ -142,14 +122,50 @@ PyObject *scribus_newtext(PyObject* /* self */, PyObject* args)
 	return PyString_FromString(ScCore->primaryMainWindow()->doc->Items->at(i)->itemName().toUtf8());
 }
 
+PyObject *scribus_newtable(PyObject* /* self */, PyObject* args)
+{
+	double x, y, w, h;
+	int numRows, numColumns;
+	char *Name = const_cast<char*>("");
+	if (!PyArg_ParseTuple(args, "ddddii|es", &x, &y, &w, &h, &numRows, &numColumns, "utf-8", &Name))
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	if (numRows < 1 || numColumns < 1)
+	{
+		PyErr_SetString(PyExc_ValueError, QObject::tr("Both numRows and numColumns must be greater than 0.","python error").toLocal8Bit().constData());
+		return nullptr;
+	}
+	int i = ScCore->primaryMainWindow()->doc->itemAdd(PageItem::Table, PageItem::Unspecified,
+								pageUnitXToDocX(x),
+								pageUnitYToDocY(y),
+								ValueToPoint(w),
+								ValueToPoint(h),
+								0,                    // Unused.
+								CommonStrings::None,  // Unused.
+								CommonStrings::None); // Unused.
+	PageItem_Table *table = ScCore->primaryMainWindow()->doc->Items->at(i)->asTable();
+	table->insertRows(0, numRows - 1);
+	table->insertColumns(0, numColumns - 1);
+	table->adjustTableToFrame();
+	table->adjustFrameToTable();
+	if (strlen(Name) > 0)
+	{
+		QString objName = QString::fromUtf8(Name);
+		if (!ItemExists(objName))
+			ScCore->primaryMainWindow()->doc->Items->at(i)->setItemName(objName);
+	}
+	return PyString_FromString(table->itemName().toUtf8());
+}
+
 PyObject *scribus_newline(PyObject* /* self */, PyObject* args)
 {
 	double x, y, w, h;
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "dddd|es", &x, &y, &w, &h, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	x = pageUnitXToDocX(x);
 	y = pageUnitYToDocY(y);
 	w = pageUnitXToDocX(w);
@@ -159,13 +175,13 @@ PyObject *scribus_newline(PyObject* /* self */, PyObject* args)
 //		PyErr_SetString(NameExistsError,
 //						QObject::tr("An object with the requested name already exists.",
 //									"python error"));
-//		return NULL;
+//		return nullptr;
 //	}
 	int i = ScCore->primaryMainWindow()->doc->itemAdd(PageItem::Line, PageItem::Unspecified,
 							   x, y, w, h,
-							   ScCore->primaryMainWindow()->doc->toolSettings.dWidth,
-							   ScCore->primaryMainWindow()->doc->toolSettings.dBrush,
-							   ScCore->primaryMainWindow()->doc->toolSettings.dPen, true);
+							   ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeLineWidth,
+							   ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeFillColor,
+							   ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeLineColor);
 	PageItem *it = ScCore->primaryMainWindow()->doc->Items->at(i);
 	it->setRotation(xy2Deg(w-x, h-y));
 	it->setWidthHeight(sqrt(pow(x-w, 2.0) + pow(y-h, 2.0)), 1.0);
@@ -190,12 +206,10 @@ PyObject *scribus_newline(PyObject* /* self */, PyObject* args)
 		it->PoLine.translate(0, -np2.y());
 		ScCore->primaryMainWindow()->view->MoveItem(0, np2.y(), it);
 	}
-	ScCore->primaryMainWindow()->view->SizeItem(it->PoLine.WidthHeight().x(),
-						 it->PoLine.WidthHeight().y(), i, false, false, false);
+	ScCore->primaryMainWindow()->view->SizeItem(it->PoLine.widthHeight().x(),
+						 it->PoLine.widthHeight().y(), i, false, false, false);
 	ScCore->primaryMainWindow()->view->AdjustItemSize(it);*/
-//	if (Name != EMPTY_STRING)
-//		it->setItemName(QString::fromUtf8(Name));
-	if (Name != EMPTY_STRING)
+	if (strlen(Name) > 0)
 	{
 		QString objName = QString::fromUtf8(Name);
 		if (!ItemExists(objName))
@@ -209,26 +223,26 @@ PyObject *scribus_polyline(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	PyObject *il;
-	// FIXME: PyList_Check failing will cause the function to return NULL w/o an exception. Separarate out the check.
+	// FIXME: PyList_Check failing will cause the function to return nullptr w/o an exception. Separarate out the check.
 	if ((!PyArg_ParseTuple(args, "O|es", &il, "utf-8", &Name)) || (!PyList_Check(il)))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	int len = PyList_Size(il);
 	if (len < 4)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Point list must contain at least two points (four values).","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	if ((len % 2) != 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Point list must contain an even number of values.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 //	if (ItemExists(QString::fromUtf8(Name)))
 //	{
 //		PyErr_SetString(NameExistsError, QObject::tr("An object with the requested name already exists.","python error").toLocal8Bit().constData());
-//		return NULL;
+//		return nullptr;
 //	}
 	double x, y, w, h;
 	int i = 0;
@@ -236,7 +250,7 @@ PyObject *scribus_polyline(PyObject* /* self */, PyObject* args)
 	i++;
 	y = pageUnitYToDocY(static_cast<double>(PyFloat_AsDouble(PyList_GetItem(il, i))));
 	i++;
-	int ic = ScCore->primaryMainWindow()->doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, x, y, 1, 1,	ScCore->primaryMainWindow()->doc->toolSettings.dWidth, ScCore->primaryMainWindow()->doc->toolSettings.dBrush, ScCore->primaryMainWindow()->doc->toolSettings.dPen, true);
+	int ic = ScCore->primaryMainWindow()->doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, x, y, 1, 1,	ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeLineWidth, ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeFillColor, ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeLineColor);
 	PageItem *it = ScCore->primaryMainWindow()->doc->Items->at(ic);
 	it->PoLine.resize(2);
 	it->PoLine.setPoint(0, 0, 0);
@@ -263,20 +277,16 @@ PyObject *scribus_polyline(PyObject* /* self */, PyObject* args)
 	if (np2.x() < 0)
 	{
 		it->PoLine.translate(-np2.x(), 0);
-		ScCore->primaryMainWindow()->doc->MoveItem(np2.x(), 0, it);
+		ScCore->primaryMainWindow()->doc->moveItem(np2.x(), 0, it);
 	}
 	if (np2.y() < 0)
 	{
 		it->PoLine.translate(0, -np2.y());
-		ScCore->primaryMainWindow()->doc->MoveItem(0, np2.y(), it);
+		ScCore->primaryMainWindow()->doc->moveItem(0, np2.y(), it);
 	}
-	ScCore->primaryMainWindow()->doc->SizeItem(it->PoLine.WidthHeight().x(), it->PoLine.WidthHeight().y(), ic, false, false, false);
-	ScCore->primaryMainWindow()->doc->AdjustItemSize(it);
-//	if (Name != EMPTY_STRING)
-//	{
-//		it->setItemName(QString::fromUtf8(Name));
-//	}
-	if (Name != EMPTY_STRING)
+	ScCore->primaryMainWindow()->doc->sizeItem(it->PoLine.widthHeight().x(), it->PoLine.widthHeight().y(), it, false, false, false);
+	ScCore->primaryMainWindow()->doc->adjustItemSize(it);
+	if (strlen(Name) > 0)
 	{
 		QString objName = QString::fromUtf8(Name);
 		if (!ItemExists(objName))
@@ -290,26 +300,26 @@ PyObject *scribus_polygon(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	PyObject *il;
-	// FIXME: PyList_Check failing will cause the function to return NULL w/o an exception. Separarate out the check.
+	// FIXME: PyList_Check failing will cause the function to return nullptr w/o an exception. Separarate out the check.
 	if ((!PyArg_ParseTuple(args, "O|es", &il, "utf-8", &Name)) || (!PyList_Check(il)))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	int len = PyList_Size(il);
 	if (len < 6)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Point list must contain at least three points (six values).","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	if ((len % 2) != 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Point list must contain an even number of values.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 //	if (ItemExists(QString::fromUtf8(Name)))
 //	{
 //		PyErr_SetString(NameExistsError, QObject::tr("An object with the requested name already exists.","python error").toLocal8Bit().constData());
-//		return NULL;
+//		return nullptr;
 //	}
 	double x, y, w, h;
 	int i = 0;
@@ -317,7 +327,7 @@ PyObject *scribus_polygon(PyObject* /* self */, PyObject* args)
 	i++;
 	y = pageUnitYToDocY(static_cast<double>(PyFloat_AsDouble(PyList_GetItem(il, i))));
 	i++;
-	int ic = ScCore->primaryMainWindow()->doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, x, y, 1, 1,	ScCore->primaryMainWindow()->doc->toolSettings.dWidth, ScCore->primaryMainWindow()->doc->toolSettings.dBrush, ScCore->primaryMainWindow()->doc->toolSettings.dPen, true);
+	int ic = ScCore->primaryMainWindow()->doc->itemAdd(PageItem::Polygon, PageItem::Unspecified, x, y, 1, 1,	ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeLineWidth, ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeFillColor, ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeLineColor);
 	PageItem *it = ScCore->primaryMainWindow()->doc->Items->at(ic);
 	it->PoLine.resize(2);
 	it->PoLine.setPoint(0, 0, 0);
@@ -349,18 +359,16 @@ PyObject *scribus_polygon(PyObject* /* self */, PyObject* args)
 	if (np2.x() < 0)
 	{
 		it->PoLine.translate(-np2.x(), 0);
-		ScCore->primaryMainWindow()->doc->MoveItem(np2.x(), 0, it);
+		ScCore->primaryMainWindow()->doc->moveItem(np2.x(), 0, it);
 	}
 	if (np2.y() < 0)
 	{
 		it->PoLine.translate(0, -np2.y());
-		ScCore->primaryMainWindow()->doc->MoveItem(0, np2.y(), it);
+		ScCore->primaryMainWindow()->doc->moveItem(0, np2.y(), it);
 	}
-	ScCore->primaryMainWindow()->doc->SizeItem(it->PoLine.WidthHeight().x(), it->PoLine.WidthHeight().y(), ic, false, false, false);
-	ScCore->primaryMainWindow()->doc->AdjustItemSize(it);
-//	if (Name != EMPTY_STRING)
-//		it->setItemName(QString::fromUtf8(Name));
-	if (Name != EMPTY_STRING)
+	ScCore->primaryMainWindow()->doc->sizeItem(it->PoLine.widthHeight().x(), it->PoLine.widthHeight().y(), it, false, false, false);
+	ScCore->primaryMainWindow()->doc->adjustItemSize(it);
+	if (strlen(Name) > 0)
 	{
 		QString objName = QString::fromUtf8(Name);
 		if (!ItemExists(objName))
@@ -373,26 +381,26 @@ PyObject *scribus_bezierline(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	PyObject *il;
-	// FIXME: PyList_Check failing will cause the function to return NULL w/o an exception. Separarate out the check.
+	// FIXME: PyList_Check failing will cause the function to return nullptr w/o an exception. Separarate out the check.
 	if ((!PyArg_ParseTuple(args, "O|es", &il, "utf-8", &Name)) || (!PyList_Check(il)))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	int len = PyList_Size(il);
 	if (len < 8)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Point list must contain at least four points (eight values).","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	if ((len % 6) != 0)
 	{
 		PyErr_SetString(PyExc_ValueError, QObject::tr("Point list must have a multiple of six values.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 //	if (ItemExists(QString::fromUtf8(Name)))
 //	{
 //		PyErr_SetString(NameExistsError, QObject::tr("An object with the requested name already exists.","python error").toLocal8Bit().constData());
-//		return NULL;
+//		return nullptr;
 //	}
 	double x, y, w, h, kx, ky, kx2, ky2;
 	int i = 0;
@@ -409,7 +417,7 @@ PyObject *scribus_bezierline(PyObject* /* self */, PyObject* args)
 	ky2 = pageUnitYToDocY(static_cast<double>(PyFloat_AsDouble(PyList_GetItem(il, i))));
 	i++;
 	//int ic = ScCore->primaryMainWindow()->view->PaintPolyLine(x, y, 1, 1,	ScCore->primaryMainWindow()->doc->toolSettings.dWidth, ScCore->primaryMainWindow()->doc->toolSettings.dBrush, ScCore->primaryMainWindow()->doc->toolSettings.dPen);
-	int ic = ScCore->primaryMainWindow()->doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, x, y, 1, 1,	ScCore->primaryMainWindow()->doc->toolSettings.dWidth, ScCore->primaryMainWindow()->doc->toolSettings.dBrush, ScCore->primaryMainWindow()->doc->toolSettings.dPen, true);
+	int ic = ScCore->primaryMainWindow()->doc->itemAdd(PageItem::PolyLine, PageItem::Unspecified, x, y, 1, 1,	ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeLineWidth, ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeFillColor, ScCore->primaryMainWindow()->doc->itemToolPrefs().shapeLineColor);
 	PageItem *it = ScCore->primaryMainWindow()->doc->Items->at(ic);
 	it->PoLine.resize(2);
 	it->PoLine.setPoint(0, 0, 0);
@@ -442,18 +450,16 @@ PyObject *scribus_bezierline(PyObject* /* self */, PyObject* args)
 	if (np2.x() < 0)
 	{
 		it->PoLine.translate(-np2.x(), 0);
-		ScCore->primaryMainWindow()->doc->MoveItem(np2.x(), 0, it);
+		ScCore->primaryMainWindow()->doc->moveItem(np2.x(), 0, it);
 	}
 	if (np2.y() < 0)
 	{
 		it->PoLine.translate(0, -np2.y());
-		ScCore->primaryMainWindow()->doc->MoveItem(0, np2.y(), it);
+		ScCore->primaryMainWindow()->doc->moveItem(0, np2.y(), it);
 	}
-	ScCore->primaryMainWindow()->doc->SizeItem(it->PoLine.WidthHeight().x(), it->PoLine.WidthHeight().y(), ic, false, false, false);
-	ScCore->primaryMainWindow()->doc->AdjustItemSize(it);
-//	if (Name != EMPTY_STRING)
-//		it->setItemName(QString::fromUtf8(Name));
-	if (Name != EMPTY_STRING)
+	ScCore->primaryMainWindow()->doc->sizeItem(it->PoLine.widthHeight().x(), it->PoLine.widthHeight().y(), it, false, false, false);
+	ScCore->primaryMainWindow()->doc->adjustItemSize(it);
+	if (strlen(Name) > 0)
 	{
 		QString objName = QString::fromUtf8(Name);
 		if (!ItemExists(objName))
@@ -472,38 +478,35 @@ PyObject *scribus_pathtext(PyObject* /* self */, PyObject* args)
 	char *TextB = const_cast<char*>("");
 	char *PolyB = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "ddeses|es", &x, &y, "utf-8", &TextB, "utf-8", &PolyB, "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 //	if (ItemExists(QString::fromUtf8(Name)))
 //	{
 //		PyErr_SetString(NameExistsError, QObject::tr("An object with the requested name already exists.","python error"));
-//		return NULL;
+//		return nullptr;
 //	}
 	//FIXME: Why use GetItem not GetUniqueItem? Maybe use GetUniqueItem and use the exceptions
 	// its sets for us?
-	int i = GetItem(QString::fromUtf8(TextB));
-	int ii = GetItem(QString::fromUtf8(PolyB));
-	if ((i == -1) || (ii == -1))
+	PageItem *i = GetItem(QString::fromUtf8(TextB));
+	PageItem *ii = GetItem(QString::fromUtf8(PolyB));
+	if ((i == nullptr) || (ii == nullptr))
 	{
 		PyErr_SetString(NotFoundError, QObject::tr("Object not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	ScCore->primaryMainWindow()->doc->m_Selection->clear();
-	ScCore->primaryMainWindow()->doc->m_Selection->addItem(ScCore->primaryMainWindow()->doc->Items->at(i));
-	ScCore->primaryMainWindow()->doc->m_Selection->addItem(ScCore->primaryMainWindow()->doc->Items->at(ii));
-	PageItem *it = ScCore->primaryMainWindow()->doc->Items->at(i);
+	ScCore->primaryMainWindow()->doc->m_Selection->addItem(i);
+	ScCore->primaryMainWindow()->doc->m_Selection->addItem(ii);
 	ScCore->primaryMainWindow()->view->ToPathText();
-	ScCore->primaryMainWindow()->doc->MoveItem(pageUnitXToDocX(x) - it->xPos(), pageUnitYToDocY(y) - it->yPos(), it);
-//	if (Name != EMPTY_STRING)
-//		it->setItemName(QString::fromUtf8(Name));
-	if (Name != EMPTY_STRING)
+	ScCore->primaryMainWindow()->doc->moveItem(pageUnitXToDocX(x) - i->xPos(), pageUnitYToDocY(y) - i->yPos(), i);
+	if (strlen(Name) > 0)
 	{
 		QString objName = QString::fromUtf8(Name);
 		if (!ItemExists(objName))
-			ScCore->primaryMainWindow()->doc->Items->at(i)->setItemName(objName);
+			i->setItemName(objName);
 	}
-	return PyString_FromString(it->itemName().toUtf8());
+	return PyString_FromString(i->itemName().toUtf8());
 }
 
 
@@ -513,17 +516,16 @@ PyObject *scribus_deleteobj(PyObject* /* self */, PyObject* args)
 {
 	char *Name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "|es", "utf-8", &Name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *i = GetUniqueItem(QString::fromUtf8(Name));
-	if (i == NULL)
-		return NULL;
+	if (i == nullptr)
+		return nullptr;
 	ScCore->primaryMainWindow()->doc->m_Selection->clear();
 	ScCore->primaryMainWindow()->doc->m_Selection->addItem(i);
 	ScCore->primaryMainWindow()->doc->itemSelection_DeleteItem();
-//	Py_INCREF(Py_None);
-//	return Py_None;
+
 	Py_RETURN_NONE;
 }
 
@@ -536,12 +538,12 @@ PyObject *scribus_textflow(PyObject* /* self */, PyObject* args)
 	int state = -1;
 
 	if (!PyArg_ParseTuple(args, "es|i", "utf-8", &name, &state))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *i = GetUniqueItem(QString::fromUtf8(name));
-	if (i == NULL)
-		return NULL;
+	if (i == nullptr)
+		return nullptr;
 	if (state == -1)
 	{
 		if (i->textFlowAroundObject())
@@ -549,18 +551,17 @@ PyObject *scribus_textflow(PyObject* /* self */, PyObject* args)
 		else
 			i->setTextFlowMode(PageItem::TextFlowUsesFrameShape);
 	}
-	else if( state == (int) PageItem::TextFlowDisabled )
+	else if (state == (int) PageItem::TextFlowDisabled )
 		i->setTextFlowMode(PageItem::TextFlowDisabled);
-	else if( state == (int) PageItem::TextFlowUsesFrameShape )
+	else if (state == (int) PageItem::TextFlowUsesFrameShape )
 		i->setTextFlowMode(PageItem::TextFlowUsesFrameShape);
-	else if( state == (int) PageItem::TextFlowUsesBoundingBox )
+	else if (state == (int) PageItem::TextFlowUsesBoundingBox )
 		i->setTextFlowMode(PageItem::TextFlowUsesBoundingBox);
-	else if( state == (int) PageItem::TextFlowUsesContourLine )
+	else if (state == (int) PageItem::TextFlowUsesContourLine )
 		i->setTextFlowMode(PageItem::TextFlowUsesContourLine);
 	ScCore->primaryMainWindow()->view->DrawNew();
 	ScCore->primaryMainWindow()->slotDocCh(true);
-//	Py_INCREF(Py_None);
-//	return Py_None;
+
 	Py_RETURN_NONE;
 }
 
@@ -569,14 +570,54 @@ PyObject *scribus_objectexists(PyObject* /* self */, PyObject* args)
 {
 	char* name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "|es", "utf-8", &name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	if (ItemExists(QString::fromUtf8(name)))
 		return PyBool_FromLong(static_cast<long>(true));
 	return PyBool_FromLong(static_cast<long>(false));
 }
 
+
+/*
+ * Vaclav Smilauer, 2017-21-21
+ * Return style name of the object (or currently selected object)
+ */
+PyObject *scribus_getstyle(PyObject* /* self */, PyObject* args)
+{
+	char *name = const_cast<char*>("");
+	if (!PyArg_ParseTuple(args, "|es", "utf-8", &name))
+		return NULL;
+	if (!checkHaveDocument())
+		return NULL;
+	PageItem *item = GetUniqueItem(QString::fromUtf8(name));
+	if (item == NULL)
+		return NULL;
+	if ((item->itemType() != PageItem::TextFrame) && (item->itemType() != PageItem::PathText))
+	{
+		PyErr_SetString(WrongFrameTypeError, QObject::tr("Cannot get style of a non-text frame.", "python error").toLocal8Bit().constData());
+		return NULL;
+	}
+
+	int selectionLength = item->itemText.selectionLength();	
+	if (selectionLength > 0)
+	{
+		int selectionStart = item->itemText.startOfSelection();
+		const ParagraphStyle& currentStyle = item->itemText.paragraphStyle(selectionStart);
+		if (currentStyle.hasParent())
+			return PyString_FromString(currentStyle.parentStyle()->name().toUtf8());
+	}
+	else
+	{
+		const ParagraphStyle& itemDefaultStyle = item->itemText.defaultStyle();
+		if (itemDefaultStyle.hasParent())
+			return PyString_FromString(itemDefaultStyle.parentStyle()->name().toUtf8());
+	}
+	Py_RETURN_NONE;
+};
+
+
+
 /*
  * Craig Ringer, 2004-09-09
  * Apply the named style to the currently selected object.
@@ -587,16 +628,16 @@ PyObject *scribus_setstyle(PyObject* /* self */, PyObject* args)
 	char *style = const_cast<char*>("");
 	char *name = const_cast<char*>("");
 	if (!PyArg_ParseTuple(args, "es|es", "utf-8", &style, "utf-8", &name))
-		return NULL;
-	if(!checkHaveDocument())
-		return NULL;
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
 	PageItem *item = GetUniqueItem(QString::fromUtf8(name));
-	if (item == NULL)
-		return NULL;
+	if (item == nullptr)
+		return nullptr;
 	if ((item->itemType() != PageItem::TextFrame) && (item->itemType() != PageItem::PathText))
 	{
-		PyErr_SetString(WrongFrameTypeError, QObject::tr("Cannot set style on a non-text frame.","python error").toLocal8Bit().constData());
-		return NULL;
+		PyErr_SetString(WrongFrameTypeError, QObject::tr("Cannot set style on a non-text frame.", "python error").toLocal8Bit().constData());
+		return nullptr;
 	}
 
 	// First, find the style number associated with the requested style
@@ -604,45 +645,142 @@ PyObject *scribus_setstyle(PyObject* /* self */, PyObject* args)
 	// we can't find it, raise PyExc_Exception.
 	// FIXME: Should use a more specific exception.
 	bool found = false;
-	uint styleid = 0;
+	QString paraStyleName = QString::fromUtf8(style);
 	// We start at zero here because it's OK to match an internal name
-	int docParagraphStylesCount=ScCore->primaryMainWindow()->doc->paragraphStyles().count();
+	ScribusDoc*  currentDoc = ScCore->primaryMainWindow()->doc;
+	ScribusView* currentView = ScCore->primaryMainWindow()->view;
+	ScribusMainWindow* currentWin = ScCore->primaryMainWindow();
+	const StyleSet<ParagraphStyle> &docParagraphStyles = currentDoc->paragraphStyles();
+	int docParagraphStylesCount = docParagraphStyles.count();
 	for (int i=0; i < docParagraphStylesCount; ++i)
 	{
-		if (ScCore->primaryMainWindow()->doc->paragraphStyles()[i].name() == QString::fromUtf8(style)) {
+		if (docParagraphStyles[i].name() == paraStyleName) {
 			found = true;
-			styleid = i;
 			break;
 		}
 	}
 	if (!found) {
 		// whoops, the user specified an invalid style, complain loudly.
 		PyErr_SetString(NotFoundError, QObject::tr("Style not found.","python error").toLocal8Bit().constData());
-		return NULL;
+		return nullptr;
 	}
 	// for current item only
-	if (ScCore->primaryMainWindow()->doc->m_Selection->count() == 0 || name != EMPTY_STRING)
+	if (currentDoc->m_Selection->count() == 0 || (strlen(name) > 0))
 	{
+		// Store text selection as clearing object selection
+		// will also clear text selection
+		int selectionStart = -1;
+		int selectionLength = item->itemText.selectionLength();
+		if (selectionLength > 0)
+			selectionStart = item->itemText.startOfSelection();
 		// quick hack to always apply on the right frame - pv
-		ScCore->primaryMainWindow()->view->Deselect(true);
+		currentView->Deselect(true);
 		//CB I dont think we need to draw here. Its faster if we dont.
-		ScCore->primaryMainWindow()->view->SelectItem(item, false);
+		currentView->SelectItem(item, false);
+		// Restore text selection if necessary
+		if (selectionStart >= 0)
+		{
+			item->itemText.deselectAll();
+			item->itemText.select(selectionStart, selectionLength);
+			item->HasSel = true;
+		}
 		// Now apply the style.
-		int mode = ScCore->primaryMainWindow()->doc->appMode;
-		ScCore->primaryMainWindow()->doc->appMode = modeEdit;
-		ScCore->primaryMainWindow()->setNewParStyle(QString::fromUtf8(style));
-		ScCore->primaryMainWindow()->doc->appMode = mode;
+		int mode = currentDoc->appMode;
+		currentDoc->appMode = modeEdit;
+		currentWin->setNewParStyle(paraStyleName);
+		currentDoc->appMode = mode;
 	}
 	else // for multiple selection
 	{
+		int mode = currentDoc->appMode;
+		currentDoc->appMode = modeNormal;
+		currentDoc->itemSelection_SetNamedParagraphStyle(paraStyleName);
+		currentDoc->appMode = mode;
+	}
+
+	Py_RETURN_NONE;
+}
+
+/*
+* Jean Ghali, 2017-07-18
+* Apply the named character style to the currently selected object.
+* pv, 2004-09-13, optionaly param objectName + "check the page" stuff
+*/
+PyObject *scribus_setcharstyle(PyObject* /* self */, PyObject* args)
+{
+	char *style = const_cast<char*>("");
+	char *name = const_cast<char*>("");
+	if (!PyArg_ParseTuple(args, "es|es", "utf-8", &style, "utf-8", &name))
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+	PageItem *item = GetUniqueItem(QString::fromUtf8(name));
+	if (item == nullptr)
+		return nullptr;
+	if ((item->itemType() != PageItem::TextFrame) && (item->itemType() != PageItem::PathText))
+	{
+		PyErr_SetString(WrongFrameTypeError, QObject::tr("Cannot set character style on a non-text frame.", "python error").toLocal8Bit().constData());
+		return nullptr;
+	}
+
+	// First, find the style number associated with the requested style
+	// by scanning through the styles looking for the name. If
+	// we can't find it, raise PyExc_Exception.
+	// FIXME: Should use a more specific exception.
+	bool found = false;
+	QString charStyleName = QString::fromUtf8(style);
+	// We start at zero here because it's OK to match an internal name
+	ScribusDoc*  currentDoc = ScCore->primaryMainWindow()->doc;
+	ScribusView* currentView = ScCore->primaryMainWindow()->view;
+	ScribusMainWindow* currentWin = ScCore->primaryMainWindow();
+	const StyleSet<CharStyle> &docCharStyles = currentDoc->charStyles();
+	int docCharacterStylesCount = docCharStyles.count();
+	for (int i = 0; i < docCharacterStylesCount; ++i)
+	{
+		if (docCharStyles[i].name() == charStyleName) {
+			found = true;
+			break;
+		}
+	}
+	if (!found) {
+		// whoops, the user specified an invalid style, complain loudly.
+		PyErr_SetString(NotFoundError, QObject::tr("Character style not found.", "python error").toLocal8Bit().constData());
+		return nullptr;
+	}
+	// for current item only
+	if (currentDoc->m_Selection->count() == 0 || (strlen(name) > 0))
+	{
+		// Store text selection as clearing object selection
+		// will also clear text selection
+		int selectionStart = -1;
+		int selectionLength = item->itemText.selectionLength();
+		if (selectionLength > 0)
+			selectionStart = item->itemText.startOfSelection();
+		// quick hack to always apply on the right frame - pv
+		currentView->Deselect(true);
+		//CB I dont think we need to draw here. Its faster if we dont.
+		currentView->SelectItem(item, false);
+		// Restore text selection if necessary
+		if (selectionStart >= 0)
+		{
+			item->itemText.deselectAll();
+			item->itemText.select(selectionStart, selectionLength);
+			item->HasSel = true;
+		}	
+		// Now apply the style.
 		int mode = ScCore->primaryMainWindow()->doc->appMode;
-		ScCore->primaryMainWindow()->doc->appMode = modeNormal;
-		ScCore->primaryMainWindow()->doc->itemSelection_ApplyParagraphStyle(ScCore->primaryMainWindow()->doc->paragraphStyles()[styleid]);
-		ScCore->primaryMainWindow()->doc->appMode = mode;
+		currentDoc->appMode = modeEdit;
+		currentWin->setNewCharStyle(charStyleName);
+		currentDoc->appMode = mode;
+	}
+	else // for multiple selection
+	{
+		int mode = currentDoc->appMode;
+		currentDoc->appMode = modeNormal;
+		currentDoc->itemSelection_SetNamedCharStyle(charStyleName);
+		currentDoc->appMode = mode;
 	}
 
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
@@ -653,42 +791,101 @@ PyObject *scribus_setstyle(PyObject* /* self */, PyObject* args)
 PyObject *scribus_getstylenames(PyObject* /* self */)
 {
 	PyObject *styleList;
-	if(!checkHaveDocument())
-		return NULL;
+	if (!checkHaveDocument())
+		return nullptr;
+	const auto& paragraphStyles = ScCore->primaryMainWindow()->doc->paragraphStyles();
+
 	styleList = PyList_New(0);
-	for (int i=0; i < ScCore->primaryMainWindow()->doc->paragraphStyles().count(); ++i)
+	for (int i = 0; i < paragraphStyles.count(); ++i)
 	{
-		if (PyList_Append(styleList, PyString_FromString(ScCore->primaryMainWindow()->doc->paragraphStyles()[i].name().toUtf8())))
+		if (PyList_Append(styleList, PyString_FromString(paragraphStyles[i].name().toUtf8())))
 		{
 			// An exception will have already been set by PyList_Append apparently.
-			return NULL;
+			return nullptr;
 		}
 	}
 	return styleList;
 }
 
+PyObject *scribus_getcharstylenames(PyObject* /* self */)
+{
+	PyObject *charStyleList;
+	if (!checkHaveDocument())
+		return nullptr;
+	const auto& charStyles = ScCore->primaryMainWindow()->doc->charStyles();
+
+	charStyleList = PyList_New(0);
+	for (int i = 0; i < charStyles.count(); ++i)
+	{
+		if (PyList_Append(charStyleList, PyString_FromString(charStyles[i].name().toUtf8())))
+		{
+			// An exception will have already been set by PyList_Append apparently.
+			return nullptr;
+		}
+	}
+	return charStyleList;
+}
+
 PyObject *scribus_duplicateobject(PyObject * /* self */, PyObject *args)
 {
 	char* name = const_cast<char*>("");
-	if (!PyArg_ParseTuple(args, "|es", "utf-8", &name)) {
-		return NULL;
-	}
-	if(!checkHaveDocument()) {
-		return NULL;
-	}
+	if (!PyArg_ParseTuple(args, "|es", "utf-8", &name))
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+
 	// Is there a special name given? Yes -> add this to selection
+	ScribusMainWindow* currentWin = ScCore->primaryMainWindow();
+	ScribusDoc* currentDoc = currentWin->doc;
+
 	PageItem *i = GetUniqueItem(QString::fromUtf8(name));
-	if (i != NULL) {
-		ScCore->primaryMainWindow()->doc->m_Selection->clear();
-		ScCore->primaryMainWindow()->doc->m_Selection->addItem(i);
-	}
-	else
-		return NULL;
+	if (i == nullptr)
+		return nullptr;
+	currentDoc->m_Selection->clear();
+	currentDoc->m_Selection->addItem(i);
+
 	// do the duplicate
-	ScCore->primaryMainWindow()->slotEditCopy();
+	currentWin->slotEditCopy();
+	currentWin->slotEditPaste();
+
+	Py_RETURN_NONE;
+}
+
+PyObject *scribus_copyobject(PyObject * /* self */, PyObject *args)
+{
+	char* name = const_cast<char*>("");
+	if (!PyArg_ParseTuple(args, "|es", "utf-8", &name))
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+
+	// Is there a special name given? Yes -> add this to selection
+	ScribusMainWindow* currentWin = ScCore->primaryMainWindow();
+	ScribusDoc* currentDoc = currentWin->doc;
+
+	PageItem *i = GetUniqueItem(QString::fromUtf8(name));
+	if (i == nullptr)
+		return nullptr;
+	currentDoc->m_Selection->clear();
+	currentDoc->m_Selection->addItem(i);
+
+	// do the copy
+	currentWin->slotEditCopy();
+
+	Py_RETURN_NONE;
+}
+
+PyObject *scribus_pasteobject(PyObject * /* self */, PyObject *args)
+{
+	char* name = const_cast<char*>("");
+	if (!PyArg_ParseTuple(args, "|es", "utf-8", &name))
+		return nullptr;
+	if (!checkHaveDocument())
+		return nullptr;
+
+	// do the paste
 	ScCore->primaryMainWindow()->slotEditPaste();
-//	Py_INCREF(Py_None);
-//	return Py_None;
+
 	Py_RETURN_NONE;
 }
 
@@ -697,6 +894,6 @@ with header files structure untouched (docstrings are kept near declarations)
 PV */
 void cmdobjdocwarnings()
 {
-    QStringList s;
-    s << scribus_newrect__doc__ <<scribus_newellipse__doc__ << scribus_newimage__doc__ << scribus_newtext__doc__ << scribus_newline__doc__ <<scribus_polyline__doc__ << scribus_polygon__doc__ << scribus_bezierline__doc__ <<scribus_pathtext__doc__ <<scribus_deleteobj__doc__ <<scribus_textflow__doc__ <<scribus_objectexists__doc__ <<scribus_setstyle__doc__ <<scribus_getstylenames__doc__ <<scribus_duplicateobject__doc__;
+	QStringList s;
+	s << scribus_newrect__doc__ <<scribus_newellipse__doc__ << scribus_newimage__doc__ << scribus_newtext__doc__ << scribus_newtable__doc__ << scribus_newline__doc__ <<scribus_polyline__doc__ << scribus_polygon__doc__ << scribus_bezierline__doc__ <<scribus_pathtext__doc__ <<scribus_deleteobj__doc__ <<scribus_textflow__doc__ <<scribus_objectexists__doc__ <<scribus_getstyle__doc__ <<scribus_setstyle__doc__ <<scribus_getstylenames__doc__ <<scribus_getcharstylenames__doc__ <<scribus_duplicateobject__doc__ <<scribus_copyobject__doc__ <<scribus_pasteobject__doc__;
 }
