@@ -36,6 +36,42 @@ selected item is used.\n\
 /*! Get font */
 PyObject *scribus_getfont(PyObject * /*self*/, PyObject* args);
 
+/*! docstring */
+PyDoc_STRVAR(scribus_getfontfeatures__doc__,
+QT_TR_NOOP("getFontFeatures([\"name\"]) -> string\n\
+\n\
+Returns the font features for the text frame \"name\". If this text frame\n\
+has some text selected the value assigned to the first character\n\
+of the selection is returned. If \"name\" is not given the currently\n\
+selected item is used.\n\
+"));
+/*! Get fontfeatures */
+PyObject *scribus_getfontfeatures(PyObject * /*self*/, PyObject* args);
+
+/*! docstring */
+PyDoc_STRVAR(scribus_gettextcolor__doc__,
+	QT_TR_NOOP("getTextColor([\"name\"]) -> string\n\
+\n\
+Returns the name of the text color used for text frame \"name\".\n\
+If this text frame has some text selected the value assigned\n\
+to the first character of the selection is returned.\n\
+If \"name\" is not given the currently selected item is used.\n\
+"));
+/*! Returns color of the line */
+PyObject *scribus_gettextcolor(PyObject * /*self*/, PyObject* args);
+
+/*! docstring */
+PyDoc_STRVAR(scribus_gettextshade__doc__,
+	QT_TR_NOOP("getTextShade([\"name\"]) -> integer\n\
+\n\
+Returns the shade of text color used for text frame \"name\".\n\
+If this text frame has some text selected the value assigned\n\
+to the first character of the selection is returned.\n\
+If \"name\" is not given the currently selected item is used.\n\
+"));
+/*! Returns color of the line */
+PyObject *scribus_gettextshade(PyObject * /*self*/, PyObject* args);
+
 /*! docstring */
 PyDoc_STRVAR(scribus_gettextsize__doc__,
 QT_TR_NOOP("getTextLength([\"name\"]) -> integer\n\
@@ -101,6 +137,16 @@ is not given the currently selected item is used.\n\
 /*! Get text distances */
 PyObject *scribus_gettextdistances(PyObject */*self*/, PyObject* args);
 
+/*! docstring */
+PyDoc_STRVAR(scribus_gettextverticalalignment__doc__,
+	QT_TR_NOOP("getTextVerticalAlignment([\"name\"]) -> integer\n\
+\n\
+Gets the vertical alignment of text inside text frame \"name\". If \"name\" is not\n\
+given the currently selected item is used.\n\
+"));
+/*! Get columns */
+PyObject *scribus_gettextverticalalignment(PyObject * /*self*/, PyObject* args);
+
 /*! docstring */
 PyDoc_STRVAR(scribus_getcolumngap__doc__,
 QT_TR_NOOP("getColumnGap([\"name\"]) -> float\n\
@@ -147,6 +193,17 @@ May throw IndexError for an insertion out of bounds.\n\
 /*! Insert text */
 PyObject *scribus_inserttext(PyObject * /*self*/, PyObject* args);
 
+/*! docstring */
+PyDoc_STRVAR(scribus_inserthtmltext__doc__,
+QT_TR_NOOP("insertHTMLText(\"file\", [\"name\"])\n\
+\n\
+Inserts the text from \"file\" into the text frame \"name\".\n\
+Text must be UTF encoded (see setText() as reference). If \"name\" is\n\
+not given the currently selected Item is used.\n\
+"));
+/*! Insert HTML text */
+PyObject *scribus_inserthtmltext(PyObject * /*self*/, PyObject* args);
+
 /*! docstring */
 PyDoc_STRVAR(scribus_setfont__doc__,
 QT_TR_NOOP("setFont(\"font\", [\"name\"])\n\
@@ -160,6 +217,19 @@ May throw ValueError if the font cannot be found.\n\
 /*! Set font */
 PyObject *scribus_setfont(PyObject * /*self*/, PyObject* args);
 
+/*! docstring */
+PyDoc_STRVAR(scribus_setfontfeatures__doc__,
+QT_TR_NOOP("setFontFeatures(\"fontfeature\", [\"name\"])\n\
+\n\
+Sets the font features of the text frame \"name\" to \"fontfeature\". If there is some text\n\
+selected only the selected text is changed.  If \"name\" is not given the\n\
+currently selected item is used.\n\
+\n\
+May throw ValueError if the font cannot be found.\n\
+"));
+/*! Set font features */
+PyObject *scribus_setfontfeatures(PyObject * /*self*/, PyObject* args);
+
 /*! docstring */
 PyDoc_STRVAR(scribus_setfontsize__doc__,
 QT_TR_NOOP("setFontSize(size, [\"name\"])\n\
@@ -187,6 +257,20 @@ May throw ValueError if the line spacing is out of bounds.\n\
 /*! Set line space */
 PyObject *scribus_setlinespace(PyObject * /*self*/, PyObject* args);
 
+/*! docstring */
+PyDoc_STRVAR(scribus_setlinespacemode__doc__,
+QT_TR_NOOP("setLineSpacingMode(mode, [\"name\"])\n\
+\n\
+Sets the line spacing mode of the text frame \"name\" to \"mode\".\n\
+If \"name\" is not given the currently selected\n\
+item is used.\n\
+Mode values are the same as in createParagraphStyle.\n\
+\n\
+May throw ValueError if the mode is out of bounds.\n\
+"));
+/*! Set line space mode */
+PyObject *scribus_setlinespacemode(PyObject * /*self*/, PyObject* args);
+
 /*! docstring */
 PyDoc_STRVAR(scribus_settextdistances__doc__,
 QT_TR_NOOP("setTextDistances(left, right, top, bottom, [\"name\"])\n\
@@ -235,7 +319,33 @@ be one of the ALIGN_ constants defined in this module - see dir(scribus).\n\
 May throw ValueError for an invalid alignment constant.\n\
 "));
 /*! Set alignt */
-PyObject *scribus_setalign(PyObject * /*self*/, PyObject* args);
+PyObject *scribus_setalignment(PyObject * /*self*/, PyObject* args);
+
+/*! docstring */
+PyDoc_STRVAR(scribus_settextverticalalignment__doc__,
+	QT_TR_NOOP("setTextVerticalAlignment(align, [\"name\"])\n\
+\n\
+Sets the vertical alignment of text inside text frame \"name\" to the specified\n\
+alignment. If \"name\" is not given the currently selected item is used. \"align\" \n\
+should be one of the ALIGNV constants defined in this module - see dir(scribus).\n\
+\n\
+May throw ValueError for an invalid alignment constant.\n\
+"));
+/*! Set vertical alignment */
+PyObject *scribus_settextverticalalignment(PyObject * /*self*/, PyObject* args);
+
+/*! docstring */
+PyDoc_STRVAR(scribus_setdirection__doc__,
+QT_TR_NOOP("setTextDirection(direction, [\"name\"])\n\
+\n\
+Sets the text direction of the text frame \"name\" to the specified direction.\n\
+If \"name\" is not given the currently selected item is used. \"direction\" should\n\
+be one of the DIRECTION_ constants defined in this module - see dir(scribus).\n\
+\n\
+May throw ValueError for an invalid direction constant.\n\
+"));
+/*! Set direction */
+PyObject *scribus_setdirection(PyObject * /*self*/, PyObject* args);
 
 /*! docstring */
 PyDoc_STRVAR(scribus_selecttext__doc__,
