@@ -18,13 +18,11 @@ for which a new license (GPL+exception) is in place.
 #include "pslib.h"
 #include "scpaths.h"
 #include "scribuscore.h"
+#include "scribusdoc.h"
 #include "util_file.h"
 #include "util_ghostscript.h"
 #include "util_printer.h"
 
-#ifdef HAVE_CUPS
-#include <cups/cups.h>
-#endif
 // these functions are located at utils.cpp
 bool SCRIBUS_API loadText(QString nam, QString *Buffer);
 void SCRIBUS_API ReOrderText(ScribusDoc *doc, ScribusView *view);
@@ -67,49 +65,47 @@ static void Printer_dealloc(Printer* self)
 static PyObject * Printer_new(PyTypeObject *type, PyObject * /*args*/, PyObject * /*kwds*/)
 {
 // do not create new object if there is no opened document
-	if (!ScCore->primaryMainWindow()->HaveDoc) {
-		PyErr_SetString(PyExc_SystemError, "Need to open document first");
-		return NULL;
+	if (!checkHaveDocument()) {
+		return nullptr;
 	}
 
-	Printer *self;
-	self = (Printer *)type->tp_alloc(type, 0);
-	if (self != NULL) {
+	Printer *self = (Printer *)type->tp_alloc(type, 0);
+	if (self != nullptr) {
 // set allPrinters attribute
 		self->allPrinters = PyList_New(0);
-		if (self->allPrinters == NULL){
+		if (self->allPrinters == nullptr){
 			Py_DECREF(self);
-			return NULL;
+			return nullptr;
 		}
 // set printer attribute
 		self->printer = PyString_FromString("");
-		if (self->printer == NULL){
+		if (self->printer == nullptr){
 			Py_DECREF(self);
-			return NULL;
+			return nullptr;
 		}
 // set file attribute
 		self->file = PyString_FromString("");
-		if (self->file == NULL){
+		if (self->file == nullptr){
 			Py_DECREF(self);
-			return NULL;
+			return nullptr;
 		}
 // set cmd attribute
 		self->cmd = PyString_FromString("");
-		if (self->cmd == NULL){
+		if (self->cmd == nullptr){
 			Py_DECREF(self);
-			return NULL;
+			return nullptr;
 		}
 // set pages attribute
 		self->pages = PyList_New(0);
-		if (self->pages == NULL){
+		if (self->pages == nullptr){
 			Py_DECREF(self);
-			return NULL;
+			return nullptr;
 		}
 // set separation attribute
 		self->separation = PyString_FromString("No");
-		if (self->separation == NULL){
+		if (self->separation == nullptr){
 			Py_DECREF(self);
-			return NULL;
+			return nullptr;
 		}
 // set color attribute
 		self->color = 1;
@@ -131,6 +127,9 @@ static PyObject * Printer_new(PyTypeObject *type, PyObject * /*args*/, PyObject
 
 static int Printer_init(Printer *self, PyObject * /*args*/, PyObject * /*kwds*/)
 {
+	if (!checkHaveDocument()) {
+		return -1;
+	}
 // pool system for installed printers
 // most code is stolen and little adopted from druck.cpp
 	PyObject *allPrinters = PyList_New(0);
@@ -154,20 +153,20 @@ static int Printer_init(Printer *self, PyObject * /*args*/, PyObject * /*kwds*/)
 	PyList_Append(self->allPrinters, tmp2);
 	Py_DECREF(tmp2);
 // as defaut set to print into file
-	PyObject *printer = NULL;
+	PyObject *printer = nullptr;
 	printer = PyString_FromString("File");
 	if (printer){
 		Py_DECREF(self->printer);
 		self->printer = printer;
 	}
 // set defaul name of file to print into
-	QString tf = ScCore->primaryMainWindow()->doc->PDF_Options.fileName;
+	QString tf(ScCore->primaryMainWindow()->doc->pdfOptions().fileName);
 	if (tf.isEmpty()) {
-		QFileInfo fi = QFileInfo(ScCore->primaryMainWindow()->doc->DocName);
+		QFileInfo fi = QFileInfo(ScCore->primaryMainWindow()->doc->documentFileName());
 		tf = fi.path()+"/"+fi.baseName()+".pdf";
 	}
-	PyObject *file = NULL;
-	file = PyString_FromString(tf.toAscii());
+	PyObject *file = nullptr;
+	file = PyString_FromString(tf.toLatin1());
 	if (file){
 		Py_DECREF(self->file);
 		self->file = file;
@@ -176,7 +175,7 @@ static int Printer_init(Printer *self, PyObject * /*args*/, PyObject * /*kwds*/)
 		return -1;
 	}
 // alternative printer commands default to ""
-	PyObject *cmd = NULL;
+	PyObject *cmd = nullptr;
 	cmd = PyString_FromString("");
 	if (cmd){
 		Py_DECREF(self->cmd);
@@ -184,25 +183,21 @@ static int Printer_init(Printer *self, PyObject * /*args*/, PyObject * /*kwds*/)
 	}
 // if document exist when created Printer instance
 // set to print all pages
-	PyObject *pages = NULL;
-	int num = 0;
-	if (ScCore->primaryMainWindow()->HaveDoc)
-		// which one should I use ???
-		// new = ScCore->primaryMainWindow()->view->Pages.count()
-		num = ScCore->primaryMainWindow()->doc->Pages->count();
+	PyObject *pages = nullptr;
+	int num = ScCore->primaryMainWindow()->doc->Pages->count();
 	pages = PyList_New(num);
 	if (pages){
 		Py_DECREF(self->pages);
 		self->pages = pages;
 	}
 	for (int i = 0; i<num; i++) {
-		PyObject *tmp=NULL;
+		PyObject *tmp=nullptr;
 		tmp = PyInt_FromLong((long)i+1L); // instead of 1 put here first page number
 		if (tmp)
 			PyList_SetItem(self->pages, i, tmp);
 	}
 // do not print separation
-	PyObject *separation = NULL;
+	PyObject *separation = nullptr;
 	separation = PyString_FromString("No");
 	if (separation){
 		Py_DECREF(self->separation);
@@ -233,7 +228,7 @@ static PyMemberDef Printer_members[] = {
 	{const_cast<char*>("mph"), T_INT, offsetof(Printer, mph), 0, const_cast<char*>("Mirror Pages Horizontal\n\tTrue\n\tFalse  --  Default")},
 	{const_cast<char*>("mpv"), T_INT, offsetof(Printer, mpv), 0, const_cast<char*>("Mirror Pages Vertical\n\t True\n\tFalse  --  Default")},
 	{const_cast<char*>("ucr"), T_INT, offsetof(Printer, ucr), 0, const_cast<char*>("Apply Under Color Removal\n\tTrue  --  Default\n\tFalse")},
-	{NULL, 0, 0, 0, NULL} // sentinel
+	{nullptr, 0, 0, 0, nullptr} // sentinel
 };
 
 /* Here begins Getter & Setter functions */
@@ -258,7 +253,7 @@ static PyObject *Printer_getprinter(Printer *self, void * /*closure*/)
 
 static int Printer_setprinter(Printer *self, PyObject *value, void * /*closure*/)
 {
-	if (value == NULL) {
+	if (value == nullptr) {
 		PyErr_SetString(PyExc_TypeError, "Cannot delete 'printer' attribute.");
 		return -1;
 	}
@@ -289,7 +284,7 @@ static PyObject *Printer_getfile(Printer *self, void * /*closure*/)
 
 static int Printer_setfile(Printer *self, PyObject *value, void * /*closure*/)
 {
-	if (value == NULL) {
+	if (value == nullptr) {
 		PyErr_SetString(PyExc_TypeError, "Cannot delete 'file' attribute.");
 		return -1;
 	}
@@ -311,7 +306,7 @@ static PyObject *Printer_getcmd(Printer *self, void * /*closure*/)
 
 static int Printer_setcmd(Printer *self, PyObject *value, void * /*closure*/)
 {
-	if (value == NULL) {
+	if (value == nullptr) {
 		PyErr_SetString(PyExc_TypeError, "Cannot delete 'cmd' attribute.");
 		return -1;
 	}
@@ -333,7 +328,7 @@ static PyObject *Printer_getpages(Printer *self, void * /*closure*/)
 
 static int Printer_setpages(Printer *self, PyObject *value, void * /*closure*/)
 {
-	if (value == NULL) {
+	if (value == nullptr) {
 		PyErr_SetString(PyExc_TypeError, "Cannot delete 'pages' attribute.");
 		return -1;
 	}
@@ -367,7 +362,7 @@ static PyObject *Printer_getseparation(Printer *self, void * /*closure*/)
 
 static int Printer_setseparation(Printer *self, PyObject *value, void * /*closure*/)
 {
-	if (value == NULL) {
+	if (value == nullptr) {
 		PyErr_SetString(PyExc_TypeError, "Cannot delete 'separation' attribute.");
 		return -1;
 	}
@@ -383,24 +378,23 @@ static int Printer_setseparation(Printer *self, PyObject *value, void * /*closur
 
 
 static PyGetSetDef Printer_getseters [] = {
-	{const_cast<char*>("allPrinters"), (getter)Printer_getallPrinters, (setter)Printer_setallPrinters, const_cast<char*>("List of installed printers  --  read only"), NULL},
-	{const_cast<char*>("printer"), (getter)Printer_getprinter, (setter)Printer_setprinter, const_cast<char*>("Name of printer to use.\nDefault is 'File' for printing into file"), NULL},
-	{const_cast<char*>("file"), (getter)Printer_getfile, (setter)Printer_setfile, const_cast<char*>("Name of file to print into"), NULL},
-	{const_cast<char*>("cmd"), (getter)Printer_getcmd, (setter)Printer_setcmd, const_cast<char*>("Alternative Printer Command"), NULL},
-	{const_cast<char*>("pages"), (getter)Printer_getpages, (setter)Printer_setpages, const_cast<char*>("List of pages to be printed"), NULL},
-	{const_cast<char*>("separation"), (getter)Printer_getseparation, (setter)Printer_setseparation, const_cast<char*>("Print separationl\n\t 'No'  -- Default\n\t 'All'\n\t 'Cyan'\n\t 'Magenta'\n\t 'Yellow'\n\t 'Black'\nBeware of misspelling because check is not performed"), NULL},
-	{NULL, NULL, NULL, NULL, NULL}  // sentinel
+	{const_cast<char*>("allPrinters"), (getter)Printer_getallPrinters, (setter)Printer_setallPrinters, const_cast<char*>("List of installed printers  --  read only"), nullptr},
+	{const_cast<char*>("printer"), (getter)Printer_getprinter, (setter)Printer_setprinter, const_cast<char*>("Name of printer to use.\nDefault is 'File' for printing into file"), nullptr},
+	{const_cast<char*>("file"), (getter)Printer_getfile, (setter)Printer_setfile, const_cast<char*>("Name of file to print into"), nullptr},
+	{const_cast<char*>("cmd"), (getter)Printer_getcmd, (setter)Printer_setcmd, const_cast<char*>("Alternative Printer Command"), nullptr},
+	{const_cast<char*>("pages"), (getter)Printer_getpages, (setter)Printer_setpages, const_cast<char*>("List of pages to be printed"), nullptr},
+	{const_cast<char*>("separation"), (getter)Printer_getseparation, (setter)Printer_setseparation, const_cast<char*>("Print separationl\n\t 'No'  -- Default\n\t 'All'\n\t 'Cyan'\n\t 'Magenta'\n\t 'Yellow'\n\t 'Black'\nBeware of misspelling because check is not performed"), nullptr},
+	{nullptr, nullptr, nullptr, nullptr, nullptr}  // sentinel
 };
 
 // Here we actually print
 static PyObject *Printer_print(Printer *self)
 {
-	if (!ScCore->primaryMainWindow()->HaveDoc) {
-		PyErr_SetString(PyExc_SystemError, "Need to open documetnt first");
-		return NULL;
+	if (!checkHaveDocument()) {
+		return nullptr;
 	}
 // copied from void ScribusMainWindow::slotFilePrint() in file scribus.cpp
-	QString fna, prn, cmd, scmd, cc, data, SepName;
+	QString fna, prn, cmd, cc, SepName;
 	QString printcomm;
 	bool fil, PSfile;
 	PSfile = false;
@@ -408,19 +402,19 @@ static PyObject *Printer_print(Printer *self)
 //    ReOrderText(ScCore->primaryMainWindow()->doc, ScCore->primaryMainWindow()->view);
 	prn = QString(PyString_AsString(self->printer));
 	fna = QString(PyString_AsString(self->file));
-	fil = (QString(PyString_AsString(self->printer)) == QString("File")) ? true : false;
+	fil = QString(PyString_AsString(self->printer)) == QString("File");
 	std::vector<int> pageNs;
 	PrintOptions options;
 	for (int i = 0; i < PyList_Size(self->pages); ++i) {
 		options.pageNumbers.push_back((int)PyInt_AsLong(PyList_GetItem(self->pages, i)));
 	}
-	int Nr = (self->copies < 1) ? 1 : self->copies;
+	int copyCount = (self->copies < 1) ? 1 : self->copies;
 	SepName = QString(PyString_AsString(self->separation));
 	options.printer   = prn;
 	options.prnEngine = (PrintEngine) self->pslevel;
 	options.toFile    = fil;
 	options.separationName = SepName;
-	options.outputSeparations = (SepName == QString("No")) ?  false : true;
+	options.outputSeparations = SepName != QString("No");
 	options.useColor = self->color;
 	options.mirrorH  = self->mph;
 	options.mirrorV  = self->mpv;
@@ -430,17 +424,13 @@ static PyObject *Printer_print(Printer *self)
 	options.registrationMarks = false;
 	options.colorMarks = false;
 	options.markOffset = 0.0;
-	options.bleeds.Top    = 0.0;
-	options.bleeds.Left   = 0.0;
-	options.bleeds.Right  = 0.0;
-	options.bleeds.Bottom = 0.0;
+	options.bleeds.set(0, 0, 0, 0);
 	if (!PrinterUtil::checkPrintEngineSupport(options.printer, options.prnEngine, options.toFile))
 		options.prnEngine = PrinterUtil::getDefaultPrintEngine(options.printer, options.toFile);
 	printcomm = QString(PyString_AsString(self->cmd));
 	QMap<QString, QMap<uint, FPointArray> > ReallyUsed;
 	ReallyUsed.clear();
 	ScCore->primaryMainWindow()->doc->getUsedFonts(ReallyUsed);
-	PrefsManager *prefsManager=PrefsManager::instance();
 
 #if defined(_WIN32)
 	if (!options.toFile)
@@ -458,11 +448,11 @@ static PyObject *Printer_print(Printer *self)
 	}
 #endif
 
-	PSLib *dd = new PSLib(options, true, prefsManager->appPrefs.AvailFonts, ReallyUsed, ScCore->primaryMainWindow()->doc->PageColors, false, true);
-	if (dd != NULL)
+	PSLib *dd = new PSLib(options, true, PrefsManager::instance().appPrefs.fontPrefs.AvailFonts, ReallyUsed, ScCore->primaryMainWindow()->doc->PageColors, false, true);
+	if (dd != nullptr)
 	{
 		if (!fil)
-			fna = QDir::toNativeSeparators(ScPaths::getTempFileDir()+"/tmp.ps");
+			fna = QDir::toNativeSeparators(ScPaths::tempFileDir()+"/tmp.ps");
 		PSfile = dd->PS_set_file(fna);
 		fna = QDir::toNativeSeparators(fna);
 		if (PSfile)
@@ -476,8 +466,8 @@ static PyObject *Printer_print(Printer *self)
 				{
 					QString tmp;
 					QStringList opts;
-					opts.append( QString("-dDEVICEWIDTHPOINTS=%1").arg(tmp.setNum(ScCore->primaryMainWindow()->doc->pageWidth)) );
-					opts.append( QString("-dDEVICEHEIGHTPOINTS=%1").arg(tmp.setNum(ScCore->primaryMainWindow()->doc->pageHeight)) );
+					opts.append( QString("-dDEVICEWIDTHPOINTS=%1").arg(tmp.setNum(ScCore->primaryMainWindow()->doc->pageWidth())) );
+					opts.append( QString("-dDEVICEHEIGHTPOINTS=%1").arg(tmp.setNum(ScCore->primaryMainWindow()->doc->pageHeight())) );
 					convertPS2PS(fna, fna+".tmp", opts, options.prnEngine);
 					moveFile( fna + ".tmp", fna );
 				}
@@ -495,12 +485,8 @@ static PyObject *Printer_print(Printer *self)
 				else
 				{
 					cmd = "lpr -P" + prn;
-					if (Nr > 1)
-						cmd += " -#" + cc.setNum(Nr);
-#ifdef HAVE_CUPS
-// This need yet to be implemented by object Printer
-//					cmd += printer->PrinterOpts;
-#endif
+					if (copyCount > 1)
+						cmd += " -#" + cc.setNum(copyCount);
 					cmd += " "+fna;
 				}
 				system(cmd.toLocal8Bit().constData());
@@ -510,52 +496,50 @@ static PyObject *Printer_print(Printer *self)
 		else {
 			delete dd;
 			PyErr_SetString(PyExc_SystemError, "Printing failed");
-			return NULL;
+			return nullptr;
 		}
 		delete dd;
 	}
-//	Py_INCREF(Py_None);
-//	return Py_None;
 	Py_RETURN_NONE;
 }
 
 static PyMethodDef Printer_methods[] = {
 	{const_cast<char*>("printNow"), (PyCFunction)Printer_print, METH_NOARGS, const_cast<char*>("Prints selected pages.")},
-	{NULL, (PyCFunction)(0), 0, NULL} // sentinel
+	{nullptr, (PyCFunction)(nullptr), 0, nullptr} // sentinel
 };
 
 PyTypeObject Printer_Type = {
-	PyObject_HEAD_INIT(NULL)   // PyObject_VAR_HEAD
+	PyObject_HEAD_INIT(nullptr)   // PyObject_VAR_HEAD
 	0,			 //
-	const_cast<char*>("Printer"), // char *tp_name; /* For printing, in format "<module>.<name>" */
+	const_cast<char*>("scribus.Printer"), // char *tp_name; /* For printing, in format "<module>.<name>" */
 	sizeof(Printer),   // int tp_basicsize, /* For allocation */
 	0,		       // int tp_itemsize; /* For allocation */
 
 	/* Methods to implement standard operations */
 
 	(destructor) Printer_dealloc, //     destructor tp_dealloc;
-	0, //     printfunc tp_print;
-	0, //     getattrfunc tp_getattr;
-	0, //     setattrfunc tp_setattr;
-	0, //     cmpfunc tp_compare;
-	0, //     reprfunc tp_repr;
+	nullptr, //     printfunc tp_print;
+	nullptr, //     getattrfunc tp_getattr;
+	nullptr, //     setattrfunc tp_setattr;
+	nullptr, //     cmpfunc tp_compare;
+	nullptr, //     reprfunc tp_repr;
 
 	/* Method suites for standard classes */
 
-	0, //     PyNumberMethods *tp_as_number;
-	0, //     PySequenceMethods *tp_as_sequence;
-	0, //     PyMappingMethods *tp_as_mapping;
+	nullptr, //     PyNumberMethods *tp_as_number;
+	nullptr, //     PySequenceMethods *tp_as_sequence;
+	nullptr, //     PyMappingMethods *tp_as_mapping;
 
 	/* More standard operations (here for binary compatibility) */
 
-	0, //     hashfunc tp_hash;
-	0, //     ternaryfunc tp_call;
-	0, //     reprfunc tp_str;
-	0, //     getattrofunc tp_getattro;
-	0, //     setattrofunc tp_setattro;
+	nullptr, //     hashfunc tp_hash;
+	nullptr, //     ternaryfunc tp_call;
+	nullptr, //     reprfunc tp_str;
+	nullptr, //     getattrofunc tp_getattro;
+	nullptr, //     setattrofunc tp_setattro;
 
 	/* Functions to access object as input/output buffer */
-	0, //     PyBufferProcs *tp_as_buffer;
+	nullptr, //     PyBufferProcs *tp_as_buffer;
 
 	/* Flags to define presence of optional/expanded features */
 	Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE,    // long tp_flags;
@@ -564,43 +548,43 @@ PyTypeObject Printer_Type = {
 
 	/* Assigned meaning in release 2.0 */
 	/* call function for all accessible objects */
-	0, //     traverseproc tp_traverse;
+	nullptr, //     traverseproc tp_traverse;
 
 	/* delete references to contained objects */
-	0, //     inquiry tp_clear;
+	nullptr, //     inquiry tp_clear;
 
 	/* Assigned meaning in release 2.1 */
 	/* rich comparisons */
-	0, //     richcmpfunc tp_richcompare;
+	nullptr, //     richcmpfunc tp_richcompare;
 
 	/* weak reference enabler */
 	0, //     long tp_weaklistoffset;
 
 	/* Added in release 2.2 */
 	/* Iterators */
-	0, //     getiterfunc tp_iter;
-	0, //     iternextfunc tp_iternext;
+	nullptr, //     getiterfunc tp_iter;
+	nullptr, //     iternextfunc tp_iternext;
 
 	/* Attribute descriptor and subclassing stuff */
 	Printer_methods, //     struct PyMethodDef *tp_methods;
 	Printer_members, //     struct PyMemberDef *tp_members;
 	Printer_getseters, //     struct PyGetSetDef *tp_getset;
-	0, //     struct _typeobject *tp_base;
-	0, //     PyObject *tp_dict;
-	0, //     descrgetfunc tp_descr_get;
-	0, //     descrsetfunc tp_descr_set;
+	nullptr, //     struct _typeobject *tp_base;
+	nullptr, //     PyObject *tp_dict;
+	nullptr, //     descrgetfunc tp_descr_get;
+	nullptr, //     descrsetfunc tp_descr_set;
 	0, //     long tp_dictoffset;
 	(initproc)Printer_init, //     initproc tp_init;
-	0, //     allocfunc tp_alloc;
+	nullptr, //     allocfunc tp_alloc;
 	Printer_new, //     newfunc tp_new;
-	0, //     freefunc tp_free; /* Low-level free-memory routine */
-	0, //     inquiry tp_is_gc; /* For PyObject_IS_GC */
-	0, //     PyObject *tp_bases;
-	0, //     PyObject *tp_mro; /* method resolution order */
-	0, //     PyObject *tp_cache;
-	0, //     PyObject *tp_subclasses;
-	0, //     PyObject *tp_weaklist;
-	0, //     destructor tp_del;
+	nullptr, //     freefunc tp_free; /* Low-level free-memory routine */
+	nullptr, //     inquiry tp_is_gc; /* For PyObject_IS_GC */
+	nullptr, //     PyObject *tp_bases;
+	nullptr, //     PyObject *tp_mro; /* method resolution order */
+	nullptr, //     PyObject *tp_cache;
+	nullptr, //     PyObject *tp_subclasses;
+	nullptr, //     PyObject *tp_weaklist;
+	nullptr, //     destructor tp_del;
 
 #ifdef COUNT_ALLOCS
 	/* these must be last and never explicitly initialized */
