@@ -28,6 +28,11 @@ HunspellDialog::HunspellDialog(QWidget *parent, ScribusDoc *doc, StoryText *iTex
 	m_iText=iText;
 	m_returnToDefaultLang=false;
 	m_primaryLangIndex=0;
+
+	m_dictionaryMap = nullptr;
+	m_hspellerMap = nullptr;
+	m_wfList = nullptr;
+	m_wfListIndex = 0;
 }
 
 void HunspellDialog::set(QMap<QString, QString>* dictionaryMap, QMap<QString, HunspellDict*> *hspellerMap, QList<WordsFound> *wfList)
@@ -37,17 +42,21 @@ void HunspellDialog::set(QMap<QString, QString>* dictionaryMap, QMap<QString, Hu
 	m_wfList=wfList;
 	bool b=languagesComboBox->blockSignals(true);
 	languagesComboBox->clear();
-	QMap<QString, QString>::iterator it = m_dictionaryMap->begin();
-	while (it != m_dictionaryMap->end())
+
+	QStringList langsToAdd;
+	auto it = m_dictionaryMap->cbegin();
+	while (it != m_dictionaryMap->cend())
 	{
 		QString lang=LanguageManager::instance()->getLangFromAbbrev(it.key(), true);
-		languagesComboBox->addItem(!lang.isEmpty() ? lang : it.key());
+		if (!langsToAdd.contains(lang))
+			langsToAdd<<(!lang.isEmpty() ? lang : it.key());
 		++it;
 	}
+	languagesComboBox->addItems(langsToAdd);
 	languagesComboBox->setCurrentIndex(0);
 	m_primaryLangIndex=0;
 	languagesComboBox->blockSignals(b);
-	wfListIndex=0;
+	m_wfListIndex=0;
 	goToNextWord(0);
 }
 
@@ -67,15 +76,15 @@ void HunspellDialog::goToNextWord(int i)
 		languagesComboBox->blockSignals(b);
 	}
 	if (i>=0)
-		wfListIndex=i;
+		m_wfListIndex=i;
 	else
 	{
 		do {
-			++wfListIndex;
-		} while (wfListIndex<m_wfList->count() && (m_wfList->at(wfListIndex).changed || m_wfList->at(wfListIndex).ignore));
-		//qDebug()<<"selected word index"<<wfListIndex;
+			++m_wfListIndex;
+		} while (m_wfListIndex<m_wfList->count() && (m_wfList->at(m_wfListIndex).changed || m_wfList->at(m_wfListIndex).ignore));
+		//qDebug()<<"selected word index"<<m_wfListIndex;
 	}
-	if (wfListIndex>=m_wfList->count())
+	if (m_wfListIndex>=m_wfList->count())
 	{
 		statusLabel->setText(tr("Spelling check complete"));
 		suggestionsListWidget->clear();
@@ -86,9 +95,8 @@ void HunspellDialog::goToNextWord(int i)
 		ignoreAllPushButton->setEnabled(false);
 		return;
 	}
-	else
-		statusLabel->setText("");
-	currWF=m_wfList->at(wfListIndex);
+	statusLabel->setText("");
+	currWF=m_wfList->at(m_wfListIndex);
 	setLanguageCombo(currWF.lang);
 	updateSuggestions(currWF.replacements);
 
@@ -97,14 +105,13 @@ void HunspellDialog::goToNextWord(int i)
 	sentence.insert(currWF.end-sentencePos+currWF.changeOffset,"</b></font>");
 	sentence.insert(currWF.start-sentencePos+currWF.changeOffset,"<font color=red><b>");
 	sentenceTextEdit->setText(sentence);
-
 }
 
 void HunspellDialog::ignoreAllWords()
 {
-	if (wfListIndex < 0 || wfListIndex >= m_wfList->count())
+	if (m_wfListIndex < 0 || m_wfListIndex >= m_wfList->count())
 		return;
-	QString wordToIgnore = m_wfList->at(wfListIndex).w;
+	QString wordToIgnore = m_wfList->at(m_wfListIndex).w;
 	//Do we start from 0 or from the instance of the word where we are... 0 for now
 	for (int i = 0; i < m_wfList->count(); ++i)
 		if (m_wfList->at(i).w == wordToIgnore)
@@ -115,27 +122,27 @@ void HunspellDialog::ignoreAllWords()
 void HunspellDialog::changeWord()
 {
 	//If we have ignored a word or its already changed, skip to the next.
-	if(m_wfList->at(wfListIndex).ignore || m_wfList->at(wfListIndex).changed)
+	if (m_wfList->at(m_wfListIndex).ignore || m_wfList->at(m_wfListIndex).changed)
 		goToNextWord();
-	replaceWord(wfListIndex);
+	replaceWord(m_wfListIndex);
 	goToNextWord();
 }
 
 void HunspellDialog::changeAllWords()
 {
-	if(m_wfList->at(wfListIndex).ignore && !m_wfList->at(wfListIndex).changed)
+	if (m_wfList->at(m_wfListIndex).ignore && !m_wfList->at(m_wfListIndex).changed)
 		return;
-	QString wordToChange=m_wfList->at(wfListIndex).w;
+	QString wordToChange=m_wfList->at(m_wfListIndex).w;
 	//Do we start from 0 or from the instance of the word where we are... 0 for now
-	for(int i=0;i<m_wfList->count();++i)
-		if(m_wfList->at(i).w==wordToChange)
+	for (int i=0;i<m_wfList->count();++i)
+		if (m_wfList->at(i).w==wordToChange)
 			replaceWord(i);
 	goToNextWord();
 }
 
 void HunspellDialog::replaceWord(int i)
 {
-	//TODO: rehypenate after the replacement
+	//TODO: rehyphenate after the replacement
 	QString newText(suggestionsListWidget->currentItem()->text());
 	int lengthDiff=m_iText->replaceWord(m_wfList->at(i).start+m_wfList->at(i).changeOffset, newText);
 	if (lengthDiff!=0)
@@ -150,50 +157,48 @@ void HunspellDialog::replaceWord(int i)
 void HunspellDialog::languageComboChanged(const QString &newLanguage)
 {
 	m_returnToDefaultLang=true;
-	QString wordLang = LanguageManager::instance()->getAbbrevFromLang(newLanguage, true, false, 1);
+	QString wordLang=LanguageManager::instance()->getAbbrevFromLang(newLanguage, false);
 	if (!m_hspellerMap->contains(wordLang) )
 	{
 		//qDebug()<<"hspeller"<<wordLang<<"does not exist";
-		wordLang=LanguageManager::instance()->getAbbrevFromLang(newLanguage, true, false, 2);
-		if (!m_hspellerMap->contains(wordLang) )
-		//{
-			//qDebug()<<"hspeller"<<wordLang<<"ALTERNATIVE does not exist";
-			return;
-		//}
-		//else
-			//qDebug()<<"hspeller"<<wordLang<<"ALTERNATIVE exists";
-	}
-
-	if (m_wfList->count() == 0)
 		return;
-	if (wfListIndex >= m_wfList->count())
-		wfListIndex = 0;
-
-	QString word = m_wfList->at(wfListIndex).w;
-	if ((*m_hspellerMap)[wordLang]->spell(word) == 0)
+	}
+	//qDebug()<<wordLang<<newLanguage;
+	if (m_wfList->count()==0)
+		return;
+	if (m_wfListIndex>=m_wfList->count())
+		m_wfListIndex=0;
+	QString word=m_wfList->at(m_wfListIndex).w;
+	if ((*m_hspellerMap)[wordLang]->spell(word)==0)
 	{
 		QStringList replacements = (*m_hspellerMap)[wordLang]->suggest(word);
 		updateSuggestions(replacements);
 	}
 	else
 	{
-		(*m_wfList)[wfListIndex].changed = true;
-		m_docChanged = true;
+		(*m_wfList)[m_wfListIndex].changed=true;
+		m_docChanged=true;
 		goToNextWord();
 	}
 }
 
 void HunspellDialog::setLanguageCombo(const QString &newLangAbbrev)
 {
-	QMap<QString, QString>::iterator it = m_dictionaryMap->begin();
-	int i = 0;
-	while (it != m_dictionaryMap->end())
+	bool found = false;
+
+	auto it = m_dictionaryMap->cbegin();
+	while (it != m_dictionaryMap->cend())
 	{
 		if (it.key() == newLangAbbrev)
+		{
+			found=true;
 			break;
-		++i;
+		}
 		++it;
 	}
+	int i = 0;
+	if (found)
+		i = languagesComboBox->findText(LanguageManager::instance()->getLangFromAbbrev(newLangAbbrev,true));
 	bool b = languagesComboBox->blockSignals(true);
 	languagesComboBox->setCurrentIndex(i);
 	languagesComboBox->blockSignals(b);
