@@ -14,6 +14,7 @@ for which a new license (GPL+exception) is in place.
 #include <QAction>
 #include <QByteArray>
 #include <QDebug>
+#include <QFlags>
 #include <QKeySequence>
 #include <QList>
 #include <QPolygon>
@@ -25,157 +26,20 @@ for which a new license (GPL+exception) is in place.
 #include <vector>
 
 #include "scribusapi.h"
-#include "colormgmt/sccolormgmtstructs.h"
+
+#include "annotation.h"
+#include "fpointarray.h"
+#include "pageitem.h"
+#include "margins.h"
+#include "sccolorstructs.h"
+#include "scfonts.h"
 #include "scimagestructs.h"
 #include "sctextstruct.h"
-#include "scfonts.h"
-#include "fpointarray.h"
 #include "vgradient.h"
-#include "annotation.h"
-#include "pageitem.h"
+#include "colormgmt/sccolormgmtstructs.h"
 
 extern bool SCRIBUS_API compareDouble(double, double);
 
-typedef struct
-{
-	int r;
-	int g;
-	int b;
-	void getValues(int& vr, int& vg, int& vb) {vr = r; vg = g; vb = b;}
-} RGBColor;
-
-typedef struct
-{
-	int c;
-	int m;
-	int y;
-	int k;
-	void getValues(int& vc, int& vm, int& vy, int& vk) {vc = c; vm = m; vy = y; vk = k;}
-} CMYKColor;
-
-struct CopyPasteBuffer
-{
-	PageItem::ItemType PType;
-	double Xpos;
-	double Ypos;
-	double Width;
-	double Height;
-	double RadRect;
-	int FrameType;
-	bool ClipEdited;
-	double Pwidth;
-	QString Pcolor;
-	QString Pcolor2;
-	double Shade;
-	double Shade2;
-	int FillRule;
-	QString GrColor;
-	QString GrColor2;
-	int GrShade;
-	int GrShade2;
-	VGradient fill_gradient;
-	int GrType;
-	QString pattern;
-	double patternScaleX;
-	double patternScaleY;
-	double patternOffsetX;
-	double patternOffsetY;
-	double patternRotation;
-	double GrStartX;
-	double GrStartY;
-	double GrEndX;
-	double GrEndY;
-	QString TxtStroke;
-	QString TxtFill;
-	int ShTxtStroke;
-	int ShTxtFill;
-	int TxtScale;
-	int TxtScaleV;
-	int TxTStyle;
-	int TxTBase;
-	int TxtShadowX;
-	int TxtShadowY;
-	int TxtOutline;
-	int TxtUnderPos;
-	int TxtUnderWidth;
-	int TxtStrikePos;
-	int TxtStrikeWidth;
-	double Rot;
-	int PLineArt;
-	int PLineJoin;
-	int PLineEnd;
-	double LineSp;
-	int LineSpMode;
-	int ExtraV;
-	double LocalScX;
-	double LocalScY;
-	double LocalX;
-	double LocalY;
-	bool PicArt;
-	bool flippedH;
-	bool flippedV;
-	bool isPrintable;
-	bool isBookmark;
-	bool m_isAnnotation;
-	Annotation m_annotation;
-	QString AnName;
-	double Extra;
-	double TExtra;
-	double BExtra;
-	double RExtra;
-	FirstLineOffsetPolicy firstLineOffsetP;
-	QString Pfile;
-	QString Pfile2;
-	QString Pfile3;
-	QString IProfile;
-	QString EmProfile;
-	eRenderIntent IRender;
-	bool UseEmbedded;
-	QString itemText;
-	QPolygon Clip;
-	FPointArray PoLine;
-	FPointArray ContourLine;
-	bool PoShow;
-	double BaseOffs;
-	int textPathType;
-	bool textPathFlipped;
-	int TextflowMode;
-	int textAlignment;
-	QString IFont;
-	int ISize;
-	QStack<int> Groups;
-	int LayerNr;
-	bool ScaleType;
-	bool AspectRatio;
-	bool Locked;
-	bool LockRes;
-	double Transparency;
-	double TranspStroke;
-	int TransBlend;
-	int TransBlendS;
-	bool Reverse;
-	QString NamedLStyle;
-	QString Language;
-	QString guiLanguage;
-	int Cols;
-	double ColGap;
-	QList<ParagraphStyle::TabRecord> TabValues;
-	QVector<double> DashValues;
-	double DashOffset;
-	bool isTableItem;
-	bool TopLine;
-	bool LeftLine;
-	bool RightLine;
-	bool BottomLine;
-	int LeftLinkID;
-	int RightLinkID;
-	int TopLinkID;
-	int BottomLinkID;
-	int startArrowIndex;
-	int endArrowIndex;
-	ObjAttrVector pageItemAttributes;
-};
-
 struct CopyContentsBuffer
 {
 	PageItem::ItemType sourceType;
@@ -184,6 +48,7 @@ struct CopyContentsBuffer
 	double LocalScY;
 	double LocalX;
 	double LocalY;
+	double LocalRot;
 	double ItemX;
 	double ItemY;
 	ScImageEffectList effects;
@@ -192,24 +57,6 @@ struct CopyContentsBuffer
 	eRenderIntent renderingIntent;
 };
 
-/** \brief Pagemargins and bleeds*/
-class MarginStruct
-{
-	public:
-		MarginStruct() : Top(0), Left(0), Bottom(0), Right(0) {}
-		MarginStruct(double top, double left, double bottom, double right) :
-			Top(top), Left(left), Bottom(bottom), Right(right) {}
-		MarginStruct(const MarginStruct& rhs) {Top=rhs.Top;Bottom=rhs.Bottom;Left=rhs.Left;Right=rhs.Right;}
-		void set(double top, double left, double bottom, double right) {Top=top;Bottom=bottom;Left=left;Right=right;}
-		void resetToZero() {Top=0.0;Bottom=0.0;Left=0.0;Right=0.0;}
-		bool hasNonZeroValue() { return Top!=0.0 || Bottom!=0.0 || Left!=0.0 || Right!=0.0;}
-		void print() {qDebug() << Top << Left << Bottom << Right;}
-		double Top;
-		double Left;
-		double Bottom;
-		double Right;
-};
-
 struct DocPagesSetup
 {
 	DocPagesSetup() : pageArrangement(0), firstPageLocation(0), firstPageNumber(0), orientation(0), autoTextFrames(false), columnDistance(0), columnCount(1) {}
@@ -285,6 +132,20 @@ struct SingleLine
 	}
 };
 
+struct Bullet  //used by style reader while importing ODT files
+{
+	QString name;
+	QString charStr;
+	double indent;
+	double firstLineIndent;
+	double tabPosition;
+	CharStyle* style;
+	Bullet() : indent(0.0), firstLineIndent(0.0), tabPosition(0.0), style(nullptr) {}
+	Bullet(QString n, QString ch) : name(n), charStr(ch), indent(0.0), firstLineIndent(0.0), tabPosition(0.0), style(nullptr) {}
+	Bullet(QString n, QString ch, double first, double ind, double tab, CharStyle* chStyle)
+	    { name = n; charStr = ch, firstLineIndent = first; indent = ind; tabPosition = tab; style = chStyle; }
+};
+
 struct ArrowDesc
 {
 	QString name;
@@ -294,6 +155,13 @@ struct ArrowDesc
 
 struct PDFPresentationData
 {
+	PDFPresentationData()
+	{
+		pageEffectDuration = 1;
+		pageViewDuration = 1;
+		effectType = 0;
+		Dm = M = Di = 0;
+	}
 	int pageEffectDuration;
 	int pageViewDuration;
 	int effectType;
@@ -349,6 +217,7 @@ struct PrintOptions
 	bool includePDFMarks;
 	int  copies;
 	PrintEngine prnEngine;
+	double markLength;
 	double markOffset;
 	MarginStruct bleeds;
 	std::vector<int> pageNumbers;
@@ -373,58 +242,32 @@ public:
 	}
 };
 
-//CB: Using 17 for OffConflictLayers as we moved onto 17 in 150svn
 typedef enum {
-	MissingGlyph=1,
-	TextOverflow=2,
-	ObjectNotOnPage=3,
-	MissingImage=4,
-	ImageDPITooLow=5,
-	Transparency=6,
-	PDFAnnotField=7,
-	PlacedPDF=8,
-	ImageDPITooHigh=9,
-	ImageIsGIF=10,
-	BlendMode=11,
-	WrongFontInAnnotation=12,
-	OffConflictLayers=17
+	MissingGlyph = 1,
+	TextOverflow = 2,
+	ObjectNotOnPage = 3,
+	MissingImage = 4,
+	ImageDPITooLow = 5,
+	Transparency = 6,
+	PDFAnnotField = 7,
+	PlacedPDF = 8,
+	ImageDPITooHigh = 9,
+	ImageIsGIF = 10,
+	BlendMode = 11,
+	WrongFontInAnnotation = 12,
+	NotCMYKOrSpot = 13,
+	DeviceColorsAndOutputIntent = 14,
+	FontNotEmbedded = 15,
+	EmbeddedFontIsOpenType = 16,
+	OffConflictLayers = 17,
+	PartFilledImageFrame = 18,
+	MarksChanged = 19,
+	AppliedMasterDifferentSide = 20,
+	EmptyTextFrame = 21
 } PreflightError;
 
 typedef QMap<PreflightError, int> errorCodes;
 
-enum AppMode
-{
-	modeNormal,
-	modeDrawShapes,
-	modeStoryEditor,
-	modeDrawImage,
-	modeDrawText,
-	modeMagnifier,
-	modeEdit,
-	modeDrawLine,
-	modeRotation,
-	modeLinkFrames,
-	modeUnlinkFrames,
-	modeDrawRegularPolygon,
-	modeDrawBezierLine,
-	modeInsertPDFButton,
-	modeInsertPDFTextfield,
-	modeInsertPDFCheckbox,
-	modeInsertPDFCombobox,
-	modeInsertPDFListbox,
-	modeInsertPDFTextAnnotation,
-	modeInsertPDFLinkAnnotation,
-	modeDrawFreehandLine,
-	modeDrawTable,
-	modePanning,
-	modeMeasurementTool,
-	modeEditGradientVectors,
-	modeEyeDropper,
-	modeCopyProperties,
-	modeEditClip,
-	modeDrawLatex,
-	modeImportObject
-};
 
 struct AlignObjs
 {
@@ -436,7 +279,7 @@ struct AlignObjs
 	double y2;
 	double width;
 	double height;
-	QList<PageItem*> Objects;
+	PageItem* Object;
 };
 
 /*! \brief Human readable orientations */
@@ -447,12 +290,46 @@ enum PageOrientation
 	customPage = 30
 };
 
+/**
+* Flags for ScribusView PageToPixmap()
+*/
+enum PageToPixmapFlag {
+	Pixmap_NoFlags = 0,
+	Pixmap_DrawFrame = 1 << 0,
+	Pixmap_DrawBackground = 1 << 1,
+	Pixmap_DrawWhiteBackground = 1 << 2,
+	Pixmap_DontReloadImages = 1 << 3,
+};
+
+Q_DECLARE_FLAGS(PageToPixmapFlags, PageToPixmapFlag);
+Q_DECLARE_OPERATORS_FOR_FLAGS(PageToPixmapFlags);
+
+enum UpdateRequests
+{
+	reqColorsUpdate = 1,
+	reqCharStylesUpdate = 2,
+	reqParaStylesUpdate = 4,
+	reqTextStylesUpdate = 6,
+	reqArrowStylesUpdate = 8,
+	reqLineStylesUpdate  = 16,
+	reqSymbolsUpdate     = 32,
+	reqDefFontListUpdate = 64,
+	reqDocFontListUpdate = 128,
+	reqStyleComboDocUpdate = 256,
+	reqCmsOptionsUpdate  = 512,
+	reqCustomShapeUpdate = 1024,
+	reqInlinePalUpdate   = 2048,
+	reqMarksUpdate       = 4096,
+	reqNumUpdate         = 8192,
+	reqUpdateAll = 65535
+};
+
 //! \brief Common type for guides list
 typedef QList<double> Guides;
 
 //! \brief from ols scribusXml
-struct Linked 
-{ 
+struct Linked
+{
 	int Start;
 	int StPag;
 };
@@ -465,10 +342,95 @@ public:
 	QString ParaStyle;
 	LastStyles() {
 		StyleStart = 0;
-		ParaStyle = "";
 	}
 };
 
+class AttributeValue
+{
+	public:
+		AttributeValue() : valid(false)
+		{
+		}
+		AttributeValue(const QString& val)
+		{
+			if (val.isEmpty() || (val == ""))
+			{
+				valid = false;
+				value.clear();
+			}
+			else
+			{
+				valid = true;
+				value = val;
+			}
+		}
+		bool valid;
+		QString value;
+};
+
+//TODO: Dict license showing, URL background unzipping, checksumming, pkg mgr platforms warning
+struct DownloadItem
+{
+	QString lang;
+	QString version;
+	QString files;
+	QString extractfiles;
+	QString url;
+	QString desc;
+	QString license;
+	QString filetype;
+	QString type;
+	QString movetofile;
+	QString source;
+	bool download;
+};
+
+struct DownloadData
+{
+	int id;
+	QUrl url;
+	QString name;
+	QString downloadLocation;
+	QString destinationLocation;
+	QString destinationName;
+	typedef enum {New, Started, Paused, Finished, Successful, Failed} DownloadState;
+	DownloadState state;
+};
+
+enum
+{
+	GS_ALL,
+	GS_MARGINS,
+	GS_FRAMES,
+	GS_GRID,
+	GS_GUIDES,
+	GS_BASELINE,
+	GS_LINKS,
+	GS_CONTROLS,
+	GS_RULERMODE,
+	GS_RULERS,
+	GS_COLUMNBORDERS,
+	GS_LAYERMARKERS,
+	GS_BLEED,
+	GS_MAX
+};
+
+enum
+{
+	PAL_ALL,
+	PAL_PROPERTIES,
+	PAL_OUTLINE,
+	PAL_SCRAPBOOK,
+	PAL_LAYER,
+	PAL_PAGE,
+	PAL_BOOKMARK,
+	PAL_7_UNUSED,
+	PAL_UNDO,
+	PAL_VERIFIER,
+	PAL_DOWNLOADS,
+	PAL_TEXT,
+	PAL_MAX
+};
+
 #endif
 
-
