@@ -18,7 +18,7 @@ class SCRIBUS_API CreateRange : public QDialog, Ui::CreateRange
 {
 	Q_OBJECT
 	public:
-		CreateRange(QString currText, int pageCount, QWidget* parent, Qt::WFlags fl=0);
+		CreateRange(const QString& currText, int pageCount, QWidget* parent);
 		~CreateRange();
 		void getCreateRangeData(CreateRangeData&);
 		
@@ -33,7 +33,7 @@ class SCRIBUS_API CreateRange : public QDialog, Ui::CreateRange
 		void basicSelectRangeTypeEven();
 		void basicSelectRangeTypeOdd();
 		void basicSelectRangeType(int);
-		void advSpinChange(int);
+		void advSpinChange();
 		
 	protected:
 		int m_PageCount;
