@@ -24,20 +24,23 @@ for which a new license (GPL+exception) is in place.
 #include <algorithm>
 #include "util.h"
 #include <zlib.h>
-
+#include <qglobal.h>
+#include <QApplication>
 #include <QCryptographicHash>
 #include <QDomElement>
 #include <QMessageBox>
 #include <QProcess>
 
 #include "pageitem.h"
-#include "scribus.h"
+#include "pageitem_table.h"
+#include "scribusview.h"
 #include "scribusdoc.h"
 #include "scpainter.h"
+#include "ui/scmessagebox.h"
 
-#include <signal.h>
+#include <csignal>
 
-#if !defined(_WIN32) && !defined(Q_OS_MAC) && !defined(Q_OS_HAIKU) 
+#if !defined(_WIN32) && !defined(Q_OS_MAC) 
 #include <execinfo.h>
 #include <cxxabi.h>
 #endif
@@ -47,22 +50,16 @@ for which a new license (GPL+exception) is in place.
 
 using namespace std;
 
-void sDebug(QString message)
-{
-	qDebug("%s", message.toAscii().constData());
-}
-
 QString cleanupLang(const QString& lang)
 {
 	int dotIndex = lang.indexOf(QChar('.'));
 	if (dotIndex < 0)
 		return lang;
 
-	QString cleanLang = lang.left(dotIndex);
-	return cleanLang;
+	return lang.left(dotIndex);
 }
 
-int System(const QString exename, const QStringList & args, const QString fileStdErr, const QString fileStdOut, bool* cancel)
+int System(const QString& exename, const QStringList & args, const QString& fileStdErr, const QString& fileStdOut, const bool* cancel)
 {
 	QProcess proc;
 	if (!fileStdOut.isEmpty())
@@ -70,22 +67,21 @@ int System(const QString exename, const QStringList & args, const QString fileSt
 	if (!fileStdErr.isEmpty())
 		proc.setStandardErrorFile(fileStdErr);
 	proc.start(exename, args);
-	if (proc.waitForStarted(5000))
+	if (proc.waitForStarted(15000))
 	{
-		while (!proc.waitForFinished(5000))
+		while (!proc.waitForFinished(15000))
 		{
 			qApp->processEvents();
-			if (cancel && (*cancel == true))
+			if (cancel && (*cancel))
 			{
 				proc.kill();
 				break;
 			}
 		}
 	}
-	if (cancel && (*cancel == true))
+	if (cancel && (*cancel))
 		return -1;
-	int ex = proc.exitCode();
-	return ex;
+	return proc.exitCode();
 }
 
 // On Windows, return short path name, else return longPath;
@@ -128,32 +124,25 @@ QString getLongPathName(const QString & shortPath)
 	return longPath;
 }
 
-QString GetAttr(QDomElement *el, QString at, QString def)
-{
-	return el->attribute(at, def);
-}
-
-
 // Legacy implementation of LoadText with incorrect
 // handling of unicode data. This should be retired.
 // Use loadRawText instead.
 // FIXME XXX
 //
-bool loadText(QString filename, QString *Buffer)
+bool loadText(const QString& filename, QString *Buffer)
 {
 	QFile f(filename);
 	QFileInfo fi(f);
 	if (!fi.exists())
 		return false;
-	bool ret;
 	QByteArray bb(f.size(), ' ');
-	if (f.open(QIODevice::ReadOnly))
-	{
-		f.read(bb.data(), f.size());
-		f.close();
-		for (int posi = 0; posi < bb.size(); ++posi)
-			*Buffer += QChar(bb[posi]);
-		/*
+	if (!f.open(QIODevice::ReadOnly))
+		return false;
+	f.read(bb.data(), f.size());
+	f.close();
+	for (int i = 0; i < bb.size(); ++i)
+		*Buffer += QChar(bb[i]);
+	/*
 		int len = bb.size();
 		int oldLen = Buffer->length();
 		Buffer->setLength( oldLen + len + 1);
@@ -164,11 +153,7 @@ bool loadText(QString filename, QString *Buffer)
 		*ucsString++ = *data++;
 		*ucsString = 0;
 		*/
-		ret = true;
-	}
-	else
-		ret = false;
-	return ret;
+	return true;
 }
 
 bool loadRawText(const QString & filename, QByteArray & buf)
@@ -277,17 +262,17 @@ QByteArray CompressArray(const QByteArray& in)
 
 char *toAscii85( quint32 value, bool& allZero )
 {
-	int digit, i;
+	int digit;
 	static char asciiVal[6];
 	allZero = true;
-    for (i = 0; i < 5; ++i) 
+	for (int i = 0; i < 5; ++i)
 	{
 		digit = value % 85;
 		if (digit != 0)
 			allZero = false;
 		asciiVal[4-i] = digit + 33;
 		value = (value - digit) / 85;
-    }
+	}
 	asciiVal[5] = 0;
 	return asciiVal;
 }
@@ -313,11 +298,11 @@ char *toHex( uchar u )
 QString String2Hex(QString *in, bool lang)
 {
 	int i = 0;
-	QString out("");
-	for( int xi = 0; xi < in->length(); ++xi )
+	QString out;
+	for (int j = 0; j < in->length(); ++j)
 	{
 		// Qt4 .cell() added ???
-		out += toHex(QChar(in->at(xi)).cell());
+		out += toHex(QChar(in->at(j)).cell());
 		++i;
 		if ((i>40) && (lang))
 		{
@@ -328,23 +313,15 @@ QString String2Hex(QString *in, bool lang)
 	return out;
 }
 
-QByteArray ComputeMD5Sum(QByteArray *in)
-{
-	return QCryptographicHash::hash(*in, QCryptographicHash::Md5);
-//	QByteArray MDsum(16, ' ');
-//	md5_buffer (in->data(), in->size(), reinterpret_cast<void*>(MDsum.data()));
-//	return MDsum;
-}
-
-QString Path2Relative(QString Path, const QString& baseDir)
+QString Path2Relative(const QString& Path, const QString& baseDir)
 {
 	QDir d(baseDir);
 	return d.relativeFilePath(Path);
 }
 
-QString Relative2Path(QString File, const QString& baseDir)
+QString Relative2Path(const QString& File, const QString& baseDir)
 {
-	QString   absPath;
+	QString absPath;
 	QFileInfo fi(File);
 	if (File.isEmpty())
 		absPath = File;
@@ -367,16 +344,18 @@ QString Relative2Path(QString File, const QString& baseDir)
 // check if the file exists, if it does, ask if they're sure
 // return true if they're sure, else return false;
 
-bool overwrite(QWidget *parent, QString filename)
+bool overwrite(QWidget *parent, const QString& filename)
 {
 	bool retval = true;
 	QFileInfo fi(filename);
 	if (fi.exists())
 	{
 		QString fn = QDir::toNativeSeparators(filename);
-		int t = QMessageBox::warning(parent, QObject::tr("File exists"),
+		int t = ScMessageBox::warning(parent, QObject::tr("File exists"),
 									 "<qt>"+ QObject::tr("A file named '%1' already exists.<br/>Do you want to replace it with the file you are saving?").arg(fn) +"</qt>",
-											 QMessageBox::Ok | QMessageBox::Cancel, QMessageBox::Cancel);
+											 QMessageBox::Ok | QMessageBox::Cancel,
+											 QMessageBox::Cancel,	// GUI default
+											 QMessageBox::Ok);	// batch default
 		if (t == QMessageBox::Cancel)
 			retval = false;
 	}
@@ -395,26 +374,24 @@ void WordAndPara(PageItem* currItem, int *w, int *p, int *c, int *wN, int *pN, i
 	bool first = true;
 	PageItem *nextItem = currItem;
 	PageItem *nbl = currItem;
-	while (nextItem != 0)
+	while (nextItem != nullptr)
 	{
-		if (nextItem->prevInChain() != 0)
+		if (nextItem->prevInChain() != nullptr)
 			nextItem = nextItem->prevInChain();
 		else
 			break;
 	}
-	while (nextItem != 0)
+	while (nextItem != nullptr)
 	{
 		for (int a = qMax(nextItem->firstInFrame(),0); a <= nextItem->lastInFrame() && a < nextItem->itemText.length(); ++a)
 		{
 			QChar b = nextItem->itemText.text(a);
 			if (b == SpecialChars::PARSEP)
-			{
 				para++;
-			}
 			if ((!b.isLetterOrNumber()) && (Dat.isLetterOrNumber()) && (!first))
-			{
 				ww++;
-			}
+			if (b.isSurrogate())
+				++a;
 			cc++;
 			Dat = b;
 			first = false;
@@ -422,19 +399,18 @@ void WordAndPara(PageItem* currItem, int *w, int *p, int *c, int *wN, int *pN, i
 		nbl = nextItem;
 		nextItem = nextItem->nextInChain();
 	}
-	if (nbl->frameOverflows()) {
+	if (nbl->frameOverflows())
+	{
 		paraN++;
 		for (int a = nbl->lastInFrame()+1; a < nbl->itemText.length(); ++a)
 		{
 			QChar b = nbl->itemText.text(a);
 			if (b == SpecialChars::PARSEP)
-			{
 				paraN++;
-			}
 			if ((!b.isLetterOrNumber()) && (Dat.isLetterOrNumber()) && (!first))
-			{
 				wwN++;
-			}
+			if (b.isSurrogate())
+				++a;
 			ccN++;
 			Dat = b;
 			first = false;
@@ -463,18 +439,18 @@ void ReOrderText(ScribusDoc *currentDoc, ScribusView *view)
 	double savScale = view->scale();
 	view->setScale(1.0);
 	currentDoc->RePos = true;
-	QImage pgPix(10, 10, QImage::Format_ARGB32);
+	QImage pgPix(10, 10, QImage::Format_ARGB32_Premultiplied);
 	QRect rd; // = QRect(0,0,9,9);
 	ScPainter *painter = new ScPainter(&pgPix, pgPix.width(), pgPix.height());
-	for (int azz=0; azz<currentDoc->MasterItems.count(); ++azz)
+	for (auto it = currentDoc->MasterItems.begin(); it != currentDoc->MasterItems.end(); ++it)
 	{
-		PageItem *currItem = currentDoc->MasterItems.at(azz);
+		PageItem* currItem = *it;
 		if (currItem->itemType() == PageItem::PathText)
 			currItem->DrawObj(painter, rd);
 	}
-	for (int azz=0; azz<currentDoc->Items->count(); ++azz)
+	for (auto it = currentDoc->Items->begin(); it != currentDoc->Items->end(); ++it)
 	{
-		PageItem *currItem = currentDoc->Items->at(azz);
+		PageItem* currItem = *it;
 		currItem->layout();
 		if (currItem->itemType() == PageItem::PathText)
 			currItem->DrawObj(painter, rd); //FIXME: this should be replaced by code in layout()
@@ -490,11 +466,9 @@ void ReOrderText(ScribusDoc *currentDoc, ScribusView *view)
 \param s2 second string
 \retval bool t/f related s1>s2
  */
-bool compareQStrings(QString s1, QString s2)
+bool compareQStrings(const QString& s1, const QString& s2)
 {
-	if (QString::localeAwareCompare(s1, s2) >= 0)
-		return false;
-	return true;
+	return QString::localeAwareCompare(s1, s2) < 0;
 }
 
 QStringList sortQStringList(QStringList aList)
@@ -505,7 +479,7 @@ QStringList sortQStringList(QStringList aList)
 	for (it = aList.begin(); it != aList.end(); ++it)
 		sortList.push_back(*it);
 	std::sort(sortList.begin(), sortList.end(), compareQStrings);
-	for(uint i = 0; i < sortList.size(); i++)
+	for (uint i = 0; i < sortList.size(); i++)
 		retList.append(sortList[i]);
 	return retList;
 }
@@ -526,17 +500,21 @@ QString checkFileExtension(const QString &currName, const QString &extension)
 		newName+=extension.toLower();
 		return newName;
 	}
-	//If filename doesnt end with the period+extension, add it on
+	//If filename doesn't end with the period+extension, add it on
 	QString dotExt("." + extension.toLower());
 	if (!newName.endsWith(dotExt, Qt::CaseInsensitive))
 		newName+=dotExt;
 	return newName;
 }
 
-QString getFileNameByPage(ScribusDoc* currDoc, uint pageNo, QString extension)
+QString getFileNameByPage(ScribusDoc* currDoc, uint pageNo, const QString& extension, const QString& prefix)
 {
 	uint number = pageNo + currDoc->FirstPnum;
-	QString defaultName = currDoc->DocName;
+	QString defaultName;
+	if (!prefix.isNull())
+		defaultName=prefix;
+	else
+		defaultName=currDoc->documentFileName();
 	if (defaultName.isNull())
 		defaultName = "export";
 	else
@@ -547,28 +525,51 @@ QString getFileNameByPage(ScribusDoc* currDoc, uint pageNo, QString extension)
 	return QString("%1-%2%3.%4").arg(defaultName).arg(QObject::tr("page", "page export")).arg(number, 3, 10, QChar('0')).arg(extension);
 }
 
-const QString getStringFromSequence(DocumentSectionType type, uint position)
+const QString getStringFromSequence(NumFormat type, uint position, const QString& asterix)
 {
-	QString retVal("");
-	switch( type )
+	QString retVal;
+
+	const QString english("abcdefghijklmnopqrstuvwxyz");
+	const QString arabic("أبتثجحخدذرزسشصضطظعغفقكلمنهوي");
+	const QString abjad("أبجدهوزحطيكلمنسعفصقرشتثخذضظغ");
+
+	switch (type)
 	{
 		case Type_1_2_3:
 			retVal=QString::number(position);
 			break;
+		case Type_1_2_3_ar:
+			retVal=QLocale("ar").toString(position);
+			break;
 		case Type_A_B_C:
-			retVal=numberToLetterSequence(position).toUpper();
+			retVal = numberToLetterSequence(english, position).toUpper();
 			break;
 		case Type_a_b_c:
-			retVal=numberToLetterSequence(position);
+			retVal = numberToLetterSequence(english, position);
+			break;
+		case Type_Alphabet_ar:
+			retVal = numberToLetterSequence(arabic, position);
+			break;
+		case Type_Abjad_ar:
+			retVal = numberToLetterSequence(abjad, position);
+			break;
+		case Type_Hebrew:
+			retVal = numberToHebrew(position);
 			break;
 		case Type_I_II_III:
-			retVal=arabicToRoman(position);
+			retVal = numberToRoman(position);
 			break;
 		case Type_i_ii_iii:
 			//well, for lower case people will want that, even if its "wrong"
 			//ie, X=10, x=10000
-			retVal=arabicToRoman(position).toLower();
+			retVal = numberToRoman(position).toLower();
 			break;
+		case Type_asterix:
+			for (uint a=1; a <= position; ++a)
+				retVal.append(asterix);
+			break;
+		case Type_CJK:
+			retVal = numberToCJK(position);
 		case Type_None:
 			break;
 		default:
@@ -577,25 +578,32 @@ const QString getStringFromSequence(DocumentSectionType type, uint position)
 	return retVal;
 }
 
-const QString numberToLetterSequence(uint i)
+const QString numberToLetterSequence(const QString& letters, uint num)
 {
-	QString retVal("");
+	QString retVal;
 	unsigned digits = 1;
 	unsigned offset = 0;
-	uint column=i;
-	--column;
+	uint column = num - 1;
 
-	if( column > 4058115285U ) return  QString("@");
+	// FIXME: what is the heck is this?
+	if (column > 4058115285U)
+		return  QString("@");
 
-	for( unsigned limit = 26; column >= limit+offset; limit *= 26, digits++ )
+	for (unsigned limit = 28; column >= limit+offset; limit *= letters.length(), digits++)
 		offset += limit;
 
-	for( unsigned c = column - offset; digits; --digits, c/=26 )
-		retVal.prepend( QChar( 'a' + (c%26) ) );
+	for (unsigned c = column - offset; digits; --digits, c /= letters.length())
+	{
+		uint i = c % letters.length();
+		if (i < static_cast<uint>(letters.length()))
+			retVal.prepend(letters.at(i));
+		else
+			retVal.prepend(QChar::Null);
+	}
 	return retVal;
 }
 
-const QString arabicToRoman(uint i)
+const QString numberToRoman(uint i)
 {
 	QString roman("");
 	int arabic = i;
@@ -703,11 +711,10 @@ const QString arabicToRoman(uint i)
 }
 
 //CB Moved from scribus.cpp
-void parsePagesString(QString pages, std::vector<int>* pageNs, int sourcePageCount)
+void parsePagesString(const QString& pages, std::vector<int>* pageNs, int sourcePageCount)
 {
 	QString tmp(pages);
 	QString token;
-	int from, to, pageNr;
 	do
 	{
 		if (tmp.indexOf(",") == -1)
@@ -729,8 +736,8 @@ void parsePagesString(QString pages, std::vector<int>* pageNs, int sourcePageCou
 		}
 		else if (token.indexOf("-") != -1) // import a range of source doc pages
 		{
-			from = QString(token.left(token.indexOf("-"))).toInt();
-			to = QString(token.right(token.length() - token.indexOf("-") - 1)).toInt();
+			int from = QString(token.left(token.indexOf("-"))).toInt();
+			int to = QString(token.right(token.length() - token.indexOf("-") - 1)).toInt();
 			if ((from != 0) && (to != 0))
 			{
 				if (from > sourcePageCount)
@@ -753,23 +760,17 @@ void parsePagesString(QString pages, std::vector<int>* pageNs, int sourcePageCou
 		}
 		else // import single source doc page
 		{
-			pageNr = token.toInt();
+			int pageNr = token.toInt();
 			if ((pageNr > 0) && (pageNr <= sourcePageCount))
 				pageNs->push_back(pageNr);
 		}
 	} while (!tmp.isEmpty());
 }
 
-void tDebug(QString message)
-{
-	QDateTime debugTime;
-	qDebug("%s", QString("%1\t%2").arg(debugTime.currentDateTime().toString("hh:mm:ss:zzz")).arg(message).toAscii().constData());
-}
 
-
-QString readLinefromDataStream(QDataStream &s)
+QString readLineFromDataStream(QDataStream &s)
 {
-	QString ret = "";
+	QString ret;
 	uchar charData;
 	while (!s.atEnd())
 	{
@@ -789,7 +790,7 @@ QString readLinefromDataStream(QDataStream &s)
 	return ret.trimmed();
 }
 
-void setCurrentComboItem(QComboBox *box, QString text)
+void setCurrentComboItem(QComboBox *box, const QString& text)
 {
 	bool sigBlocked = box->blockSignals(true);
 	int ind = box->findText(text);
@@ -798,15 +799,63 @@ void setCurrentComboItem(QComboBox *box, QString text)
 	box->blockSignals(sigBlocked);
 }
 
+void setCurrentComboItemFromData(QComboBox *box, const QString& data)
+{
+	bool sigBlocked = box->blockSignals(true);
+	int ind = box->findData(data);
+	if (ind > -1)
+		box->setCurrentIndex(ind);
+	box->blockSignals(sigBlocked);
+}
+
+void removeComboItem(QComboBox *box, const QString& text)
+{
+	bool sigBlocked = box->blockSignals(true);
+	int ind = box->findText(text);
+	if (ind > -1)
+		box->removeItem(ind);
+	box->blockSignals(sigBlocked);
+}
+
+QString readAdobeUniCodeString(QDataStream &s)
+{
+	QString ret;
+	quint32 len;
+	s >> len;
+	for (quint32 i = 0; i < len; i++)
+	{
+		quint16 ch;
+		s >> ch;
+		if (ch != 0)
+			ret.append(QChar(ch));
+	}
+	return ret;
+}
+
+QString readAdobeUniCodeString16(QDataStream &s)
+{
+	QString ret;
+	quint16 len;
+	s >> len;
+	for (quint16 i = 0; i < len; i++)
+	{
+		quint16 ch;
+		s >> ch;
+		if (ch != 0)
+			ret.append(QChar(ch));
+	}
+	return ret;
+}
+
 QString getDashString(int dashtype, double linewidth)
 {
 	QString dashString;
 	QVector<double> dashArray;
 	getDashArray(dashtype, linewidth, dashArray);
-	for (int a = 0; a < dashArray.size(); ++a)
+	for (int i = 0; i < dashArray.size(); ++i)
 	{
-		dashString += QString::number(dashArray.at(a));
-		if (a < (dashArray.size() - 1))
+		dashString += QString::number(dashArray.at(i));
+		if (i < (dashArray.size() - 1))
 			dashString += " ";
 	}
 	return dashString;
@@ -817,9 +866,8 @@ void getDashArray(int dashtype, double linewidth, QVector<float> &dashArray)
 	QVector<double> tmp;
 	getDashArray(dashtype, linewidth, tmp);
 	dashArray.clear();
-	for (int i = 0; i < tmp.count(); ++i) {
+	for (int i = 0; i < tmp.count(); ++i)
 		dashArray << static_cast<float>(tmp[i]);
-	}
 }
 
 void getDashArray(int dashtype, double linewidth, QVector<double> &dashArray)
@@ -950,48 +998,311 @@ void getDashArray(int dashtype, double linewidth, QVector<double> &dashArray)
 	}
 }
 
-/**
- * Print a backtrace
- * Please never commit code that uses it.
- * @param nFrames specify how much frames you want to be printed
- */
-void printBacktrace ( int nFrames )
+bool convertOldTable(ScribusDoc *m_Doc, PageItem* gItem, QList<PageItem*> &gpL, QStack<QList<PageItem *> > *groupStackT, QList<PageItem *> *target)
 {
-#if !defined(_WIN32) && !defined(Q_OS_MAC) && !defined(Q_OS_OPENBSD)  && !defined(Q_OS_FREEBSD) && !defined(Q_OS_HAIKU)
-	void ** trace = new void*[nFrames + 1];
-	char **messages = ( char ** ) NULL;
-	int i, trace_size = 0;
+	QList<double> colWidths;
+	QList<double> rowHeights;
 
-	trace_size = backtrace ( trace, nFrames + 1 );
-	messages = backtrace_symbols ( trace, trace_size );
-	if ( messages )
+	// 1. Although this was not intended, legacy tables allowed user to link frames together
+	// New table do not support that, so if one frame has any link, we stop the conversion
+	// here, those frame will be converted to a standard group.
+	// 2. Pre-1.4.3 versions had a bug where item TopLink/LeftLink/BottomLink/RightLink were
+	// lost when copy/pasting tables. Exit conversion too so these broken tables can be
+	// converted to standard groups (at least until we find a good way to process that case)
+	bool hasTableLinks = false;
+	bool hasTextLinks = false;
+	for (int i = 0; i < gpL.count(); i++)
 	{
-		for ( i=1; i < trace_size; ++i )
-		{
-			QString msg ( messages[i] );
-			int sep1 ( msg.indexOf ( "(" ) );
-			int sep2 ( msg.indexOf ( "+" ) );
-			QString mName (	msg.mid ( sep1 + 1,sep2-sep1 -1) );
-			
-			QString name;
-			if(mName.startsWith("_Z"))
-			{
-				char* outbuf = 0; 
-				size_t length = 0;
-				int status = 0; 
-				outbuf = abi::__cxa_demangle(mName.trimmed().toAscii().data(), outbuf, &length, &status);
-				name = QString::fromAscii( outbuf,length );
-				if(!status)
-					free(outbuf);
-			}
-			else
-				name = mName;
-			QString bts ( "[ScBT] %1. %2" );
-			qDebug ("%s", bts.arg( i ).arg( name ).toUtf8().data() );
-		}
-		free ( messages );
+		PageItem* it = gpL[i];
+		it->isTableItem = false;
+		if (it->nextInChain() || it->prevInChain())
+			hasTextLinks = true;
+		if (it->m_leftLink || it->m_rightLink || it->m_bottomLink || it->m_topLink)
+			hasTableLinks = true;
 	}
-	delete[] trace;
-#endif
+
+	if (!hasTableLinks || hasTextLinks)
+		return false;
+
+	PageItem *topLeft = nullptr;
+	for (int i = 0; i < gpL.count(); i++)
+	{
+		PageItem* it = gpL[i];
+		if ((it->m_topLink == nullptr) && (it->m_leftLink == nullptr))	// we got the topleft item
+		{
+			topLeft = it;
+			PageItem *tl = it;
+			while (tl->m_rightLink != nullptr)
+			{
+				colWidths.append(tl->width());
+				tl = tl->m_rightLink;
+			}
+			colWidths.append(tl->width());
+			while (tl->m_bottomLink != nullptr)
+			{
+				rowHeights.append(tl->height());
+				tl = tl->m_bottomLink;
+			}
+			rowHeights.append(tl->height());
+			break;
+		}
+	}
+
+	// Check we have found enough rows and columns so that no item will disappear
+	if ((colWidths.count() * rowHeights.count()) < gpL.count())
+		return false;
+
+	m_Doc->dontResize = true;
+	int z = m_Doc->itemAdd(PageItem::Table, PageItem::Unspecified, gItem->xPos(), gItem->yPos(), gItem->width(), gItem->height(), 0.0, CommonStrings::None, CommonStrings::None);
+	PageItem_Table* currItem = m_Doc->Items->takeAt(z)->asTable();
+
+	currItem->m_layerID = gItem->m_layerID;
+	currItem->OwnPage = gItem->OwnPage;
+	currItem->OnMasterPage = gItem->OnMasterPage;
+
+	currItem->insertRows(0, rowHeights.count()-1);
+	m_Doc->dontResize = true;
+	currItem->insertColumns(0, colWidths.count()-1);
+	m_Doc->dontResize = true;
+	for (int i = 0; i < rowHeights.count(); i++)
+	{
+		currItem->resizeRow(i, rowHeights[i]);
+	}
+	m_Doc->dontResize = true;
+	for (int i = 0; i < colWidths.count(); i++)
+	{
+		currItem->resizeColumn(i, colWidths[i]);
+	}
+	TableBorder border(0.0, Qt::SolidLine, CommonStrings::None, 100);
+	currItem->setLeftBorder(border);
+	currItem->setTopBorder(border);
+	currItem->setRightBorder(border);
+	currItem->setBottomBorder(border);
+	m_Doc->dontResize = true;
+	PageItem *tr = topLeft;
+	int rowCount = 0;
+	int colCount = 0;
+	while (rowCount < rowHeights.count())
+	{
+		PageItem *tl = tr;
+		while (colCount < colWidths.count())
+		{
+			currItem->cellAt(rowCount, colCount).textFrame()->itemText = tl->itemText.copy();
+			currItem->cellAt(rowCount, colCount).setFillColor(tl->fillColor());
+			currItem->cellAt(rowCount, colCount).setFillShade(tl->fillShade());
+			currItem->cellAt(rowCount, colCount).setLeftBorder(border);
+			currItem->cellAt(rowCount, colCount).setTopBorder(border);
+			currItem->cellAt(rowCount, colCount).setRightBorder(border);
+			currItem->cellAt(rowCount, colCount).setBottomBorder(border);
+			if ((tl->lineColor() != CommonStrings::None) && (tl->lineWidth() != 0.0))
+			{
+				TableBorder bb(tl->lineWidth(), tl->lineStyle(), tl->lineColor(), tl->lineShade());
+				if (tl->LeftLine)
+					currItem->cellAt(rowCount, colCount).setLeftBorder(bb);
+				if (tl->TopLine)
+					currItem->cellAt(rowCount, colCount).setTopBorder(bb);
+				if (tl->RightLine)
+					currItem->cellAt(rowCount, colCount).setRightBorder(bb);
+				if (tl->BottomLine)
+					currItem->cellAt(rowCount, colCount).setBottomBorder(bb);
+			}
+			if (colCount == colWidths.count()-1)
+				break;
+			colCount++;
+			tl = tl->m_rightLink;
+		}
+		if (rowCount == rowHeights.count()-1)
+			break;
+		colCount = 0;
+		rowCount++;
+		tr = tr->m_bottomLink;
+	}
+	m_Doc->dontResize = true;
+	currItem->setLayer(gItem->m_layerID);
+	currItem->setMasterPage(gItem->OwnPage, gItem->OnMasterPage);
+	currItem->adjustFrameToTable();
+	if (target != nullptr)
+	{
+		int ind = target->indexOf(gItem);
+		target->replace(ind, currItem);
+	}
+	else
+	{
+		int ind = m_Doc->FrameItems.key(gItem);
+		m_Doc->FrameItems.remove(ind);
+		m_Doc->FrameItems.insert(ind, currItem);
+	}
+	if (groupStackT != nullptr)
+	{
+		if (groupStackT->count() > 0)
+		{
+			int ii = groupStackT->top().indexOf(gItem);
+			if (ii >= 0)
+				groupStackT->top().replace(ii, currItem);
+		}
+	}
+	while (!gpL.isEmpty())
+	{
+		PageItem* item = gpL.takeFirst();
+		if (item->asTextFrame())
+			item->dropLinks();
+		delete item;
+	}
+	delete gItem;
+
+	return true;
 }
 
+void setWidgetBoldFont(QWidget* w, bool wantBold)
+{
+	QFont f(w->font());
+	f.setBold(wantBold);
+	w->setFont(f);
+}
+
+void getUniqueName(QString &name, const QStringList& list, const QString& separator, bool prepend)
+{
+	if (!list.contains(name))
+		return;
+	int token = -1;
+	QString newName;
+	do {
+		token++;
+		if (prepend)
+			newName = separator + QString::number(token) + " " + name;
+		else
+			newName = name + separator + QString::number(token);
+	} while (list.contains(newName));
+	name = newName;
+}
+
+const QString numberToHebrew(uint i)
+{
+	const QString hebrew("אבגדהוזחטיכלמנסעפצקרשת");
+	QString result;
+
+	if (i > 999)
+	{
+		result.append(numberToHebrew(i / 1000));
+		result.append(QChar(0x05F3));
+		i %= 1000;
+	}
+
+	int hundreds = i / 100;
+	int tens = (i - hundreds * 100) / 10;
+	int ones = i % 10;
+
+	while (hundreds > 4)
+	{
+		result.append(hebrew.at(21));
+		hundreds -= 4;
+	}
+
+	if (hundreds)
+		result.append(hebrew.at(hundreds + 17));
+
+	if (tens == 1 && ones == 5)
+		result.append("טו");
+	else if (tens == 1 && ones == 6)
+		result.append("טז");
+	else
+	{
+		if (tens)
+			result.append(hebrew.at(tens + 8));
+		if (ones)
+			result.append(hebrew.at(ones - 1));
+	}
+
+	return result;
+}
+
+const QString numberToCJK(uint i)
+{
+	QString result;
+	if (i<10)
+		result = QString(cjkDigit(i));
+
+	if (i>9 && i<=99)
+	{
+		int tens=i/10;
+		int ones=i%10;
+		if (tens!=1)
+			result.append(cjkDigit(tens));
+		result.append(cjkDigit(10));
+		if (ones!=0)
+			result.append(cjkDigit(ones));
+	}
+
+	if (i>99 && i<=999)
+	{
+		int hundreds=i/100;
+		int tens=(i-hundreds*100)/10;
+		int ones=i%10;
+		result.append(cjkDigit(hundreds));
+		result.append(cjkDigit(100));
+		if (tens!=0)
+		{
+			result.append(cjkDigit(tens));
+			result.append(cjkDigit(10));
+		}
+		else if (ones!=0)
+			result.append(cjkDigit(0));
+		if (ones!=0)
+			result.append(cjkDigit(ones));
+	}
+	return result;
+}
+
+
+QChar cjkDigit(uint i)
+{
+	switch (i)
+	{
+		case 0:
+			return QChar(0x96f6);
+			break;
+		case 1:
+			return QChar(0x4e00);
+			break;
+		case 2:
+			return QChar(0x4e8c);
+			break;
+		case 3:
+			return QChar(0x4e09);
+			break;
+		case 4:
+			return QChar(0x56db);
+			break;
+		case 5:
+			return QChar(0x4e94);
+			break;
+		case 6:
+			return QChar(0x516d);
+			break;
+		case 7:
+			return QChar(0x4e03);
+			break;
+		case 8:
+			return QChar(0x516b);
+			break;
+		case 9:
+			return QChar(0x4e5d);
+			break;
+		case 10:
+			return QChar(0x5341);
+			break;
+		case 100:
+			return QChar(0x767e);
+			break;
+		case 1000:
+			return QChar(0x5343);
+			break;
+		case 10000:
+			return QChar(0x842c);
+			break;
+		case 100000000:
+			return QChar(0x5104);
+			break;
+	}
+	return QChar::Null;
+}
